@@ -1,7 +1,7 @@
 #ifndef __Module_H
 #define __Module_H
 /*
- * Copyright (c) 1998-2004 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2008 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,23 +18,24 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: Module.h,v 1.37 2004/06/13 04:56:53 steve Exp $"
-#endif
+
 
 # include  <list>
 # include  <map>
-# include  "svector.h"
+# include  <vector>
+# include  <utility>
 # include  "StringHeap.h"
 # include  "HName.h"
 # include  "named.h"
+# include  "PScope.h"
 # include  "LineInfo.h"
 # include  "netlist.h"
-class PData;
-class PEvent;
+# include  "pform_types.h"
 class PExpr;
 class PEIdent;
 class PGate;
+class PGenerate;
+class PSpecPath;
 class PTask;
 class PFunction;
 class PWire;
@@ -48,7 +49,7 @@ class NetScope;
  * therefore the handle for grasping the described circuit.
  */
 
-class Module : public LineInfo {
+class Module : public PScope, public LineInfo {
 
 	/* The module ports are in general a vector of port_t
 	   objects. Each port has a name and an ordered list of
@@ -58,7 +59,7 @@ class Module : public LineInfo {
     public:
       struct port_t {
 	    perm_string name;
-	    svector<PEIdent*> expr;
+	    vector<PEIdent*> expr;
       };
 
     public:
@@ -67,21 +68,14 @@ class Module : public LineInfo {
       explicit Module(perm_string name);
       ~Module();
 
+	/* Initially false. This is set to true if the module has been
+	   declared as a library module. This makes the module
+	   ineligible for being chosen as an implicit root. It has no
+	   other effect. */
+      bool library_flag;
+
       NetNet::Type default_nettype;
 
-	/* The module has parameters that are evaluated when the
-	   module is elaborated. During parsing, I put the parameters
-	   into this map. */
-      struct param_expr_t {
-	    PExpr*expr;
-	    PExpr*msb;
-	    PExpr*lsb;
-	    bool signed_flag;
-      };
-      map<perm_string,param_expr_t>parameters;
-      map<perm_string,param_expr_t>localparams;
-
-
 	/* specparams are simpler then other params, in that they have
 	   no type information. They are merely constant
 	   expressions. */
@@ -91,7 +85,8 @@ class Module : public LineInfo {
 	   new parameters within the module, but may be used to set
 	   values within this module (when instantiated) or in other
 	   instantiated modules. */
-      map<hname_t,PExpr*>defparms;
+      typedef pair<pform_name_t,PExpr*> named_expr_t;
+      list<named_expr_t>defparms;
 
         /* Parameters may be overridden at instantiation time;
            the overrides do not contain explicit parameter names,
@@ -103,13 +98,7 @@ class Module : public LineInfo {
 
 	/* This is an array of port descriptors, which is in turn a
 	   named array of PEident pointers. */
-      svector<port_t*> ports;
-
-	/* Keep a table of named events declared in the module. */
-      map<perm_string,PEvent*>events;
-
-	/* Keep a table of datum variables declared in the module. */
-      map<hname_t,PData*>datum;
+      vector<port_t*> ports;
 
       map<perm_string,PExpr*> attributes;
 
@@ -117,95 +106,50 @@ class Module : public LineInfo {
 	   set by the `timescale directive. */
       int time_unit, time_precision;
 
-      perm_string mod_name() const { return name_; }
+	/* Task definitions within this module */
+      map<perm_string,PTask*> tasks;
+      map<perm_string,PFunction*> funcs;
+
+	/* The module has a list of genvars that may be used in
+	   various generate schemes. */
+      map<perm_string,LineInfo*> genvars;
+
+	/* the module has a list of generate schemes that appear in
+	   the module definition. These are used at elaboration time. */
+      list<PGenerate*> generate_schemes;
+
+      list<PSpecPath*> specify_paths;
+
+	// The mod_name() is the name of the module type.
+      perm_string mod_name() const { return pscope_name(); }
 
       void add_gate(PGate*gate);
 
-	// The add_wire method adds a wire by name, but only if the
-	// wire name doesn't already exist. Either way, the result is
-	// the existing wire or the pointer passed in.
-      PWire* add_wire(PWire*wire);
-
-      void add_behavior(PProcess*behave);
-      void add_task(perm_string name, PTask*def);
-      void add_function(perm_string name, PFunction*def);
-
       unsigned port_count() const;
-      const svector<PEIdent*>& get_port(unsigned idx) const;
+      const vector<PEIdent*>& get_port(unsigned idx) const;
       unsigned find_port(const char*name) const;
 
-	// Find a wire by name. This is used for connecting gates to
-	// existing wires, etc.
-      PWire* get_wire(const hname_t&name) const;
       PGate* get_gate(perm_string name);
 
-      const map<hname_t,PWire*>& get_wires() const;
       const list<PGate*>& get_gates() const;
-      const list<PProcess*>& get_behaviors() const;
 
       void dump(ostream&out) const;
       bool elaborate(Design*, NetScope*scope) const;
 
-      bool elaborate_scope(Design*, NetScope*scope) const;
+      typedef map<perm_string,NetExpr*> replace_t;
+      bool elaborate_scope(Design*, NetScope*scope, const replace_t&rep);
 
       bool elaborate_sig(Design*, NetScope*scope) const;
 
     private:
-      perm_string name_;
-
-      map<hname_t,PWire*> wires_;
       list<PGate*> gates_;
-      list<PProcess*> behaviors_;
-      map<perm_string,PTask*> tasks_;
-      map<perm_string,PFunction*> funcs_;
+
+      static void elaborate_parm_item_(perm_string name, const param_expr_t&cur,
+				       Design*des, NetScope*scope);
 
     private: // Not implemented
       Module(const Module&);
       Module& operator= (const Module&);
 };
 
-
-/*
- * $Log: Module.h,v $
- * Revision 1.37  2004/06/13 04:56:53  steve
- *  Add support for the default_nettype directive.
- *
- * Revision 1.36  2004/05/25 19:21:06  steve
- *  More identifier lists use perm_strings.
- *
- * Revision 1.35  2004/02/20 18:53:33  steve
- *  Addtrbute keys are perm_strings.
- *
- * Revision 1.34  2004/02/20 06:22:56  steve
- *  parameter keys are per_strings.
- *
- * Revision 1.33  2004/02/18 17:11:54  steve
- *  Use perm_strings for named langiage items.
- *
- * Revision 1.32  2003/06/20 00:53:19  steve
- *  Module attributes from the parser
- *  through to elaborated form.
- *
- * Revision 1.31  2003/06/13 19:10:45  steve
- *  Properly manage real variables in subscopes.
- *
- * Revision 1.30  2003/03/06 04:37:12  steve
- *  lex_strings.add module names earlier.
- *
- * Revision 1.29  2003/02/27 06:45:11  steve
- *  specparams as far as pform.
- *
- * Revision 1.28  2003/01/26 21:15:58  steve
- *  Rework expression parsing and elaboration to
- *  accommodate real/realtime values and expressions.
- *
- * Revision 1.27  2002/08/19 02:39:16  steve
- *  Support parameters with defined ranges.
- *
- * Revision 1.26  2002/08/12 01:34:58  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.25  2002/05/19 23:37:28  steve
- *  Parse port_declaration_lists from the 2001 Standard.
- */
 #endif
