@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2007 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,28 +16,21 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: PWire.cc,v 1.10 2002/08/12 01:34:58 steve Exp $"
-#endif
 
 # include "config.h"
 # include  "PWire.h"
+# include  "PExpr.h"
 # include  <assert.h>
 
-PWire::PWire(const hname_t&n, NetNet::Type t, NetNet::PortType pt)
-: hname_(n), type_(t), port_type_(pt), signed_(false), isint_(false),
-lidx_(0), ridx_(0)
-{
-      if (t == NetNet::INTEGER) {
-	    type_ = NetNet::REG;
-	    signed_ = true;
-	    isint_ = true;
-      }
-}
-
-PWire::PWire(char*n, NetNet::Type t, NetNet::PortType pt)
-: hname_(n), type_(t), port_type_(pt), signed_(false), isint_(false),
-lidx_(0), ridx_(0)
+PWire::PWire(perm_string n,
+	     NetNet::Type t,
+	     NetNet::PortType pt,
+	     ivl_variable_type_t dt)
+: name_(n), type_(t), port_type_(pt), data_type_(dt),
+  signed_(false), isint_(false),
+  port_msb_(0), port_lsb_(0), port_set_(false),
+  net_msb_(0), net_lsb_(0), net_set_(false), error_cnt_(0),
+    lidx_(0), ridx_(0), discipline_(0)
 {
       if (t == NetNet::INTEGER) {
 	    type_ = NetNet::REG;
@@ -51,9 +44,9 @@ NetNet::Type PWire::get_wire_type() const
       return type_;
 }
 
-const hname_t& PWire::path() const
+perm_string PWire::basename() const
 {
-      return hname_;
+      return name_;
 }
 
 bool PWire::set_wire_type(NetNet::Type t)
@@ -108,6 +101,25 @@ bool PWire::set_port_type(NetNet::PortType pt)
       }
 }
 
+bool PWire::set_data_type(ivl_variable_type_t dt)
+{
+      if (data_type_ != IVL_VT_NO_TYPE) {
+	    if (data_type_ != dt)
+		  return false;
+	    else
+		  return true;
+      }
+
+      assert(data_type_ == IVL_VT_NO_TYPE);
+      data_type_ = dt;
+      return true;
+}
+
+ivl_variable_type_t PWire::get_data_type() const
+{
+      return data_type_;
+}
+
 void PWire::set_signed(bool flag)
 {
       signed_ = flag;
@@ -123,56 +135,76 @@ bool PWire::get_isint() const
       return isint_;
 }
 
-void PWire::set_range(PExpr*m, PExpr*l)
+void PWire::set_range(PExpr*m, PExpr*l, PWSRType type)
 {
-      msb_ = svector<PExpr*>(msb_,m);
-      lsb_ = svector<PExpr*>(lsb_,l);
+      switch (type) {
+	  case SR_PORT:
+	    if (port_set_) {
+		  cerr << get_fileline() << ": error: Port ``" << name_
+		       << "'' has already been declared a port." << endl;
+		  error_cnt_ += 1;
+	    } else {
+		  port_msb_ = m;
+		  port_lsb_ = l;
+		  port_set_ = true;
+	    }
+	    return;
+
+	  case SR_NET:
+	    if (net_set_) {
+		  cerr << get_fileline() << ": error: Net ``" << name_
+		       << "'' has already been declared." << endl;
+		  error_cnt_ += 1;
+	    } else {
+		  net_msb_ = m;
+		  net_lsb_ = l;
+		  net_set_ = true;
+	    }
+	    return;
+
+	  case SR_BOTH:
+	    if (port_set_ || net_set_) {
+		  if (port_set_) {
+		        cerr << get_fileline() << ": error: Port ``" << name_
+		             << "'' has already been declared a port." << endl;
+		        error_cnt_ += 1;
+		  }
+		  if (net_set_) {
+		        cerr << get_fileline() << ": error: Net ``" << name_
+		             << "'' has already been declared." << endl;
+		        error_cnt_ += 1;
+		  }
+	    } else {
+		  port_msb_ = m;
+		  port_lsb_ = l;
+		  port_set_ = true;
+		  net_msb_ = m;
+		  net_lsb_ = l;
+		  net_set_ = true;
+	    }
+	    return;
+      }
 }
 
 void PWire::set_memory_idx(PExpr*ldx, PExpr*rdx)
 {
-      assert(lidx_ == 0);
-      assert(ridx_ == 0);
-      assert(type_ == NetNet::REG);
-      lidx_ = ldx;
-      ridx_ = rdx;
+      if (lidx_ != 0 || ridx_ != 0) {
+	    cerr << get_fileline() << ": error: Array ``" << name_
+	         << "'' has already been declared." << endl;
+	    error_cnt_ += 1;
+      } else {
+            lidx_ = ldx;
+            ridx_ = rdx;
+      }
 }
 
-/*
- * $Log: PWire.cc,v $
- * Revision 1.10  2002/08/12 01:34:58  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.9  2002/06/21 04:59:35  steve
- *  Carry integerness throughout the compilation.
- *
- * Revision 1.8  2002/01/26 05:28:28  steve
- *  Detect scalar/vector declarion mismatch.
- *
- * Revision 1.7  2001/12/03 04:47:14  steve
- *  Parser and pform use hierarchical names as hname_t
- *  objects instead of encoded strings.
- *
- * Revision 1.6  2001/07/25 03:10:48  steve
- *  Create a config.h.in file to hold all the config
- *  junk, and support gcc 3.0. (Stephan Boettcher)
- *
- * Revision 1.5  2001/01/06 02:29:35  steve
- *  Support arrays of integers.
- *
- * Revision 1.4  2000/12/11 00:31:43  steve
- *  Add support for signed reg variables,
- *  simulate in t-vvm signed comparisons.
- *
- * Revision 1.3  2000/02/23 02:56:54  steve
- *  Macintosh compilers do not support ident.
- *
- * Revision 1.2  1999/09/10 05:02:09  steve
- *  Handle integers at task parameters.
- *
- * Revision 1.1  1999/06/17 05:34:42  steve
- *  Clean up interface of the PWire class,
- *  Properly match wire ranges.
- *
- */
+void PWire::set_discipline(ivl_discipline_t d)
+{
+      assert(discipline_ == 0);
+      discipline_ = d;
+}
 
+ivl_discipline_t PWire::get_discipline(void) const
+{
+      return discipline_;
+}
