@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2006 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2008 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,17 +16,15 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: cprop.cc,v 1.47.2.7 2007/02/26 19:51:38 steve Exp $"
-#endif
 
 # include "config.h"
 
+# include  <cstdlib>
 # include  "netlist.h"
 # include  "netmisc.h"
 # include  "functor.h"
 # include  "compiler.h"
-# include  <assert.h>
+# include  "ivl_assert.h"
 
 
 
@@ -48,9 +46,6 @@ struct cprop_functor  : public functor_t {
       virtual void lpm_ff(Design*des, NetFF*obj);
       virtual void lpm_logic(Design*des, NetLogic*obj);
       virtual void lpm_mux(Design*des, NetMux*obj);
-      virtual void lpm_mux_large(Design*des, NetMux*obj);
-      virtual void lpm_ram_dq(Design*des, NetRamDq*obj);
-      bool lpm_ram_dq_const_address_(Design*des, NetRamDq*obj);
 };
 
 void cprop_functor::signal(Design*des, NetNet*obj)
@@ -59,86 +54,6 @@ void cprop_functor::signal(Design*des, NetNet*obj)
 
 void cprop_functor::lpm_add_sub(Design*des, NetAddSub*obj)
 {
-	// For now, only additions are handled.
-      if (obj->attribute(perm_string::literal("LPM_Direction")) != verinum("ADD"))
-	    return;
-
-	// If the low bit on the A side is 0, then eliminate it from
-	// the adder, and pass the B side directly to the
-	// result. Don't reduce the adder smaller then a 1-bit
-	// adder. These will be eliminated later.
-      while ((obj->width() > 1)
-	     && obj->pin_DataA(0).nexus()->drivers_constant()
-	     && (obj->pin_DataA(0).nexus()->driven_value() == verinum::V0)) {
-
-	    NetAddSub*tmp = 0;
-	    tmp = new NetAddSub(obj->scope(), obj->name(), obj->width()-1);
-	      //connect(tmp->pin_Aclr(), obj->pin_Aclr());
-	      //connect(tmp->pin_Add_Sub(), obj->pin_Add_Sub());
-	      //connect(tmp->pin_Clock(), obj->pin_Clock());
-	      //connect(tmp->pin_Cin(), obj->pin_Cin());
-	    connect(tmp->pin_Cout(), obj->pin_Cout());
-	      //connect(tmp->pin_Overflow(), obj->pin_Overflow());
-	    for (unsigned idx = 0 ;  idx < tmp->width() ;  idx += 1) {
-		  connect(tmp->pin_DataA(idx), obj->pin_DataA(idx+1));
-		  connect(tmp->pin_DataB(idx), obj->pin_DataB(idx+1));
-		  connect(tmp->pin_Result(idx), obj->pin_Result(idx+1));
-	    }
-	    connect(obj->pin_Result(0), obj->pin_DataB(0));
-	    delete obj;
-	    des->add_node(tmp);
-	    obj = tmp;
-	    count += 1;
-      }
-
-	// Now do the same thing on the B side.
-      while ((obj->width() > 1)
-	     && obj->pin_DataB(0).nexus()->drivers_constant()
-	     && (obj->pin_DataB(0).nexus()->driven_value() == verinum::V0)) {
-
-	    NetAddSub*tmp = 0;
-	    tmp = new NetAddSub(obj->scope(), obj->name(), obj->width()-1);
-	      //connect(tmp->pin_Aclr(), obj->pin_Aclr());
-	      //connect(tmp->pin_Add_Sub(), obj->pin_Add_Sub());
-	      //connect(tmp->pin_Clock(), obj->pin_Clock());
-	      //connect(tmp->pin_Cin(), obj->pin_Cin());
-	    connect(tmp->pin_Cout(), obj->pin_Cout());
-	      //connect(tmp->pin_Overflow(), obj->pin_Overflow());
-	    for (unsigned idx = 0 ;  idx < tmp->width() ;  idx += 1) {
-		  connect(tmp->pin_DataA(idx), obj->pin_DataA(idx+1));
-		  connect(tmp->pin_DataB(idx), obj->pin_DataB(idx+1));
-		  connect(tmp->pin_Result(idx), obj->pin_Result(idx+1));
-	    }
-	    connect(obj->pin_Result(0), obj->pin_DataA(0));
-	    delete obj;
-	    des->add_node(tmp);
-	    obj = tmp;
-	    count += 1;
-      }
-
-	// If the adder is only 1 bit wide, then replace it with the
-	// simple logic gate.
-      if (obj->width() == 1) {
-	    NetLogic*tmp;
-	    if (obj->pin_Cout().is_linked()) {
-		  tmp = new NetLogic(obj->scope(),
-				     obj->scope()->local_symbol(),
-				     3, NetLogic::AND);
-		  connect(tmp->pin(0), obj->pin_Cout());
-		  connect(tmp->pin(1), obj->pin_DataA(0));
-		  connect(tmp->pin(2), obj->pin_DataB(0));
-		  des->add_node(tmp);
-	    }
-	    tmp = new NetLogic(obj->scope(), obj->name(), 3, NetLogic::XOR);
-	    connect(tmp->pin(0), obj->pin_Result(0));
-	    connect(tmp->pin(1), obj->pin_DataA(0));
-	    connect(tmp->pin(2), obj->pin_DataB(0));
-	    delete obj;
-	    des->add_node(tmp);
-	    count += 1;
-	    return;
-      }
-
 }
 
 void cprop_functor::lpm_compare(Design*des, NetCompare*obj)
@@ -157,6 +72,8 @@ void cprop_functor::lpm_compare(Design*des, NetCompare*obj)
 
 void cprop_functor::lpm_compare_eq_(Design*des, NetCompare*obj)
 {
+#if 0
+	/* XXXX Need to reimplement this code to account for vectors. */
       NetScope*scope = obj->scope();
 
       unsigned const_count = 0;
@@ -252,7 +169,7 @@ void cprop_functor::lpm_compare_eq_(Design*des, NetCompare*obj)
 	   with a simple XOR gate. */
       if (top == 1) {
 	    NetLogic*tmp = new NetLogic(scope, obj->name(), 3,
-					NetLogic::XNOR);
+					NetLogic::XNOR, 1);
 	    connect(tmp->pin(0), obj->pin_AEB());
 	    connect(tmp->pin(1), obj->pin_DataA(0));
 	    connect(tmp->pin(2), obj->pin_DataB(0));
@@ -275,6 +192,7 @@ void cprop_functor::lpm_compare_eq_(Design*des, NetCompare*obj)
       delete obj;
       des->add_node(tmp);
       count += 1;
+#endif
 }
 
 void cprop_functor::lpm_ff(Design*des, NetFF*obj)
@@ -282,63 +200,30 @@ void cprop_functor::lpm_ff(Design*des, NetFF*obj)
 	// Look for and count unlinked FF outputs. Note that if the
 	// Data and Q pins are connected together, they can be removed
 	// from the circuit, since it doesn't do anything.
-      unsigned unlinked_count = 0;
-      for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1) {
-	    if (connected(obj->pin_Data(idx), obj->pin_Q(idx))
-		&& (! obj->pin_Sclr().is_linked())
-		&& (! obj->pin_Sset().is_linked())
-		&& (! obj->pin_Aclr().is_linked())
-		&& (! obj->pin_Aset().is_linked())) {
-		  obj->pin_Data(idx).unlink();
-		  obj->pin_Q(idx).unlink();
-	    }
-	    if (! obj->pin_Q(idx).is_linked())
-		  unlinked_count += 1;
-      }
 
-	// If the entire FF is unlinked, remove the whole thing.
-      if (unlinked_count == obj->width()) {
+      if (connected(obj->pin_Data(), obj->pin_Q())
+	  && (! obj->pin_Sclr().is_linked())
+	  && (! obj->pin_Sset().is_linked())
+	  && (! obj->pin_Aclr().is_linked())
+	  && (! obj->pin_Aset().is_linked())) {
+	    obj->pin_Data().unlink();
+	    obj->pin_Q().unlink();
 	    delete obj;
-	    count += 1;
-	    return;
-      }
-
-	// If some of the FFs are unconnected, make a new FF array
-	// that does not include the useless FF devices.
-      if (unlinked_count > 0) {
-	    NetFF*tmp = new NetFF(obj->scope(), obj->name(),
-				  obj->width()-unlinked_count);
-	    connect(tmp->pin_Clock(), obj->pin_Clock());
-	    connect(tmp->pin_Enable(), obj->pin_Enable());
-	    connect(tmp->pin_Aload(), obj->pin_Aload());
-	    connect(tmp->pin_Aset(), obj->pin_Aset());
-	    connect(tmp->pin_Aclr(), obj->pin_Aclr());
-	    connect(tmp->pin_Sload(), obj->pin_Sload());
-	    connect(tmp->pin_Sset(), obj->pin_Sset());
-	    connect(tmp->pin_Sclr(), obj->pin_Sclr());
-
-	    unsigned tidx = 0;
-	    for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1)
-		  if (obj->pin_Q(idx).is_linked()) {
-			connect(tmp->pin_Data(tidx), obj->pin_Data(idx));
-			connect(tmp->pin_Q(tidx), obj->pin_Q(idx));
-			tidx += 1;
-		  }
-
-	    assert(tidx == obj->width() - unlinked_count);
-	    delete obj;
-	    des->add_node(tmp);
-	    count += 1;
-	    return;
       }
 }
 
 void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 {
+#if 0
       NetScope*scope = obj->scope();
+#endif
 
       switch (obj->type()) {
-
+#if 0
+	      /* XXXX This old code assumed that the individual bit
+		 slices could be replaced with different gates. They
+		 cannot when the device takes atomic vectors, so this
+		 needs to be rewritten. XXXX */
 	  case NetLogic::NAND:
 	  case NetLogic::AND: {
 		unsigned top = obj->pin_count();
@@ -453,12 +338,12 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 			  case NetLogic::AND:
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::BUF);
+					       NetLogic::BUF, 1);
 			    break;
 			  case NetLogic::NAND:
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::NOT);
+					       NetLogic::NOT, 1);
 			    break;
 			  default:
 			    assert(0);
@@ -484,7 +369,7 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		if (top < obj->pin_count()) {
 		      NetLogic*tmp = new NetLogic(scope,
 						  obj->name(), top,
-						  obj->type());
+						  obj->type(), 1);
 		      tmp->rise_time(obj->rise_time());
 		      tmp->fall_time(obj->fall_time());
 		      tmp->decay_time(obj->decay_time());
@@ -501,7 +386,12 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		}
 		break;
 	  }
-
+#endif
+#if 0
+	      /* XXXX This old code assumed that the individual bit
+		 slices could be replaced with different gates. They
+		 cannot when the device takes atomic vectors, so this
+		 needs to be rewritten. XXXX */
 
 	  case NetLogic::NOR:
 	  case NetLogic::OR: {
@@ -601,12 +491,12 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 			  case NetLogic::OR:
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::BUF);
+					       NetLogic::BUF, 1);
 			    break;
 			  case NetLogic::NOR:
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::NOT);
+					       NetLogic::NOT, 1);
 			    break;
 			  default:
 			    assert(0);
@@ -623,7 +513,7 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		      delete obj;
 		      count += 1;
 		      return;
-		}
+v		}
 
 		  /* Finally, this cleans up the gate by creating a
 		     new [N]OR gate that has the right number of
@@ -631,7 +521,7 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		if (top < obj->pin_count()) {
 		      NetLogic*tmp = new NetLogic(scope,
 						  obj->name(), top,
-						  obj->type());
+						  obj->type(), 1);
 		      tmp->rise_time(obj->rise_time());
 		      tmp->fall_time(obj->fall_time());
 		      tmp->decay_time(obj->decay_time());
@@ -648,7 +538,12 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		}
 		break;
 	  }
-
+#endif
+#if 0
+	      /* XXXX This old code assumed that the individual bit
+		 slices could be replaced with different gates. They
+		 cannot when the device takes atomic vectors, so this
+		 needs to be rewritten. XXXX */
 	  case NetLogic::XNOR:
 	  case NetLogic::XOR: {
 		unsigned top = obj->pin_count();
@@ -770,11 +665,11 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		      if (obj->type() == NetLogic::XOR)
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::NOT);
+					       NetLogic::NOT, 1);
 		      else
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::BUF);
+					       NetLogic::BUF, 1);
 
 		      tmp->rise_time(obj->rise_time());
 		      tmp->fall_time(obj->fall_time());
@@ -799,11 +694,11 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		      if (obj->type() == NetLogic::XOR)
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::BUF);
+					       NetLogic::BUF, 1);
 		      else
 			    tmp = new NetLogic(scope,
 					       obj->name(), 2,
-					       NetLogic::NOT);
+					       NetLogic::NOT, 1);
 
 		      tmp->rise_time(obj->rise_time());
 		      tmp->fall_time(obj->fall_time());
@@ -825,7 +720,7 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		if (top < obj->pin_count()) {
 		      NetLogic*tmp = new NetLogic(scope,
 						  obj->name(), top,
-						  obj->type());
+						  obj->type(), 1);
 		      des->add_node(tmp);
 		      tmp->pin(0).drive0(obj->pin(0).drive0());
 		      tmp->pin(0).drive1(obj->pin(0).drive1());
@@ -838,386 +733,121 @@ void cprop_functor::lpm_logic(Design*des, NetLogic*obj)
 		}
 		break;
 	    }
-
+#endif
 	  default:
 	    break;
       }
 }
 
+#if 0
+static void replace_with_mos(Design*des, NetMux*obj, NetLogic::TYPE type)
+{
+      NetScope*scope = obj->scope();
+      NetLogic*tmp = new NetLogic(obj->scope(),
+				  scope->local_symbol(),
+				  3, type, obj->width());
+
+      des->add_node(tmp);
+
+      connect(obj->pin_Result(), tmp->pin(0));
+      connect(obj->pin_Data(type==NetLogic::PMOS? 0 : 1),  tmp->pin(1));
+
+      if (obj->width() == 1) {
+	      /* Special case that the expression is 1 bit
+		 wide. Connect the select directly to the enable. */
+	    connect(obj->pin_Sel(), tmp->pin(2));
+
+      } else {
+	      /* General case that the expression is arbitrarily
+		 wide. Replicate the enable signal (which we
+		 assume is 1 bit wide) to match the expression,
+		 and connect the enable vector to the enable
+		 input of the gate. */
+	    NetReplicate*rtmp = new NetReplicate(scope,
+						 scope->local_symbol(),
+						 obj->width(),
+						 obj->width());
+	    des->add_node(rtmp);
+
+	    connect(obj->pin_Sel(), rtmp->pin(1));
+	    connect(tmp->pin(2), rtmp->pin(0));
+
+	    NetNet*rsig = new NetNet(scope, scope->local_symbol(),
+				     NetNet::WIRE, obj->width());
+	    rsig->local_flag(true);
+	    rsig->data_type(IVL_VT_LOGIC);
+	    connect(tmp->pin(2), rsig->pin(0));
+      }
+
+      delete obj;
+}
+#endif
+
 /*
  * This detects the case where the mux selects between a value and
- * Vz. In this case, replace the device with a bufif with the sel
+ * Vz. In this case, replace the device with a mos with the sel
  * input used to enable the output.
  */
 void cprop_functor::lpm_mux(Design*des, NetMux*obj)
 {
-      if (obj->size() > 2) {
-	    lpm_mux_large(des, obj);
-	    return;
-      }
       if (obj->size() != 2)
 	    return;
       if (obj->sel_width() != 1)
 	    return;
 
-      NetScope*scope = obj->scope();
-
-      bool flag = true;
-      for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1) {
-	    bool cflag_a = obj->pin_Data(idx, 0).nexus()->drivers_constant();
-	    bool cflag_b = obj->pin_Data(idx, 1).nexus()->drivers_constant();
-
-	      /* If both data inputs are constant, we'll be able to do
-		 a substitution. */
-	    if (cflag_a && cflag_b)
-		  continue;
-
-	    verinum::V va = cflag_a
-		  ? obj->pin_Data(idx, 0).nexus()->driven_value()
-		  : verinum::Vx;
-	    verinum::V vb = cflag_b
-		  ? obj->pin_Data(idx, 1).nexus()->driven_value()
-		  : verinum::Vx;
-
-	      /* If only one Data input is constant, but a constant
-		 HiZ, then we will be able to to a bufif
-		 substitution. */
-	    if (cflag_a && va==verinum::Vz)
-		  continue;
-
-	    if (cflag_b && vb==verinum::Vz)
-		  continue;
-
-	      /* Otherwise, we cannot accurately do a substitution. If
-		 one input is non-constant, then that input may have a
-		 HiZ value, and there is no Verilog logic other then a
-		 MUX that can pass a HiZ value. */
-	    flag = false;
-      }
-
-      if (! flag) {
-	    return;
-      }
-
-	/* Create a temporary net to hold the result value that we
-	   build up. When we are done, we will connect this to the
-	   Result pins of the mux, then delete the mux *and* this
-	   net. That will cause all the proper connectivity to
-	   happen. */
-      NetNet result_tmp(scope, scope->local_symbol(),
-			NetNet::WIRE, obj->width());
-
-	/* We know that every slice has at least one constant. Run
-	   through the slices again, creating boolean devices to
-	   replace the MUX slice. */
-
-      for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1) {
-
-	      /* If the Sel==0 input is constant Z, make a bufif1. */
-	    if (obj->pin_Data(idx, 0).nexus()->drivers_constant()
-		&& obj->pin_Data(idx, 0).nexus()->driven_value()==verinum::Vz) {
-
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      3, NetLogic::BUFIF1);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Data(idx,1), tmp->pin(1));
-		  connect(obj->pin_Sel(0), tmp->pin(2));
-		  des->add_node(tmp);
-		  continue;
-	    }
-
-	      /* If the Sel==1 input is constant Z, make a bufif0. */
-	    if (obj->pin_Data(idx, 1).nexus()->drivers_constant()
-		&& obj->pin_Data(idx, 1).nexus()->driven_value()==verinum::Vz) {
-
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      3, NetLogic::BUFIF0);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Data(idx,0), tmp->pin(1));
-		  connect(obj->pin_Sel(0), tmp->pin(2));
-		  des->add_node(tmp);
-		  continue;
-	    }
-
-	      /* At this point, the only cases that are left are where
-		 the data inputs are both constant, and neither are
-		 HiZ. From this we know how to generate the output
-		 from only the S input. */
-	    if (! (obj->pin_Data(idx, 0).nexus()->drivers_constant()
-		   && obj->pin_Data(idx, 1).nexus()->drivers_constant())) {
-		  cerr << obj->get_line() << ": internal error: "
-		       << "Drivers not constant where expected?"
-		       << " obj->width()=" << obj->width()
-		       << " idx=" << idx
-		       << endl;
-		  obj->dump_node(cerr, 4);
-	    }
-	    assert(obj->pin_Data(idx, 0).nexus()->drivers_constant()
-		   && obj->pin_Data(idx, 1).nexus()->drivers_constant());
-
-
-	    verinum::V a = obj->pin_Data(idx, 0).nexus()->driven_value();
-	    verinum::V b = obj->pin_Data(idx, 1).nexus()->driven_value();
-
-	    if (a == b) {
-		  connect(result_tmp.pin(idx), obj->pin_Data(idx,0));
-		  continue;
-	    }
-
-	    if (a == verinum::V0 && b == verinum::V1) {
-		  connect(result_tmp.pin(idx), obj->pin_Sel(0));
-		  continue;
-	    }
-
-	    if (a == verinum::V1 && b == verinum::V0) {
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      2, NetLogic::NOT);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Sel(0), tmp->pin(1));
-		  des->add_node(tmp);
-		  continue;
-	    }
-
-	      /* A==0: Q = B & S */
-	    if (a == verinum::V0) {
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      3, NetLogic::AND);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Data(idx,1), tmp->pin(1));
-		  connect(obj->pin_Sel(0), tmp->pin(2));
-		  des->add_node(tmp);
-		  continue;
-	    }
-
-	      /* B==1: Q = A | S */
-	    if (b == verinum::V1) {
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      3, NetLogic::OR);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Data(idx,0), tmp->pin(1));
-		  connect(obj->pin_Sel(0), tmp->pin(2));
-		  des->add_node(tmp);
-		  continue;
-	    }
-	      /* A==1: Q = B | ~S */
-	    if (a == verinum::V1) {
-		  NetLogic*inv = new NetLogic(scope,
-					      scope->local_symbol(),
-					      2, NetLogic::NOT);
-		  NetNet*invs = new NetNet(scope,
-					   scope->local_symbol(),
-					   NetNet::TRI, 1);
-		  invs->local_flag(true);
-		  connect(inv->pin(0), invs->pin(0));
-		  connect(inv->pin(1), obj->pin_Sel(0));
-		  des->add_node(inv);
-
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      3, NetLogic::OR);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Data(idx,1), tmp->pin(1));
-		  connect(inv->pin(0), tmp->pin(2));
-		  des->add_node(tmp);
-		  continue;
-	    }
-	      /* B==0: Q = A & ~S */
-	    if (b == verinum::V0) {
-		  NetLogic*inv = new NetLogic(scope,
-					      scope->local_symbol(),
-					      2, NetLogic::NOT);
-		  NetNet*invs = new NetNet(scope,
-					   scope->local_symbol(),
-					   NetNet::TRI, 1);
-		  invs->local_flag(true);
-		  connect(inv->pin(0), invs->pin(0));
-		  connect(inv->pin(1), obj->pin_Sel(0));
-		  des->add_node(inv);
-
-		  NetLogic*tmp = new NetLogic(scope,
-					      scope->local_symbol(),
-					      3, NetLogic::AND);
-		  connect(result_tmp.pin(idx), tmp->pin(0));
-		  connect(obj->pin_Data(idx,0), tmp->pin(1));
-		  connect(inv->pin(0), tmp->pin(2));
-		  des->add_node(tmp);
-		  continue;
-	    }
-
-	    assert(0);
-
-      }
-
-      for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1)
-	    connect(result_tmp.pin(idx), obj->pin_Result(idx));
-
-      delete obj;
-      count += 1;
-}
-
-void cprop_functor::lpm_mux_large(Design*des, NetMux*obj)
-{
-      NetScope*scope = obj->scope();
-      unsigned width = obj->width();
-      unsigned size = obj->size();
-
-	/* This test looks for bit slices that are constant
-	   throughout. If we find any, we can reduce the width of the
-	   MUX to eliminate the fixed value. */
-
-	/* After the following for loop, this array of bools will
-	   contain "true" for each bit slice that is constant and
-	   identical, and "false" otherwise. The reduce_width will
-	   count the number of true entries in the flags array. */
-      bool*flags = new bool[width];
-      unsigned reduce_width = 0;
-
-      for (unsigned bit = 0 ;  bit < width ;  bit += 1) {
-
-	    flags[bit] = true;
-
-	      /* If not even the first selection is constant, then the
-		 slice cannot be reduced. */
-	    if (! obj->pin_Data(bit, 0).nexus()->drivers_constant()) {
-		  flags[bit] = false;
-		  continue;
-	    }
-
-	      /* If any of the remaining selections in non-consant, or
-		 constant with a different value, then this slice
-		 cannot be reduced. */
-	    verinum::V val = obj->pin_Data(bit, 0).nexus()->driven_value();
-
-	    for (unsigned idx = 1; flags[bit] && idx < size ;  idx += 1) {
-
-		  if (!obj->pin_Data(bit,idx).nexus()->drivers_constant()) {
-			flags[bit] = false;
-			break;
-		  }
-
-		  if (val != obj->pin_Data(bit,idx).nexus()->driven_value()) {
-			flags[bit] = false;
-			break;
-		  }
-	    }
-
-	    if (! flags[bit]) {
-		    /* This bit slice is too complex. Go on. */
-		  continue;
-	    }
-
-	    reduce_width += 1;
-      }
-
-	/* If no slices can be reduced, then we are finished. */
-      if (reduce_width == 0) {
-	    delete[]flags;
-	    return;
-      }
-
-	/* Handle the very special case that all the slices can be
-	   reduced. We don't need a MUX at all! */
-      if (reduce_width == width) {
-	    for (unsigned idx = 0 ;  idx < width ;  idx += 1)
-		  connect(obj->pin_Result(idx), obj->pin_Data(idx,0));
-
-	    delete obj;
-	    count += 1;
-	    delete[]flags;
-	    return;
-      }
-
-	/* Create a reduced mux with the same name and size, but fewer
-	   slices. Connect all the slices that we are keeping. */
-      NetMux*tmp = new NetMux(scope, obj->name(),
-			      width-reduce_width, size, obj->sel_width());
-      tmp->set_line(*obj);
-
-      for (unsigned idx = 0 ;  idx < obj->sel_width() ;  idx += 1)
-	    connect(obj->pin_Sel(idx), tmp->pin_Sel(idx));
-
-      unsigned dst_bit = 0;
-
-      for (unsigned bit = 0 ;  bit < width ;  bit += 1) {
-	    if (flags[bit]) {
-		  connect(obj->pin_Result(bit), obj->pin_Data(bit,0));
-		  continue;
-	    }
-
-	    connect(obj->pin_Result(bit), tmp->pin_Result(dst_bit));
-
-	    for (unsigned idx = 0 ;  idx < size ;  idx += 1)
-		  connect(obj->pin_Data(bit,idx), tmp->pin_Data(dst_bit,idx));
-
-	    dst_bit += 1;
-      }
-
-	/* Add the new node. Delete the old node. Signal that we
-	   change the design and may use a rescan. */
-      des->add_node(tmp);
-      delete obj;
-      delete[]flags;
-      count += 1;
-}
-
-void cprop_functor::lpm_ram_dq(Design*des, NetRamDq*obj)
-{
-      if (lpm_ram_dq_const_address_(des,obj))
-	    return;
-
-}
-
+#if 0
 /*
- * Try to evaluate a constant address input. If we find it, then
- * replace the NetRamDq with a direct link to the addressed word.
+ * This is slower than the actual MUXZ so we are skipping this for now.
+ * If we had a half mux functor this could be faster and more compact
+ * so I'm leaving the code for future reference.
  */
-bool cprop_functor::lpm_ram_dq_const_address_(Design*des, NetRamDq*obj)
-{
-      NetMemory*mem = obj->mem();
-      NetNet* reg = mem->reg_from_explode();
-
-	/* I only know how to do this on exploded memories. */
-      if (reg == 0)
-	    return false;
-
-      verinum sel (0UL, obj->awidth());
-      for (unsigned idx = 0 ;  idx < obj->awidth() ;  idx += 1) {
-	    if (! obj->pin_Address(idx).nexus()->drivers_constant())
-		  return false;
-
-	    sel.set(idx, obj->pin_Address(idx).nexus()->driven_value());
+	/* If the first input is all constant Vz, then replace the
+	   NetMux with an array of NMOS devices, with the enable
+	   connected to the select input. */
+      if (obj->pin_Data(0).nexus()->drivers_constant() &&
+          obj->pin_Data(0).nexus()->driven_value() == verinum::Vz) {
+	    replace_with_mos(des, obj, NetLogic::NMOS);
+	    count += 1;
+	    return;
       }
 
-      unsigned long address = sel.as_ulong();
-
-	/* If the address is outside the ram, then leave this to the
-	   code generator to figure out. */
-      if (address >= obj->size())
-	    return false;
-
-
-      unsigned base = address * obj->width();
-      assert(base+obj->width() <= reg->pin_count());
-
-      for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1)
-	    connect(reg->pin(base+idx), obj->pin_Q(idx));
-
-      if (debug_cprop) {
-	    cerr << obj->get_line() << ": debug: Replace read port with"
-		 << " fixed link to word " << address << "." << endl;
+	/* If instead the second input is all constant Vz, replace the
+	   NetMux with an array of PMOS devices. */
+      if (obj->pin_Data(1).nexus()->drivers_constant() &&
+          obj->pin_Data(1).nexus()->driven_value() == verinum::Vz) {
+	    replace_with_mos(des, obj, NetLogic::PMOS);
+	    count += 1;
+	    return;
       }
+#endif
 
-      delete obj;
-      count += 1;
-      return true;
+	/* If the select input is constant, then replace with a BUFZ */
+      bool flag = obj->pin_Sel().nexus()->drivers_constant();
+      verinum::V sel_val = flag? obj->pin_Sel().nexus()->driven_value() : verinum::Vx;
+      if ((sel_val != verinum::Vz) && (sel_val != verinum::Vx)) {
+	    NetBUFZ*tmp = new NetBUFZ(obj->scope(), obj->name(), obj->width());
+	    tmp->set_line(*obj);
+
+	    if (debug_optimizer)
+		  cerr << obj->get_fileline() << ": debug: "
+		       << "Replace binary MUX with constant select=" << sel_val
+		       << " with a BUFZ to the selected input." << endl;
+
+	    tmp->rise_time(obj->rise_time());
+	    tmp->fall_time(obj->fall_time());
+	    tmp->decay_time(obj->decay_time());
+
+	    connect(tmp->pin(0), obj->pin_Result());
+	    if (sel_val == verinum::V1)
+		  connect(tmp->pin(1), obj->pin_Data(1));
+	    else
+		  connect(tmp->pin(1), obj->pin_Data(0));
+	    delete obj;
+	    des->add_node(tmp);
+	    count += 1;
+      }
 }
 
-
 /*
  * This functor looks to see if the constant is connected to nothing
  * but signals. If that is the case, delete the dangling constant and
@@ -1232,15 +862,17 @@ struct cprop_dc_functor  : public functor_t {
 void cprop_dc_functor::lpm_const(Design*des, NetConst*obj)
 {
 	// 'bz constant values drive high impedance to whatever is
-	// connected to it. In other words, it is a noop.
+	// connected to it. In other words, it is a noop. But that is
+	// only true if *all* the bits of the vectors.
       { unsigned tmp = 0;
-        for (unsigned idx = 0 ;  idx < obj->pin_count() ;  idx += 1)
+	ivl_assert(*obj, obj->pin_count()==1);
+	for (unsigned idx = 0 ;  idx < obj->width() ;  idx += 1) {
 	      if (obj->value(idx) == verinum::Vz) {
-		    obj->pin(idx).unlink();
 		    tmp += 1;
 	      }
+	}
 
-	if (tmp == obj->pin_count()) {
+	if (tmp == obj->width()) {
 	      delete obj;
 	      return;
 	}
@@ -1256,7 +888,7 @@ void cprop_dc_functor::lpm_const(Design*des, NetConst*obj)
 	    for (Link*clnk = nex->first_nlink()
 		       ; clnk ; clnk = clnk->next_nlink()) {
 
-		  NetObj*cur;
+		  NetPins*cur;
 		  unsigned pin;
 		  clnk->cur_link(cur, pin);
 
@@ -1282,7 +914,7 @@ void cprop_dc_functor::lpm_const(Design*des, NetConst*obj)
 	    for (Link*clnk = nex->first_nlink()
 		       ; clnk ; clnk = clnk->next_nlink()) {
 
-		  NetObj*cur;
+		  NetPins*cur;
 		  unsigned pin;
 		  clnk->cur_link(cur, pin);
 
@@ -1333,69 +965,3 @@ void cprop(Design*des)
       cprop_dc_functor dc;
       des->functor(&dc);
 }
-
-/*
- * $Log: cprop.cc,v $
- * Revision 1.47.2.7  2007/02/26 19:51:38  steve
- *  Spelling fixes (larry doolittle)
- *
- * Revision 1.47.2.6  2006/11/12 01:20:45  steve
- *  Prevent constant mux outputs from confusing itself.
- *
- * Revision 1.47.2.5  2006/04/23 04:26:13  steve
- *  Constant propagate addresses through NetRamDq read ports.
- *
- * Revision 1.47.2.4  2005/09/11 02:50:51  steve
- *  Fix overly agressive constant propagation through MUX causing lost Z bits.
- *
- * Revision 1.47.2.3  2005/08/28 22:00:39  steve
- *  Reduce mux slices that are constant throughout range.
- *
- * Revision 1.47.2.2  2005/08/28 19:51:02  steve
- *  More thorough constant propagation through MUX devices.
- *
- * Revision 1.47  2004/02/20 18:53:34  steve
- *  Addtrbute keys are perm_strings.
- *
- * Revision 1.46  2003/11/08 17:53:34  steve
- *  Do not remove constants accessible to VPI.
- *
- * Revision 1.45  2003/10/31 02:40:06  steve
- *  Donot elide FF that has set or clr connections.
- *
- * Revision 1.44  2003/04/25 05:06:32  steve
- *  Handle X values in constant == nets.
- *
- * Revision 1.43  2003/03/06 00:28:41  steve
- *  All NetObj objects have lex_string base names.
- *
- * Revision 1.42  2003/02/26 01:29:24  steve
- *  LPM objects store only their base names.
- *
- * Revision 1.41  2003/01/30 16:23:07  steve
- *  Spelling fixes.
- *
- * Revision 1.40  2003/01/27 05:09:17  steve
- *  Spelling fixes.
- *
- * Revision 1.39  2002/08/20 04:12:22  steve
- *  Copy gate delays when doing gate delay substitutions.
- *
- * Revision 1.38  2002/08/12 01:34:58  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.37  2002/06/25 01:33:22  steve
- *  Cache calculated driven value.
- *
- * Revision 1.36  2002/06/24 01:49:38  steve
- *  Make link_drive_constant cache its results in
- *  the Nexus, to improve cprop performance.
- *
- * Revision 1.35  2002/05/26 01:39:02  steve
- *  Carry Verilog 2001 attributes with processes,
- *  all the way through to the ivl_target API.
- *
- *  Divide signal reference counts between rval
- *  and lval references.
- */
-
