@@ -32,28 +32,28 @@
 
 #include <windows.h>
 
-/* Macros used for compiling and linking */
+static void setup_ivl_environment();
+static void assign(char **ptr, char *str);
 
-#define IVERILOG_VPI_CC        "gcc"				/* no .exe extension  */
-#define IVERILOG_VPI_CXX       "gcc"				/* no .exe extension  */
-#define IVERILOG_VPI_CFLAGS    "-O"					/* -I appended  later */
-#define IVERILOG_VPI_LD        "gcc"				/* no .exe extension  */
-#define IVERILOG_VPI_LDFLAGS   "-shared -Wl,--enable-auto-image-base"
-#define IVERILOG_VPI_LDLIBS    "-lveriuser -lvpi"	/* -L prepended later */
+/* The compile options: compiler, flags, etc. are in here */
+#include "config.h"
 
 /* pointers to global strings */
 
 static struct global_strings {
-	char *pCCSRC;		/* list of C source files */
-	char *pCXSRC;		/* list of C++ source files */
+	char *pCCSRC;		/* list of C source files (*.c) */
+	char *pCXSRC;		/* list of C++ source files (*.cc, *.cpp) */
 	char *pOBJ;			/* list of object files */
 	char *pLIB;			/* list of library files */
+	char *pINCS;			/* list of include directories */
+	char *pDEFS;			/* list of definitions */
 	char *pOUT;			/* output file name (.vpi extension), if 0 length then no source files specified */
 	char *pMINGW;		/* path to MinGW directory */
 	char *pIVL;			/* path to IVL directory */
 	char *pCFLAGS;		/* CFLAGS option */
 	char *pLDLIBS;		/* LDLIBS option */
 	char *pNewPath;		/* new PATH environment variable setting */
+	char *pLD;		/* what to use for a linker */
 } gstr;
 
 
@@ -70,12 +70,15 @@ static void myExit(int exitVal)
 	deInitDynString(gstr.pCXSRC);
 	deInitDynString(gstr.pOBJ);
 	deInitDynString(gstr.pLIB);
+	deInitDynString(gstr.pINCS);
+	deInitDynString(gstr.pDEFS);
 	deInitDynString(gstr.pOUT);
 	deInitDynString(gstr.pMINGW);
 	deInitDynString(gstr.pIVL);
 	deInitDynString(gstr.pCFLAGS);
 	deInitDynString(gstr.pLDLIBS);
 	deInitDynString(gstr.pNewPath);
+	free(gstr.pLD);
 
 	exit(exitVal);
 }
@@ -84,9 +87,9 @@ static void myExit(int exitVal)
 
 static void usage()
 {
-	fprintf(stderr,"usage: iverilog-vpi [--name=name] [-llibrary] [-mingw=dir] [-ivl=dir] sourcefile...\n");
-	fprintf(stderr,"   or  iverilog-vpi -mingw=dir\n");
-	fprintf(stderr,"   or  iverilog-vpi -ivl=dir\n");
+	fprintf(stderr,"usage: iverilog-vpi" IVERILOG_SUFFIX " [src and obj files]...\n");
+	fprintf(stderr,"   or  iverilog-vpi" IVERILOG_SUFFIX " -mingw=dir\n");
+	fprintf(stderr,"   or  iverilog-vpi" IVERILOG_SUFFIX " -ivl=dir\n");
 	myExit(1);
 }
 
@@ -110,12 +113,16 @@ static void init()
 	initDynString(&gstr.pCXSRC);
 	initDynString(&gstr.pOBJ);
 	initDynString(&gstr.pLIB);
+	initDynString(&gstr.pINCS);
+	initDynString(&gstr.pDEFS);
 	initDynString(&gstr.pOUT);
 	initDynString(&gstr.pMINGW);
 	initDynString(&gstr.pIVL);
 	initDynString(&gstr.pCFLAGS);
 	initDynString(&gstr.pLDLIBS);
 	initDynString(&gstr.pNewPath);
+	  /* By default use the C compiler to link the programs. */
+	assign(&gstr.pLD, IVERILOG_VPI_CC);
 }
 
 /* return true if "str" is terminated with with "end", case insensitive */
@@ -277,63 +284,124 @@ static int parse(int argc, char *argv[])
 
 	char dot_c_ext[]    = ".c";
 	char dot_cc_ext[]   = ".cc";
+	char dot_cpp_ext[]  = ".cpp";
 	char dot_o_ext[]    = ".o";
 	char name_option[]  = "--name=";
 	char lib_option[]   = "-l";
+	char inc_option[]   = "-I";
 	char mingw_option[] = "-mingw=";
 	char ivl_option[]   = "-ivl=";
+	char def_option[]   = "-D";
 
-	if (argc == 1)
-		return 0;
+	if (argc == 1) return 0;
 
 	for (idx=1; idx<argc; ++idx) {
-		if (endsIn(dot_c_ext,argv[idx])) {				/* check for C source files */
+		  /* Check for C source files (*.c) */
+		if (endsIn(dot_c_ext, argv[idx])) {
 			++srcFileCnt;
-			append(&gstr.pCCSRC,argv[idx]);
-			append(&gstr.pCCSRC," ");
+			append(&gstr.pCCSRC, argv[idx]);
+			append(&gstr.pCCSRC, " ");
 			if (!*gstr.pOUT)
-			   assignn(&gstr.pOUT,argv[idx],strlen(argv[idx])-strlen(dot_c_ext));
+			   assignn(&gstr.pOUT, argv[idx],
+			           strlen(argv[idx])-strlen(dot_c_ext));
 		}
-		else if (endsIn(dot_cc_ext,argv[idx])) {		/* check for C++ source files */
+		  /* Check for C++ source files (*.cc) */
+		else if (endsIn(dot_cc_ext, argv[idx])) {
+			  /* We need to link with the C++ compiler. */
+			assign(&gstr.pLD, IVERILOG_VPI_CXX);
 			++srcFileCnt;
-			append(&gstr.pCXSRC,argv[idx]);
-			append(&gstr.pCXSRC," ");
+			append(&gstr.pCXSRC, argv[idx]);
+			append(&gstr.pCXSRC, " ");
 			if (!*gstr.pOUT)
-			   assignn(&gstr.pOUT,argv[idx],strlen(argv[idx])-strlen(dot_cc_ext));
+			   assignn(&gstr.pOUT, argv[idx],
+			           strlen(argv[idx])-strlen(dot_cc_ext));
 		}
-		else if (endsIn(dot_o_ext,argv[idx])) {			/* check for compiled object files */
+		  /* Check for C++ source files (*.cpp) */
+		else if (endsIn(dot_cpp_ext, argv[idx])) {
+			  /* We need to link with the C++ compiler. */
+			assign(&gstr.pLD, IVERILOG_VPI_CXX);
+			++srcFileCnt;
+			append(&gstr.pCXSRC, argv[idx]);
+			append(&gstr.pCXSRC, " ");
+			if (!*gstr.pOUT)
+			   assignn(&gstr.pOUT, argv[idx],
+			           strlen(argv[idx])-strlen(dot_cpp_ext));
+		}
+		  /* Check for compiled object files */
+		else if (endsIn(dot_o_ext, argv[idx])) {
 			++srcFileCnt;
-			append(&gstr.pOBJ,argv[idx]);
 			append(&gstr.pOBJ," ");
+			append(&gstr.pOBJ, argv[idx]);
 			if (!*gstr.pOUT)
-			   assignn(&gstr.pOUT,argv[idx],strlen(argv[idx])-strlen(dot_o_ext));
+			   assignn(&gstr.pOUT, argv[idx],
+			           strlen(argv[idx])-strlen(dot_o_ext));
 		}
-		else if (startsWith(name_option,argv[idx])) {	/* check for --name option */
-			assignn(&gstr.pOUT,argv[idx]+sizeof(name_option)-1,strlen(argv[idx])-(sizeof(name_option)-1));
+		  /* Check for the -mingw option */
+		else if (startsWith(mingw_option, argv[idx]))
+			assignn(&gstr.pMINGW, argv[idx]+sizeof(mingw_option)-1,
+			        strlen(argv[idx])-(sizeof(mingw_option)-1));
+		  /* Check for the -ivl   option */
+		else if (startsWith(ivl_option, argv[idx]))
+			assignn(&gstr.pIVL, argv[idx]+sizeof(ivl_option)-1,
+			        strlen(argv[idx])-(sizeof(ivl_option)-1));
+		  /* Check for the --name option */
+		else if (startsWith(name_option, argv[idx])) {
+			assignn(&gstr.pOUT, argv[idx]+sizeof(name_option)-1,
+			        strlen(argv[idx])-(sizeof(name_option)-1));
 		}
-		else if (startsWith(lib_option,argv[idx])) {	/* check for -l option */
-			append(&gstr.pLIB,argv[idx]);
-			append(&gstr.pLIB," ");
+		  /* Check for the -l option */
+		else if (startsWith(lib_option, argv[idx])) {
+			append(&gstr.pLIB, " ");
+			append(&gstr.pLIB, argv[idx]);
 		}
-		else if (startsWith(mingw_option,argv[idx]))	/* check for -mingw option */
-			assignn(&gstr.pMINGW,argv[idx]+sizeof(mingw_option)-1,strlen(argv[idx])-(sizeof(mingw_option)-1));
-		else if (startsWith(ivl_option,argv[idx]))		/* check for -ivl   option */
-			assignn(&gstr.pIVL,argv[idx]+sizeof(ivl_option)-1,strlen(argv[idx])-(sizeof(ivl_option)-1));
-		else
-			return 0;	/* different from iverilog-vpi.sh, we don't ignore accept arguments */
+		  /* Check for the -I option */
+		else if (startsWith(inc_option, argv[idx])) {
+			append(&gstr.pINCS, " ");
+			append(&gstr.pINCS, argv[idx]);
+		}
+		  /* Check for the -D option */
+		else if (startsWith(def_option, argv[idx])) {
+			append(&gstr.pDEFS, " ");
+			append(&gstr.pDEFS, argv[idx]);
+		}
+		  /* Check for the --cflags option */
+		else if (stricmp("--cflags", argv[idx]) == 0) {
+			setup_ivl_environment();
+			printf("%s\n", gstr.pCFLAGS);
+			myExit(0);
+		}
+		  /* Check for the --ldflags option */
+		else if (stricmp("--ldflags", argv[idx]) == 0) {
+			printf("%s\n", IVERILOG_VPI_LDFLAGS);
+			myExit(0);
+		}
+		  /* Check for the --ldlibs option */
+		else if (stricmp("--ldlibs", argv[idx]) == 0) {
+			setup_ivl_environment();
+			printf("%s\n", gstr.pLDLIBS);
+			myExit(0);
+		}
+		  /* Check for the --install-dir option */
+		else if (stricmp("--install-dir", argv[idx]) == 0) {
+			setup_ivl_environment();
+			printf("%s\\\\lib\\\\ivl" IVERILOG_SUFFIX "\\\\.\n", gstr.pIVL);
+			myExit(0);
+		}
+		  /* This is different than iverilog-vpi.sh, we don't
+		   * ignore unknown arguments */
+		else return 0;
 	}
 
-	if (0 == srcFileCnt)
-		assign(&gstr.pOUT,"");				/* in case they used --name with no source files */
+	  /* In case there is a --name without source/object files */
+	if (0 == srcFileCnt) assign(&gstr.pOUT,"");
 
-	if (!*gstr.pOUT) {						/* normally it's an error if there are no *.c,*.cc,*.o files */
-		if (!*gstr.pMINGW && !*gstr.pIVL)	/* unless they are just setting the IVL or MinGW registry entries */
-			usage();
-	}
-	else {
-		append(&gstr.pOUT,".vpi");			/* the result file should have a .vpi extension */
-		append(&gstr.pOUT," ");
-	}
+	  /* Normally it's an error if there are no source or object files */
+	  /* Unless we are setting the IVL or MinGW registry entries */
+	if (!*gstr.pOUT) {
+		if (!*gstr.pMINGW && !*gstr.pIVL) return 0;
+	} else
+		  /* We have a valid result file so add the .vpi extension */
+		append(&gstr.pOUT, ".vpi");
 
 	return 1;
 }
@@ -376,14 +444,14 @@ static void checkIvlDir(char *root)
 	initDynString(&path);
 	assign(&path,gstr.pIVL);
 	appendBackSlash(&path);
-	append(&path,"bin\\vvp.exe");
+	append(&path,"bin\\vvp" IVERILOG_SUFFIX ".exe");
 
 	irv = _stat(path,&stat_buf);
 	deInitDynString(path);
 
 	if (irv) {
 		fprintf(stderr,"error: %s does not appear to be the valid root directory of\n",root);
-		fprintf(stderr,"       Icarus Verilog.  Use the -ivl option of iverilog-vpi.exe to\n");
+		fprintf(stderr,"       Icarus Verilog.  Use the -ivl option of iverilog-vpi" IVERILOG_SUFFIX " to\n");
 		fprintf(stderr,"       point to the Icarus Verilog root directory.  For a Windows\n");
 		fprintf(stderr,"       command shell the option would be something like -ivl=c:\\iverilog\n");
 		fprintf(stderr,"       For a Cygwin shell the option would be something like\n");
@@ -403,9 +471,7 @@ static void setup_mingw_environment()
 	if (*gstr.pMINGW) {
 		checkMingwDir(gstr.pMINGW);
 		SetRegistryKey(IVL_REGKEY_MINGW,gstr.pMINGW);
-	}
-	else
-		if (!GetRegistryKey(IVL_REGKEY_MINGW,&gstr.pMINGW)) {
+	} else if (!GetRegistryKey(IVL_REGKEY_MINGW,&gstr.pMINGW)) {
 			fprintf(stderr,"error: can not locate the MinGW root directory, use the -mingw option of\n");
 			fprintf(stderr,"       iverilog-vpi.exe to point to the MinGW root directory.  For\n");
 			fprintf(stderr,"       a Windows command shell the option would be something like\n");
@@ -414,13 +480,16 @@ static void setup_mingw_environment()
 			myExit(5);
 		}
 
-	assign(&gstr.pNewPath,"PATH=");			/* create new path */
+	  /* Create new path with MinGW in it */
+	assign(&gstr.pNewPath,"PATH=");
 	append(&gstr.pNewPath,gstr.pMINGW);
 	appendBackSlash(&gstr.pNewPath);
+	append(&gstr.pNewPath, "\\");
 	append(&gstr.pNewPath,"bin;");
 	append(&gstr.pNewPath,pOldPATH);
 
-	_putenv(gstr.pNewPath);					/* place new path in environment variable */
+	  /* Place new path in environment */
+	_putenv(gstr.pNewPath);
 }
 
 /* see if we can find iverilog root */
@@ -432,67 +501,73 @@ static void setup_ivl_environment()
 	if (*gstr.pIVL) {
 		checkIvlDir(gstr.pIVL);
 		SetRegistryKey(IVL_REGKEY_IVL,gstr.pIVL);
-	}
-	else
-		if (!GetRegistryKey(IVL_REGKEY_IVL,&gstr.pIVL)) {
+	} else if (!GetRegistryKey(IVL_REGKEY_IVL,&gstr.pIVL)) {
 			fprintf(stderr,"error: can not locate the Icarus Verilog root directory, use the -ivl option\n");
-			fprintf(stderr,"       of iverilog-vpi.exe to point to the Icarus Verilog root directory.\n");
+			fprintf(stderr,"       of iverilog-vpi" IVERILOG_SUFFIX " to point to the Icarus Verilog root directory.\n");
 			fprintf(stderr,"       For a Windows command shell the option would be something like\n");
 			fprintf(stderr,"       -ivl=c:\\iverilog  For a Cygwin shell the option would be something\n");
 			fprintf(stderr,"       like -ivl=c:\\\\iverilog\n");
 			myExit(6);
 		}
 
-	/* build up the CFLAGS option string */
-
-	assign(&gstr.pCFLAGS,IVERILOG_VPI_CFLAGS);
-	append(&gstr.pCFLAGS," -I");
+	  /* Build up the CFLAGS option string */
+	assign(&gstr.pCFLAGS,IVERILOG_VPI_CFLAGS " -I");
 	append(&gstr.pCFLAGS,gstr.pIVL);
 	appendBackSlash(&gstr.pCFLAGS);
-	append(&gstr.pCFLAGS,"include");
-
-	/* build up the LDFLAGS option string */
+	append(&gstr.pCFLAGS,"\\include\\\\iverilog" IVERILOG_SUFFIX);
 
+	  /* Build up the LDFLAGS option string */
 	assign(&gstr.pLDLIBS,"-L");
 	append(&gstr.pLDLIBS,gstr.pIVL);
 	appendBackSlash(&gstr.pLDLIBS);
-	append(&gstr.pLDLIBS,"lib ");
-	append(&gstr.pLDLIBS,IVERILOG_VPI_LDLIBS);
+	append(&gstr.pLDLIBS,"\\lib " IVERILOG_VPI_LDLIBS);
 }
 
 /* compile source modules */
 
-static void compile(char *pSource, char **pObject, char *ext, int *compile_errors, char *compiler)
+static void compile(char *pSource, char **pObject, int *compile_errors, char *compiler)
 {
 	char *ptr1 = pSource;
-	char *ptr2 = strchr(pSource,' ');
-	char *buf=0,*src=0,*obj=0;
+	char *ptr2 = strchr(ptr1, ' ');
+	char *buf=0, *src=0, *obj=0;
 
 	while (ptr2) {
 		int len = ptr2 - ptr1;
-		assignn(&src,ptr1,len);
+		char *ostart;
+		int olen;
+		assignn(&src, ptr1, len);
 
-		assignn(&obj,ptr1,len-strlen(ext));		/* strip off the extension */
-		append (&obj,".o");
+		  /* Build the object file name */
+		ostart = strrchr(ptr1, '/');
+		if (ostart == NULL) ostart = ptr1;
+		else ostart += 1;
+		olen = strrchr(ptr1, '.') - ostart;
+		assignn(&obj, ostart, olen);
+		append(&obj, ".o");
 
-		assign (&buf,compiler);
-		append (&buf," -c -o ");
-		append (&buf,obj);
-		append (&buf," ");
-		append (&buf,gstr.pCFLAGS);
-		append (&buf," ");
-		append (&buf,src);
+		  /* Build the compile line */
+		assign(&buf, compiler);
+		append(&buf, " -c -o ");
+		append(&buf, obj);
+		append(&buf, " ");
+		append(&buf, gstr.pDEFS);
+		append(&buf, " ");
+		append(&buf, gstr.pCFLAGS);
+		append(&buf, " ");
+		append(&buf, gstr.pINCS);
+		append(&buf, " ");
+		append(&buf, src);
 
-		append (pObject,obj);
-		append (pObject," ");
+		append (pObject, " ");
+		append (pObject, obj);
 
-		printf("%s\n",buf);
+		printf("Compiling %s...\n", src);
 
-		if (system(buf))
-			++*compile_errors;
+		if (system(buf)) ++*compile_errors;
 
-		ptr1 = ptr2 + 1;					/* advance to next token */
-		ptr2 = strchr(ptr1,' ');
+		  /* advance to next token */
+		ptr1 = ptr2 + 1;
+		ptr2 = strchr(ptr1, ' ');
 	}
 
 	free(buf);
@@ -507,43 +582,44 @@ static void compile_and_link()
 	char *buf=0;
 	int iRet, compile_errors = 0;
 
-	/* print out the mingw and ivl directories to help the user debug problems */
+	  /* To make the output match iverilog-vpi.sh do not print out the
+	   * root directories */
 
-	printf("info:  %s will be used as the MinGW root directory.\n",gstr.pMINGW);
+//	printf("MinGW root directory:  %s.\n", gstr.pMINGW);
 	checkMingwDir(gstr.pMINGW);
 
-	printf("info:  %s will be used as the Icarus Verilog root directory.\n",gstr.pIVL);
+//	printf("Icarus Verilog root directory:  %s.\n", gstr.pIVL);
 	checkIvlDir(gstr.pIVL);
 
-	/* compile */
-
-	compile(gstr.pCCSRC,&gstr.pOBJ,".c" ,&compile_errors,IVERILOG_VPI_CC );		/* compile the C   source files */
-	compile(gstr.pCXSRC,&gstr.pOBJ,".cc",&compile_errors,IVERILOG_VPI_CXX);		/* compile the C++ source files */
+	  /* compile the C source files (*.c) */
+	compile(gstr.pCCSRC, &gstr.pOBJ, &compile_errors, IVERILOG_VPI_CC );
+	  /* compile the C++ source files (*.cc, *.cpp) */
+	compile(gstr.pCXSRC, &gstr.pOBJ, &compile_errors, IVERILOG_VPI_CXX);
 
 	if (compile_errors) {
-		fprintf(stderr,"iverilog-vpi: Some %d files failed to compile.\n",compile_errors);
+		fprintf(stderr,"iverilog-vpi: %d file(s) failed to compile.\n",
+		        compile_errors);
 		myExit(2);
 	}
 
-	/* link */
+	  /* link */
+	assign(&buf, gstr.pLD);
+	append(&buf, " -o ");
+	append(&buf, gstr.pOUT);
+	append(&buf, " ");
+	append(&buf, IVERILOG_VPI_LDFLAGS);
+	append(&buf, " ");
+	append(&buf, gstr.pOBJ);
+	append(&buf, " ");
+	append(&buf, gstr.pLIB);
+	append(&buf, " ");
+	append(&buf, gstr.pLDLIBS);
 
-	assign(&buf,IVERILOG_VPI_LD);
-	append(&buf," -o ");
-	append(&buf,gstr.pOUT);				/* has a trailing space */
-	append(&buf,IVERILOG_VPI_LDFLAGS);
-	append(&buf," ");
-	append(&buf,gstr.pOBJ)				/* has a trailing space */;
-	append(&buf,gstr.pLIB);				/* has a trailing space */
-	append(&buf,gstr.pLDLIBS);
-
-	printf("%s\n",buf);
+	printf("Making %s from %s...\n", gstr.pOUT,gstr.pOBJ);
 
 	iRet = system(buf);
-
 	free(buf);
-
-	if (iRet)
-		myExit(3);
+	if (iRet) myExit(3);
 }
 
 /* program execution starts here */
@@ -552,14 +628,13 @@ int main(int argc, char *argv[])
 {
 	init();
 
-	if (!parse(argc,argv))
-		usage();
+	if (!parse(argc,argv)) usage();
 
 	setup_mingw_environment();
 	setup_ivl_environment();
 
-	if (*gstr.pOUT)				/* are there any *.c,*.cc,*.o files specified */
-		compile_and_link();
+	  /* are there any source or object files specified */
+	if (*gstr.pOUT) compile_and_link();
 
 	myExit(0);
 	return 0; // eliminate warnings.
