@@ -19,9 +19,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: cfparse_misc.h,v 1.6 2004/02/15 18:03:30 steve Exp $"
-#endif
 
 /*
  * The vlltype supports the passing of detailed source file location
@@ -36,32 +33,11 @@ struct cfltype {
       const char*text;
 };
 # define YYLTYPE struct cfltype
-extern YYLTYPE yylloc;
 
 int cflex(void);
 int cferror(const char *);
 int cfparse(void);
+void switch_to_command_file(const char *);
+char *current_file;
 
-/*
- * $Log: cfparse_misc.h,v $
- * Revision 1.6  2004/02/15 18:03:30  steve
- *  Cleanup of warnings.
- *
- * Revision 1.5  2003/09/26 21:25:58  steve
- *  Warnings cleanup.
- *
- * Revision 1.4  2002/08/12 01:35:01  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.3  2002/01/02 02:39:34  steve
- *  Use my own cfltype to defend against bison 1.30.
- *
- * Revision 1.2  2001/11/12 18:47:32  steve
- *  Support +incdir in command files, and ignore other
- *  +args flags. Also ignore -a and -v flags.
- *
- * Revision 1.1  2001/11/12 01:26:36  steve
- *  More sophisticated command file parser.
- *
- */
 #endif
