@@ -18,9 +18,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: globals.h,v 1.19 2003/11/18 06:31:46 steve Exp $"
-#endif
 
 # include  <stddef.h>
 
@@ -31,10 +28,10 @@ extern const char*base;
 extern char* iconfig_path;
 extern char* iconfig_common_path;
 
-  /* Ths is the optional -M<dependfile> value, if one was supplied. */
+  /* This is the optional -M<dependfile> value, if one was supplied. */
 extern const char*depfile;
 
-  /* Ths is the optional -N<path> value, if one was supplied. */
+  /* This is the optional -N<path> value, if one was supplied. */
 extern const char*npath;
 
   /* This is the name of the output file that the user selected. */
@@ -49,11 +46,14 @@ extern int synth_flag;
   /* This is the name of the selected target. */
 extern const char*targ;
 
+  /* This is the integer-width argument that will be passed to ivl. */
+extern unsigned integer_width;
+
   /* Perform variable substitutions on the string. */
 extern char* substitutions(const char*str);
 
   /* Add the name to the list of source files. */
-extern void process_file_name(const char*name);
+extern void process_file_name(const char*name, int lib_flag);
 
   /* Add the name to the list of library directories. */
 extern void process_library_switch(const char*name);
@@ -75,30 +75,4 @@ extern char warning_flags[];
 extern char* library_flags;
 extern char* library_flags2;
 
-/*
- * $Log: globals.h,v $
- * Revision 1.19  2003/11/18 06:31:46  steve
- *  Remove the iverilog.conf file.
- *
- * Revision 1.18  2003/11/13 04:09:49  steve
- *  Pass flags through the temporary config file.
- *
- * Revision 1.17  2003/11/01 04:21:57  steve
- *  Add support for a target static config file.
- *
- * Revision 1.16  2002/08/12 01:35:01  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.15  2002/06/23 20:10:51  steve
- *  Variable substitution in command files.
- *
- * Revision 1.14  2002/05/28 20:40:37  steve
- *  ivl indexes the search path for libraries, and
- *  supports case insensitive module-to-file lookup.
- *
- * Revision 1.13  2002/05/28 00:50:40  steve
- *  Add the ivl -C flag for bulk configuration
- *  from the driver, and use that to run library
- *  modules through the preprocessor.
- */
 #endif
