@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,11 +16,9 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: main.c,v 1.65.2.6 2007/05/30 17:48:26 steve Exp $"
-#endif
 
 # include "config.h"
+# include "version.h"
 
 const char NOTICE[] =
 "  This program is free software; you can redistribute it and/or modify\n"
@@ -33,18 +31,20 @@ const char NOTICE[] =
 "  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 "  GNU General Public License for more details.\n"
 "\n"
-"  You should have received a copy of the GNU General Public License\n"
-"  along with this program; if not, write to the Free Software\n"
-"  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA\n"
+"  You should have received a copy of the GNU General Public License along\n"
+"  with this program; if not, write to the Free Software Foundation, Inc.,\n"
+"  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.\n"
 ;
 
 const char HELP[] =
-"Usage: iverilog [-ESvV] [-B base] [-c cmdfile] [-g1|-g2|-g3.0]\n"
+"Usage: iverilog [-ESvV] [-B base] [-c cmdfile|-f cmdfile]\n"
+"                [-g1995|-g2001|-g2005] [-g<feature>]\n"
 "                [-D macro[=defn]] [-I includedir] [-M depfile] [-m module]\n"
 "                [-N file] [-o filename] [-p flag=value]\n"
 "                [-s topmodule] [-t target] [-T min|typ|max]\n"
 "                [-W class] [-y dir] [-Y suf] source_file(s)\n"
-"See man page for details.";
+"\n"
+"See the man page for details.";
 
 #define MAXSIZE 4096
 
@@ -52,6 +52,7 @@ const char HELP[] =
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <assert.h>
 
 #include <sys/types.h>
@@ -88,10 +89,6 @@ extern const char*optarg;
 # define IVL_ROOT "."
 #endif
 
-#ifndef IVL_ROOT_VARIABLE
-# define IVL_ROOT_VARIABLE "IVERILOG_ROOT"
-#endif
-
 # include  "globals.h"
 #include "cfparse_misc.h"   /* cfparse() */
 
@@ -111,12 +108,26 @@ const char*npath = 0;
 const char*targ  = "vvp";
 const char*depfile = 0;
 
-const char*generation = "3.0";
+const char*generation = "2005";
+const char*gen_specify = "no-specify";
+const char*gen_xtypes = "xtypes";
+const char*gen_icarus = "icarus-misc";
+const char*gen_io_range_error = "io-range-error";
+const char*gen_strict_ca_eval = "no-strict-ca-eval";
+const char*gen_verilog_ams = "no-verilog-ams";
+const char*gen_system_verilog = "no-system-verilog";
+
+/* Boolean: true means use a default include dir, false means don't */
+int gen_std_include = 1;
+
+/* Boolean: true means add the local file directory to the start
+   of the include list. */
+int gen_relative_include = 0;
 
 char warning_flags[16] = "";
 
-char*inc_list = 0;
-char*def_list = 0;
+unsigned integer_width = 32;
+
 char*mod_list = 0;
 char*command_filename = 0;
 
@@ -127,15 +138,19 @@ char*source_path = 0;
 FILE*source_file = 0;
 unsigned source_count = 0;
 
+char*defines_path = 0;
+FILE*defines_file = 0;
+
 char*iconfig_path = 0;
 FILE*iconfig_file = 0;
 
+char*compiled_defines_path = 0;
+
 static char iconfig_common_path_buf[4096] = "";
 char*iconfig_common_path = iconfig_common_path_buf;
 
 int synth_flag = 0;
 int verbose_flag = 0;
-int command_file = 0;
 
 FILE *fp;
 
@@ -144,6 +159,48 @@ char tmp[MAXSIZE];
 
 static char ivl_root[MAXSIZE];
 
+/* Structure to keep a FIFO list of the command files */
+typedef struct t_command_file {
+      char *filename;
+      struct t_command_file *next;
+} s_command_file, *p_command_file;
+p_command_file cmd_file_head = NULL;  /* The FIFO head */
+p_command_file cmd_file_tail = NULL;  /* The FIFO tail */
+
+/* Function to add a command file name to the FIFO. */
+void add_cmd_file(const char* filename)
+{
+      p_command_file new;
+
+      new = (p_command_file) malloc(sizeof(s_command_file));
+      new->filename = strdup(filename);
+      new->next = NULL;
+      if (cmd_file_head == NULL) {
+            cmd_file_head = new;
+            cmd_file_tail = new;
+      } else {
+            cmd_file_tail->next = new;
+            cmd_file_tail = new;
+      }
+}
+
+/* Function to return the top command file name from the FIFO. */
+char *get_cmd_file()
+{
+      char *filename;
+
+      if (cmd_file_head == NULL) filename = NULL;
+      else {
+            p_command_file head;
+
+            filename = cmd_file_head->filename;
+            head = cmd_file_head;
+            cmd_file_head = cmd_file_head->next;
+            free(head);
+      }
+      return filename;
+}
+
 #ifdef __MINGW32__
 # include  <io.h>
 # include  <fcntl.h>
@@ -156,7 +213,7 @@ static FILE*fopen_safe(const char*path)
       if (fd != -1)
 	    file = _fdopen(fd, "w");
 
-       return file;
+      return file;
 }
 #else
 # include  <fcntl.h>
@@ -169,7 +226,7 @@ static FILE*fopen_safe(const char*path)
       if (fd != -1)
 	    file = fdopen(fd, "w");
 
-       return file;
+      return file;
 }
 #endif
 
@@ -210,18 +267,120 @@ static const char*my_tempfile(const char*str, FILE**fout)
       return pathbuf;
 }
 
+static int t_version_only(void)
+{
+      int rc;
+      remove(source_path);
+      free(source_path);
+
+      fflush(0);
+      snprintf(tmp, sizeof tmp, "%s%civlpp -V", pbase, sep);
+      rc = system(tmp);
+      if (rc != 0) {
+	    fprintf(stderr, "Unable to get version from \"%s\"\n", tmp);
+      }
+
+      fflush(0);
+      snprintf(tmp, sizeof tmp, "%s%civl -V -C%s -C%s", pbase, sep,
+	       iconfig_path, iconfig_common_path);
+      rc = system(tmp);
+      if (rc != 0) {
+	    fprintf(stderr, "Unable to get version from \"%s\"\n", tmp);
+      }
+
+      if ( ! getenv("IVERILOG_ICONFIG")) {
+	    remove(iconfig_path);
+	    free(iconfig_path);
+	    remove(defines_path);
+	    free(defines_path);
+	    remove(compiled_defines_path);
+	    free(compiled_defines_path);
+      }
+
+      return 0;
+}
+
+static void build_preprocess_command(int e_flag)
+{
+      snprintf(tmp, sizeof tmp, "%s%civlpp %s%s -F%s -f%s -p%s ",
+	       pbase,sep, verbose_flag?" -v":"",
+	       e_flag?"":" -L", defines_path, source_path,
+	       compiled_defines_path);
+}
+
+static int t_preprocess_only(void)
+{
+      int rc;
+      char*cmd;
+      unsigned ncmd;
+
+      build_preprocess_command(1);
+
+      ncmd = strlen(tmp);
+      cmd = malloc(ncmd+1);
+      strcpy(cmd, tmp);
+
+      if (strcmp(opath,"-") != 0) {
+	    snprintf(tmp, sizeof tmp, " > %s", opath);
+	    cmd = realloc(cmd, ncmd+strlen(tmp)+1);
+	    strcpy(cmd+ncmd, tmp);
+	    ncmd += strlen(tmp);
+      }
+
+      if (verbose_flag)
+	    printf("preprocess: %s\n", cmd);
+
+      rc = system(cmd);
+      remove(source_path);
+      free(source_path);
+
+      if ( ! getenv("IVERILOG_ICONFIG")) {
+	    remove(iconfig_path);
+	    free(iconfig_path);
+	    remove(defines_path);
+	    free(defines_path);
+	    remove(compiled_defines_path);
+	    free(compiled_defines_path);
+      }
+
+      if (rc != 0) {
+	    if (WIFEXITED(rc)) {
+		  fprintf(stderr, "errors preprocessing Verilog program.\n");
+		  return WEXITSTATUS(rc);
+	    }
+
+	    fprintf(stderr, "Command signaled: %s\n", cmd);
+	    free(cmd);
+	    return -1;
+      }
+
+      free(cmd);
+      return 0;
+}
+
 /*
  * This is the default target type. It looks up the bits that are
  * needed to run the command from the configuration file (which is
  * already parsed for us) so we can handle must of the generic cases.
  */
-static int t_default(char*cmd, unsigned ncmd)
+static int t_compile()
 {
       unsigned rc;
+
+	/* Start by building the preprocess command line. */
+      build_preprocess_command(0);
+
+      size_t ncmd = strlen(tmp);
+      char*cmd = malloc(ncmd + 1);
+      strcpy(cmd, tmp);
+
 #ifdef __MINGW32__
       unsigned ncmd_start = ncmd;
+#else
+      int rtn;
 #endif
 
+	/* Build the ivl command and pipe it to the preprocessor. */
       snprintf(tmp, sizeof tmp, " | %s/ivl", base);
       rc = strlen(tmp);
       cmd = realloc(cmd, ncmd+rc+1);
@@ -266,123 +425,79 @@ static int t_default(char*cmd, unsigned ncmd)
       }
 #endif
 
-#ifdef DEBUG
-      char *original_cmd = malloc( strlen( cmd ) + (size_t)1 );
-      strcpy( original_cmd, cmd );
-
-      cmd = realloc( cmd, strlen( original_cmd ) + (size_t)19 ); /* more than enough memory for the ivlpp command and ">/tmp/ivlpp_stdout" */
-      if( cmd == NULL ){
-	exit( EXIT_FAILURE );
-      }
-
-      char *debug_ivl = malloc( strlen( original_cmd ) + (size_t)5 ); /* more than enough memory for "gdb " and ivl program_name */
-      strcpy( debug_ivl, "gdb " );
-
-      static char whitespace[] = " \t\f\r\v\n";
-      char *token;
-      int found_pipe = 0;
-      unsigned long token_counter = 0UL;
-
-      strcpy( cmd, "" );
-      for( token = strtok( original_cmd, whitespace ); token != NULL; token = strtok( NULL, whitespace ) ){
-
-	if( strcmp( token, "|" ) == 0 ){
-	  found_pipe = 1;
-	}
-
-	if( found_pipe ){
-	  token_counter++;
-	}
-
-	if( token_counter == 0UL ){
-	  strcat( cmd, token );
-	  strcat( cmd, " " ); /* harmless space after last token, and it was already there anyway */
-	}
-
-	if( token_counter == 2UL ){
-	  strcat( debug_ivl, token );
-	}
-
-	if( token_counter > 2UL ){
-	  printf( "%s\n", token );
-	}
-
-      }
-
-      free( original_cmd );
-      strcat( cmd, ">/tmp/ivlpp_stdout" );
-#endif
 
       if (verbose_flag)
 	    printf("translate: %s\n", cmd);
 
 
       rc = system(cmd);
-#ifdef DEBUG
-      if( verbose_flag ){
-	printf( "%s\n", debug_ivl );
-      }
-
-      system( debug_ivl ); /* For simplicity, ignore any return value, which is implementation dependent anyway. */
-      free( debug_ivl );
-#endif
-      remove(source_path);
-      if ( ! getenv("IVERILOG_ICONFIG"))
+      if ( ! getenv("IVERILOG_ICONFIG")) {
+	    remove(source_path);
+	    free(source_path);
 	    remove(iconfig_path);
+	    free(iconfig_path);
+	    remove(defines_path);
+	    free(defines_path);
+	    remove(compiled_defines_path);
+	    free(compiled_defines_path);
+      }
+#ifdef __MINGW32__  /* MinGW just returns the exit status, so return it! */
+      free(cmd);
+      return rc;
+#else
+      rtn = 0;
       if (rc != 0) {
 	    if (rc == 127) {
 		  fprintf(stderr, "Failed to execute: %s\n", cmd);
-		  return 1;
+		  rtn = 1;
+	    } else if (WIFEXITED(rc)) {
+		  rtn = WEXITSTATUS(rc);
+	    } else {
+		  fprintf(stderr, "Command signaled: %s\n", cmd);
+		  rtn = -1;
 	    }
-
-	    if (WIFEXITED(rc))
-		  return WEXITSTATUS(rc);
-
-	    fprintf(stderr, "Command signaled: %s\n", cmd);
-	    return -1;
       }
 
-      return 0;
+      free(cmd);
+      return rtn;
+#endif
 }
 
 
 static void process_warning_switch(const char*name)
 {
       if (strcmp(name,"all") == 0) {
-	    strcat(warning_flags, "iptu");
-
+	    process_warning_switch("implicit");
+	    process_warning_switch("portbind");
+	    process_warning_switch("timescale");
       } else if (strcmp(name,"implicit") == 0) {
-	    if (! strchr(warning_flags+2, 'i'))
+	    if (! strchr(warning_flags, 'i'))
 		  strcat(warning_flags, "i");
       } else if (strcmp(name,"portbind") == 0) {
-	    if (! strchr(warning_flags+2, 'p'))
+	    if (! strchr(warning_flags, 'p'))
 		  strcat(warning_flags, "p");
       } else if (strcmp(name,"timescale") == 0) {
-	    if (! strchr(warning_flags+2, 't'))
+	    if (! strchr(warning_flags, 't'))
 		  strcat(warning_flags, "t");
-      } else if (strcmp(name,"unused") == 0) {
-	    if (! strchr(warning_flags+2, 'u'))
-		  strcat(warning_flags, "u");
+	/* Since the infinite loop check is not part of 'all' it
+	 * does not have a no- version. */
+      } else if (strcmp(name,"infloop") == 0) {
+	    if (! strchr(warning_flags, 'l'))
+		  strcat(warning_flags, "l");
       } else if (strcmp(name,"no-implicit") == 0) {
-	    char*cp = strchr(warning_flags+2, 'i');
+	    char*cp = strchr(warning_flags, 'i');
 	    if (cp) while (*cp) {
 		  cp[0] = cp[1];
 		  cp += 1;
 	    }
       } else if (strcmp(name,"no-portbind") == 0) {
-	    char*cp = strchr(warning_flags+2, 'p');
+	    char*cp = strchr(warning_flags, 'p');
 	    if (cp) while (*cp) {
 		  cp[0] = cp[1];
 		  cp += 1;
 	    }
       } else if (strcmp(name,"no-timescale") == 0) {
-	    char*cp = strchr(warning_flags+2, 't');
-	    if (cp) while (*cp) {
-		  cp[0] = cp[1];
-		  cp += 1;
-	    }
-      } else if (strcmp(name,"no-unused") == 0) {
-	    char*cp = strchr(warning_flags+2, 'u');
+	    char*cp = strchr(warning_flags, 't');
 	    if (cp) while (*cp) {
 		  cp[0] = cp[1];
 		  cp += 1;
@@ -407,32 +522,12 @@ void process_library2_switch(const char *name)
 
 void process_include_dir(const char *name)
 {
-      if (inc_list == 0) {
-	    inc_list = malloc(strlen(" -I")+strlen(name)+1);
-	    strcpy(inc_list, " -I");
-	    strcat(inc_list, name);
-      } else {
-	    inc_list = realloc(inc_list, strlen(inc_list)
-			       + strlen(" -I")
-			       + strlen(name) + 1);
-	    strcat(inc_list, " -I");
-	    strcat(inc_list, name);
-      }
+      fprintf(defines_file, "I:%s\n", name);
 }
 
 void process_define(const char*name)
 {
-      if (def_list == 0) {
-	    def_list = malloc(strlen(" -D")+strlen(name)+1);
-	    strcpy(def_list, " -D");
-	    strcat(def_list, name);
-      } else {
-	    def_list = realloc(def_list, strlen(def_list)
-			       + strlen(" -D")
-			       + strlen(name) + 1);
-	    strcat(def_list, " -D");
-	    strcat(def_list, name);
-      }
+      fprintf(defines_file,"D:%s\n", name);
 }
 
 /*
@@ -441,7 +536,7 @@ void process_define(const char*name)
  * .sft suffix, and if so pass that as a sys_func file. Otherwise, it
  * is a Verilog source file to be written into the file list.
  */
-void process_file_name(const char*name)
+void process_file_name(const char*name, int lib_flag)
 {
       if (strlen(name) > 4 && strcasecmp(".sft", name+strlen(name)-4) == 0) {
 	    fprintf(iconfig_file,"sys_func:%s\n", name);
@@ -449,27 +544,105 @@ void process_file_name(const char*name)
       } else {
 	    fprintf(source_file, "%s\n", name);
 	    source_count += 1;
+	    if (lib_flag)
+		  fprintf(iconfig_file,"library_file:%s\n", name);
       }
 }
 
 int process_generation(const char*name)
 {
-      if (strcmp(name,"1") == 0)
-	    generation = "1";
+      if (strcmp(name,"1995") == 0)
+	    generation = "1995";
 
-      else if (strcmp(name,"2") == 0)
-	    generation = "2";
+      else if (strcmp(name,"2001") == 0)
+	    generation = "2001";
 
-      else if (strcmp(name,"3.0") == 0)
-	    generation = "3.0";
+      else if (strcmp(name,"2005") == 0)
+	    generation = "2005";
+
+      else if (strcmp(name,"1") == 0) { /* Deprecated: use 1995 */
+	    generation = "1995";
+	    gen_xtypes = "no-xtypes";
+	    gen_icarus = "no-icarus-misc";
+
+      } else if (strcmp(name,"2") == 0) { /* Deprecated: use 2001 */
+	    generation = "2001";
+	    gen_xtypes = "no-xtypes";
+	    gen_icarus = "no-icarus-misc";
+
+      } else if (strcmp(name,"2x") == 0) { /* Deprecated: use 2005/xtypes */
+	    generation = "2005";
+	    gen_xtypes = "xtypes";
+	    gen_icarus = "icarus-misc";
+
+      } else if (strcmp(name,"xtypes") == 0)
+	    gen_xtypes = "xtypes";
+
+      else if (strcmp(name,"no-xtypes") == 0)
+	    gen_xtypes = "no-xtypes";
+
+      else if (strcmp(name,"icarus-misc") == 0)
+	    gen_icarus = "icarus-misc";
+
+      else if (strcmp(name,"no-icarus-misc") == 0)
+	    gen_icarus = "no-icarus-misc";
+
+      else if (strcmp(name,"specify") == 0)
+	    gen_specify = "specify";
+
+      else if (strcmp(name,"no-specify") == 0)
+	    gen_specify = "no-specify";
+
+      else if (strcmp(name,"std-include") == 0)
+	     gen_std_include = 1;
+
+      else if (strcmp(name,"no-std-include") == 0)
+	     gen_std_include = 0;
+      
+      else if (strcmp(name,"relative-include") == 0)
+	    gen_relative_include = 1;
+      
+      else if (strcmp(name,"no-relative-include") == 0)
+	    gen_relative_include = 0;
+
+      else if (strcmp(name,"io-range-error") == 0)
+	    gen_io_range_error = "io-range-error";
+
+      else if (strcmp(name,"no-io-range-error") == 0)
+	    gen_io_range_error = "no-io-range-error";
+
+      else if (strcmp(name,"strict-ca-eval") == 0)
+	    gen_strict_ca_eval = "strict-ca-eval";
+
+      else if (strcmp(name,"no-strict-ca-eval") == 0)
+	    gen_strict_ca_eval = "no-strict-ca-eval";
+
+      else if (strcmp(name,"verilog-ams") == 0)
+	    gen_verilog_ams = "verilog-ams";
+
+      else if (strcmp(name,"no-verilog-ams") == 0)
+	    gen_verilog_ams = "no-verilog-ams";
+      
+      else if (strcmp(name,"system-verilog") == 0)
+	    gen_system_verilog = "system-verilog";
 
       else {
 	    fprintf(stderr, "Unknown/Unsupported Language generation "
-		    "%s\n", name);
+		    "%s\n\n", name);
 	    fprintf(stderr, "Supported generations are:\n");
-	    fprintf(stderr, "    1   -- IEEE1364-1995 (Verilog 1)\n"
-		            "    2   -- IEEE1364-2001 (Verilog 2001)\n"
-		            "    3.0 -- SystemVerilog 3.0\n");
+	    fprintf(stderr, "    1995 -- IEEE1364-1995\n"
+		            "    2001 -- IEEE1364-2001\n"
+		            "    2005 -- IEEE1364-2005\n"
+		            "Other generation flags:\n"
+		            "    specify | no-specify\n"
+		            "    verilog-ams | no-verilog-ams\n"
+		            "    std-include | no-std-include\n"
+		            "    relative-include | no-relative-include\n"
+		            "    xtypes | no-xtypes\n"
+		            "    icarus-misc | no-icarus-misc\n"
+		            "    io-range-error | no-io-range-error\n"
+		            "    strict-ca-eval | no-strict-ca-eval\n"
+					"    system-verilog\n");
 	    return 1;
       }
 
@@ -477,12 +650,25 @@ int process_generation(const char*name)
 }
 
 /*
- * This function fills in th ivl_root directory with the appropriate
- * path, based on operating system, build time configuration, and
- * environment.
+ * If it exists add the SFT file for the given module.
  */
-static char* get_root_dir()
+void add_sft_file(const char *module)
 {
+      char *file;
+
+      file = (char *) malloc(strlen(base)+1+strlen(module)+4+1);
+      sprintf(file, "%s%c%s.sft", base, sep, module);
+      if (access(file, R_OK) == 0)
+            fprintf(iconfig_file, "sys_func:%s\n", file);
+      free(file);
+}
+
+int main(int argc, char **argv)
+{
+      int e_flag = 0;
+      int version_flag = 0;
+      int opt, idx;
+
 #ifdef __MINGW32__
       { char * s;
 	char basepath[1024];
@@ -509,56 +695,16 @@ static char* get_root_dir()
 	s = strrchr(ivl_root, sep);
 	if (s) *s = 0;
 	strcat(ivl_root, "\\lib\\ivl");
+
+	base = ivl_root;
       }
+
 #else
-
-      { char*var = 0;
-
-	  /* In other systems, use the configured root. */
-        strcpy(ivl_root, IVL_ROOT);
-
-	  /* In any case, the IVL_ROOT variable can be used to override
-	     the install location at run time. If it is set, use that
-	     value instead. */
-	if ( (var = getenv(IVL_ROOT_VARIABLE)) ) {
-	      strncpy(ivl_root, var, MAXSIZE);
-	      strcat(ivl_root, "/lib/ivl");
-	}
-      }
-#endif
-
-      return ivl_root;
-}
-
-int main(int argc, char **argv)
-{
-      char*cmd;
-      unsigned ncmd;
-      int e_flag = 0;
-      int version_flag = 0;
-      int opt, idx, rc;
-
-      base = get_root_dir();
-
-#ifdef __MINGW32__
-	/* Under Windows, there is no point checking the root
-	   directory. */
-#else
-	/* On other systems, check that the root directory seems
-	   reasonable. */
-      { size_t tmp_len = strlen(base) + 16;
-        char*  tmp = malloc(tmp_len);
-	snprintf(tmp, tmp_len, "%s%civl", base, sep);
-
-	if (0 != access(tmp, F_OK)) {
-	      fprintf(stderr, "%s: INSTALLATION ERROR: Directory %s"
-		      " does not contain ivl files.\n", argv[0], base);
-	      fprintf(stderr, "%s:                   : Check installation"
-		      " or set " IVL_ROOT_VARIABLE " correctly.\n", argv[0]);
-	      return 1;
-	}
-	free(tmp);
-      }
+        /* In a UNIX environment, the IVL_ROOT from the Makefile is
+	   dependable. It points to the $prefix/lib/ivl directory,
+	   where the sub-parts are installed. */
+      strcpy(ivl_root, IVL_ROOT);
+      base = ivl_root;
 #endif
 
 	/* Create a temporary file for communicating input parameters
@@ -571,6 +717,21 @@ int main(int argc, char **argv)
 	    return 1;
       }
 
+      defines_path = strdup(my_tempfile("ivrlg2", &defines_file));
+      if (NULL == defines_file) {
+	    fprintf(stderr, "%s: Error opening temporary file %s\n",
+		    argv[0], defines_path);
+	    fprintf(stderr, "%s: Please check TMP or TMPDIR.\n", argv[0]);
+
+	    fclose(source_file);
+	    remove(source_path);
+	    return 1;
+      }
+
+      fprintf(defines_file, "D:__ICARUS__=1\n");
+      if (strcmp(gen_verilog_ams,"verilog-ams") == 0)
+	    fprintf(defines_file, "D:__VAMS_ENABLE__=1\n");
+
 	/* Create another temporary file for passing configuration
 	   information to ivl. */
 
@@ -591,10 +752,22 @@ int main(int argc, char **argv)
 	    fprintf(stderr, "%s: Please check TMP or TMPDIR.\n", argv[0]);
 	    fclose(source_file);
 	    remove(source_path);
+
+	    fclose(defines_file);
+	    remove(defines_path);
 	    return 1;
       }
 
-      while ((opt = getopt(argc, argv, "B:c:D:Ef:g:hI:M:m:N::o:p:Ss:T:t:vVW:y:Y:")) != EOF) {
+	/* Create a temporary file (I only really need the path) that
+	   can carry preprocessor precompiled defines to the library. */
+      { FILE*tmp_file = 0;
+	compiled_defines_path = strdup(my_tempfile("ivrli", &tmp_file));
+	if (tmp_file) {
+	      fclose(tmp_file);
+	}
+      }
+
+      while ((opt = getopt(argc, argv, "B:c:D:d:Ef:g:hI:M:m:N::o:p:Ss:T:t:vVW:y:Y:")) != EOF) {
 
 	    switch (opt) {
 		case 'B':
@@ -608,30 +781,30 @@ int main(int argc, char **argv)
 			base=optarg;
 		  }
 		  break;
- 		case 'c':
-		  command_filename = malloc(strlen(optarg)+1);
- 		  strcpy(command_filename, optarg);
- 		  break;
+		case 'c':
+		case 'f':
+		  add_cmd_file(optarg);
+		  break;
 		case 'D':
 		  process_define(optarg);
 		  break;
 		case 'E':
 		  e_flag = 1;
 		  break;
-		case 'f':
-		  fprintf(stderr, "warning: The -f flag is moved to -p\n");
 		case 'p':
 		  fprintf(iconfig_file, "flag:%s\n", optarg);
 		  break;
+		case 'd':
+		  fprintf(iconfig_file, "debug:%s\n", optarg);
+		  break;
 
 		case 'g':
-		  rc = process_generation(optarg);
-		  if (rc != 0)
+		  if (process_generation(optarg) != 0)
 			return -1;
 		  break;
- 		case 'h':
- 		  fprintf(stderr, "%s\n", HELP);
- 		  return 1;
+		case 'h':
+		  fprintf(stderr, "%s\n", HELP);
+		  return 1;
 
 		case 'I':
 		  process_include_dir(optarg);
@@ -643,6 +816,7 @@ int main(int argc, char **argv)
 
 		case 'm':
 		  fprintf(iconfig_file, "module:%s\n", optarg);
+		  add_sft_file(optarg);
 		  break;
 
 		case 'N':
@@ -700,12 +874,9 @@ int main(int argc, char **argv)
 	    pbase = base;
 
       if (version_flag || verbose_flag) {
-	    printf("Icarus Verilog version " VERSION " ($Name:  $)\n");
-	    printf("Copyright 1998-2003 Stephen Williams\n");
+	    printf("Icarus Verilog version " VERSION " (" VERSION_TAG ")\n\n");
+	    printf("Copyright 1998-2009 Stephen Williams\n\n");
 	    puts(NOTICE);
-
-	    if (version_flag)
-		  return 0;
       }
 
 	/* Make a common conf file path to reflect the target. */
@@ -720,188 +891,117 @@ int main(int argc, char **argv)
 	   how to handle them. */
       fprintf(iconfig_file, "sys_func:%s%csystem.sft\n", base, sep);
 
+	/* If verilog-2005 is enabled or icarus-misc or verilog-ams,
+	 * then include the v2005_math library. */
+      if (strcmp(generation, "2005") == 0 ||
+          strcmp(gen_icarus, "icarus-misc") == 0 ||
+          strcmp(gen_verilog_ams, "verilog-ams") == 0) {
+	    fprintf(iconfig_file, "sys_func:%s%cv2005_math.sft\n", base, sep);
+	    fprintf(iconfig_file, "module:v2005_math\n");
+      }
+	/* If verilog-ams or icarus_misc is enabled, then include the
+	 * va_math module as well. */
+      if (strcmp(gen_verilog_ams,"verilog-ams") == 0 ||
+          strcmp(gen_icarus, "icarus-misc") == 0) {
+	    fprintf(iconfig_file, "sys_func:%s%cva_math.sft\n", base, sep);
+	    fprintf(iconfig_file, "module:va_math\n");
+      }
+
       if (mtm != 0) fprintf(iconfig_file, "-T:%s\n", mtm);
       fprintf(iconfig_file, "generation:%s\n", generation);
+      fprintf(iconfig_file, "generation:%s\n", gen_specify);
+      fprintf(iconfig_file, "generation:%s\n", gen_xtypes);
+      fprintf(iconfig_file, "generation:%s\n", gen_io_range_error);
+      fprintf(iconfig_file, "generation:%s\n", gen_strict_ca_eval);
+      fprintf(iconfig_file, "generation:%s\n", gen_verilog_ams);
+      fprintf(iconfig_file, "generation:%s\n", gen_icarus);
+      fprintf(iconfig_file, "generation:%s\n", gen_system_verilog);
       fprintf(iconfig_file, "warnings:%s\n", warning_flags);
       fprintf(iconfig_file, "out:%s\n", opath);
       if (depfile) fprintf(iconfig_file, "depfile:%s\n", depfile);
 
-      if (command_filename) {
+      while ( (command_filename = get_cmd_file()) ) {
 	    int rc;
 
 	    if (( fp = fopen(command_filename, "r")) == NULL ) {
-		  fprintf(stderr, "%s: Can't open %s\n",
-			  argv[0], command_filename);
+		  fprintf(stderr, "%s: cannot open command file %s "
+			  "for reading.\n", argv[0], command_filename);
 		  return 1;
 	    }
 
 	    cfreset(fp, command_filename);
 	    rc = cfparse();
 	    if (rc != 0) {
-		  fprintf(stderr, "%s: error reading command file\n",
-			  command_filename);
+		  fprintf(stderr, "%s: parsing failed in base command "
+		          "file %s.\n", argv[0], command_filename);
 		  return 1;
 	    }
+            free(command_filename);
+      }
+
+      if (depfile) {
+	    fprintf(defines_file, "M:%s\n", depfile);
       }
 
 	/* Finally, process all the remaining words on the command
 	   line as file names. */
       for (idx = optind ;  idx < argc ;  idx += 1)
-	    process_file_name(argv[idx]);
+	    process_file_name(argv[idx], 0);
 
+	/* If the use of a default include directory is not
+	   specifically disabled, then write that directory as the
+	   very last include directory to use... always. */
+      if (gen_std_include) {
+	    fprintf(defines_file, "I:%s%cinclude\n", base, sep);
+      }
+
+      if (gen_relative_include) {
+	    fprintf(defines_file, "relative include:true\n");
+      } else {
+	    fprintf(defines_file, "relative include:false\n");
+      }
 
       fclose(source_file);
       source_file = 0;
 
-      if (source_count == 0) {
-	    fprintf(stderr, "%s: No input files.\n", argv[0]);
- 	    fprintf(stderr, "%s\n", HELP);
+      fclose(defines_file);
+      defines_file = 0;
+
+	/* If we are planning on opening a dependencies file, then
+	   open and truncate it here. The other phases of compilation
+	   will append to the file, so this is necessray to make sure
+	   it starts out empty. */
+      if (depfile) {
+	    FILE*fd = fopen(depfile, "w");
+	    fclose(fd);
+      }
+
+      if (source_count == 0 && !version_flag) {
+	    fprintf(stderr, "%s: no source files.\n\n%s\n", argv[0], HELP);
 	    return 1;
       }
 
-
-	/* Start building the preprocess command line. */
-
-      sprintf(tmp, "%s%civlpp %s%s -D__ICARUS__=1 -f%s ", pbase,sep,
-	      verbose_flag?" -v":"",
-	      e_flag?"":" -L", source_path);
-
-      ncmd = strlen(tmp);
-      cmd = malloc(ncmd + 1);
-      strcpy(cmd, tmp);
-
-      if (inc_list) {
-	    cmd = realloc(cmd, ncmd + strlen(inc_list) + 1);
-	    strcat(cmd, inc_list);
-	    ncmd += strlen(inc_list);
-      }
-
-      if (def_list) {
-	    cmd = realloc(cmd, ncmd + strlen(def_list) + 1);
-	    strcat(cmd, def_list);
-	    ncmd += strlen(def_list);
-      }
-
-      if (depfile) {
-	    cmd = realloc(cmd, ncmd + strlen(depfile) + 5);
-	    strcat(cmd, " -M ");
-	    strcat(cmd, depfile);
-	    ncmd += strlen(depfile) + 4;
-      }
-
-	/* If the -E flag was given on the command line, then all we
-	   do is run the preprocessor and put the output where the
-	   user wants it. */
-      if (e_flag) {
-	    int rc;
-	    if (strcmp(opath,"-") != 0) {
-		  sprintf(tmp, " > %s", opath);
-		  cmd = realloc(cmd, ncmd+strlen(tmp)+1);
-		  strcpy(cmd+ncmd, tmp);
-		  ncmd += strlen(tmp);
-	    }
-
-	    if (verbose_flag)
-		  printf("preprocess: %s\n", cmd);
-
-	    rc = system(cmd);
-	    remove(source_path);
-	    fclose(iconfig_file);
-	    if ( ! getenv("IVERILOG_ICONFIG"))
-		  remove(iconfig_path);
-
-	    if (rc != 0) {
-		  if (WIFEXITED(rc)) {
-			fprintf(stderr, "errors preprocessing Verilog program.\n");
-			return WEXITSTATUS(rc);
-		  }
-
-		  fprintf(stderr, "Command signaled: %s\n", cmd);
-		  return -1;
-	    }
-
-	    return 0;
-      }
+      fprintf(iconfig_file, "iwidth:%u\n", integer_width);
 
 	/* Write the preprocessor command needed to preprocess a
 	   single file. This may be used to preprocess library
 	   files. */
-      fprintf(iconfig_file, "ivlpp:%s%civlpp -D__ICARUS__ -L %s %s\n",
-	      pbase, sep,
-	      inc_list? inc_list : "",
-	      def_list? def_list : "");
+      fprintf(iconfig_file, "ivlpp:%s%civlpp -L -F%s -P%s\n",
+	      pbase, sep, defines_path, compiled_defines_path);
 
 	/* Done writing to the iconfig file. Close it now. */
       fclose(iconfig_file);
 
-      return t_default(cmd, ncmd);
+	/* If we're only here for the version output, then we're done. */
+      if (version_flag)
+	    return t_version_only();
 
-      return 0;
+	/* If the -E flag was given on the command line, then all we
+	   do is run the preprocessor and put the output where the
+	   user wants it. */
+      if (e_flag)
+	    return t_preprocess_only();
+
+	/* Otherwise, this is a full compile. */
+      return t_compile();
 }
-
-/*
- * $Log: main.c,v $
- * Revision 1.65.2.6  2007/05/30 17:48:26  steve
- *  DEBUG aids. (Alan Feldstein)
- *
- * Revision 1.65.2.5  2006/07/07 21:31:50  steve
- *  Root dir variable does not include lib/ivl components.
- *
- * Revision 1.65.2.4  2006/06/27 01:30:20  steve
- *  Fix unused var warning for mingw32 build.
- *
- * Revision 1.65.2.3  2006/06/14 03:01:49  steve
- *  Remove redundant call to get_root_dir.
- *
- * Revision 1.65.2.2  2006/06/12 00:16:53  steve
- *  Add support for -Wunused warnings.
- *
- * Revision 1.65.2.1  2006/03/26 21:47:26  steve
- *  More installation directory flexibility.
- *
- * Revision 1.65  2004/06/17 14:47:22  steve
- *  Add a .sft file for the system functions.
- *
- * Revision 1.64  2004/03/10 04:51:25  steve
- *  Add support for system function table files.
- *
- * Revision 1.63  2004/02/15 18:03:30  steve
- *  Cleanup of warnings.
- *
- * Revision 1.62  2003/12/12 04:36:48  steve
- *  Fix make check to support -tconf configuration method.
- *
- * Revision 1.61  2003/11/18 06:31:46  steve
- *  Remove the iverilog.conf file.
- *
- * Revision 1.60  2003/11/13 05:55:33  steve
- *  Move the DLL= flag to target config files.
- *
- * Revision 1.59  2003/11/13 04:09:49  steve
- *  Pass flags through the temporary config file.
- *
- * Revision 1.58  2003/11/01 04:21:57  steve
- *  Add support for a target static config file.
- *
- * Revision 1.57  2003/10/26 22:43:42  steve
- *  Improve -V messages,
- *
- * Revision 1.56  2003/09/26 21:25:58  steve
- *  Warnings cleanup.
- *
- * Revision 1.55  2003/09/23 05:57:15  steve
- *  Pass -m flag from driver via iconfig file.
- *
- * Revision 1.54  2003/09/22 01:12:09  steve
- *  Pass more ivl arguments through the iconfig file.
- *
- * Revision 1.53  2003/08/26 16:26:02  steve
- *  ifdef idents correctly.
- *
- * Revision 1.52  2003/02/22 04:55:36  steve
- *  portbind adds p, not i, flag.
- *
- * Revision 1.51  2003/02/22 04:12:49  steve
- *  Add the portbind warning.
- */
-
