@@ -1,6 +1,6 @@
 
 const char COPYRIGHT[] =
-          "Copyright (c) 1998-2004 Stephen Williams (steve@icarus.com)";
+          "Copyright (c) 1998-2009 Stephen Williams (steve@icarus.com)";
 
 /*
  *    This source code is free software; you can redistribute it
@@ -18,11 +18,9 @@ const char COPYRIGHT[] =
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: main.cc,v 1.86.2.3 2006/06/12 00:16:53 steve Exp $"
-#endif
 
 # include "config.h"
+# include "version.h"
 
 const char NOTICE[] =
 "  This program is free software; you can redistribute it and/or modify\n"
@@ -35,15 +33,16 @@ const char NOTICE[] =
 "  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 "  GNU General Public License for more details.\n"
 "\n"
-"  You should have received a copy of the GNU General Public License\n"
-"  along with this program; if not, write to the Free Software\n"
-"  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA\n"
+"  You should have received a copy of the GNU General Public License along\n"
+"  with this program; if not, write to the Free Software Foundation, Inc.,\n"
+"  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.\n"
 ;
 
 # include  <stdio.h>
 # include  <iostream>
 # include  <fstream>
 # include  <queue>
+# include  <cstring>
 # include  <list>
 # include  <map>
 # include  <unistd.h>
@@ -56,9 +55,12 @@ const char NOTICE[] =
 #endif
 # include  "pform.h"
 # include  "parse_api.h"
+# include  "PGenerate.h"
 # include  "netlist.h"
 # include  "target.h"
 # include  "compiler.h"
+# include  "discipline.h"
+# include  "t-dll.h"
 
 #if defined(__MINGW32__) && !defined(HAVE_GETOPT_H)
 extern "C" int getopt(int argc, char*argv[], const char*fmt);
@@ -75,18 +77,27 @@ extern "C" const char*optarg;
 /* Count errors detected in flag processing. */
 unsigned flag_errors = 0;
 
-const char VERSION[] = "$Name:  $";
-
 const char*basedir = ".";
 
-const char*target = "null";
-
+/*
+ * These are the language support control flags. These support which
+ * language features (the generation) to support. The generation_flag
+ * is a major mode, and the gn_* flags control specific sub-features.
+ */
 generation_t generation_flag = GN_DEFAULT;
+bool gn_icarus_misc_flag = true;
+bool gn_cadence_types_flag = true;
+bool gn_specify_blocks_flag = true;
+bool gn_io_range_error_flag = true;
+bool gn_strict_ca_eval_flag = false;
+bool gn_verilog_ams_flag = false;
+bool gn_system_verilog_flag = false;
 
 map<string,const char*> flags;
 char*vpi_module_list = 0;
 
 map<perm_string,unsigned> missing_modules;
+map<perm_string,bool> library_file_map;
 
 list<const char*> library_suff;
 
@@ -103,7 +114,7 @@ FILE *depend_file = NULL;
 bool warn_implicit  = false;
 bool warn_timescale = false;
 bool warn_portbinding = false;
-bool warn_unused = false;
+bool warn_inf_loop = false;
 
 bool error_implicit = false;
 
@@ -113,20 +124,31 @@ bool error_implicit = false;
 bool debug_scopes = false;
 bool debug_eval_tree = false;
 bool debug_elaborate = false;
-bool debug_synth = false;
-bool debug_cprop = false;
+bool debug_elab_pexpr = false;
+bool debug_synth2 = false;
+bool debug_optimizer = false;
+
+/*
+ * Miscellaneous flags.
+ */
+bool disable_virtual_pins = false;
+unsigned long array_size_limit = 16777216;  // Minimum required by IEEE-1364?
 
 /*
  * Verbose messages enabled.
  */
 bool verbose_flag = false;
 
+unsigned integer_width = 32;
+
 /*
  * Keep a heap of identifier strings that I encounter. This is a more
  * efficient way to allocate those strings.
  */
 StringHeapLex lex_strings;
 
+StringHeapLex filename_strings;
+
 /*
  * In library searches, Windows file names are never case sensitive.
  */
@@ -146,7 +168,6 @@ extern void synth(Design*des);
 extern void synth2(Design*des);
 extern void syn_rules(Design*des);
 extern void nodangle(Design*des);
-extern void xnfio(Design*des);
 
 typedef void (*net_func)(Design*);
 static struct net_func_map {
@@ -158,7 +179,6 @@ static struct net_func_map {
       { "synth",   &synth },
       { "synth2",  &synth2 },
       { "syn-rules",   &syn_rules },
-      { "xnfio",   &xnfio },
       { 0, 0 }
 };
 
@@ -184,17 +204,66 @@ const char *net_func_to_name(const net_func func)
 
 static void process_generation_flag(const char*gen)
 {
-      if (strcmp(gen,"1") == 0)
+      if (strcmp(gen,"1") == 0) { // FIXME: Deprecated for 1995
 	    generation_flag = GN_VER1995;
 
-      else if (strcmp(gen,"2") == 0)
+      } else if (strcmp(gen,"2") == 0) { // FIXME: Deprecated for 2001
 	    generation_flag = GN_VER2001;
 
-      else if (strcmp(gen,"3.0") == 0)
-	    generation_flag = GN_SYSVER30;
+      } else if (strcmp(gen,"2x") == 0) { // FIXME: Deprecated for 2001
+	    generation_flag = GN_VER2001;
+	    gn_icarus_misc_flag = true;
 
-      else
-	    generation_flag = GN_DEFAULT;
+      } else if (strcmp(gen,"1995") == 0) {
+	    generation_flag = GN_VER1995;
+
+      } else if (strcmp(gen,"2001") == 0) {
+	    generation_flag = GN_VER2001;
+
+      } else if (strcmp(gen,"2005") == 0) {
+	    generation_flag = GN_VER2005;
+
+      } else if (strcmp(gen,"icarus-misc") == 0) {
+	    gn_icarus_misc_flag = true;
+
+      } else if (strcmp(gen,"no-icarus-misc") == 0) {
+	    gn_icarus_misc_flag = false;
+
+      } else if (strcmp(gen,"xtypes") == 0) {
+	    gn_cadence_types_flag = true;
+
+      } else if (strcmp(gen,"no-xtypes") == 0) {
+	    gn_cadence_types_flag = false;
+
+      } else if (strcmp(gen,"specify") == 0) {
+	    gn_specify_blocks_flag = true;
+
+      } else if (strcmp(gen,"no-specify") == 0) {
+	    gn_specify_blocks_flag = false;
+
+      } else if (strcmp(gen,"verilog-ams") == 0) {
+	    gn_verilog_ams_flag = true;
+
+      } else if (strcmp(gen,"no-verilog-ams") == 0) {
+	    gn_verilog_ams_flag = false;
+
+      } else if (strcmp(gen,"io-range-error") == 0) {
+	    gn_io_range_error_flag = true;
+
+      } else if (strcmp(gen,"no-io-range-error") == 0) {
+	    gn_io_range_error_flag = false;
+
+      } else if (strcmp(gen,"strict-ca-eval") == 0) {
+	    gn_strict_ca_eval_flag = true;
+
+      } else if (strcmp(gen,"no-strict-ca-eval") == 0) {
+	    gn_strict_ca_eval_flag = false;
+
+      } else if (strcmp(gen,"system-verilog") == 0) {
+        gn_system_verilog_flag = true;
+
+	  } else {
+      }
 }
 
 static void parm_to_flagmap(const string&flag)
@@ -214,6 +283,9 @@ static void parm_to_flagmap(const string&flag)
       flags[key] = value;
 }
 
+static void find_module_mention(map<perm_string,bool>&check_map, Module*m);
+static void find_module_mention(map<perm_string,bool>&check_map, PGenerate*s);
+
 /*
  * Read the contents of a config file. This file is a temporary
  * configuration file made by the compiler driver to carry the bulky
@@ -237,7 +309,7 @@ static void parm_to_flagmap(const string&flag)
  *    -T:<min/typ/max>
  *        Select which expression to use.
  *
- *    -t:<target>
+ *    -t:<target>    (obsolete)
  *        Usually, "-t:dll"
  *
  *    basedir:<path>
@@ -255,13 +327,22 @@ static void parm_to_flagmap(const string&flag)
  *    functor:<name>
  *        Append a named functor to the processing path.
  *
- *    generation:<1|2|3.0>
+ *    generation:<1|2|2x|xtypes|no-xtypes|specify|no-specify>
  *        This is the generation flag
  *
  *    ivlpp:<preprocessor command>
  *        This specifies the ivlpp command line used to process
  *        library modules as I read them in.
  *
+ *    iwidth:<bits>
+ *        This specifies the width of integer variables. (that is,
+ *        variables declared using the "integer" keyword.)
+ *
+ *    library_file:<path>
+ *        This marks that a source file with the given path is a
+ *        library. Any modules in that file are marked as library
+ *        modules.
+ *
  *    module:<name>
  *        Load a VPI module.
  *
@@ -314,21 +395,24 @@ static void read_iconfig_file(const char*ipath)
 		  basedir = strdup(cp);
 
 	    } else if (strcmp(buf, "debug") == 0) {
-		  if (strcmp(cp, "scope") == 0) {
+		  if (strcmp(cp, "scopes") == 0) {
 			debug_scopes = true;
-			cerr << "debug: Enable scope debug" << endl;
+			cerr << "debug: Enable scopes debug" << endl;
 		  } else if (strcmp(cp,"eval_tree") == 0) {
 			debug_eval_tree = true;
 			cerr << "debug: Enable eval_tree debug" << endl;
 		  } else if (strcmp(cp,"elaborate") == 0) {
 			debug_elaborate = true;
 			cerr << "debug: Enable elaborate debug" << endl;
-		  } else if (strcmp(cp,"synth") == 0) {
-			debug_synth = true;
-			cerr << "debug: Enable synthesis debug" << endl;
-		  } else if (strcmp(cp,"cprop") == 0) {
-			debug_cprop = true;
-			cerr << "debug: Enable cprop debug" << endl;
+		  } else if (strcmp(cp,"elab_pexpr") == 0) {
+			debug_elab_pexpr = true;
+			cerr << "debug: Enable elaborate-pexpr debug" << endl;
+		  } else if (strcmp(cp,"synth2") == 0) {
+			debug_synth2 = true;
+			cerr << "debug: Enable synth2 debug" << endl;
+		  } else if (strcmp(cp,"optimizer") == 0) {
+			debug_optimizer = true;
+			cerr << "debug: Enable optimizer debug" << endl;
 		  } else {
 		  }
 
@@ -358,6 +442,13 @@ static void read_iconfig_file(const char*ipath)
 	    } else if (strcmp(buf, "ivlpp") == 0) {
 		  ivlpp_string = strdup(cp);
 
+	    } else if (strcmp(buf, "iwidth") == 0) {
+		  integer_width = strtoul(cp,0,10);
+
+	    } else if (strcmp(buf, "library_file") == 0) {
+		  perm_string path = filename_strings.make(cp);
+		  library_file_map[path] = true;
+
 	    } else if (strcmp(buf,"module") == 0) {
 		  if (vpi_module_list == 0) {
 			vpi_module_list = strdup(cp);
@@ -388,15 +479,15 @@ static void read_iconfig_file(const char*ipath)
 		      case 'i':
 			warn_implicit = true;
 			break;
+		      case 'l':
+			warn_inf_loop = true;
+			break;
 		      case 'p':
 			warn_portbinding = true;
 			break;
 		      case 't':
 			warn_timescale = true;
 			break;
-		      case 'u':
-			warn_unused = true;
-			break;
 		      default:
 			break;
 		  }
@@ -411,7 +502,7 @@ static void read_iconfig_file(const char*ipath)
 		  library_suff.push_back(strdup(cp));
 
 	    } else if (strcmp(buf,"-t") == 0) {
-		  target = strdup(cp);
+		    // NO LONGER USED
 
 	    } else if (strcmp(buf,"-T") == 0) {
 		  if (strcmp(cp,"min") == 0) {
@@ -461,6 +552,7 @@ int main(int argc, char*argv[])
 {
       bool help_flag = false;
       bool times_flag = false;
+      bool version_flag = false;
 
       const char* net_path = 0;
       const char* pf_path = 0;
@@ -504,10 +596,8 @@ int main(int argc, char*argv[])
 #          endif
 	    break;
 	  case 'V':
-	    cout << "Icarus Verilog version " << VERSION << endl;
-	    cout << COPYRIGHT << endl;
-	    cout << endl << NOTICE << endl;
-	    return 0;
+	    version_flag = true;
+	    break;
 	  default:
 	    flag_errors += 1;
 	    break;
@@ -516,8 +606,20 @@ int main(int argc, char*argv[])
       if (flag_errors)
 	    return flag_errors;
 
+      if (version_flag) {
+	    cout << "\nIcarus Verilog Parser/Elaborator version "
+		 << VERSION << " (" << VERSION_TAG << ")" << endl << endl;
+	    cout << COPYRIGHT << endl << endl;
+	    cout << NOTICE << endl;
+
+	    dll_target_obj.test_version(flags["DLL"]);
+
+	    return 0;
+      }
+
       if (help_flag) {
-	    cout << "Icarus Verilog version " << VERSION << endl <<
+	    cout << "Icarus Verilog Parser/Elaborator version "
+		 << VERSION << " (" << VERSION_TAG << ")"  << endl <<
 "usage: ivl <options> <file>\n"
 "options:\n"
 "\t-C <name>        Config file from driver.\n"
@@ -548,6 +650,32 @@ int main(int argc, char*argv[])
 	      }
       }
 
+      lexor_keyword_mask = 0;
+      switch (generation_flag) {
+        case GN_VER1995:
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_1995;
+	  break;
+        case GN_VER2001:
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_1995;
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_2001;
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_2001_CONFIG;
+	  break;
+        case GN_VER2005:
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_1995;
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_2001;
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_2001_CONFIG;
+	  lexor_keyword_mask |= GN_KEYWORDS_1364_2005;
+	  break;
+      }
+
+      if (gn_cadence_types_flag)
+	    lexor_keyword_mask |= GN_KEYWORDS_ICARUS;
+      
+	  if (gn_system_verilog_flag)
+	    lexor_keyword_mask |= GN_KEYWORDS_1800_2005;
+
+      if (gn_verilog_ams_flag)
+	    lexor_keyword_mask |= GN_KEYWORDS_VAMS_2_3;
 
       if (verbose_flag) {
 	    if (times_flag)
@@ -561,19 +689,52 @@ int main(int argc, char*argv[])
 		case GN_VER2001:
 		  cout << "IEEE1364-2001";
 		  break;
-		case GN_SYSVER30:
-		  cout << "SystemVerilog 3.0";
+		case GN_VER2005:
+		  cout << "IEEE1364-2005";
 		  break;
 	    }
 
+	    if (gn_verilog_ams_flag)
+		  cout << ",verilog-ams";
+
+	    if (gn_specify_blocks_flag)
+		  cout << ",specify";
+	    else
+		  cout << ",no-specify";
+
+	    if (gn_cadence_types_flag)
+		  cout << ",xtypes";
+	    else
+		  cout << ",no-xtypes";
+
+	    if (gn_icarus_misc_flag)
+		  cout << ",icarus-misc";
+	    else
+		  cout << ",no-icarus-misc";
+
 	    cout << endl << "PARSING INPUT" << endl;
       }
 
+      const char *flag_tmp = flags["DISABLE_VIRTUAL_PINS"];
+      if (flag_tmp) disable_virtual_pins = strcmp(flag_tmp,"true")==0;
+      flag_tmp = flags["ARRAY_SIZE_LIMIT"];
+      if (flag_tmp) array_size_limit = strtoul(flag_tmp,NULL,0);
+
 	/* Parse the input. Make the pform. */
       int rc = pform_parse(argv[optind]);
 
       if (pf_path) {
 	    ofstream out (pf_path);
+	    out << "PFORM DUMP NATURES:" << endl;
+	    for (map<perm_string,ivl_nature_t>::iterator cur = natures.begin()
+		       ; cur != natures.end() ; cur ++ ) {
+		  pform_dump(out, (*cur).second);
+	    }
+	    out << "PFORM DUMP DISCIPLINES:" << endl;
+	    for (map<perm_string,ivl_discipline_t>::iterator cur = disciplines.begin()
+		       ; cur != disciplines.end() ; cur ++ ) {
+		  pform_dump(out, (*cur).second);
+	    }
 	    out << "PFORM DUMP MODULES:" << endl;
 	    for (map<perm_string,Module*>::iterator mod = pform_modules.begin()
 		       ; mod != pform_modules.end()
@@ -604,25 +765,26 @@ int main(int argc, char*argv[])
 	    for (mod = pform_modules.begin()
 		       ; mod != pform_modules.end()
 		       ; mod++) {
-		  list<PGate*> gates = (*mod).second->get_gates();
-		  list<PGate*>::const_iterator gate;
-		  for (gate = gates.begin(); gate != gates.end(); gate++) {
-			PGModule *mod = dynamic_cast<PGModule*>(*gate);
-			if (mod) {
-			      // Note that this module has been instantiated
-			      mentioned_p[mod->get_type()] = true;
-			}
-		  }
+		  find_module_mention(mentioned_p, mod->second);
 	    }
 
 	    for (mod = pform_modules.begin()
 		       ; mod != pform_modules.end()
 		       ; mod++) {
-		  if (mentioned_p[(*mod).second->mod_name()] == false) {
-			if (verbose_flag)
-			      cout << " " << (*mod).second->mod_name();
-			roots.push_back((*mod).second->mod_name());
-		  }
+
+		    /* Don't choose library modules. */
+		  if ((*mod).second->library_flag)
+			continue;
+
+		    /* Don't choose modules instantiated in other
+		       modules. */
+		  if (mentioned_p[(*mod).second->mod_name()])
+			continue;
+
+		    /* What's left might as well be chosen as a root. */
+		  if (verbose_flag)
+			cout << " " << (*mod).second->mod_name();
+		  roots.push_back((*mod).second->mod_name());
 	    }
 	    if (verbose_flag)
 		  cout << endl;
@@ -666,7 +828,7 @@ int main(int argc, char*argv[])
 
       des->set_flags(flags);
 
-	/* Done iwth all the pform data. Delete the modules. */
+	/* Done with all the pform data. Delete the modules. */
       for (map<perm_string,Module*>::iterator idx = pform_modules.begin()
 		 ; idx != pform_modules.end() ; idx ++) {
 
@@ -691,6 +853,11 @@ int main(int argc, char*argv[])
 	    func(des);
       }
 
+      if (verbose_flag) {
+	    cout << "CALCULATING ISLANDS" << endl;
+      }
+      des->join_islands();
+
       if (net_path) {
 	    if (verbose_flag)
 		  cerr<<" dumping netlist to " <<net_path<< "..." <<endl;
@@ -715,14 +882,21 @@ int main(int argc, char*argv[])
       }
 
       if (verbose_flag) {
-	    cout << "CODE GENERATION -t "<<target<< endl;
+	    cout << "CODE GENERATION" << endl;
       }
 
-      bool emit_rc;
-      emit_rc = emit(des, target);
-      if (!emit_rc) {
-	    cerr << "error: Code generation had errors." << endl;
-	    return 1;
+      if (int emit_rc = des->emit(&dll_target_obj)) {
+	    if (emit_rc > 0) {
+		  cerr << "error: Code generation had "
+		       << emit_rc << " errors."
+		       << endl;
+		  return 1;
+	    }
+	    if (emit_rc < 0) {
+		  cerr << "error: Code generator failure: " << emit_rc << endl;
+		  return -1;
+	    }
+	    assert(emit_rc);
       }
 
       if (verbose_flag) {
@@ -763,82 +937,39 @@ int main(int argc, char*argv[])
       return des? des->errors : 1;
 }
 
-/*
- * $Log: main.cc,v $
- * Revision 1.86.2.3  2006/06/12 00:16:53  steve
- *  Add support for -Wunused warnings.
- *
- * Revision 1.86.2.2  2006/04/23 04:25:45  steve
- *  Add cprop debugging.
- *
- * Revision 1.86.2.1  2006/04/01 01:37:24  steve
- *  Add synth debug flag
- *
- * Revision 1.86  2004/10/04 01:10:53  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.85  2004/09/25 01:58:44  steve
- *  Add a debug_elaborate flag
- *
- * Revision 1.84  2004/09/10 23:51:42  steve
- *  Fix the evaluation of constant ternary expressions.
- *
- * Revision 1.83  2004/09/05 17:44:42  steve
- *  Add support for module instance arrays.
- *
- * Revision 1.82  2004/03/10 04:51:24  steve
- *  Add support for system function table files.
- *
- * Revision 1.81  2004/02/18 17:11:56  steve
- *  Use perm_strings for named langiage items.
- *
- * Revision 1.80  2004/02/15 00:19:29  steve
- *  Report elaboration errors without crashing.
- *
- * Revision 1.79  2003/11/26 01:37:14  steve
- *  Properly initialize vpi_module_list with system.
- *
- * Revision 1.78  2003/11/13 05:55:33  steve
- *  Move the DLL= flag to target config files.
- *
- * Revision 1.77  2003/11/13 04:09:49  steve
- *  Pass flags through the temporary config file.
- *
- * Revision 1.76  2003/11/13 03:10:38  steve
- *  ivl -F and -t flags are onpassed throught the -C file.
- *
- * Revision 1.75  2003/11/10 20:59:03  steve
- *  Design::get_flag returns const char* instead of string.
- *
- * Revision 1.74  2003/11/01 04:22:30  steve
- *  Accept functors in the config file.
- *
- * Revision 1.73  2003/10/26 22:43:42  steve
- *  Improve -V messages,
- *
- * Revision 1.72  2003/09/26 02:17:14  steve
- *  Delete pform when done with it.
- *
- * Revision 1.71  2003/09/25 00:25:14  steve
- *  Summary list of missing modules.
- *
- * Revision 1.70  2003/09/23 05:57:36  steve
- *  Pass -m flag from driver via iconfig file.
- *
- * Revision 1.69  2003/09/22 01:12:08  steve
- *  Pass more ivl arguments through the iconfig file.
- *
- * Revision 1.68  2003/06/20 00:53:19  steve
- *  Module attributes from the parser
- *  through to elaborated form.
- *
- * Revision 1.67  2003/04/24 05:25:27  steve
- *  Dump design even on errors.
- *
- * Revision 1.66  2003/03/01 06:25:30  steve
- *  Add the lex_strings string handler, and put
- *  scope names and system task/function names
- *  into this table. Also, permallocate event
- *  names from the beginning.
- */
+static void find_module_mention(map<perm_string,bool>&check_map, Module*mod)
+{
+      list<PGate*> gates = mod->get_gates();
+      list<PGate*>::const_iterator gate;
+      for (gate = gates.begin(); gate != gates.end(); gate++) {
+	    PGModule*tmp = dynamic_cast<PGModule*>(*gate);
+	    if (tmp) {
+		    // Note that this module has been instantiated
+		  check_map[tmp->get_type()] = true;
+	    }
+      }
 
+      list<PGenerate*>::const_iterator cur;
+      for (cur = mod->generate_schemes.begin()
+		 ; cur != mod->generate_schemes.end() ; cur ++) {
+	    find_module_mention(check_map, *cur);
+      }
+}
+
+static void find_module_mention(map<perm_string,bool>&check_map, PGenerate*schm)
+{
+      list<PGate*>::const_iterator gate;
+      for (gate = schm->gates.begin(); gate != schm->gates.end(); gate++) {
+	    PGModule*tmp = dynamic_cast<PGModule*>(*gate);
+	    if (tmp) {
+		    // Note that this module has been instantiated
+		  check_map[tmp->get_type()] = true;
+	    }
+      }
+
+      list<PGenerate*>::const_iterator cur;
+      for (cur = schm->generate_schemes.begin()
+		 ; cur != schm->generate_schemes.end() ; cur ++) {
+	    find_module_mention(check_map, *cur);
+      }
+}
