@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2002-2008 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,26 +16,27 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: net_expr.cc,v 1.23 2004/10/04 01:10:54 steve Exp $"
-#endif
 
 # include  "config.h"
 # include  "netlist.h"
 # include  "compiler.h"
+# include  "netmisc.h"
 # include  <iostream>
 
-NetExpr::TYPE NetExpr::expr_type() const
+/*
+ * the grand default data type is a logic vector.
+ */
+ivl_variable_type_t NetExpr::expr_type() const
 {
-      return ET_VECTOR;
+      return IVL_VT_LOGIC;
 }
 
 /*
  * Create an add/sub node from the two operands. Make a best guess of
  * the
  */
-NetEBAdd::NetEBAdd(char op, NetExpr*l, NetExpr*r)
-: NetEBinary(op, l, r)
+NetEBAdd::NetEBAdd(char op__, NetExpr*l, NetExpr*r, bool lossless_flag)
+: NetEBinary(op__, l, r)
 {
       NetEConst* tmp;
 
@@ -45,41 +46,61 @@ NetEBAdd::NetEBAdd(char op, NetExpr*l, NetExpr*r)
 	   one. This expands the expression to account for the largest
 	   possible result.
 
+	   Remember to handle the special case of an unsized constant,
+	   which we define to be at least "integer_width" bits.
+
 	   The set_width applied to a constant value will only
 	   truncate the constant so far as it can still hold its
 	   logical value, so this is safe to do. */
       if ( (tmp = dynamic_cast<NetEConst*>(r))
 	   && (! tmp->has_width())
-	   && (tmp->expr_width() > l->expr_width()) ) {
+	   && (tmp->expr_width() > l->expr_width() || integer_width > l->expr_width()) ) {
+
+	    verinum tmp_v = trim_vnum(tmp->value());
+	    unsigned target_width = l->expr_width();
+	    if (target_width < tmp_v.len())
+		  target_width = tmp_v.len();
+	    if (lossless_flag)
+		  target_width += 1;
+	    if (target_width < integer_width)
+		  target_width = integer_width;
 
-	    unsigned target_width = l->expr_width() + 1;
 	    r->set_width(target_width);
 
 	      /* Note: This constant value will not gain a defined
 		 width from this. Make sure. */
 	    assert(! r->has_width() );
 
+	    expr_width(target_width);
+
       } else if ( (tmp = dynamic_cast<NetEConst*>(l))
 	   && (! tmp->has_width())
-	   && (tmp->expr_width() > r->expr_width()) ) {
+		  && (tmp->expr_width() > r->expr_width() || integer_width > r->expr_width()) ) {
+
+	    verinum tmp_v = trim_vnum(tmp->value());
+	    unsigned target_width = r->expr_width();
+	    if (target_width < tmp_v.len())
+		  target_width = tmp_v.len();
+	    if (lossless_flag)
+		  target_width += 1;
+	    if (target_width < integer_width)
+		  target_width = integer_width;
 
-	    unsigned target_width = r->expr_width() + 1;
 	    l->set_width(target_width);
 
 	      /* Note: This constant value will not gain a defined
 		 width from this. Make sure. */
 	    assert(! l->has_width() );
 
-      }
+	    expr_width(target_width);
 
-	/* Now that we have the operand sizes the way we like, or as
-	   good as we are going to get them, set the size of myself. */
-      if (r->expr_width() > l->expr_width()) {
-
-	    expr_width(r->expr_width());
+      } else if (r->expr_width() > l->expr_width()) {
+	    unsigned loss_pad = lossless_flag? 1 : 0;
+	    expr_width(r->expr_width() + loss_pad);
 
       } else {
-	    expr_width(l->expr_width());
+	    unsigned loss_pad = lossless_flag? 1 : 0;
+	    expr_width(l->expr_width() + loss_pad);
       }
 
       cast_signed(l->has_sign() && r->has_sign());
@@ -96,15 +117,15 @@ NetEBAdd* NetEBAdd::dup_expr() const
       return result;
 }
 
-NetExpr::TYPE NetEBAdd::expr_type() const
+ivl_variable_type_t NetEBAdd::expr_type() const
 {
-      if (left_->expr_type() == ET_REAL)
-	    return ET_REAL;
+      if (left_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
 
-      if (right_->expr_type() == ET_REAL)
-	    return ET_REAL;
+      if (right_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
 
-      return ET_VECTOR;
+      return IVL_VT_LOGIC;
 }
 
 /*
@@ -115,9 +136,12 @@ NetExpr::TYPE NetEBAdd::expr_type() const
  * expression. Otherwise, the netlist will have to allow the
  * expressions to have different widths.
  */
-NetEBComp::NetEBComp(char op, NetExpr*l, NetExpr*r)
-: NetEBinary(op, l, r)
+NetEBComp::NetEBComp(char op__, NetExpr*l, NetExpr*r)
+: NetEBinary(op__, l, r)
 {
+	// The output of compare is always unsigned.
+      cast_signed_base_(false);
+
       if (NetEConst*tmp = dynamic_cast<NetEConst*>(r)) do {
 
 	    if (tmp->has_width())
@@ -161,8 +185,23 @@ bool NetEBComp::has_width() const
       return true;
 }
 
-NetEBDiv::NetEBDiv(char op, NetExpr*l, NetExpr*r)
-: NetEBinary(op, l, r)
+ivl_variable_type_t NetEBComp::expr_type() const
+{
+	// Case compare always returns BOOL
+      if (op() == 'E' || op() == 'N')
+	    return IVL_VT_BOOL;
+
+      if (left()->expr_type() == IVL_VT_LOGIC)
+	    return IVL_VT_LOGIC;
+
+      if (right()->expr_type() == IVL_VT_LOGIC)
+	    return IVL_VT_LOGIC;
+
+      return IVL_VT_BOOL;
+}
+
+NetEBDiv::NetEBDiv(char op__, NetExpr*l, NetExpr*r)
+: NetEBinary(op__, l, r)
 {
       unsigned w = l->expr_width();
       if (r->expr_width() > w)
@@ -183,29 +222,50 @@ NetEBDiv* NetEBDiv::dup_expr() const
       return result;
 }
 
-NetExpr::TYPE NetEBDiv::expr_type() const
+ivl_variable_type_t NetEBDiv::expr_type() const
 {
-      if (left_->expr_type() == ET_REAL)
-	    return ET_REAL;
+      if (left_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
 
-      if (right_->expr_type() == ET_REAL)
-	    return ET_REAL;
+      if (right_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
 
-      return ET_VECTOR;
+      return IVL_VT_LOGIC;
 }
 
-NetEBMult::NetEBMult(char op, NetExpr*l, NetExpr*r)
-: NetEBinary(op, l, r)
+NetEBMinMax::NetEBMinMax(char op__, NetExpr*l, NetExpr*r)
+: NetEBinary(op__, l, r)
 {
-      expr_width(l->expr_width() + r->expr_width());
-      cast_signed(l->has_sign() && r->has_sign());
+      expr_width( max(l->expr_width(), r->expr_width()) );
+      cast_signed(l->has_sign() || r->has_sign());
+}
 
-	/* If it turns out that this is not a signed expression, then
-	   cast the signedness out of the operands as well. */
-      if (! has_sign()) {
-	    l->cast_signed(false);
-	    r->cast_signed(false);
-      }
+NetEBMinMax::~NetEBMinMax()
+{
+}
+
+ivl_variable_type_t NetEBMinMax::expr_type() const
+{
+      if (left_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
+      if (right_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
+
+      return IVL_VT_LOGIC;
+}
+
+NetEBMult::NetEBMult(char op__, NetExpr*l, NetExpr*r)
+: NetEBinary(op__, l, r)
+{
+      if (expr_type() == IVL_VT_REAL)
+	    expr_width(1);
+      else
+	    expr_width(l->expr_width() + r->expr_width());
+
+      if (expr_type() == IVL_VT_REAL)
+	    cast_signed(true);
+      else
+	    cast_signed(l->has_sign() && r->has_sign());
 }
 
 NetEBMult::~NetEBMult()
@@ -219,24 +279,55 @@ NetEBMult* NetEBMult::dup_expr() const
       return result;
 }
 
-NetExpr::TYPE NetEBMult::expr_type() const
+ivl_variable_type_t NetEBMult::expr_type() const
 {
-      if (left_->expr_type() == ET_REAL)
-	    return ET_REAL;
+      if (left_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
 
-      if (right_->expr_type() == ET_REAL)
-	    return ET_REAL;
+      if (right_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
 
-      return ET_VECTOR;
+      return IVL_VT_LOGIC;
 }
 
-NetEBShift::NetEBShift(char op, NetExpr*l, NetExpr*r)
-: NetEBinary(op, l, r)
+NetEBPow::NetEBPow(char op__, NetExpr*l, NetExpr*r)
+: NetEBinary(op__, l, r)
+{
+      assert(op__ == 'p');
+      /* This is incorrect! a * (2^b - 1) is close. */
+      expr_width(l->expr_width()+r->expr_width());
+      cast_signed(l->has_sign() || r->has_sign());
+}
+
+NetEBPow::~NetEBPow()
+{
+}
+
+NetEBPow* NetEBPow::dup_expr() const
+{
+      NetEBPow*result = new NetEBPow(op_, left_->dup_expr(),
+				     right_->dup_expr());
+      result->set_line(*this);
+      return result;
+}
+
+ivl_variable_type_t NetEBPow::expr_type() const
+{
+      if (right_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
+      if (left_->expr_type() == IVL_VT_REAL)
+	    return IVL_VT_REAL;
+
+      return IVL_VT_LOGIC;
+}
+
+NetEBShift::NetEBShift(char op__, NetExpr*l, NetExpr*r)
+: NetEBinary(op__, l, r)
 {
       expr_width(l->expr_width());
 
 	// The >>> is signed if the left operand is signed.
-      if (op == 'R') cast_signed(l->has_sign());
+      if (op__ == 'R') cast_signed(l->has_sign());
 }
 
 NetEBShift::~NetEBShift()
@@ -289,7 +380,8 @@ void NetEConcat::set(unsigned idx, NetExpr*e)
 
 NetEConcat* NetEConcat::dup_expr() const
 {
-      NetEConcat*dup = new NetEConcat(parms_.count(), repeat_);
+      NetEConcat*dup = new NetEConcat(parms_.count(), 0);
+      dup->set_line(*this);
       for (unsigned idx = 0 ;  idx < parms_.count() ;  idx += 1)
 	    if (parms_[idx]) {
 		  NetExpr*tmp = parms_[idx]->dup_expr();
@@ -298,7 +390,11 @@ NetEConcat* NetEConcat::dup_expr() const
 	    }
 
 
+      dup->repeat_ = repeat_? repeat_->dup_expr() : 0;
+      dup->repeat_value_ = repeat_value_;
+      dup->repeat_calculated_ = repeat_calculated_;
       dup->expr_width(expr_width());
+
       return dup;
 }
 
@@ -307,24 +403,16 @@ unsigned NetEConcat::repeat()
       if (repeat_calculated_)
 	    return repeat_value_;
 
-      assert(repeat_);
-
-      if (! dynamic_cast<NetEConst*>(repeat_)) {
-	    NetExpr*tmp = repeat_->eval_tree();
-	    if (tmp != 0) {
-		  delete repeat_;
-		  repeat_ = tmp;
-	    }
-      }
+      eval_expr(repeat_);
 
       NetEConst*repeat_const = dynamic_cast<NetEConst*>(repeat_);
 
 	/* This should not be possible, as it was checked earlier to
 	   assure that this is a constant expression. */
       if (repeat_const == 0) {
-	    cerr << get_line() << ": internal error: repeat expression "
+	    cerr << get_fileline() << ": internal error: repeat expression "
 		 << "is not a compile time constant." << endl;
-	    cerr << get_line() << ":               : Expression is: "
+	    cerr << get_fileline() << ":               : Expression is: "
 		 << *repeat_ << endl;
 	    repeat_calculated_ = true;
 	    repeat_value_ = 1;
@@ -349,6 +437,7 @@ unsigned NetEConcat::repeat() const
 NetECReal::NetECReal(const verireal&val)
 : value_(val)
 {
+      expr_width(1);
 }
 
 NetECReal::~NetECReal()
@@ -372,9 +461,9 @@ NetECReal* NetECReal::dup_expr() const
       return tmp;
 }
 
-NetExpr::TYPE NetECReal::expr_type() const
+ivl_variable_type_t NetECReal::expr_type() const
 {
-      return ET_REAL;
+      return IVL_VT_REAL;
 }
 
 NetECRealParam::NetECRealParam(NetScope*s, perm_string n, const verireal&v)
@@ -403,8 +492,15 @@ NetEParam::NetEParam()
 }
 
 NetEParam::NetEParam(Design*d, NetScope*s, perm_string n)
-: des_(d), scope_(s), name_(n)
+    : des_(d), scope_(s), reference_(scope_->find_parameter(n))
 {
+      cast_signed_base_(reference_->second.signed_flag);
+}
+
+NetEParam::NetEParam(Design*d, NetScope*s, ref_t ref)
+    : des_(d), scope_(s), reference_(ref)
+{
+      cast_signed_base_(reference_->second.signed_flag);
 }
 
 NetEParam::~NetEParam()
@@ -416,9 +512,14 @@ bool NetEParam::has_width() const
       return false;
 }
 
+ivl_variable_type_t NetEParam::expr_type() const
+{
+      return (*reference_).second.type;
+}
+
 NetEParam* NetEParam::dup_expr() const
 {
-      NetEParam*tmp = new NetEParam(des_, scope_, name_);
+      NetEParam*tmp = new NetEParam(des_, scope_, reference_);
       tmp->set_line(*this);
       return tmp;
 }
@@ -450,15 +551,7 @@ bool NetESelect::has_width() const
       return true;
 }
 
-bool NetESelect::set_width(unsigned w)
-{
-      if (expr_width() == 1)
-	    return true;
-      else
-	    return false;
-}
-
-NetESFunc::NetESFunc(const char*n, NetExpr::TYPE t,
+NetESFunc::NetESFunc(const char*n, ivl_variable_type_t t,
 		     unsigned width, unsigned np)
 : name_(0), type_(t)
 {
@@ -509,95 +602,21 @@ NetExpr* NetESFunc::parm(unsigned idx)
       return parms_[idx];
 }
 
-NetExpr::TYPE NetESFunc::expr_type() const
+ivl_variable_type_t NetESFunc::expr_type() const
 {
       return type_;
 }
 
-/*
- * $Log: net_expr.cc,v $
- * Revision 1.23  2004/10/04 01:10:54  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.22  2004/02/20 06:22:56  steve
- *  parameter keys are per_strings.
- *
- * Revision 1.21  2003/08/28 04:11:19  steve
- *  Spelling patch.
- *
- * Revision 1.20  2003/06/18 03:55:18  steve
- *  Add arithmetic shift operators.
- *
- * Revision 1.19  2003/06/15 18:53:20  steve
- *  Operands of unsigned multiply are unsigned.
- *
- * Revision 1.18  2003/05/30 02:55:32  steve
- *  Support parameters in real expressions and
- *  as real expressions, and fix multiply and
- *  divide with real results.
- *
- * Revision 1.17  2003/05/20 15:05:33  steve
- *  Do not try to set constants to width 0.
- *
- * Revision 1.16  2003/03/15 18:08:43  steve
- *  Comparison operators do have defined width.
- *
- * Revision 1.15  2003/03/15 04:46:29  steve
- *  Better organize the NetESFunc return type guesses.
- *
- * Revision 1.14  2003/03/01 06:25:30  steve
- *  Add the lex_strings string handler, and put
- *  scope names and system task/function names
- *  into this table. Also, permallocate event
- *  names from the beginning.
- *
- * Revision 1.13  2003/02/06 17:50:23  steve
- *  Real constants have no defined vector width
- *
- * Revision 1.12  2003/01/27 00:14:37  steve
- *  Support in various contexts the $realtime
- *  system task.
- *
- * Revision 1.11  2003/01/26 21:15:58  steve
- *  Rework expression parsing and elaboration to
- *  accommodate real/realtime values and expressions.
- *
- * Revision 1.10  2002/11/09 01:40:19  steve
- *  Postpone parameter width check to evaluation.
- *
- * Revision 1.9  2002/11/06 02:25:13  steve
- *  No need to keep excess width from an
- *  unsigned constant value, if it can
- *  be trimmed safely.
- *
- * Revision 1.8  2002/10/19 22:59:49  steve
- *  Redo the parameter vector support to allow
- *  parameter names in range expressions.
- *
- * Revision 1.7  2002/09/01 03:01:48  steve
- *  Properly cast signedness of parameters with ranges.
- *
- * Revision 1.6  2002/08/12 01:34:59  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.5  2002/06/06 18:57:18  steve
- *  Use standard name for iostream.
- *
- * Revision 1.4  2002/05/25 16:51:37  steve
- *  include iostream for gcc 3.1
- *
- * Revision 1.3  2002/05/05 21:11:50  steve
- *  Put off evaluation of concatenation repeat expresions
- *  until after parameters are defined. This allows parms
- *  to be used in repeat expresions.
- *
- *  Add the builtin $signed system function.
- *
- * Revision 1.2  2002/01/29 22:36:31  steve
- *  include config.h to eliminate warnings.
- *
- * Revision 1.1  2002/01/28 01:39:45  steve
- *  Add ne_expr.cc
- *
- */
+NetEAccess::NetEAccess(NetBranch*br, ivl_nature_t nat)
+: branch_(br), nature_(nat)
+{
+}
 
+NetEAccess::~NetEAccess()
+{
+}
+
+ivl_variable_type_t NetEAccess::expr_type() const
+{
+      return IVL_VT_REAL;
+}
