@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2004 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2008 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,9 +16,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: pform_dump.cc,v 1.88.2.1 2006/07/10 00:21:54 steve Exp $"
-#endif
 
 # include "config.h"
 
@@ -29,8 +26,10 @@
  * module in question.
  */
 # include  "pform.h"
-# include  "PData.h"
 # include  "PEvent.h"
+# include  "PGenerate.h"
+# include  "PSpec.h"
+# include  "discipline.h"
 # include  <iostream>
 # include  <iomanip>
 # include  <typeinfo>
@@ -41,6 +40,12 @@ ostream& operator << (ostream&out, const PExpr&obj)
       return out;
 }
 
+ostream& operator << (ostream&out, const PEventStatement&obj)
+{
+      obj.dump_inline(out);
+      return out;
+}
+
 ostream& operator << (ostream&o, const PDelays&d)
 {
       d.dump_delays(o);
@@ -71,6 +76,112 @@ ostream& operator<< (ostream&o, PGate::strength_t str)
       return o;
 }
 
+ostream& operator<< (ostream&out, perm_string that)
+{
+      out << that.str();
+      return out;
+}
+
+ostream& operator<< (ostream&out, const index_component_t&that)
+{
+      out << "[";
+      switch (that.sel) {
+	  case index_component_t::SEL_BIT:
+	    out << *that.msb;
+	    break;
+	  case index_component_t::SEL_PART:
+	    out << *that.msb << ":" << *that.lsb;
+	    break;
+	  case index_component_t::SEL_IDX_UP:
+	    out << *that.msb << "+:" << *that.lsb;
+	    break;
+	  case index_component_t::SEL_IDX_DO:
+	    out << *that.msb << "-:" << *that.lsb;
+	    break;
+	  default:
+	    out << "???";
+	    break;
+      }
+      out << "]";
+      return out;
+}
+
+ostream& operator<< (ostream&out, const name_component_t&that)
+{
+      out << that.name.str();
+
+      typedef std::list<index_component_t>::const_iterator index_it_t;
+      for (index_it_t idx = that.index.begin()
+		 ; idx != that.index.end() ;  idx++) {
+
+	    out << *idx;
+      }
+      return out;
+}
+
+ostream& operator<< (ostream&o, const pform_name_t&that)
+{
+      pform_name_t::const_iterator cur;
+
+      cur = that.begin();
+      o << *cur;
+
+      cur++;
+      while (cur != that.end()) {
+	    o << "." << *cur;
+	    cur++;
+      }
+
+      return o;
+}
+
+std::ostream& operator << (std::ostream&out, ivl_process_type_t pt)
+{
+      switch (pt) {
+	  case IVL_PR_INITIAL:
+	    out << "initial";
+	    break;
+	  case IVL_PR_ALWAYS:
+	    out << "always";
+	    break;
+      }
+      return out;
+}
+
+std::ostream& operator << (std::ostream&out, ivl_dis_domain_t dom)
+{
+      switch (dom) {
+	  case IVL_DIS_NONE:
+	    out << "no-domain";
+	    break;
+	  case IVL_DIS_DISCRETE:
+	    out << "discrete";
+	    break;
+	  case IVL_DIS_CONTINUOUS:
+	    out << "continuous";
+	    break;
+	  default:
+	    assert(0);
+	    break;
+      }
+      return out;
+}
+
+static void dump_attributes_map(ostream&out,
+				const map<perm_string,PExpr*>&attributes,
+				int ind)
+{
+      for (map<perm_string,PExpr*>::const_iterator idx = attributes.begin()
+		 ; idx != attributes.end() ; idx++ ) {
+
+	    out << setw(ind) << "" << "(* " << (*idx).first;
+	    if ((*idx).second) {
+		  out << " = " << *(*idx).second;
+	    }
+	    out << " *)" << endl;
+      }
+}
+
 void PExpr::dump(ostream&out) const
 {
       out << typeid(*this).name();
@@ -102,9 +213,9 @@ void PECallFunction::dump(ostream &out) const
 {
       out << path_ << "(";
 
-      if (parms_.count() > 0) {
+      if (parms_.size() > 0) {
 	    if (parms_[0]) parms_[0]->dump(out);
-	    for (unsigned idx = 1; idx < parms_.count(); ++idx) {
+	    for (unsigned idx = 1; idx < parms_.size(); ++idx) {
 		  out << ", ";
 		  if (parms_[idx]) parms_[idx]->dump(out);
 	    }
@@ -144,13 +255,6 @@ void PENumber::dump(ostream&out) const
 void PEIdent::dump(ostream&out) const
 {
       out << path_;
-      if (msb_) {
-	    out << "[" << *msb_;
-	    if (lsb_) {
-		  out << ":" << *lsb_;
-	    }
-	    out << "]";
-      }
 }
 
 void PEString::dump(ostream&out) const
@@ -165,11 +269,30 @@ void PETernary::dump(ostream&out) const
 
 void PEUnary::dump(ostream&out) const
 {
-      out << op_ << "(" << *expr_ << ")";
+      switch (op_) {
+	  case 'm':
+	    out << "abs";
+	    break;
+	  default:
+	    out << op_;
+	    break;
+      }
+      out << "(" << *expr_ << ")";
 }
 
 void PEBinary::dump(ostream&out) const
 {
+	/* Handle some special cases, where the operators are written
+	   in function notation. */
+      if (op_ == 'm') {
+	    out << "min(" << *left_ << "," << *right_ << ")";
+	    return;
+      }
+      if (op_ == 'M') {
+	    out << "min(" << *left_ << "," << *right_ << ")";
+	    return;
+      }
+
       out << "(" << *left_ << ")";
       switch (op_) {
 	  case 'a':
@@ -184,12 +307,21 @@ void PEBinary::dump(ostream&out) const
 	  case 'l':
 	    out << "<<";
 	    break;
+	  case 'L':
+	    out << "<=";
+	    break;
 	  case 'n':
 	    out << "!=";
 	    break;
 	  case 'N':
 	    out << "!==";
 	    break;
+	  case 'p':
+	    out << "**";
+	    break;
+	  case 'R':
+	    out << ">>>";
+	    break;
 	  case 'r':
 	    out << ">>";
 	    break;
@@ -201,47 +333,53 @@ void PEBinary::dump(ostream&out) const
 }
 
 
-void PWire::dump(ostream&out) const
+void PWire::dump(ostream&out, unsigned ind) const
 {
-      out << "    " << type_;
+      out << setw(ind) << "" << type_;
 
       switch (port_type_) {
 	  case NetNet::PIMPLICIT:
-	    out << " (implicit input)";
+	    out << " implicit input";
 	    break;
 	  case NetNet::PINPUT:
-	    out << " (input)";
+	    out << " input";
 	    break;
 	  case NetNet::POUTPUT:
-	    out << " (output)";
+	    out << " output";
 	    break;
 	  case NetNet::PINOUT:
-	    out << " (input output)";
+	    out << " inout";
 	    break;
 	  case NetNet::NOT_A_PORT:
 	    break;
       }
 
+      out << " " << data_type_;
+
       if (signed_) {
 	    out << " signed";
       }
 
-      assert(msb_.count() == lsb_.count());
-      for (unsigned idx = 0 ;  idx < msb_.count() ;  idx += 1) {
-
-	    if (msb_[idx] == 0) {
-		  assert(lsb_[idx] == 0);
-		  out << " <scalar>";
+      if (discipline_) {
+	    out << " discipline<" << discipline_->name() << ">";
+      }
 
+      if (port_set_) {
+	    if (port_msb_ == 0) {
+		  out << " port<scalar>";
 	    } else {
-		  if (lsb_[idx])
-			out << " [" << *msb_[idx] << ":" << *lsb_[idx] << "]";
-		  else
-			out << " [" << *msb_[idx] << "]";
+		  out << " port[" << *port_msb_ << ":" << *port_lsb_ << "]";
+	    }
+      }
+      if (net_set_) {
+	    if (net_msb_ == 0) {
+		  out << " net<scalar>";
+	    } else {
+		  out << " net[" << *net_msb_ << ":" << *net_lsb_ << "]";
 	    }
       }
 
-      out << " " << hname_;
+      out << " " << name_;
 
 	// If the wire has indices, dump them.
       if (lidx_ || ridx_) {
@@ -252,14 +390,7 @@ void PWire::dump(ostream&out) const
       }
 
       out << ";" << endl;
-      for (map<perm_string,PExpr*>::const_iterator idx = attributes.begin()
-		 ; idx != attributes.end()
-		 ; idx ++) {
-	    out << "        " << (*idx).first;
-	    if ((*idx).second)
-		  out << " = " << *(*idx).second;
-	    out << endl;
-      }
+      dump_attributes_map(out, attributes, 8);
 }
 
 void PGate::dump_pins(ostream&out) const
@@ -291,9 +422,9 @@ void PGate::dump_delays(ostream&out) const
       delay_.dump_delays(out);
 }
 
-void PGate::dump(ostream&out) const
+void PGate::dump(ostream&out, unsigned ind) const
 {
-      out << "    " << typeid(*this).name() << " ";
+      out << setw(ind) << "" << typeid(*this).name() << " ";
       delay_.dump_delays(out);
       out << " " << get_name() << "(";
       dump_pins(out);
@@ -301,45 +432,53 @@ void PGate::dump(ostream&out) const
 
 }
 
-void PGAssign::dump(ostream&out) const
+void PGAssign::dump(ostream&out, unsigned ind) const
 {
-      out << "    assign (" << strength0() << "0 " << strength1() << "1) ";
+      out << setw(ind) << "";
+      out << "assign (" << strength0() << "0 " << strength1() << "1) ";
       dump_delays(out);
       out << " " << *pin(0) << " = " << *pin(1) << ";" << endl;
 }
 
-void PGBuiltin::dump(ostream&out) const
+void PGBuiltin::dump(ostream&out, unsigned ind) const
 {
+      out << setw(ind) << "";
       switch (type()) {
 	  case PGBuiltin::BUFIF0:
-	    out << "    bufif0 ";
+	    out << "bufif0 ";
 	    break;
 	  case PGBuiltin::BUFIF1:
-	    out << "    bufif1 ";
+	    out << "bufif1 ";
 	    break;
 	  case PGBuiltin::NOTIF0:
-	    out << "    bufif0 ";
+	    out << "bufif0 ";
 	    break;
 	  case PGBuiltin::NOTIF1:
-	    out << "    bufif1 ";
+	    out << "bufif1 ";
 	    break;
 	  case PGBuiltin::NAND:
-	    out << "    nand ";
+	    out << "nand ";
 	    break;
 	  case PGBuiltin::NMOS:
-	    out << "    nmos ";
+	    out << "nmos ";
 	    break;
 	  case PGBuiltin::RNMOS:
-	    out << "    rnmos ";
+	    out << "rnmos ";
 	    break;
 	  case PGBuiltin::RPMOS:
-	    out << "    rpmos ";
+	    out << "rpmos ";
 	    break;
 	  case PGBuiltin::PMOS:
-	    out << "    pmos ";
+	    out << "pmos ";
+	    break;
+	  case PGBuiltin::RCMOS:
+	    out << "rcmos ";
+	    break;
+	  case PGBuiltin::CMOS:
+	    out << "cmos ";
 	    break;
 	  default:
-	    out << "    builtin gate ";
+	    out << "builtin gate ";
       }
 
       out << "(" << strength0() << "0 " << strength1() << "1) ";
@@ -355,15 +494,19 @@ void PGBuiltin::dump(ostream&out) const
       out << ");" << endl;
 }
 
-void PGModule::dump(ostream&out) const
+void PGModule::dump(ostream&out, unsigned ind) const
 {
-      out << "    " << type_ << " ";
+      out << setw(ind) << "" << type_ << " ";
 
 	// If parameters are overridden by order, dump them.
-      if (overrides_) {
+      if (overrides_ && overrides_->count() > 0) {
 	    assert(parms_ == 0);
             out << "#(";
-	    out << *((*overrides_)[0]);
+
+	    if ((*overrides_)[0] == 0)
+		  out << "<nil>";
+	    else
+		  out << *((*overrides_)[0]);
 	    for (unsigned idx = 1 ;  idx < overrides_->count() ;  idx += 1) {
 	          out << "," << *((*overrides_)[idx]);
 	    }
@@ -416,44 +559,54 @@ void Statement::dump(ostream&out, unsigned ind) const
 	   so just print the C++ typeid and let the user figure
 	   it out. */
       out << setw(ind) << "";
-      out << "/* " << get_line() << ": " << typeid(*this).name()
+      out << "/* " << get_fileline() << ": " << typeid(*this).name()
 	  << " */ ;" << endl;
-      dump_attributes(out, ind+2);
+      dump_attributes_map(out, attributes, ind+2);
 }
 
-void Statement::dump_attributes(ostream&out, unsigned ind) const
+void AContrib::dump(ostream&out, unsigned ind) const
 {
-      for (map<perm_string,PExpr*>::const_iterator idx = attributes.begin()
-		 ; idx != attributes.end()
-		 ; idx ++) {
-	    out << setw(ind) << "" << "(* " << (*idx).first;
-	    if ((*idx).second)
-		  out << " = " << *(*idx).second;
-	    out << " *)" << endl;
-      }
+      out << setw(ind) << "";
+      out << *lval_ << " <+ " << *rval_
+	  << "; /* " << get_fileline() << " */"
+	  << endl;
 }
 
 void PAssign::dump(ostream&out, unsigned ind) const
 {
-      out << setw(ind) << "";
-      out << *lval() << " = " << delay_ << " " << *rval() << ";";
-      out << "  /* " << get_line() << " */" << endl;
+      out << setw(ind) << "" << *lval() << " = ";
+      if (delay_) out << "#" << *delay_ << " ";
+      if (count_) out << "repeat(" << *count_ << ") ";
+      if (event_) out << *event_ << " ";
+      out << *rval() << ";" << "  /* " << get_fileline() << " */" << endl;
 }
 
 void PAssignNB::dump(ostream&out, unsigned ind) const
 {
-      out << setw(ind) << "";
-      out << *lval() << " <= " << delay_ << " " << *rval() << ";";
-      out << "  /* " << get_line() << " */" << endl;
+      out << setw(ind) << "" << *lval() << " <= ";
+      if (delay_) out << "#" << *delay_ << " ";
+      if (count_) out << "repeat(" << *count_ << ") ";
+      if (event_) out << *event_ << " ";
+      out << *rval() << ";" << "  /* " << get_fileline() << " */" << endl;
 }
 
 void PBlock::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "begin";
-      if (name_ != 0)
-	    out << " : " << name_;
+      if (pscope_name() != 0)
+	    out << " : " << pscope_name();
       out << endl;
 
+      if (pscope_name() != 0) {
+            dump_parameters_(out, ind+2);
+
+            dump_localparams_(out, ind+2);
+
+            dump_events_(out, ind+2);
+
+	    dump_wires_(out, ind+2);
+      }
+
       for (unsigned idx = 0 ;  idx < list_.count() ;  idx += 1) {
 	    if (list_[idx])
 		  list_[idx]->dump(out, ind+2);
@@ -481,7 +634,7 @@ void PCallTask::dump(ostream&out, unsigned ind) const
 	    out << ")";
       }
 
-      out << "; /* " << get_line() << " */" << endl;
+      out << "; /* " << get_fileline() << " */" << endl;
 }
 
 void PCase::dump(ostream&out, unsigned ind) const
@@ -498,9 +651,8 @@ void PCase::dump(ostream&out, unsigned ind) const
 	    out << "casez";
 	    break;
       }
-      out << " (" << *expr_ << ") /* " << get_line() << " */" << endl;
-
-      dump_attributes(out, ind+2);
+      out << " (" << *expr_ << ") /* " << get_fileline() << " */" << endl;
+      dump_attributes_map(out, attributes, ind+2);
 
       for (unsigned idx = 0 ;  idx < items_->count() ;  idx += 1) {
 	    PCase::Item*cur = (*items_)[idx];
@@ -544,19 +696,19 @@ void PCondit::dump(ostream&out, unsigned ind) const
 void PCAssign::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "assign " << *lval_ << " = " << *expr_
-	  << "; /* " << get_line() << " */" << endl;
+	  << "; /* " << get_fileline() << " */" << endl;
 }
 
 void PDeassign::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "deassign " << *lval_ << "; /* "
-	  << get_line() << " */" << endl;
+	  << get_fileline() << " */" << endl;
 }
 
 void PDelayStatement::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "#" << *delay_ << " /* " <<
-	    get_line() << " */";
+	    get_fileline() << " */";
       if (statement_) {
 	    out << endl;
 	    statement_->dump(out, ind+2);
@@ -568,7 +720,7 @@ void PDelayStatement::dump(ostream&out, unsigned ind) const
 void PDisable::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "disable " << scope_ << "; /* "
-	  << get_line() << " */" << endl;
+	  << get_fileline() << " */" << endl;
 }
 
 void PEventStatement::dump(ostream&out, unsigned ind) const
@@ -593,15 +745,32 @@ void PEventStatement::dump(ostream&out, unsigned ind) const
       }
 }
 
+void PEventStatement::dump_inline(ostream&out) const
+{
+      assert(statement_ == 0);
+
+      if (expr_.count() == 0) {
+	    out << "@* ";
+
+      } else {
+	    out << "@(" << *(expr_[0]);
+	    if (expr_.count() > 1)
+		  for (unsigned idx = 1 ;  idx < expr_.count() ;  idx += 1)
+			out << " or " << *(expr_[idx]);
+
+	    out << ")";
+      }
+}
+
 void PForce::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "force " << *lval_ << " = " << *expr_
-	  << "; /* " << get_line() << " */" << endl;
+	  << "; /* " << get_fileline() << " */" << endl;
 }
 
 void PForever::dump(ostream&out, unsigned ind) const
 {
-      out << setw(ind) << "" << "forever /* " << get_line() << " */" << endl;
+      out << setw(ind) << "" << "forever /* " << get_fileline() << " */" << endl;
       statement_->dump(out, ind+3);
 }
 
@@ -616,6 +785,7 @@ void PForStatement::dump(ostream&out, unsigned ind) const
 void PFunction::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "function ";
+      if (is_auto_) cout << "automatic ";
       switch (return_type_.type) {
 	  case PTF_NONE:
 	    out << "?none? ";
@@ -623,6 +793,9 @@ void PFunction::dump(ostream&out, unsigned ind) const
 	  case PTF_REG:
 	    out << "reg ";
 	    break;
+	  case PTF_REG_S:
+	    out << "reg_s ";
+	    break;
 	  case PTF_INTEGER:
 	    out << "integer ";
 	    break;
@@ -642,15 +815,23 @@ void PFunction::dump(ostream&out, unsigned ind) const
 	    out << "] ";
       }
 
-      out << name_ << ";" << endl;
+      out << pscope_name() << ";" << endl;
 
       if (ports_)
 	    for (unsigned idx = 0 ;  idx < ports_->count() ;  idx += 1) {
 		  out << setw(ind) << "";
 		  out << "input ";
-		  out << (*ports_)[idx]->path() << ";" << endl;
+		  out << (*ports_)[idx]->basename() << ";" << endl;
 	    }
 
+      dump_parameters_(out, ind);
+
+      dump_localparams_(out, ind);
+
+      dump_events_(out, ind);
+
+      dump_wires_(out, ind);
+
       if (statement_)
 	    statement_->dump(out, ind);
       else
@@ -660,7 +841,7 @@ void PFunction::dump(ostream&out, unsigned ind) const
 void PRelease::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "release " << *lval_ << "; /* "
-	  << get_line() << " */" << endl;
+	  << get_fileline() << " */" << endl;
 }
 
 void PRepeat::dump(ostream&out, unsigned ind) const
@@ -671,6 +852,9 @@ void PRepeat::dump(ostream&out, unsigned ind) const
 
 void PTask::dump(ostream&out, unsigned ind) const
 {
+      out << setw(ind) << "" << "task ";
+      if (is_auto_) cout << "automatic ";
+      out << pscope_name() << ";" << endl;
       if (ports_)
 	    for (unsigned idx = 0 ;  idx < ports_->count() ;  idx += 1) {
 		  out << setw(ind) << "";
@@ -688,9 +872,17 @@ void PTask::dump(ostream&out, unsigned ind) const
 			assert(0);
 			break;
 		  }
-		  out << (*ports_)[idx]->path() << ";" << endl;
+		  out << (*ports_)[idx]->basename() << ";" << endl;
 	    }
 
+      dump_parameters_(out, ind);
+
+      dump_localparams_(out, ind);
+
+      dump_events_(out, ind);
+
+      dump_wires_(out, ind);
+
       if (statement_)
 	    statement_->dump(out, ind);
       else
@@ -710,30 +902,234 @@ void PWhile::dump(ostream&out, unsigned ind) const
 
 void PProcess::dump(ostream&out, unsigned ind) const
 {
-      switch (type_) {
-	  case PProcess::PR_INITIAL:
-	    out << setw(ind) << "" << "initial";
-	    break;
-	  case PProcess::PR_ALWAYS:
-	    out << setw(ind) << "" << "always";
-	    break;
-      }
+      out << setw(ind) << "" << type_
+	  << " /* " << get_fileline() << " */" << endl;
 
-      out << " /* " << get_line() << " */" << endl;
-
-      for (map<perm_string,PExpr*>::const_iterator idx = attributes.begin()
-		 ; idx != attributes.end() ; idx++ ) {
-
-	    out << setw(ind+2) << "" << "(* " << (*idx).first;
-	    if ((*idx).second) {
-		  out << " = " << *(*idx).second;
-	    }
-	    out << " *)" << endl;
-      }
+      dump_attributes_map(out, attributes, ind+2);
 
       statement_->dump(out, ind+2);
 }
 
+void AProcess::dump(ostream&out, unsigned ind) const
+{
+      switch (type_) {
+	  case IVL_PR_INITIAL:
+	    out << setw(ind) << "" << "analog initial";
+	    break;
+	  case IVL_PR_ALWAYS:
+	    out << setw(ind) << "" << "analog";
+	    break;
+      }
+
+      out << " /* " << get_fileline() << " */" << endl;
+
+      dump_attributes_map(out, attributes, ind+2);
+
+      statement_->dump(out, ind+2);
+}
+
+void PSpecPath::dump(std::ostream&out, unsigned ind) const
+{
+      out << setw(ind) << "" << "specify path ";
+
+      if (condition)
+	    out << "if (" << *condition << ") ";
+
+      out << "(";
+      if (edge) {
+	    if (edge > 0)
+		  out << "posedge ";
+	    else
+		  out << "negedge ";
+      }
+
+      for (unsigned idx = 0 ;  idx < src.size() ;  idx += 1) {
+	    if (idx > 0) out << ", ";
+	    assert(src[idx]);
+	    out << src[idx];
+      }
+
+      out << " => ";
+
+      if (data_source_expression)
+	    out << "(";
+
+      for (unsigned idx = 0 ; idx < dst.size() ;  idx += 1) {
+	    if (idx > 0) out << ", ";
+	    assert(dst[idx]);
+	    out << dst[idx];
+      }
+
+      if (data_source_expression)
+	    out << " : " << *data_source_expression << ")";
+
+      out << ") = (";
+      for (unsigned idx = 0 ;  idx < delays.size() ;  idx += 1) {
+	    if (idx > 0) out << ", ";
+	    assert(delays[idx]);
+	    out << *delays[idx];
+      }
+      out << ");" << endl;
+}
+
+void PGenerate::dump(ostream&out, unsigned indent) const
+{
+      out << setw(indent) << "" << "generate(" << id_number << ")";
+
+      switch (scheme_type) {
+	  case GS_NONE:
+	    break;
+	  case GS_LOOP:
+	    out << " for ("
+		<< loop_index
+		<< "=" << *loop_init
+		<< "; " << *loop_test
+		<< "; " << loop_index
+		<< "=" << *loop_step << ")";
+	    break;
+	  case GS_CONDIT:
+	    out << " if (" << *loop_test << ")";
+	    break;
+	  case GS_ELSE:
+	    out << " else !(" << *loop_test << ")";
+	    break;
+	  case GS_CASE:
+	    out << " case (" << *loop_test << ")";
+	    break;
+	  case GS_CASE_ITEM:
+	    if (loop_test)
+		  out << " (" << *loop_test << ") == (" << *parent->loop_test << ")";
+	    else
+		  out << " default:";
+	    break;
+	  case GS_NBLOCK:
+	    out << " begin";
+      }
+
+      if (scope_name)
+	    out << " : " << scope_name;
+
+      out << endl;
+
+      dump_localparams_(out, indent+2);
+
+      dump_events_(out, indent+2);
+
+      dump_wires_(out, indent+2);
+
+      for (list<PGate*>::const_iterator idx = gates.begin()
+		 ; idx != gates.end() ;  idx++) {
+	    (*idx)->dump(out, indent+2);
+      }
+
+      for (list<PProcess*>::const_iterator idx = behaviors.begin()
+		 ; idx != behaviors.end() ;  idx++) {
+	    (*idx)->dump(out, indent+2);
+      }
+
+      for (list<AProcess*>::const_iterator idx = analog_behaviors.begin()
+		 ; idx != analog_behaviors.end() ;  idx++) {
+	    (*idx)->dump(out, indent+2);
+      }
+
+      for (list<PGenerate*>::const_iterator idx = generate_schemes.begin()
+		 ; idx != generate_schemes.end() ;  idx++) {
+	    (*idx)->dump(out, indent+2);
+      }
+
+      if (scheme_type == GS_NBLOCK) {
+	    out << setw(indent) << "" << "end endgenerate" << endl;
+      } else {
+	    out << setw(indent) << "" << "endgenerate" << endl;
+      }
+}
+
+void LexicalScope::dump_parameters_(ostream&out, unsigned indent) const
+{
+      typedef map<perm_string,param_expr_t>::const_iterator parm_iter_t;
+      for (parm_iter_t cur = parameters.begin()
+		 ; cur != parameters.end() ; cur ++) {
+	    out << setw(indent) << "" << "parameter "
+                << (*cur).second.type << " ";
+	    if ((*cur).second.signed_flag)
+		  out << "signed ";
+	    if ((*cur).second.msb)
+		  out << "[" << *(*cur).second.msb << ":"
+		      << *(*cur).second.lsb << "] ";
+	    out << (*cur).first << " = ";
+	    if ((*cur).second.expr)
+		  out << *(*cur).second.expr;
+	    else
+		  out << "/* ERROR */";
+	    for (LexicalScope::range_t*tmp = (*cur).second.range
+		       ; tmp ; tmp = tmp->next) {
+		  if (tmp->exclude_flag)
+			out << " exclude ";
+		  else
+			out << " from ";
+		  if (tmp->low_open_flag)
+			out << "(";
+		  else
+			out << "[";
+		  if (tmp->low_expr)
+			out << *(tmp->low_expr);
+		  else if (tmp->low_open_flag==false)
+			out << "-inf";
+		  else
+			out << "<nil>";
+		  out << ":";
+		  if (tmp->high_expr)
+			out << *(tmp->high_expr);
+		  else if (tmp->high_open_flag==false)
+			out << "inf";
+		  else
+			out << "<nil>";
+		  if (tmp->high_open_flag)
+			out << ")";
+		  else
+			out << "]";
+	    }
+	    out << ";" << endl;
+      }
+}
+
+void LexicalScope::dump_localparams_(ostream&out, unsigned indent) const
+{
+      typedef map<perm_string,param_expr_t>::const_iterator parm_iter_t;
+      for (parm_iter_t cur = localparams.begin()
+		 ; cur != localparams.end() ; cur ++) {
+	    out << setw(indent) << "" << "localparam ";
+	    if ((*cur).second.msb)
+		  out << "[" << *(*cur).second.msb << ":"
+		      << *(*cur).second.lsb << "] ";
+	    out << (*cur).first << " = ";
+	    if ((*cur).second.expr)
+		  out << *(*cur).second.expr << ";" << endl;
+	    else
+		  out << "/* ERROR */;" << endl;
+      }
+}
+
+void LexicalScope::dump_events_(ostream&out, unsigned indent) const
+{
+      for (map<perm_string,PEvent*>::const_iterator cur = events.begin()
+		 ; cur != events.end() ;  cur ++ ) {
+	    PEvent*ev = (*cur).second;
+	    out << setw(indent) << "" << "event " << ev->name() << "; // "
+		<< ev->get_fileline() << endl;
+      }
+}
+
+void LexicalScope::dump_wires_(ostream&out, unsigned indent) const
+{
+	// Iterate through and display all the wires.
+      for (map<perm_string,PWire*>::const_iterator wire = wires.begin()
+		 ; wire != wires.end() ; wire ++ ) {
+
+	    (*wire).second->dump(out, indent);
+      }
+}
+
 void Module::dump(ostream&out) const
 {
       if (attributes.begin() != attributes.end()) {
@@ -751,9 +1147,9 @@ void Module::dump(ostream&out) const
 	    out << " *)  ";
       }
 
-      out << "module " << name_ << ";" << endl;
+      out << "module " << mod_name() << ";" << endl;
 
-      for (unsigned idx = 0 ;  idx < ports.count() ;  idx += 1) {
+      for (unsigned idx = 0 ;  idx < ports.size() ;  idx += 1) {
 	    port_t*cur = ports[idx];
 
 	    if (cur == 0) {
@@ -762,41 +1158,27 @@ void Module::dump(ostream&out) const
 	    }
 
 	    out << "    ." << cur->name << "(" << *cur->expr[0];
-	    for (unsigned wdx = 1 ;  wdx < cur->expr.count() ;  wdx += 1) {
+	    for (unsigned wdx = 1 ;  wdx < cur->expr.size() ;  wdx += 1) {
 		  out << ", " << *cur->expr[wdx];
 	    }
 
 	    out << ")" << endl;
       }
 
-      typedef map<perm_string,param_expr_t>::const_iterator parm_iter_t;
-      typedef map<hname_t,PExpr*>::const_iterator parm_hiter_t;
-      for (parm_iter_t cur = parameters.begin()
-		 ; cur != parameters.end() ; cur ++) {
-	    out << "    parameter ";
-	    if ((*cur).second.signed_flag)
-		  out << "signed ";
-	    if ((*cur).second.msb)
-		  out << "[" << *(*cur).second.msb << ":"
-		      << *(*cur).second.lsb << "] ";
-	    out << (*cur).first << " = ";
-	    if ((*cur).second.expr)
-		  out << *(*cur).second.expr << ";" << endl;
-	    else
-		  out << "/* ERROR */;" << endl;
+      dump_parameters_(out, 4);
+
+      dump_localparams_(out, 4);
+
+      typedef map<perm_string,LineInfo*>::const_iterator genvar_iter_t;
+      for (genvar_iter_t cur = genvars.begin()
+		 ; cur != genvars.end() ; cur++) {
+	    out << "    genvar " << ((*cur).first) << ";" << endl;
       }
 
-      for (parm_iter_t cur = localparams.begin()
-		 ; cur != localparams.end() ; cur ++) {
-	    out << "    localparam ";
-	    if ((*cur).second.msb)
-		  out << "[" << *(*cur).second.msb << ":"
-		      << *(*cur).second.lsb << "] ";
-	    out << (*cur).first << " = ";
-	    if ((*cur).second.expr)
-		  out << *(*cur).second.expr << ";" << endl;
-	    else
-		  out << "/* ERROR */;" << endl;
+      typedef list<PGenerate*>::const_iterator genscheme_iter_t;
+      for (genscheme_iter_t cur = generate_schemes.begin()
+		 ; cur != generate_schemes.end() ; cur++) {
+	    (*cur)->dump(out, 4);
       }
 
       typedef map<perm_string,PExpr*>::const_iterator specparm_iter_t;
@@ -806,6 +1188,7 @@ void Module::dump(ostream&out) const
 		<< *(*cur).second << ";" << endl;
       }
 
+      typedef list<Module::named_expr_t>::const_iterator parm_hiter_t;
       for (parm_hiter_t cur = defparms.begin()
 		 ; cur != defparms.end() ;  cur ++) {
 	    out << "    defparam " << (*cur).first << " = ";
@@ -815,32 +1198,15 @@ void Module::dump(ostream&out) const
 		  out << "/* ERROR */;" << endl;
       }
 
-      for (map<perm_string,PEvent*>::const_iterator cur = events.begin()
-		 ; cur != events.end() ;  cur ++ ) {
-	    PEvent*ev = (*cur).second;
-	    out << "    event " << ev->name() << "; // "
-		<< ev->get_line() << endl;
-      }
-
-      for (map<hname_t,PData*>::const_iterator cur = datum.begin()
-		 ; cur != datum.end() ;  cur ++ ) {
-	    PData*tmp = (*cur).second;
-	    out << "    real " << tmp->name() << "; // "
-		<< tmp->get_line() << endl;
-      }
+      dump_events_(out, 4);
 
 	// Iterate through and display all the wires.
-      for (map<hname_t,PWire*>::const_iterator wire = wires_.begin()
-		 ; wire != wires_.end()
-		 ; wire ++ ) {
-
-	    (*wire).second->dump(out);
-      }
+      dump_wires_(out, 4);
 
 	// Dump the task definitions.
       typedef map<perm_string,PTask*>::const_iterator task_iter_t;
-      for (task_iter_t cur = tasks_.begin()
-		 ; cur != tasks_.end() ; cur ++) {
+      for (task_iter_t cur = tasks.begin()
+		 ; cur != tasks.end() ; cur ++) {
 	    out << "    task " << (*cur).first << ";" << endl;
 	    (*cur).second->dump(out, 6);
 	    out << "    endtask;" << endl;
@@ -848,8 +1214,8 @@ void Module::dump(ostream&out) const
 
 	// Dump the function definitions.
       typedef map<perm_string,PFunction*>::const_iterator func_iter_t;
-      for (func_iter_t cur = funcs_.begin()
-		 ; cur != funcs_.end() ; cur ++) {
+      for (func_iter_t cur = funcs.begin()
+		 ; cur != funcs.end() ; cur ++) {
 	    out << "    function " << (*cur).first << ";" << endl;
 	    (*cur).second->dump(out, 6);
 	    out << "    endfunction;" << endl;
@@ -865,13 +1231,25 @@ void Module::dump(ostream&out) const
       }
 
 
-      for (list<PProcess*>::const_iterator behav = behaviors_.begin()
-		 ; behav != behaviors_.end()
+      for (list<PProcess*>::const_iterator behav = behaviors.begin()
+		 ; behav != behaviors.end()
 		 ; behav ++ ) {
 
 	    (*behav)->dump(out, 4);
       }
 
+      for (list<AProcess*>::const_iterator idx = analog_behaviors.begin()
+		 ; idx != analog_behaviors.end() ;  idx++) {
+	    (*idx)->dump(out, 4);
+      }
+
+      for (list<PSpecPath*>::const_iterator spec = specify_paths.begin()
+		 ; spec != specify_paths.end()
+		 ; spec ++ ) {
+
+	    (*spec)->dump(out, 4);
+      }
+
       out << "endmodule" << endl;
 }
 
@@ -921,90 +1299,20 @@ void PUdp::dump(ostream&out) const
       out << "endprimitive" << endl;
 }
 
+void pform_dump(std::ostream&out, const ivl_nature_s*nat)
+{
+      out << "nature " << nat->name() << endl;
+      out << "    access " << nat->access() << ";" << endl;
+      out << "endnature" << endl;
+}
 
-/*
- * $Log: pform_dump.cc,v $
- * Revision 1.88.2.1  2006/07/10 00:21:54  steve
- *  Add support for full_case attribute.
- *
- * Revision 1.88  2004/10/04 01:10:55  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.87  2004/05/31 23:34:39  steve
- *  Rewire/generalize parsing an elaboration of
- *  function return values to allow for better
- *  speed and more type support.
- *
- * Revision 1.86  2004/05/25 19:21:07  steve
- *  More identifier lists use perm_strings.
- *
- * Revision 1.85  2004/02/20 18:53:35  steve
- *  Addtrbute keys are perm_strings.
- *
- * Revision 1.84  2004/02/20 06:22:58  steve
- *  parameter keys are per_strings.
- *
- * Revision 1.83  2004/02/18 17:11:57  steve
- *  Use perm_strings for named langiage items.
- *
- * Revision 1.82  2003/07/05 20:42:08  steve
- *  Fix some enumeration warnings.
- *
- * Revision 1.81  2003/06/20 00:53:19  steve
- *  Module attributes from the parser
- *  through to elaborated form.
- *
- * Revision 1.80  2003/06/13 19:10:46  steve
- *  Properly manage real variables in subscopes.
- *
- * Revision 1.79  2003/02/27 06:45:11  steve
- *  specparams as far as pform.
- *
- * Revision 1.78  2003/01/26 21:15:59  steve
- *  Rework expression parsing and elaboration to
- *  accommodate real/realtime values and expressions.
- *
- * Revision 1.77  2002/10/19 22:59:49  steve
- *  Redo the parameter vector support to allow
- *  parameter names in range expressions.
- *
- * Revision 1.76  2002/08/19 02:39:17  steve
- *  Support parameters with defined ranges.
- *
- * Revision 1.75  2002/08/12 01:35:00  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.74  2002/05/26 01:39:02  steve
- *  Carry Verilog 2001 attributes with processes,
- *  all the way through to the ivl_target API.
- *
- *  Divide signal reference counts between rval
- *  and lval references.
- *
- * Revision 1.73  2002/05/24 04:36:23  steve
- *  Verilog 2001 attriubtes on nets/wires.
- *
- * Revision 1.72  2002/05/23 03:08:51  steve
- *  Add language support for Verilog-2001 attribute
- *  syntax. Hook this support into existing $attribute
- *  handling, and add number and void value types.
- *
- *  Add to the ivl_target API new functions for access
- *  of complex attributes attached to gates.
- *
- * Revision 1.71  2002/05/19 23:37:28  steve
- *  Parse port_declaration_lists from the 2001 Standard.
- *
- * Revision 1.70  2002/04/21 04:59:08  steve
- *  Add support for conbinational events by finding
- *  the inputs to expressions and some statements.
- *  Get case and assignment statements working.
- *
- * Revision 1.69  2002/01/26 05:28:28  steve
- *  Detect scalar/vector declarion mismatch.
- *
- * Revision 1.68  2001/12/03 04:47:15  steve
- *  Parser and pform use hierarchical names as hname_t
- *  objects instead of encoded strings.
- */
-
+void pform_dump(std::ostream&out, const ivl_discipline_s*dis)
+{
+      out << "discipline " << dis->name() << endl;
+      out << "    domain " << dis->domain() << ";" << endl;
+      if (const ivl_nature_s*tmp = dis->potential())
+	    out << "    potential " << tmp->name() << ";" << endl;
+      if (const ivl_nature_s*tmp = dis->flow())
+	    out << "    flow " << tmp->name() << ";" << endl;
+      out << "enddiscipline" << endl;
+}
