@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,14 +16,12 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: t-dll-expr.cc,v 1.39.2.2 2006/09/15 23:56:05 steve Exp $"
-#endif
 
 # include "config.h"
 
 # include  <iostream>
 
+# include  <cstring>
 # include  "t-dll.h"
 # include  "netlist.h"
 # include  <assert.h>
@@ -38,18 +36,7 @@
  */
 static ivl_variable_type_t get_expr_type(const NetExpr*net)
 {
-      switch (net->expr_type()) {
-	  case NetExpr::ET_VOID:
-	    return IVL_VT_VOID;
-
-	  case NetExpr::ET_REAL:
-	    return IVL_VT_REAL;
-
-	  case NetExpr::ET_VECTOR:
-	    return IVL_VT_VECTOR;
-      }
-
-      return IVL_VT_VOID;
+      return net->expr_type();
 }
 
 /*
@@ -162,6 +149,22 @@ ivl_expr_t dll_target::expr_from_value_(const verinum&val)
       return expr;
 }
 
+void dll_target::expr_access_func(const NetEAccess*net)
+{
+      assert(expr_ == 0);
+	// Make a stub Branch Access Function expression node.
+      expr_ = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
+      expr_->type_  = IVL_EX_BACCESS;
+      expr_->value_ = IVL_VT_REAL;
+      expr_->file   = net->get_file();
+      expr_->lineno = net->get_lineno();
+      expr_->width_ = 1;
+      expr_->signed_= 1;
+
+      expr_->u_.branch_.branch = net->get_branch()->target_obj();
+      expr_->u_.branch_.nature = net->get_nature();
+}
+
 void dll_target::expr_binary(const NetEBinary*net)
 {
       assert(expr_ == 0);
@@ -196,6 +199,7 @@ void dll_target::expr_concat(const NetEConcat*net)
       cur->type_  = IVL_EX_CONCAT;
       cur->value_ = IVL_VT_VECTOR;
       cur->width_ = net->expr_width();
+      cur->signed_ = 0;
 
       cur->u_.concat_.rept  = net->repeat();
       cur->u_.concat_.parms = net->nparms();
@@ -211,95 +215,14 @@ void dll_target::expr_concat(const NetEConcat*net)
       expr_ = cur;
 }
 
-void dll_target::expr_memory(const NetEMemory*net)
-{
-      assert(expr_ == 0);
-      if (net->index()) {
-	    net->index()->expr_scan(this);
-	    assert(expr_);
-      }
-
-      ivl_expr_t cur = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-      assert(cur);
-
-      const NetMemory*mem = net->memory();
-
-      if (const NetNet*reg = mem->reg_from_explode()) {
-
-	    if (expr_ == 0) {
-		    // If there is no index expression for the
-		    // exploded memory, then replace it with the
-		    // entire exploded reg.
-		  cur->type_ = IVL_EX_SIGNAL;
-		  cur->value_ = IVL_VT_VECTOR;
-		  cur->width_= reg->pin_count();
-		  cur->signed_ = net->has_sign()? 1 : 0;
-		  cur->u_.signal_.sig = find_signal(des_, reg);
-		  cur->u_.signal_.lsi = 0;
-		  cur->u_.signal_.msi = cur->width_ - 1;
-
-	    } else {
-		  cur->type_ = IVL_EX_SELECT;
-		  cur->value_ = IVL_VT_VECTOR;
-		  cur->width_ = net->expr_width();
-		  cur->signed_ = net->has_sign()? 1 : 0;
-
-		    // Create an expression form of the exploded
-		    // memory. This is what the select will apply to.
-		  ivl_expr_t sig = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-		  sig->type_ = IVL_EX_SIGNAL;
-		  sig->value_ = IVL_VT_VECTOR;
-		  sig->width_ = reg->pin_count();
-		  sig->signed_ = 0;
-		  sig->u_.signal_.sig = find_signal(des_, reg);
-		  assert(sig->u_.signal_.sig);
-		  sig->u_.signal_.lsi = 0;
-		  sig->u_.signal_.msi = reg->pin_count()-1;
-		  cur->u_.binary_.lef_ = sig;
-
-		    // Create an expression of the address calculation.
-		  cur->u_.binary_.rig_ = expr_;
-
-		  if (cur->width_ > 1) {
-			ivl_expr_t mul = (ivl_expr_t)calloc(2, sizeof(struct ivl_expr_s));
-			ivl_expr_t fac = mul+1;
-
-			fac->type_  = IVL_EX_ULONG;
-			fac->value_ = IVL_VT_VECTOR;
-			fac->width_ = 8*sizeof(cur->width_);
-			fac->signed_= 0;
-			fac->u_.ulong_.value = cur->width_;
-
-			mul->type_  = IVL_EX_BINARY;
-			mul->value_ = IVL_VT_VECTOR;
-			mul->width_ = fac->width_;
-			mul->signed_= 0;
-			mul->u_.binary_.op_  = '*';
-			mul->u_.binary_.lef_ = cur->u_.binary_.rig_;
-			mul->u_.binary_.rig_ = fac;
-			cur->u_.binary_.rig_ = mul;
-		  }
-	    }
-
-      } else {
-	    cur->type_ = IVL_EX_MEMORY;
-	    cur->value_ = IVL_VT_VECTOR;
-	    cur->width_= net->expr_width();
-	    cur->signed_ = net->has_sign()? 1 : 0;
-	    cur->u_.memory_.mem_ = find_memory(des_, net->memory());
-	    cur->u_.memory_.idx_ = expr_;
-      }
-
-      expr_ = cur;
-}
-
 void dll_target::expr_const(const NetEConst*net)
 {
       assert(expr_ == 0);
 
       expr_ = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
       assert(expr_);
-      expr_->value_= IVL_VT_VECTOR;
+      expr_->value_= net->expr_type();
+      FILE_NAME(expr_, net);
 
       if (net->value().is_string()) {
 	    expr_->type_ = IVL_EX_STRING;
@@ -337,13 +260,28 @@ void dll_target::expr_const(const NetEConst*net)
 
 void dll_target::expr_param(const NetEConstParam*net)
 {
-      ivl_scope_t scope = find_scope(des_, net->scope());
-      ivl_parameter_t par = scope_find_param(scope, net->name());
+      ivl_scope_t scop = find_scope(des_, net->scope());
+      ivl_parameter_t par = scope_find_param(scop, net->name());
 
       if (par == 0) {
-	    cerr << net->get_line() << ": internal error: "
+	    cerr << net->get_fileline() << ": internal error: "
 		 << "Parameter " << net->name() << " missing from "
-		 << ivl_scope_name(scope) << endl;
+		 << ivl_scope_name(scop) << endl;
+      }
+      assert(par);
+      assert(par->value);
+      expr_ = par->value;
+}
+
+void dll_target::expr_rparam(const NetECRealParam*net)
+{
+      ivl_scope_t scop = find_scope(des_, net->scope());
+      ivl_parameter_t par = scope_find_param(scop, net->name());
+
+      if (par == 0) {
+	    cerr << net->get_fileline() << ": internal error: "
+		 << "Parameter " << net->name() << " missing from "
+		 << ivl_scope_name(scop) << endl;
       }
       assert(par);
       assert(par->value);
@@ -357,6 +295,7 @@ void dll_target::expr_creal(const NetECReal*net)
       expr_->width_  = net->expr_width();
       expr_->signed_ = 1;
       expr_->type_ = IVL_EX_REALNUM;
+      FILE_NAME(expr_, net);
       expr_->value_= IVL_VT_REAL;
       expr_->u_.real_.value = net->value().as_double();
 }
@@ -430,18 +369,8 @@ void dll_target::expr_sfunc(const NetESFunc*net)
       assert(expr);
 
       expr->type_ = IVL_EX_SFUNC;
-      switch (net->expr_type()) {
-	  case NetExpr::ET_VECTOR:
-	    expr->value_= IVL_VT_VECTOR;
-	    break;
-	  case NetExpr::ET_REAL:
-	    expr->value_= IVL_VT_REAL;
-	    break;
-	  case NetExpr::ET_VOID:
-	    assert(0);
-	    expr->value_= IVL_VT_VECTOR;
-	    break;
-      }
+      FILE_NAME(expr, net);
+      expr->value_= net->expr_type();
       expr->width_= net->expr_width();
       expr->signed_ = net->has_sign()? 1 : 0;
 	/* system function names are lex_strings strings. */
@@ -470,7 +399,8 @@ void dll_target::expr_ternary(const NetETernary*net)
       assert(expr);
 
       expr->type_  = IVL_EX_TERNARY;
-      expr->value_= IVL_VT_VECTOR;
+      FILE_NAME(expr, net);
+      expr->value_= net->expr_type();
       expr->width_ = net->expr_width();
       expr->signed_ = net->has_sign()? 1 : 0;
 
@@ -493,116 +423,43 @@ void dll_target::expr_ternary(const NetETernary*net)
 
 void dll_target::expr_signal(const NetESignal*net)
 {
+      ivl_signal_t sig = find_signal(des_, net->sig());
+
       assert(expr_ == 0);
 
+	/* If there is a word expression, generate it. */
+      ivl_expr_t word_expr = 0;
+      if (const NetExpr*word = net->word_index()) {
+	    word->expr_scan(this);
+	    assert(expr_);
+	    word_expr = expr_;
+	    expr_ = 0;
+      }
+
       expr_ = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
       assert(expr_);
 
       expr_->type_ = IVL_EX_SIGNAL;
-      expr_->value_= IVL_VT_VECTOR;
+      expr_->value_= net->expr_type();
+      expr_->file  = net->get_file();
+      expr_->lineno= net->get_lineno();
       expr_->width_= net->expr_width();
       expr_->signed_ = net->has_sign()? 1 : 0;
-      expr_->u_.signal_.sig = find_signal(des_, net->sig());
-      expr_->u_.signal_.lsi = net->lsi();
-      expr_->u_.signal_.msi = net->msi();
-}
-
-void dll_target::expr_subsignal(const NetEBitSel*net)
-{
-      assert(expr_ == 0);
-
-      ivl_expr_t expr = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-      assert(expr);
-
-      expr->type_ = IVL_EX_BITSEL;
-      expr->value_= IVL_VT_VECTOR;
-      expr->width_= net->expr_width();
-      expr->signed_ = net->has_sign()? 1 : 0;
-      expr->u_.bitsel_.sig = find_signal(des_, net->sig());
-
-      assert(expr->u_.bitsel_.sig->lsb_index == net->sig()->lsb());
-
-      net->index()->expr_scan(this);
-      assert(expr_);
-      expr->u_.bitsel_.bit = expr_;
-
-	/* If the lsb of the signal is not 0, then we are about to
-	   lose the proper offset to the normalized vector. Modify the
-	   expression to subtract the offset:
-
-	      reg [7:4] a;
-	      ... = a[x];
-
-	      becomes
-
-	      reg [3:0] a;
-	      ... = a[x-4];
-
-	    to reflect the normalizing of vectors that is done by the
-	    compiler. */
-
-      if (expr->u_.bitsel_.sig->lsb_index != 0) {
-
-	      /* Create in tmpc the constant offset (4 in the above
-		 example) to be subtracted from the index. */
-	    char*bits;
-	    long lsb = expr->u_.bitsel_.sig->lsb_index;
-	    ivl_expr_t tmpc = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-	    tmpc->type_  = IVL_EX_NUMBER;
-	    tmpc->width_ = expr->u_.bitsel_.bit->width_;
-	    tmpc->signed_ = net->index()->has_sign()? 1 : 0;
-	    tmpc->u_.number_.bits_ = bits = (char*)malloc(tmpc->width_);
-	    for (unsigned idx = 0 ;  idx < tmpc->width_ ;  idx += 1) {
-		  bits[idx] = (lsb & 1)? '1' : '0';
-		  lsb >>= 1;
-	    }
-
-	      /* Now make the subtractor (x-4 in the above example)
-		 that has as input A the index expression and input B
-		 the constant to subtract. */
-	    ivl_expr_t tmps = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-	    tmps->type_  = IVL_EX_BINARY;
-	    tmps->width_ = tmpc->width_;
-	    tmps->signed_ = net->index()->has_sign()? 1 : 0;
-	    tmps->u_.binary_.op_  = '-';
-	    tmps->u_.binary_.lef_ = expr->u_.bitsel_.bit;
-	    tmps->u_.binary_.rig_ = tmpc;
-
-	      /* Replace (x) with (x-4) */
-	    expr->u_.bitsel_.bit = tmps;
-
-	      /* If the index item distance (the distance to the next
-		 most significant bit) is not 1, then multiply the
-		 previous result to convert the index. */
-	    if (expr->u_.bitsel_.sig->lsb_dist != 1) {
-		  long dist = expr->u_.bitsel_.sig->lsb_dist;
-
-		  tmpc = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-		  tmpc->type_  = IVL_EX_NUMBER;
-		  tmpc->width_ = expr->u_.bitsel_.bit->width_;
-		  tmpc->signed_ = 1;
-		  tmpc->u_.number_.bits_ = bits = (char*)malloc(tmpc->width_);
-		  for (unsigned idx = 0 ;  idx < tmpc->width_ ;  idx += 1) {
-			bits[idx] = (dist & 1)? '1' : '0';
-			dist >>= 1;
-		  }
-
-		  tmps = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-		  tmps->type_  = IVL_EX_BINARY;
-		  tmps->width_ = tmpc->width_;
-		  tmps->signed_ = 1;
-		  tmps->u_.binary_.op_  = '*';
-		  tmps->u_.binary_.lef_ = expr->u_.bitsel_.bit;
-		  tmps->u_.binary_.rig_ = tmpc;
-
-		  expr->u_.bitsel_.bit = tmps;
-	    }
+      expr_->u_.signal_.word = word_expr;
+      expr_->u_.signal_.sig = sig;
 
+	/* Make account for the special case that this is a reference
+	   to an array as a whole. We detect this case by noting that
+	   this is an array (more than 1 word) and there is no word
+	   select expression. In that case, this is an IVL_EX_ARRAY
+	   expression instead of a SIGNAL expression. */
+      if (sig->array_words > 1 && word_expr == 0) {
+	    expr_->type_ = IVL_EX_ARRAY;
+	    expr_->width_ = 0; // Doesn't make much sense for arrays.
       }
-
-      expr_ = expr;
 }
 
+
 void dll_target::expr_ufunc(const NetEUFunc*net)
 {
       assert(expr_ == 0);
@@ -611,7 +468,8 @@ void dll_target::expr_ufunc(const NetEUFunc*net)
       assert(expr);
 
       expr->type_ = IVL_EX_UFUNC;
-      expr->value_= IVL_VT_VECTOR;
+      FILE_NAME(expr, net);
+      expr->value_= net->expr_type();
       expr->width_= net->expr_width();
       expr->signed_ = net->has_sign()? 1 : 0;
 
@@ -644,130 +502,9 @@ void dll_target::expr_unary(const NetEUnary*net)
 
       expr_ = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
       expr_->type_ = IVL_EX_UNARY;
-      expr_->value_= IVL_VT_VECTOR;
+      expr_->value_= net->expr_type();
       expr_->width_ = net->expr_width();
       expr_->signed_ = net->has_sign()? 1 : 0;
       expr_->u_.unary_.op_ = net->op();
       expr_->u_.unary_.sub_ = sub;
 }
-
-void dll_target::expr_variable(const NetEVariable*net)
-{
-      assert(expr_ == 0);
-
-      expr_ = (ivl_expr_t)calloc(1, sizeof(struct ivl_expr_s));
-      expr_->type_  = IVL_EX_VARIABLE;
-      expr_->value_ = IVL_VT_REAL;
-      expr_->width_ = 0;
-      expr_->signed_= net->has_sign()? 1 : 0;
-      expr_->u_.variable_.var = find_variable(des_, net->variable());
-}
-
-/*
- * $Log: t-dll-expr.cc,v $
- * Revision 1.39.2.2  2006/09/15 23:56:05  steve
- *  Special handling of exploded memory arguments.
- *
- * Revision 1.39.2.1  2006/03/12 07:34:19  steve
- *  Fix the memsynth1 case.
- *
- * Revision 1.39  2004/06/17 16:06:19  steve
- *  Help system function signedness survive elaboration.
- *
- * Revision 1.38  2003/07/26 03:34:43  steve
- *  Start handling pad of expressions in code generators.
- *
- * Revision 1.37  2003/06/24 01:38:03  steve
- *  Various warnings fixed.
- *
- * Revision 1.36  2003/04/22 04:48:30  steve
- *  Support event names as expressions elements.
- *
- * Revision 1.35  2003/03/10 23:40:53  steve
- *  Keep parameter constants for the ivl_target API.
- *
- * Revision 1.34  2003/03/01 06:25:30  steve
- *  Add the lex_strings string handler, and put
- *  scope names and system task/function names
- *  into this table. Also, permallocate event
- *  names from the beginning.
- *
- * Revision 1.33  2003/02/02 00:19:27  steve
- *  Terminate bits string from ivl_expr_bits.
- *
- * Revision 1.32  2003/01/30 16:23:08  steve
- *  Spelling fixes.
- *
- * Revision 1.31  2003/01/27 00:14:37  steve
- *  Support in various contexts the $realtime
- *  system task.
- *
- * Revision 1.30  2003/01/26 21:15:59  steve
- *  Rework expression parsing and elaboration to
- *  accommodate real/realtime values and expressions.
- *
- * Revision 1.29  2002/10/23 01:47:17  steve
- *  Fix synth2 handling of aset/aclr signals where
- *  flip-flops are split by begin-end blocks.
- *
- * Revision 1.28  2002/08/12 01:35:00  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.27  2002/08/04 18:28:15  steve
- *  Do not use hierarchical names of memories to
- *  generate vvp labels. -tdll target does not
- *  used hierarchical name string to look up the
- *  memory objects in the design.
- *
- * Revision 1.26  2002/06/16 20:39:12  steve
- *  Normalize run-time index expressions for bit selects
- *
- * Revision 1.25  2002/06/16 19:19:16  steve
- *  Generate runtime code to normalize indices.
- *
- * Revision 1.24  2002/05/29 22:05:54  steve
- *  Offset lvalue index expressions.
- *
- * Revision 1.23  2002/04/14 02:56:19  steve
- *  Support signed expressions through to VPI.
- *
- * Revision 1.22  2002/01/28 00:52:41  steve
- *  Add support for bit select of parameters.
- *  This leads to a NetESelect node and the
- *  vvp code generator to support that.
- *
- * Revision 1.21  2001/12/31 00:08:14  steve
- *  Support $signed cast of expressions.
- *
- * Revision 1.20  2001/10/23 04:22:41  steve
- *  Support bit selects of non-0 lsb for vectors.
- *
- * Revision 1.19  2001/10/19 21:53:24  steve
- *  Support multiple root modules (Philip Blundell)
- *
- * Revision 1.18  2001/09/15 18:27:04  steve
- *  Make configure detect malloc.h
- *
- * Revision 1.17  2001/07/27 04:51:44  steve
- *  Handle part select expressions as variants of
- *  NetESignal/IVL_EX_SIGNAL objects, instead of
- *  creating new and useless temporary signals.
- *
- * Revision 1.16  2001/07/25 03:10:49  steve
- *  Create a config.h.in file to hold all the config
- *  junk, and support gcc 3.0. (Stephan Boettcher)
- *
- * Revision 1.15  2001/07/22 00:17:49  steve
- *  Support the NetESubSignal expressions in vvp.tgt.
- *
- * Revision 1.14  2001/07/07 20:20:10  steve
- *  Pass parameters to system functions.
- *
- * Revision 1.13  2001/05/17 04:37:02  steve
- *  Behavioral ternary operators for vvp.
- *
- * Revision 1.12  2001/05/08 23:59:33  steve
- *  Add ivl and vvp.tgt support for memories in
- *  expressions and l-values. (Stephan Boettcher)
- */
-
