@@ -1,7 +1,7 @@
 #ifndef __t_dll_H
 #define __t_dll_H
 /*
- * Copyright (c) 2000-2004 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,14 +18,13 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: t-dll.h,v 1.115.2.5 2006/04/16 19:26:40 steve Exp $"
-#endif
 
 # include  "target.h"
 # include  "ivl_target.h"
+# include  "ivl_target_priv.h"
 # include  "StringHeap.h"
 # include  "netlist.h"
+# include  <vector>
 
 #if defined(__MINGW32__)
 #include <windows.h>
@@ -40,21 +39,6 @@ typedef shl_t ivl_dll_t;
 # error No DLL stub support for this target.
 #endif
 
-struct ivl_design_s {
-
-      int time_precision;
-
-      ivl_scope_t *roots_;
-      unsigned nroots_;
-
-      ivl_process_t threads_;
-
-      ivl_net_const_t*consts;
-      unsigned nconsts;
-
-      const Design*self;
-};
-
 /*
  * The DLL target type loads a named object file to handle the process
  * of scanning the netlist. When it is time to start the design, I
@@ -64,39 +48,50 @@ struct ivl_design_s {
  */
 struct dll_target  : public target_t, public expr_scan_t {
 
+	// This is a special function for loading and testing the
+	// version of a loadable target code generator.
+      void test_version(const char*target_name);
+
       bool start_design(const Design*);
       int  end_design(const Design*);
 
       bool bufz(const NetBUFZ*);
+      bool branch(const NetBranch*);
       void event(const NetEvent*);
-      void variable(const NetVariable*);
       void logic(const NetLogic*);
+      bool tran(const NetTran*);
+      bool ureduce(const NetUReduce*);
       void net_case_cmp(const NetCaseCmp*);
       void udp(const NetUDP*);
+      void lpm_abs(const NetAbs*);
       void lpm_add_sub(const NetAddSub*);
+      bool lpm_array_dq(const NetArrayDq*);
+      bool lpm_cast_int(const NetCastInt*);
+      bool lpm_cast_real(const NetCastReal*);
       void lpm_clshift(const NetCLShift*);
       void lpm_compare(const NetCompare*);
-      bool lpm_decode(const NetDecode*);
-      bool lpm_demux(const NetDemux*);
       void lpm_divide(const NetDivide*);
       void lpm_ff(const NetFF*);
-      void lpm_latch( const NetLatch * );
       void lpm_modulo(const NetModulo*);
       void lpm_mult(const NetMult*);
       void lpm_mux(const NetMux*);
-      void lpm_ram_dq(const NetRamDq*);
+      void lpm_pow(const NetPow*);
+      bool concat(const NetConcat*);
+      bool part_select(const NetPartSelect*);
+      bool replicate(const NetReplicate*);
       void net_assign(const NetAssign_*);
-      bool net_cassign(const NetCAssign*);
-      bool net_force(const NetForce*);
+      bool net_sysfunction(const NetSysFunc*);
       bool net_function(const NetUserFunc*);
       bool net_const(const NetConst*);
+      bool net_literal(const NetLiteral*);
       void net_probe(const NetEvProbe*);
+      bool sign_extend(const NetSignExtend*);
 
       bool process(const NetProcTop*);
+      bool process(const NetAnalogTop*);
       void scope(const NetScope*);
       void signal(const NetNet*);
-      void memory(const NetMemory*);
-
+      bool signal_paths(const NetNet*);
       ivl_dll_t dll_;
 
       ivl_design_s des_;
@@ -107,17 +102,20 @@ struct dll_target  : public target_t, public expr_scan_t {
 	/* These methods and members are used for forming the
 	   statements of a thread. */
       struct ivl_statement_s*stmt_cur_;
-      void proc_assign(const NetAssign*);
+      void proc_alloc(const NetAlloc*);
+      bool proc_assign(const NetAssign*);
       void proc_assign_nb(const NetAssignNB*);
       bool proc_block(const NetBlock*);
       void proc_case(const NetCase*);
       bool proc_cassign(const NetCAssign*);
       bool proc_condit(const NetCondit*);
+      bool proc_contribution(const NetContribution*);
       bool proc_deassign(const NetDeassign*);
       bool proc_delay(const NetPDelay*);
       bool proc_disable(const NetDisable*);
       bool proc_force(const NetForce*);
       void proc_forever(const NetForever*);
+      void proc_free(const NetFree*);
       bool proc_release(const NetRelease*);
       void proc_repeat(const NetRepeat*);
       void proc_stask(const NetSTask*);
@@ -130,26 +128,26 @@ struct dll_target  : public target_t, public expr_scan_t {
       void task_def(const NetScope*);
 
       struct ivl_expr_s*expr_;
+      void expr_access_func(const NetEAccess*);
       void expr_binary(const NetEBinary*);
       void expr_concat(const NetEConcat*);
-      void expr_memory(const NetEMemory*);
       void expr_const(const NetEConst*);
       void expr_creal(const NetECReal*);
       void expr_param(const NetEConstParam*);
+      void expr_rparam(const NetECRealParam*);
       void expr_event(const NetEEvent*);
       void expr_scope(const NetEScope*);
       void expr_select(const NetESelect*);
       void expr_sfunc(const NetESFunc*);
-      void expr_subsignal(const NetEBitSel*);
       void expr_ternary(const NetETernary*);
       void expr_ufunc(const NetEUFunc*);
       void expr_unary(const NetEUnary*);
       void expr_signal(const NetESignal*);
-      void expr_variable(const NetEVariable*);
 
       ivl_scope_t lookup_scope_(const NetScope*scope);
 
       ivl_attribute_s* fill_in_attributes(const Attrib*net);
+      void switch_attributes(struct ivl_switch_s *obj, const NetNode*net);
       void logic_attributes(struct ivl_net_logic_s *obj, const NetNode*net);
 
     private:
@@ -157,28 +155,42 @@ struct dll_target  : public target_t, public expr_scan_t {
 
       static ivl_scope_t find_scope(ivl_design_s &des, const NetScope*cur);
       static ivl_signal_t find_signal(ivl_design_s &des, const NetNet*net);
-      static ivl_memory_t find_memory(ivl_design_s &des, const NetMemory*net);
-      static ivl_variable_t find_variable(ivl_design_s &des, const NetVariable*net);
-
       static ivl_parameter_t scope_find_param(ivl_scope_t scope,
 					      const char*name);
 
       void add_root(ivl_design_s &des_, const NetScope *s);
 
-      ivl_lpm_t lpm_decode_ff_(const NetDecode*);
-
+      void make_assign_lvals_(const NetAssignBase*net);
       void sub_off_from_expr_(long);
       void mul_expr_by_const_(long);
 
+      void make_logic_delays_(struct ivl_net_logic_s*obj, const NetObj*net);
+      void make_lpm_delays_(struct ivl_lpm_s*obj, const NetObj*net);
+      void make_const_delays_(struct ivl_net_const_s*obj, const NetObj*net);
       void make_scope_parameters(ivl_scope_t scope, const NetScope*net);
+      void make_scope_param_expr(ivl_parameter_t cur_par, NetExpr*etmp);
+
+      ivl_event_t make_lpm_trigger(const NetEvWait*ev);
 
       static ivl_expr_t expr_from_value_(const verinum&that);
 };
 
+extern struct dll_target dll_target_obj;
+
 /*
  * These are various private declarations used by the t-dll target.
  */
 
+struct ivl_delaypath_s {
+      ivl_scope_t scope;
+      ivl_nexus_t src;
+      ivl_nexus_t condit;
+      bool conditional;
+      bool posedge;
+      bool negedge;
+      uint64_t delay[12];
+};
+
 struct ivl_event_s {
       perm_string name;
       ivl_scope_t scope;
@@ -195,6 +207,8 @@ struct ivl_event_s {
 struct ivl_expr_s {
       ivl_expr_type_t type_;
       ivl_variable_type_t value_;
+      perm_string file;
+      unsigned lineno;
 
       unsigned width_;
       unsigned signed_ : 1;
@@ -207,9 +221,9 @@ struct ivl_expr_s {
 	    } binary_;
 
 	    struct {
-		  ivl_signal_t sig;
-		  ivl_expr_t bit;
-	    } bitsel_;
+		  ivl_branch_t branch;
+		  ivl_nature_t nature;
+	    } branch_;
 
 	    struct {
 		  unsigned   rept;
@@ -232,7 +246,7 @@ struct ivl_expr_s {
 
 	    struct {
 		  ivl_signal_t sig;
-		  unsigned lsi, msi;
+		  ivl_expr_t word;
 	    } signal_;
 
 	    struct {
@@ -278,8 +292,9 @@ struct ivl_expr_s {
 	    } unary_;
 
 	    struct {
-		  ivl_variable_t var;
-	    } variable_;
+		  uint64_t value;
+	    } delay_;
+
       } u_;
 };
 
@@ -294,15 +309,14 @@ struct ivl_lpm_s {
       ivl_lpm_type_t type;
       ivl_scope_t scope;
       perm_string name;
-
-      struct ivl_attribute_s*attr;
-      unsigned nattr;
+      perm_string file;
+      unsigned lineno;
+	// Value returned by ivl_lpm_width;
+      unsigned width;
+      ivl_expr_t delay[3];
 
       union {
 	    struct ivl_lpm_ff_s {
-		  unsigned width;
-		  unsigned swid; // ram only
-		  unsigned scnt;
 		  ivl_nexus_t clk;
 		  ivl_nexus_t we;
 		  ivl_nexus_t aclr;
@@ -317,69 +331,67 @@ struct ivl_lpm_s {
 			ivl_nexus_t*pins;
 			ivl_nexus_t pin;
 		  } d;
-		  union { // ram only
-			ivl_nexus_t*pins;
-			ivl_nexus_t pin;
-		  } s;
-		  union {
-			ivl_memory_t mem; // ram only
-			ivl_lpm_t decode; // FF only
-		  } a;
 		  ivl_expr_t aset_value;
 		  ivl_expr_t sset_value;
 	    } ff;
 
-	    struct ivl_lpm_latch_s
-	    {
-	      unsigned width;
-	      ivl_nexus_s *gatePtr;
-	      ivl_nexus_s *qPtr;
-	      ivl_nexus_s *dataPtr;
-	    } latch;
-
 	    struct ivl_lpm_mux_s {
-		  unsigned width;
 		  unsigned size;
 		  unsigned swid;
 		  ivl_nexus_t*d;
-		  union {
-			ivl_nexus_t*pins;
-			ivl_nexus_t pin;
-		  } q;
-		  union {
-			ivl_nexus_t*pins;
-			ivl_nexus_t pin;
-		  } s;
+		  ivl_nexus_t q, s;
 	    } mux;
 
-	    struct ivl_lpm_demux_s {
-		  unsigned width;
-		  unsigned awid;
-		  unsigned size;
-		  ivl_nexus_t*bit_in;
-		  ivl_nexus_t *q,*d,*a;
-	    } demux;
-
 	    struct ivl_lpm_shift_s {
-		  unsigned width;
 		  unsigned select;
 		  unsigned signed_flag :1;
-		  ivl_nexus_t*q;
-		  ivl_nexus_t*d;
-		  ivl_nexus_t*s;
+		  ivl_nexus_t q, d, s;
 	    } shift;
 
 	    struct ivl_lpm_arith_s {
-		  unsigned width;
 		  unsigned signed_flag :1;
-		  ivl_nexus_t*q, *a, *b;
+		  ivl_nexus_t q,  a,  b;
 	    } arith;
 
+	    struct ivl_lpm_array_s {
+		  ivl_signal_t sig;
+		  unsigned swid;
+		  ivl_nexus_t q,  a;
+	    } array;
+
+	    struct ivl_concat_s {
+		  unsigned inputs;
+		  ivl_nexus_t*pins;
+	    } concat;
+
+	    struct ivl_part_s {
+		  unsigned base;
+		  unsigned signed_flag :1;
+		  ivl_nexus_t q, a, s;
+	    } part;
+
+	      // IVL_LPM_RE_* and IVL_LPM_SIGN_EXT use this.
+	    struct ivl_lpm_reduce_s {
+		  ivl_nexus_t q,  a;
+	    } reduce;
+
+	    struct ivl_lpm_repeat_s {
+		  unsigned count;
+		  ivl_nexus_t q, a;
+	    } repeat;
+
+	    struct ivl_lpm_sfunc_s {
+		  const char* fun_name;
+		  unsigned ports;
+		  ivl_nexus_t*pins;
+		  ivl_event_t trigger;
+	    } sfunc;
+
 	    struct ivl_lpm_ufunc_s {
 		  ivl_scope_t def;
 		  unsigned ports;
-		  unsigned *port_wid;
 		  ivl_nexus_t*pins;
+		  ivl_event_t trigger;
 	    } ufunc;
       } u_;
 };
@@ -393,40 +405,40 @@ struct ivl_lpm_s {
 enum ivl_lval_type_t {
       IVL_LVAL_REG = 0,
       IVL_LVAL_MUX = 1,
-      IVL_LVAL_MEM = 2,
+      /* IVL_LVAL_MEM = 2, / Deprecated in favor of LVAL_ARR? */
       IVL_LVAL_NET = 3, /* Only force can have NET l-values */
-      IVL_LVAL_VAR = 4
+      IVL_LVAL_ARR = 4
 };
 
 struct ivl_lval_s {
-      unsigned width_;
-      unsigned loff_;
-      unsigned type_   : 8;
+      ivl_expr_t loff;
       ivl_expr_t idx;
+      unsigned width_;
+      unsigned type_   : 8;
       union {
 	    ivl_signal_t sig;
 	    ivl_memory_t mem;
-	    ivl_variable_t var;
       } n;
 };
 
 /*
- * This object represents a vector constant, possibly signed, in a
+ * This object represents a literal constant, possibly signed, in a
  * structural context.
  */
 struct ivl_net_const_s {
+      ivl_variable_type_t type;
       unsigned width_;
       unsigned signed_ : 1;
 
       union {
+	    double real_value;
 	    char bit_[sizeof(char*)];
 	    char *bits_;
       } b;
 
-      union {
-	    ivl_nexus_t pin_;
-	    ivl_nexus_t*pins_;
-      } n;
+      ivl_nexus_t pin_;
+
+      ivl_expr_t delay[3];
 };
 
 /*
@@ -435,6 +447,7 @@ struct ivl_net_const_s {
  */
 struct ivl_net_logic_s {
       ivl_logic_t type_;
+      unsigned width_;
       ivl_udp_t udp;
 
       perm_string name_;
@@ -446,9 +459,26 @@ struct ivl_net_logic_s {
       struct ivl_attribute_s*attr;
       unsigned nattr;
 
-      unsigned delay[3];
+      ivl_expr_t delay[3];
 };
 
+struct ivl_switch_s {
+      ivl_switch_type_t type;
+      unsigned width;
+      unsigned part;
+      unsigned offset;
+
+      perm_string name;
+      ivl_scope_t scope;
+      ivl_island_t island;
+
+      struct ivl_attribute_s*attr;
+      unsigned nattr;
+
+      ivl_nexus_t pins[3];
+      perm_string file;
+      unsigned lineno;
+};
 
 /*
  * UDP definition.
@@ -456,7 +486,7 @@ struct ivl_net_logic_s {
 struct ivl_udp_s {
       perm_string name;
       unsigned nin;
-      unsigned sequ;
+      int sequ; /* boolean */
       char init;
       unsigned nrows;
       typedef const char*ccharp_t;
@@ -485,31 +515,29 @@ struct ivl_nexus_ptr_s {
 	    ivl_net_logic_t log; /* type 1 */
 	    ivl_net_const_t con; /* type 2 */
 	    ivl_lpm_t       lpm; /* type 3 */
+	    ivl_switch_t    swi; /* type 4 */
+	    ivl_branch_t    bra; /* type 5 */
       } l;
 };
 # define __NEXUS_PTR_SIG 0
 # define __NEXUS_PTR_LOG 1
 # define __NEXUS_PTR_CON 2
 # define __NEXUS_PTR_LPM 3
-
-struct ivl_nexus_s {
-      unsigned nptr_;
-      struct ivl_nexus_ptr_s*ptrs_;
-      const char*name_;
-      void*private_data;
-};
-
+# define __NEXUS_PTR_SWI 4
+# define __NEXUS_PTR_BRA 5
 
 /*
- * Memory.
+ * NOTE: ONLY allocate ivl_nexus_s objects with the included "new" operator.
  */
-struct ivl_memory_s {
-      perm_string basename_;
-      ivl_scope_t scope_;
-      unsigned width_;
-      unsigned signed_ : 1;
-      unsigned size_;
-      int root_;
+struct ivl_nexus_s {
+      ivl_nexus_s() : ptrs_(1), nexus_(0), name_(0), private_data(0) { }
+      vector<ivl_nexus_ptr_s>ptrs_;
+      const Nexus*nexus_;
+      const char*name_;
+      void*private_data;
+
+      void* operator new (size_t s);
+      void  operator delete(void*obj, size_t s); // Not implemented
 };
 
 /*
@@ -520,16 +548,21 @@ struct ivl_parameter_s {
       perm_string basename;
       ivl_scope_t scope;
       ivl_expr_t  value;
+      perm_string file;
+      unsigned lineno;
 };
 /*
- * All we know about a process it its type (initial or always) and the
+ * All we know about a process is its type (initial or always) and the
  * single statement that is it. A process also has a scope, although
  * that generally only matters for VPI calls.
  */
 struct ivl_process_s {
-      ivl_process_type_t type_;
+      ivl_process_type_t type_ : 2;
+      int analog_flag          : 1;
       ivl_scope_t scope_;
       ivl_statement_t stmt_;
+      perm_string file;
+      unsigned lineno;
 
       struct ivl_attribute_s*attr;
       unsigned nattr;
@@ -548,6 +581,10 @@ struct ivl_scope_s {
 
       perm_string name_;
       perm_string tname_;
+      perm_string file;
+      perm_string def_file;
+      unsigned lineno;
+      unsigned def_lineno;
       ivl_scope_type_t type_;
 
       unsigned nsigs_;
@@ -562,21 +599,19 @@ struct ivl_scope_s {
       unsigned nlpm_;
       ivl_lpm_t* lpm_;
 
-      unsigned nmem_;
-      ivl_memory_t* mem_;
-
-      unsigned nvar_;
-      ivl_variable_t* var_;
-
       unsigned nparam_;
       ivl_parameter_t param_;
 
 	/* Scopes that are tasks/functions have a definition. */
       ivl_statement_t def;
+      unsigned is_auto;
 
       unsigned ports;
       ivl_signal_t*port;
 
+      std::vector<ivl_switch_t>switches;
+
+      signed int time_precision :8;
       signed int time_units :8;
 
       struct ivl_attribute_s*attr;
@@ -592,12 +627,20 @@ struct ivl_scope_s {
 struct ivl_signal_s {
       ivl_signal_type_t type_;
       ivl_signal_port_t port_;
+      ivl_variable_type_t data_type;
+      ivl_discipline_t discipline;
+      perm_string file;
+      unsigned lineno;
 
       unsigned width_;
       unsigned signed_ : 1;
       unsigned isint_  : 1;
       unsigned local_  : 1;
 
+	/* For now, support only 0 or 1 array dimensions. */
+      unsigned array_dimensions_ : 1;
+      unsigned array_addr_swapped : 1;
+
 	/* These encode the run-time index for the least significant
 	   bit, and the distance to the second bit. */
       signed lsb_index;
@@ -606,10 +649,15 @@ struct ivl_signal_s {
       perm_string name_;
       ivl_scope_t scope_;
 
+      unsigned array_words;
+      int array_base;
       union {
-	    ivl_nexus_t pin_;
-	    ivl_nexus_t*pins_;
-      } n;
+	    ivl_nexus_t pin;
+	    ivl_nexus_t*pins;
+      };
+
+      ivl_delaypath_s*path;
+      unsigned npath;
 
       struct ivl_attribute_s*attr;
       unsigned nattr;
@@ -622,12 +670,27 @@ struct ivl_signal_s {
  */
 struct ivl_statement_s {
       enum ivl_statement_type_e type_;
+      perm_string file;
+      unsigned lineno;
+
       union {
-	    struct { /* IVL_ST_ASSIGN IVL_ST_ASSIGN_NB */
+	    struct { /* IVL_ST_ALLOC */
+		  ivl_scope_t scope;
+	    } alloc_;
+
+	    struct { /* IVL_ST_ASSIGN IVL_ST_ASSIGN_NB
+			IVL_ST_CASSIGN, IVL_ST_DEASSIGN */
 		  unsigned lvals_;
 		  struct ivl_lval_s*lval_;
 		  ivl_expr_t rval_;
 		  ivl_expr_t delay;
+		    // The following are only for NB event control.
+		  ivl_expr_t count;
+		  unsigned nevent;
+		  union {
+			ivl_event_t event;
+			ivl_event_t*events;
+		  };
 	    } assign_;
 
 	    struct { /* IVL_ST_BLOCK, IVL_ST_FORK */
@@ -643,13 +706,6 @@ struct ivl_statement_s {
 		  struct ivl_statement_s*case_st;
 	    } case_;
 
-	    struct { /* IVL_ST_CASSIGN, IVL_ST_DEASSIGN */
-		  unsigned lvals;
-		  struct ivl_lval_s*lval;
-		  unsigned npins;
-		  ivl_nexus_t*pins;
-	    } cassign_;
-
 	    struct { /* IVL_ST_CONDIT */
 		    /* This is the condition expression */
 		  ivl_expr_t cond_;
@@ -657,8 +713,13 @@ struct ivl_statement_s {
 		  struct ivl_statement_s*stmt_;
 	    } condit_;
 
+	    struct { /* IVL_ST_CONTRIB */
+		  ivl_expr_t lval;
+		  ivl_expr_t rval;
+	    } contrib_;
+
 	    struct { /* IVL_ST_DELAY */
-		  unsigned long delay_;
+		  uint64_t value;
 		  ivl_statement_t stmt_;
 	    } delay_;
 
@@ -675,6 +736,10 @@ struct ivl_statement_s {
 		  ivl_statement_t stmt_;
 	    } forever_;
 
+	    struct { /* IVL_ST_FREE */
+		  ivl_scope_t scope;
+	    } free_;
+
 	    struct { /* IVL_ST_STASK */
 		  const char*name_;
 		  unsigned   nparm_;
@@ -702,86 +767,51 @@ struct ivl_statement_s {
 };
 
 /*
- * This holds the details about a variable object.
+ * The FILE_NAME function is a shorthand for attaching file/line
+ * information to the statement object.
  */
-struct ivl_variable_s {
-      ivl_variable_type_t type;
-      perm_string name;
-      ivl_scope_t scope;
-};
+static inline void FILE_NAME(ivl_statement_t stmt, const LineInfo*info)
+{
+      stmt->file = info->get_file();
+      stmt->lineno = info->get_lineno();
+}
+
+static inline void FILE_NAME(ivl_expr_t expr, const LineInfo*info)
+{
+      expr->file = info->get_file();
+      expr->lineno = info->get_lineno();
+}
+
+static inline void FILE_NAME(ivl_lpm_t lpm, const LineInfo*info)
+{
+      lpm->file = info->get_file();
+      lpm->lineno = info->get_lineno();
+}
+
+static inline void FILE_NAME(ivl_scope_t scope, const NetScope*info)
+{
+      scope->file = info->get_file();
+      scope->def_file = info->get_def_file();
+      scope->lineno = info->get_lineno();
+      scope->def_lineno = info->get_def_lineno();
+}
+
+static inline void FILE_NAME(ivl_switch_t net, const LineInfo*info)
+{
+      net->file = info->get_file();
+      net->lineno = info->get_lineno();
+}
+
+static inline void FILE_NAME(ivl_process_t net, const LineInfo*info)
+{
+      net->file = info->get_file();
+      net->lineno = info->get_lineno();
+}
+
+static inline void FILE_NAME(ivl_signal_t net, const LineInfo*info)
+{
+      net->file = info->get_file();
+      net->lineno = info->get_lineno();
+}
 
-/*
- * $Log: t-dll.h,v $
- * Revision 1.115.2.5  2006/04/16 19:26:40  steve
- *  Fix handling of exploded memories with partial or missing resets.
- *
- * Revision 1.115.2.4  2006/03/26 23:09:25  steve
- *  Handle asynchronous demux/bit replacements.
- *
- * Revision 1.115.2.3  2006/03/12 07:34:19  steve
- *  Fix the memsynth1 case.
- *
- * Revision 1.115.2.2  2006/02/25 05:03:30  steve
- *  Add support for negedge FFs by using attributes.
- *
- * Revision 1.115.2.1  2006/02/19 00:11:34  steve
- *  Handle synthesis of FF vectors with l-value decoder.
- *
- * Revision 1.115  2004/10/04 01:10:56  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.114  2004/06/30 02:16:27  steve
- *  Implement signed divide and signed right shift in nets.
- *
- * Revision 1.113  2004/05/31 23:34:39  steve
- *  Rewire/generalize parsing an elaboration of
- *  function return values to allow for better
- *  speed and more type support.
- *
- * Revision 1.112  2004/02/20 06:22:58  steve
- *  parameter keys are per_strings.
- *
- * Revision 1.111  2004/02/19 07:06:57  steve
- *  LPM, logic and Variables have perm_string names.
- *
- * Revision 1.110  2004/02/19 06:57:11  steve
- *  Memory and Event names use perm_string.
- *
- * Revision 1.109  2004/02/18 17:11:58  steve
- *  Use perm_strings for named langiage items.
- *
- * Revision 1.108  2003/12/03 02:46:24  steve
- *  Add support for wait on list of named events.
- *
- * Revision 1.107  2003/11/10 20:59:04  steve
- *  Design::get_flag returns const char* instead of string.
- *
- * Revision 1.106  2003/09/03 23:33:29  steve
- *  Pass FF synchronous set values to code generator.
- *
- * Revision 1.105  2003/08/15 02:23:53  steve
- *  Add synthesis support for synchronous reset.
- *
- * Revision 1.104  2003/06/23 01:25:44  steve
- *  Module attributes make it al the way to ivl_target.
- *
- * Revision 1.103  2003/06/17 21:28:59  steve
- *  Remove short int restrictions from vvp opcodes. (part 2)
- *
- * Revision 1.102  2003/04/22 04:48:30  steve
- *  Support event names as expressions elements.
- *
- * Revision 1.101  2003/04/11 05:18:08  steve
- *  Handle signed magnitude compare all the
- *  way through to the vvp code generator.
- *
- * Revision 1.100  2003/03/10 23:40:54  steve
- *  Keep parameter constants for the ivl_target API.
- *
- * Revision 1.99  2003/03/01 06:25:30  steve
- *  Add the lex_strings string handler, and put
- *  scope names and system task/function names
- *  into this table. Also, permallocate event
- *  names from the beginning.
- */
 #endif
