@@ -18,9 +18,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: edif.h,v 1.7.2.2 2007/02/26 19:51:39 steve Exp $"
-#endif
 
 # include  <stdio.h>
 # include  <ivl_target.h>
@@ -223,43 +220,10 @@ extern void edif_add_to_joint(edif_joint_t jnt,
 			      edif_cellref_t cell,
 			      unsigned port);
 
-extern void edif_nexus_to_joint(edif_t edf, edif_joint_t jnt, ivl_nexus_t nex);
-
 /*
  * Print the entire design. This should only be done after the design
  * is completely assembled.
  */
 extern void edif_print(FILE*fd, edif_t design);
 
-/*
- * $Log: edif.h,v $
- * Revision 1.7.2.2  2007/02/26 19:51:39  steve
- *  Spelling fixes (larry doolittle)
- *
- * Revision 1.7.2.1  2005/08/27 22:29:31  steve
- *  Back-port edif_nexus_to_joint from tgt-edif.
- *
- * Revision 1.7  2003/09/03 23:34:09  steve
- *  Support synchronous set of LPM_FF devices.
- *
- * Revision 1.6  2003/08/07 05:18:04  steve
- *  Add support for OR/NOR/bufif0/bufif1.
- *
- * Revision 1.5  2003/08/07 04:04:01  steve
- *  Add an LPM device type.
- *
- * Revision 1.4  2003/06/24 03:55:00  steve
- *  Add ivl_synthesis_cell support for virtex2.
- *
- * Revision 1.3  2003/04/04 04:59:03  steve
- *  Add xlibrary celltable.
- *
- * Revision 1.2  2003/03/24 02:29:04  steve
- *  Give proper basenames to PAD signals.
- *
- * Revision 1.1  2003/03/24 00:47:54  steve
- *  Add new virtex2 architecture family, and
- *  also the new edif.h EDIF management functions.
- *
- */
 #endif
