@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,22 +16,36 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vvp.c,v 1.17 2004/10/04 01:10:57 steve Exp $"
-#endif
-
-/*
- */
 
+# include  "version.h"
 # include  "vvp_priv.h"
 # include  <string.h>
 # include  <assert.h>
 # include  <sys/types.h>
 # include  <sys/stat.h>
 
-FILE*vvp_out = 0;
+static const char*version_string =
+"Icarus Verilog VVP Code Generator " VERSION " (" VERSION_TAG ")\n\n"
+"Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)\n\n"
+"  This program is free software; you can redistribute it and/or modify\n"
+"  it under the terms of the GNU General Public License as published by\n"
+"  the Free Software Foundation; either version 2 of the License, or\n"
+"  (at your option) any later version.\n"
+"\n"
+"  This program is distributed in the hope that it will be useful,\n"
+"  but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+"  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+"  GNU General Public License for more details.\n"
+"\n"
+"  You should have received a copy of the GNU General Public License along\n"
+"  with this program; if not, write to the Free Software Foundation, Inc.,\n"
+"  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.\n"
+;
 
-inline static void draw_execute_header(ivl_design_t des)
+FILE*vvp_out = 0;
+int vvp_errors = 0;
+
+__inline__ static void draw_execute_header(ivl_design_t des)
 {
 #if !defined(__MINGW32__)
       const char*cp = ivl_design_flag(des, "VVP_EXECUTABLE");
@@ -39,10 +53,18 @@ inline static void draw_execute_header(ivl_design_t des)
 	    fprintf(vvp_out, "#! %s\n", cp);
 	    fchmod(fileno(vvp_out), 0755);
       }
+#else
+      fprintf(vvp_out, "# MinGW does not support an executable header.\n");
 #endif
+      fprintf(vvp_out, ":ivl_version \"" VERSION "\"");
+	/* I am assuming that a base release will have a blank tag. */
+      if (strcmp(VERSION_TAG, "") != 0) {
+	    fprintf(vvp_out, " \"(" VERSION_TAG ")\"");
+      }
+      fprintf(vvp_out, ";\n");
 }
 
-inline static void draw_module_declarations(ivl_design_t des)
+__inline__ static void draw_module_declarations(ivl_design_t des)
 {
       const char*cp = ivl_design_flag(des, "VPI_MODULE_LIST");
 
@@ -69,6 +91,8 @@ int target_design(ivl_design_t des)
       int rc;
       ivl_scope_t *roots;
       unsigned nroots, i;
+      unsigned size;
+      unsigned idx;
       const char*path = ivl_design_flag(des, "-o");
       assert(path);
 
@@ -82,6 +106,8 @@ int target_design(ivl_design_t des)
 	    return -1;
       }
 
+      vvp_errors = 0;
+
       draw_execute_header(des);
 
       { int pre = ivl_design_time_precision(des);
@@ -95,29 +121,34 @@ int target_design(ivl_design_t des)
 
       draw_module_declarations(des);
 
+        /* This causes all structural records to be drawn. */
       ivl_design_roots(des, &roots, &nroots);
       for (i = 0; i < nroots; i++)
 	    draw_scope(roots[i], 0);
 
+        /* Finish up any modpaths that are not yet emitted. */
+      cleanup_modpath();
+
       rc = ivl_design_process(des, draw_process, 0);
 
+        /* Dump the file name table. */
+      size = ivl_file_table_size();
+      fprintf(vvp_out, "# The file index is used to find the file name in "
+                       "the following table.\n:file_names %u;\n", size);
+      for (idx = 0; idx < size; idx++) {
+	    fprintf(vvp_out, "    \"%s\";\n", ivl_file_table_item(idx));
+      }
+
       fclose(vvp_out);
 
-      return rc;
+      return rc + vvp_errors;
 }
 
-/*
- * $Log: vvp.c,v $
- * Revision 1.17  2004/10/04 01:10:57  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.16  2003/05/16 03:22:52  steve
- *  Use fopen64 to open output file.
- *
- * Revision 1.15  2002/08/12 01:35:03  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.14  2002/08/11 23:47:04  steve
- *  Add missing Log and Ident strings.
- *
- */
+
+const char* target_query(const char*key)
+{
+      if (strcmp(key,"version") == 0)
+	    return version_string;
+
+      return 0;
+}
