@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2000 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2008 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,23 +16,39 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: verinum.cc,v 1.43.2.4 2007/03/23 23:02:31 steve Exp $"
-#endif
 
 # include "config.h"
 
 # include  "verinum.h"
 # include  <iostream>
 # include  <cassert>
+# include  <math.h> // Needed to get pow for as_double().
+# include  <stdio.h> // Needed to get snprintf for as_string().
+
+#if !defined(HAVE_LROUND)
+/*
+ * If the system doesn't provide the lround function, then we provide
+ * it ourselves here. It is simply the nearest integer, rounded away
+ * from zero.
+ */
+extern "C" long int lround(double x)
+{
+      if (x >= 0.0)
+	    return (long)floor(x+0.5);
+      else
+	    return (long)ceil(x-0.5);
+}
+#endif
+
+static verinum::V add_with_carry(verinum::V l, verinum::V r, verinum::V&c);
 
 verinum::verinum()
 : bits_(0), nbits_(0), has_len_(false), has_sign_(false), string_flag_(false)
 {
 }
 
-verinum::verinum(const V*bits, unsigned nbits, bool has_len)
-: has_len_(has_len), has_sign_(false), string_flag_(false)
+verinum::verinum(const V*bits, unsigned nbits, bool has_len__)
+: has_len_(has_len__), has_sign_(false), string_flag_(false)
 {
       nbits_ = nbits;
       bits_ = new V [nbits];
@@ -41,9 +57,63 @@ verinum::verinum(const V*bits, unsigned nbits, bool has_len)
       }
 }
 
-verinum::verinum(const std::string&str)
+static string process_verilog_string_quotes(const string&str)
+{
+      string res;
+
+      int idx = 0;
+      int str_len = str.length();
+
+      while (idx < str_len) {
+	    if (str[idx] == '\\') {
+		  idx += 1;
+		  assert(idx < str_len);
+		  switch (str[idx]) {
+		      case 'n':
+			res = res + '\n';
+			idx += 1;
+			break;
+		      case 't':
+			res = res + '\t';
+			idx += 1;
+			break;
+		      case '0':
+		      case '1':
+		      case '2':
+		      case '3':
+		      case '4':
+		      case '5':
+		      case '6':
+		      case '7': {
+			    char byte_val = 0;
+			    int odx = 0;
+			    while (odx < 3 && idx+odx < str_len
+				   && str[idx+odx] >= '0'
+				   && str[idx+odx] <= '7') {
+				  byte_val = 8*byte_val + str[idx+odx]-'0';
+				  odx += 1;
+			    }
+			    idx += odx;
+			    res = res + byte_val;
+			    break;
+		      }
+		      default:
+			res = res + str[idx];
+			idx += 1;
+			break;
+		  }
+	    } else {
+		  res = res + str[idx];
+		  idx += 1;
+	    }
+      }
+      return res;
+}
+
+verinum::verinum(const string&s)
 : has_len_(true), has_sign_(false), string_flag_(true)
 {
+      string str = process_verilog_string_quotes(s);
       nbits_ = str.length() * 8;
 
 	// Special case: The string "" is 8 bits of 0.
@@ -87,14 +157,120 @@ verinum::verinum(verinum::V val, unsigned n, bool h)
 	    bits_[idx] = val;
 }
 
-verinum::verinum(unsigned long val, unsigned n)
+verinum::verinum(uint64_t val, unsigned n)
 : has_len_(true), has_sign_(false), string_flag_(false)
 {
       nbits_ = n;
       bits_ = new V[nbits_];
       for (unsigned idx = 0 ;  idx < nbits_ ;  idx += 1) {
 	    bits_[idx] = (val&1) ? V1 : V0;
-	    val >>= 1;
+	    val >>= (uint64_t)1;
+      }
+}
+
+/* The second argument is not used! It is there to make this
+ * constructor unique. */
+verinum::verinum(double val, bool dummy)
+: has_len_(false), has_sign_(true), string_flag_(false)
+{
+      bool is_neg = false;
+      double fraction;
+      int exponent;
+      const unsigned BITS_IN_LONG = 8*sizeof(long);
+
+	/* We return `bx for a NaN or +/- infinity. */
+      if (val != val || (val && (val == 0.5*val))) {
+	    nbits_ = 1;
+	    bits_ = new V[nbits_];
+	    bits_[0] = Vx;
+	    return;
+      }
+
+	/* Convert to a positive result. */
+      if (val < 0.0) {
+	    is_neg = true;
+	    val = -val;
+      }
+
+	/* Get the exponent and fractional part of the number. */
+      fraction = frexp(val, &exponent);
+      nbits_ = exponent+1;
+      bits_ = new V[nbits_];
+      const verinum const_one(1);
+
+	/* If the value is small enough just use lround(). */
+      if (nbits_ <= BITS_IN_LONG) {
+	    long sval = lround(val);
+	    if (is_neg) sval = -sval;
+	    for (unsigned idx = 0; idx < nbits_; idx += 1) {
+		  bits_[idx] = (sval&1) ? V1 : V0;
+		  sval >>= 1;
+	    }
+	      /* Trim the result. */
+	    signed_trim();
+	    return;
+      }
+
+      unsigned nwords = (exponent-1)/BITS_IN_LONG;
+
+      fraction = ldexp(fraction, (exponent-1) % BITS_IN_LONG + 1);
+
+      if (nwords == 0) {
+	    unsigned long bits = (unsigned long) fraction;
+	    fraction = fraction - (double) bits;
+	    for (unsigned idx = 0; idx < nbits_; idx += 1) {
+		  bits_[idx] = (bits&1) ? V1 : V0;
+		  bits >>= 1;
+	    }
+	    if (fraction >= 0.5) *this = *this + const_one;
+      } else {
+	    for (int wd = nwords; wd >= 0; wd -= 1) {
+		  unsigned long bits = (unsigned long) fraction;
+		  fraction = fraction - (double) bits;
+		  unsigned max = (wd+1)*BITS_IN_LONG;
+		  if (max > nbits_) max = nbits_;
+		  for (unsigned idx = wd*BITS_IN_LONG; idx < max; idx += 1) {
+			bits_[idx] = (bits&1) ? V1 : V0;
+			bits >>= 1;
+		  }
+		  fraction = ldexp(fraction, BITS_IN_LONG);
+	    }
+	    if (fraction >= ldexp(0.5, BITS_IN_LONG)) *this = *this + const_one;
+      }
+
+	/* Convert a negative number if needed. */
+      if (is_neg) {
+	    *this = v_not(*this) + const_one;
+      }
+
+	/* Trim the result. */
+      signed_trim();
+}
+
+
+/* This is used by the double constructor above. It is needed to remove
+ * extra sign bits that can occur when calculating a negative value. */
+void verinum::signed_trim()
+{
+	/* Do we have any extra digits? */
+      unsigned tlen = nbits_-1;
+      verinum::V sign = bits_[tlen];
+      while ((tlen > 0) && (bits_[tlen] == sign)) tlen -= 1;
+
+	/* tlen now points to the first digit that is not the sign.
+	 * or bit 0. Set the length to include this bit and one proper
+	 * sign bit if needed. */
+      if (bits_[tlen] != sign) tlen += 1;
+      tlen += 1;
+
+	/* Trim the bits if needed. */
+      if (tlen < nbits_) {
+	    V* tbits = new V[tlen];
+	    for (unsigned idx = 0; idx < tlen; idx += 1)
+		  tbits[idx] = bits_[idx];
+	    delete[] bits_;
+	    bits_ = tbits;
+	    nbits_ = tlen;
       }
 }
 
@@ -134,14 +310,15 @@ verinum::verinum(const verinum&that, unsigned nbits)
       }
 }
 
-verinum::verinum(long that)
+verinum::verinum(int64_t that)
 : has_len_(false), has_sign_(true), string_flag_(false)
 {
-      long tmp;
+      int64_t tmp;
 
-      tmp = that/2;
+      if (that < 0) tmp = (that+1)/2;
+      else tmp = that/2;
       nbits_ = 1;
-      while ((tmp != 0) && (tmp != -1)) {
+      while (tmp != 0) {
 	    nbits_ += 1;
 	    tmp /= 2;
       }
@@ -151,7 +328,7 @@ verinum::verinum(long that)
       bits_ = new V[nbits_];
       for (unsigned idx = 0 ;  idx < nbits_ ;  idx += 1) {
 	    bits_[idx] = (that & 1)? V1 : V0;
-	    that /= 2;
+	    that >>= 1;
       }
 }
 
@@ -208,18 +385,7 @@ unsigned long verinum::as_ulong() const
       return val;
 }
 
-/*
- * This function returns the native long integer that represents the
- * value of this object. It accounts for sign extension if the value
- * is signed.
- *
- * If the value is undefined, return 0.
- *
- * This function presumes that the native format is 2s compliment
- * (pretty safe these days) and masks/sets bits accordingly. If the
- * value is too large for the native form, it truncates the high bits.
- */
-signed long verinum::as_long() const
+uint64_t verinum::as_ulong64() const
 {
       if (nbits_ == 0)
 	    return 0;
@@ -227,65 +393,139 @@ signed long verinum::as_long() const
       if (!is_defined())
 	    return 0;
 
-      signed long val = 0;
+      unsigned top = nbits_;
+      if (top >= (8 * sizeof(uint64_t)))
+	  top = 8 * sizeof(uint64_t);
 
-      if (has_sign_ && (bits_[nbits_-1] == V1)) {
-	    unsigned top = nbits_;
-	    if (top > (8 * sizeof(long) - 1))
-		  top = 8 * sizeof(long) - 1;
-
-	    val = -1;
-	    signed long mask = ~1L;
-	    for (unsigned idx = 0 ;  idx < top ;  idx += 1) {
-		  if (bits_[idx] == V0)
-			val &= mask;
-
-		  mask = (mask << 1) | 1L;
-	    }
-
-      } else {
-	    unsigned top = nbits_;
-	    if (top > (8 * sizeof(long) - 1))
-		  top = 8 * sizeof(long) - 1;
-
-	    signed long mask = 1;
-	    for (unsigned idx = 0 ;  idx < top ;  idx += 1, mask <<= 1)
-		  if (bits_[idx] == V1)
-			val |= mask;
-      }
+      uint64_t val = 0;
+      uint64_t mask = 1;
+      for (unsigned idx = 0 ;  idx < top ;  idx += 1, mask <<= 1)
+	    if (bits_[idx] == V1)
+		  val |= mask;
 
       return val;
 }
 
-std::string verinum::as_string() const
+/*
+ * This function returns the native long integer that represents the
+ * value of this object. It accounts for sign extension if the value
+ * is signed.
+ *
+ * If the value is undefined, return 0.
+ *
+ * This function presumes that the native format is 2s complement
+ * (pretty safe these days) and masks/sets bits accordingly. If the
+ * value is too large for the native form, it truncates the high bits.
+ */
+signed long verinum::as_long() const
+{
+#define IVLLBITS (8 * sizeof(long) - 1)
+      if (nbits_ == 0)
+	    return 0;
+
+      if (!is_defined())
+	    return 0;
+
+      signed long val = 0;
+      unsigned diag_top = 0;
+
+      unsigned top = nbits_;
+      if (top > IVLLBITS) {
+	    diag_top = top;
+	    top = IVLLBITS;
+      }
+      int lost_bits=0;
+
+      if (has_sign_ && (bits_[nbits_-1] == V1)) {
+	    val = -1;
+	    signed long mask = ~1L;
+	    for (unsigned idx = 0 ;  idx < top ;  idx += 1) {
+		  if (bits_[idx] == V0) val &= mask;
+		  mask = (mask << 1) | 1L;
+	    }
+	    if (diag_top) {
+		  for (unsigned idx = top; idx < diag_top; idx += 1) {
+			if (bits_[idx] == V0) lost_bits=1;
+		  }
+	    }
+      } else {
+	    signed long mask = 1;
+	    for (unsigned idx = 0 ;  idx < top ;  idx += 1, mask <<= 1) {
+		  if (bits_[idx] == V1) val |= mask;
+	    }
+	    if (diag_top) {
+		  for (unsigned idx = top; idx < diag_top; idx += 1) {
+			if (bits_[idx] == V1) lost_bits=1;
+		  }
+	    }
+      }
+
+      if (lost_bits) cerr << "warning: verinum::as_long() truncated " <<
+	  diag_top << " bits to " << IVLLBITS << ", returns " << val << endl;
+      return val;
+#undef IVLLBITS
+}
+
+double verinum::as_double() const
+{
+      if (nbits_ == 0) return 0.0;
+
+      double val = 0.0;
+        /* Do we have/want a signed value? */
+      if (has_sign_ && bits_[nbits_-1] == V1) {
+	    V carry = V1;
+	    for (unsigned idx = 0; idx < nbits_; idx += 1) {
+		  V sum = add_with_carry(~bits_[idx], V0, carry);
+		  if (sum == V1)
+			val += pow(2.0, (double)idx);
+	    }
+	    val *= -1.0;
+      } else {
+	    for (unsigned idx = 0; idx < nbits_; idx += 1) {
+		  if (bits_[idx] == V1)
+			val += pow(2.0, (double)idx);
+	    }
+      }
+      return val;
+}
+
+string verinum::as_string() const
 {
       assert( nbits_%8 == 0 );
       if (nbits_ == 0)
 	    return "";
 
-      char*tmp = new char[nbits_/8+1];
-      char*cp = tmp;
+      string res;
+      bool leading_nuls = true;
       for (unsigned idx = nbits_ ;  idx > 0 ;  idx -= 8) {
+	    char char_val = 0;
 	    V*bp = bits_+idx;
-	    *cp = 0;
-	    if (*(--bp) == V1) *cp |= 0x80;
-	    if (*(--bp) == V1) *cp |= 0x40;
-	    if (*(--bp) == V1) *cp |= 0x20;
-	    if (*(--bp) == V1) *cp |= 0x10;
-	    if (*(--bp) == V1) *cp |= 0x08;
-	    if (*(--bp) == V1) *cp |= 0x04;
-	    if (*(--bp) == V1) *cp |= 0x02;
-	    if (*(--bp) == V1) *cp |= 0x01;
-	    if (*cp != 0) {
-		  cp += 1;
-		  *cp = 0;
+
+	    if (*(--bp) == V1) char_val |= 0x80;
+	    if (*(--bp) == V1) char_val |= 0x40;
+	    if (*(--bp) == V1) char_val |= 0x20;
+	    if (*(--bp) == V1) char_val |= 0x10;
+	    if (*(--bp) == V1) char_val |= 0x08;
+	    if (*(--bp) == V1) char_val |= 0x04;
+	    if (*(--bp) == V1) char_val |= 0x02;
+	    if (*(--bp) == V1) char_val |= 0x01;
+	    if (char_val == 0 && leading_nuls)
+		  continue;
+
+	    if (char_val == '"' || char_val == '\\') {
+		  char tmp[5];
+		  snprintf(tmp, sizeof tmp, "\\\%03o", char_val);
+		  res = res + tmp;
+	    } else if (char_val == ' ' || isgraph(char_val)) {
+		  res = res + char_val;
+
+	    } else {
+		  char tmp[5];
+		  snprintf(tmp, sizeof tmp, "\\\%03o", char_val);
+		  res = res + tmp;
 	    }
       }
-
-      tmp[nbits_/8] = 0;
-      std::string result = std::string(tmp);
-      delete[]tmp;
-      return result;
+      return res;
 }
 
 bool verinum::is_before(const verinum&that) const
@@ -319,16 +559,37 @@ bool verinum::is_zero() const
 
 bool verinum::is_negative() const
 {
-      if (! has_sign_)
-	    return false;
+      return (bits_[nbits_-1] == V1) && has_sign();
+}
 
-      if (nbits_ == 0)
-	    return false;
+verinum pad_to_width(const verinum&that, unsigned width)
+{
+      if (that.len() >= width)
+	    return that;
 
-      if (bits_[nbits_-1] == verinum::V1)
-	    return true;
+      if (that.len() == 0) {
+	    verinum val (verinum::V0, width, that.has_len());
+	    val.has_sign(that.has_sign());
+	    return val;
+      }
 
-      return false;
+      verinum::V pad = that[that.len()-1];
+      if (pad==verinum::V1 && !that.has_sign())
+	    pad = verinum::V0;
+      if (that.has_len() && !that.has_sign()) {
+	    if (pad==verinum::Vx)
+		  pad = verinum::V0;
+	    if (pad==verinum::Vz)
+		  pad = verinum::V0;
+      }
+
+      verinum val(pad, width, that.has_len());
+
+      for (unsigned idx = 0 ;  idx < that.len() ;  idx += 1)
+	    val.set(idx, that[idx]);
+
+      val.has_sign(that.has_sign());
+      return val;
 }
 
 /*
@@ -368,12 +629,24 @@ verinum trim_vnum(const verinum&that)
       } else {
 
 	      /* If the result is unsigned and has an indefinite
-		 length, then trim off leading zeros. */
-	    tlen = 1;
-	    for (unsigned idx = tlen ;  idx < that.len() ;  idx += 1)
-		  if (that.get(idx) != verinum::V0)
-			tlen = idx+1;
+		 length, then trim off all but one leading zero. */
+	    unsigned top = that.len()-1;
+	    while ((top > 0) && (that.get(top) == verinum::V0))
+		  top -= 1;
 
+	      /* Now top is the index of the highest non-zero bit. If
+		 that turns out to the highest bit in the vector, then
+		 there is no trimming possible. */
+	    if (top+1 == that.len())
+		  return that;
+
+	      /* Make tlen wide enough to include the highest non-zero
+		 bit, plus one extra 0 bit. */
+	    tlen = top+2;
+
+	      /* This can only happen when the verinum is all zeros,
+		 so make it a single bit wide. */
+	    if (that.get(top) == verinum::V0) tlen -= 1;
       }
 
       verinum tmp (verinum::V0, tlen, false);
@@ -384,7 +657,7 @@ verinum trim_vnum(const verinum&that)
       return tmp;
 }
 
-std::ostream& operator<< (std::ostream&o, verinum::V v)
+ostream& operator<< (ostream&o, verinum::V v)
 {
       switch (v) {
 	  case verinum::V0:
@@ -404,10 +677,10 @@ std::ostream& operator<< (std::ostream&o, verinum::V v)
 }
 
 /*
- * This operator is used by various dumpers to write the verilog
+ * This operator is used by various dumpers to write the Verilog
  * number in a Verilog format.
  */
-std::ostream& operator<< (std::ostream&o, const verinum&v)
+ostream& operator<< (ostream&o, const verinum&v)
 {
       if (v.is_string()) {
 	    o << "\"" << v.as_string() << "\"";
@@ -419,25 +692,11 @@ std::ostream& operator<< (std::ostream&o, const verinum&v)
 	   output. */
       if (v.has_len()) {
 	    o << v.len();
-	    if (v.has_sign())
-		  o << "'sb";
-	    else
-		  o << "'b";
-
-	    if (v.len() == 0) {
-		  o << "0";
-		  return o;
-	    }
-
-	    for (unsigned idx = v.len() ;  idx > 0 ;  idx -= 1)
-		  o << v[idx-1];
-
-	    return o;
       }
 
 	/* If the number is fully defined (no x or z) then print it
 	   out as a decimal number. */
-      if (v.is_defined()) {
+      if (v.is_defined() && v.len() < 8*sizeof(long)) {
 	    if (v.has_sign())
 		  o << "'sd" << v.as_long();
 	    else
@@ -460,11 +719,15 @@ std::ostream& operator<< (std::ostream&o, const verinum&v)
       verinum::V trim_left = v.get(v.len()-1);
       unsigned idx;
 
-      for (idx = v.len()-1;  idx > 0;  idx -= 1)
-	    if (trim_left != v.get(idx-1))
-		  break;
+      if (v.has_sign()) {
+	    for (idx = v.len()-1;  idx > 0;  idx -= 1)
+		  if (trim_left != v.get(idx-1))
+			break;
 
-      o << trim_left;
+	    o << trim_left;
+      } else {
+	    idx = v.len();
+      }
 
       while (idx > 0) {
 	    o << v.get(idx-1);
@@ -476,37 +739,53 @@ std::ostream& operator<< (std::ostream&o, const verinum&v)
 
 verinum::V operator == (const verinum&left, const verinum&right)
 {
-      if (left.len() != right.len())
-	    return verinum::V0;
+      verinum::V left_pad = verinum::V0;
+      verinum::V right_pad = verinum::V0;
+      if (left.has_sign() && right.has_sign()) {
+	    left_pad = left.get(left.len()-1);
+	    right_pad = right.get(right.len()-1);
 
-      for (unsigned idx = 0 ;  idx < left.len() ;  idx += 1)
-	    if (left[idx] != right[idx])
+	    if (left_pad == verinum::V1 && right_pad == verinum::V0)
+		  return verinum::V1;
+	    if (left_pad == verinum::V0 && right_pad == verinum::V1)
 		  return verinum::V0;
+      }
+
+      unsigned max_len = left.len();
+      if (right.len() > max_len)
+	    max_len = right.len();
+
+      for (unsigned idx = 0 ;  idx < max_len ;  idx += 1) {
+	    verinum::V left_bit  = idx < left.len() ? left[idx]  : left_pad;
+	    verinum::V right_bit = idx < right.len()? right[idx] : right_pad;
+	    if (left_bit != right_bit)
+		  return verinum::V0;
+      }
 
       return verinum::V1;
 }
 
 verinum::V operator <= (const verinum&left, const verinum&right)
 {
+      verinum::V left_pad = verinum::V0;
+      verinum::V right_pad = verinum::V0;
       if (left.has_sign() && right.has_sign()) {
-	    if (!left.is_defined())
-		  return verinum::Vx;
-	    if (!right.is_defined())
-		  return verinum::Vx;
-	    long diff = left.as_long() - right.as_long();
-	    if (diff <= 0)
+	    left_pad = left.get(left.len()-1);
+	    right_pad = right.get(right.len()-1);
+
+	    if (left_pad == verinum::V1 && right_pad == verinum::V0)
 		  return verinum::V1;
-	    else
+	    if (left_pad == verinum::V0 && right_pad == verinum::V1)
 		  return verinum::V0;
       }
 
       unsigned idx;
       for (idx = left.len() ; idx > right.len() ;  idx -= 1) {
-	    if (left[idx-1] != verinum::V0) return verinum::V0;
+	    if (left[idx-1] != right_pad) return verinum::V0;
       }
 
       for (idx = right.len() ; idx > left.len() ;  idx -= 1) {
-	    if (right[idx-1] != verinum::V0) return verinum::V1;
+	    if (right[idx-1] != left_pad) return verinum::V1;
       }
 
       idx = right.len();
@@ -527,25 +806,25 @@ verinum::V operator <= (const verinum&left, const verinum&right)
 
 verinum::V operator < (const verinum&left, const verinum&right)
 {
+      verinum::V left_pad = verinum::V0;
+      verinum::V right_pad = verinum::V0;
       if (left.has_sign() && right.has_sign()) {
-	    if (!left.is_defined())
-		  return verinum::Vx;
-	    if (!right.is_defined())
-		  return verinum::Vx;
-	    long diff = left.as_long() - right.as_long();
-	    if (diff < 0)
+	    left_pad = left.get(left.len()-1);
+	    right_pad = right.get(right.len()-1);
+
+	    if (left_pad == verinum::V1 && right_pad == verinum::V0)
 		  return verinum::V1;
-	    else
+	    if (left_pad == verinum::V0 && right_pad == verinum::V1)
 		  return verinum::V0;
       }
 
       unsigned idx;
       for (idx = left.len() ; idx > right.len() ;  idx -= 1) {
-	    if (left[idx-1] != verinum::V0) return verinum::V0;
+	    if (left[idx-1] != right_pad) return verinum::V0;
       }
 
       for (idx = right.len() ; idx > left.len() ;  idx -= 1) {
-	    if (right[idx-1] != verinum::V0) return verinum::V1;
+	    if (right[idx-1] != left_pad) return verinum::V1;
       }
 
       while (idx > 0) {
@@ -554,6 +833,7 @@ verinum::V operator < (const verinum&left, const verinum&right)
 	    if (right[idx-1] == verinum::Vx) return verinum::Vx;
 	    if (right[idx-1] == verinum::Vz) return verinum::Vx;
 	    if (left[idx-1] > right[idx-1]) return verinum::V0;
+	    if (left[idx-1] < right[idx-1]) return verinum::V1;
 	    idx -= 1;
       }
 
@@ -650,7 +930,7 @@ verinum operator + (const verinum&left, const verinum&right)
 	    val_bits[idx] = add_with_carry(left[idx], right[idx], carry);
 
       verinum::V rpad = signed_flag? right[right.len()-1] : verinum::V0;
-      verinum::V lpad = signed_flag? left[left.len()-1] : verinum::V0;
+      verinum::V lpad = signed_flag? left[left.len()-1]   : verinum::V0;
 
       if (left.len() > right.len()) {
 
@@ -663,6 +943,50 @@ verinum operator + (const verinum&left, const verinum&right)
 		  val_bits[idx] = add_with_carry(lpad, right[idx], carry);
       }
 
+      val_bits[max] = add_with_carry(lpad, rpad, carry);
+#if 0
+      if (signed_flag) {
+	    if (val_bits[max] != val_bits[max-1])
+		  max += 1;
+      }
+#endif
+      verinum val (val_bits, max+1, false);
+      val.has_sign(signed_flag);
+
+      delete[]val_bits;
+
+      return val;
+}
+
+verinum operator - (const verinum&left, const verinum&right)
+{
+      unsigned min = left.len();
+      if (right.len() < min) min = right.len();
+
+      unsigned max = left.len();
+      if (right.len() > max) max = right.len();
+
+      bool signed_flag = left.has_sign() && right.has_sign();
+      verinum::V*val_bits = new verinum::V[max+1];
+
+      verinum::V carry = verinum::V1;
+      for (unsigned idx = 0 ;  idx < min ;  idx += 1)
+	    val_bits[idx] = add_with_carry(left[idx], ~right[idx], carry);
+
+      verinum::V rpad = signed_flag? ~right[right.len()-1] : verinum::V1;
+      verinum::V lpad = signed_flag?  left[left.len()-1]   : verinum::V0;
+
+      if (left.len() > right.len()) {
+
+	    for (unsigned idx = min ;  idx < left.len() ;  idx += 1)
+		  val_bits[idx] = add_with_carry(left[idx], rpad, carry);
+
+      } else {
+
+	    for (unsigned idx = min ;  idx < right.len() ;  idx += 1)
+		  val_bits[idx] = add_with_carry(lpad, ~right[idx], carry);
+      }
+
       if (signed_flag) {
 	    val_bits[max] = add_with_carry(lpad, rpad, carry);
 	    if (val_bits[max] != val_bits[max-1])
@@ -677,38 +1001,6 @@ verinum operator + (const verinum&left, const verinum&right)
       return val;
 }
 
-verinum operator - (const verinum&left, const verinum&r)
-{
-      verinum right;
-      unsigned min = left.len();
-      if (r.len() < min) {
-	    right = verinum(verinum::V0, min);
-	    for (unsigned idx = 0 ;  idx < r.len() ;  idx += 1)
-		  right.set(idx, r[idx]);
-
-      } else {
-	    right = r;
-      }
-
-      right = v_not(right);
-
-      unsigned max = left.len();
-      if (right.len() > max) max = right.len();
-
-      verinum val (verinum::V0, max);
-
-      verinum::V carry = verinum::V1;
-      for (unsigned idx = 0 ;  idx < min ;  idx += 1)
-	    val.set(idx, add_with_carry(left[idx], right[idx], carry));
-
-      assert(left.len() <= right.len());
-      for (unsigned idx = min ;  idx < max ;  idx += 1)
-	    val.set(idx, add_with_carry(verinum::V0, right[idx], carry));
-
-      val.has_sign(left.has_sign() && r.has_sign());
-      return val;
-}
-
 /*
  * This multiplies two verinum numbers together into a verinum
  * result. The resulting number is as large as the sum of the sizes of
@@ -736,14 +1028,18 @@ verinum operator * (const verinum&left, const verinum&right)
       verinum result(verinum::V0, left.len() + right.len(), has_len_flag);
       result.has_sign(left.has_sign() || right.has_sign());
 
-      for (unsigned rdx = 0 ;  rdx < right.len() ;  rdx += 1) {
+      verinum::V r_sign = sign_bit(right);
+      for (unsigned rdx = 0 ;  rdx < result.len() ;  rdx += 1) {
 
-	    if (right.get(rdx) == verinum::V0)
+	    verinum::V r_bit = rdx < right.len()? right.get(rdx) : r_sign;
+	    if (r_bit == verinum::V0)
 		  continue;
 
+	    verinum::V l_sign = sign_bit(left);
 	    verinum::V carry = verinum::V0;
-	    for (unsigned ldx = 0 ;  ldx < left.len() ;  ldx += 1) {
-		  result.set(ldx+rdx, add_with_carry(left[ldx],
+	    for (unsigned ldx = 0 ;  ldx < result.len()-rdx ;  ldx += 1) {
+		  verinum::V l_bit = ldx < left.len()? left[ldx] : l_sign;
+		  result.set(ldx+rdx, add_with_carry(l_bit,
 						     result[rdx+ldx],
 						     carry));
 	    }
@@ -752,6 +1048,17 @@ verinum operator * (const verinum&left, const verinum&right)
       return trim_vnum(result);
 }
 
+verinum pow(const verinum&left, const verinum&right)
+{
+      verinum result = left;
+      unsigned pow_count = right.as_ulong();
+
+      for (unsigned idx = 1 ;  idx < pow_count ;  idx += 1)
+	    result = result * left;
+
+      return result;
+}
+
 verinum operator << (const verinum&that, unsigned shift)
 {
       verinum result(verinum::V0, that.len() + shift, that.has_len());
@@ -784,7 +1091,7 @@ verinum operator >> (const verinum&that, unsigned shift)
       return result;
 }
 
-static verinum unsigned_divide(verinum num, verinum den)
+static verinum unsigned_divide(verinum num, verinum den, bool signed_result)
 {
       unsigned nwid = num.len();
       while (nwid > 0 && (num.get(nwid-1) == verinum::V0))
@@ -800,7 +1107,11 @@ static verinum unsigned_divide(verinum num, verinum den)
       den = den << (nwid-dwid);
 
       unsigned idx = nwid - dwid + 1;
-      verinum result (verinum::V0, idx);
+      verinum result (verinum::V0, signed_result ? idx + 1 : idx);
+      if (signed_result) {
+	    result.set(idx, verinum::V0);
+	    result.has_sign(true);
+      }
       while (idx > 0) {
 	    if (den <= num) {
 		  verinum dif = num - den;
@@ -814,6 +1125,34 @@ static verinum unsigned_divide(verinum num, verinum den)
       return result;
 }
 
+static verinum unsigned_modulus(verinum num, verinum den)
+{
+      unsigned nwid = num.len();
+      while (nwid > 0 && (num.get(nwid-1) == verinum::V0))
+	    nwid -= 1;
+
+      unsigned dwid = den.len();
+      while (dwid > 0 && (den.get(dwid-1) == verinum::V0))
+	    dwid -= 1;
+
+      if (dwid > nwid)
+	    return num;
+
+      den = den << (nwid-dwid);
+
+      unsigned idx = nwid - dwid + 1;
+      while (idx > 0) {
+	    if (den <= num) {
+		  verinum dif = num - den;
+		  num = dif;
+	    }
+	    den = den >> 1;
+	    idx -= 1;
+      }
+
+      return num;
+}
+
 /*
  * This operator divides the left number by the right number. If
  * either value is signed, the result is signed. If both values have a
@@ -849,16 +1188,34 @@ verinum operator / (const verinum&left, const verinum&right)
 	   result is signed or not. */
       if (result.has_sign()) {
 
-	      /* XXXX FIXME XXXX Use native unsigned division to do
-		 the work. This does not work if the result is too
-		 large for the native integer. */
-	    assert(use_len <= 8*sizeof(long));
-	    long l = left.as_long();
-	    long r = right.as_long();
-	    long v = l / r;
-	    for (unsigned idx = 0 ;  idx < use_len ;  idx += 1) {
-		  result.set(idx,  (v & 1)? verinum::V1 : verinum::V0);
-		  v >>= 1;
+	    if (use_len <= (8*sizeof(long) - 1)) {
+		  long l = left.as_long();
+		  long r = right.as_long();
+		  long v = l / r;
+		  for (unsigned idx = 0 ;  idx < use_len ;  idx += 1) {
+			result.set(idx,  (v & 1)? verinum::V1 : verinum::V0);
+			v >>= 1;
+		  }
+
+	    } else {
+		  verinum use_left, use_right;
+		  verinum zero(verinum::V0, 1, false);
+		  zero.has_sign(true);
+		  bool negative = false;
+		  if (left < zero) {
+			use_left = zero - left;
+			negative = !negative;
+		  } else {
+			use_left = left;
+		  }
+		  if (right < zero) {
+			use_right = zero - right;
+			negative = !negative;
+		  } else {
+			use_right = right;
+		  }
+		  result = unsigned_divide(use_left, use_right, true);
+		  if (negative) result = zero - result;
 	    }
 
       } else {
@@ -875,7 +1232,7 @@ verinum operator / (const verinum&left, const verinum&right)
 		  }
 
 	    } else {
-		  result = unsigned_divide(left, right);
+		  result = unsigned_divide(left, right, false);
 	    }
       }
 
@@ -909,35 +1266,51 @@ verinum operator % (const verinum&left, const verinum&right)
       result.has_sign(left.has_sign() || right.has_sign());
 
       if (result.has_sign()) {
-
-	      /* XXXX FIXME XXXX Use native unsigned division to do
-		 the work. This does not work if the result is too
-		 large for the native integer. */
-	    assert(use_len <= 8*sizeof(long));
-	    long l = left.as_long();
-	    long r = right.as_long();
-	    long v = l % r;
-	    for (unsigned idx = 0 ;  idx < use_len ;  idx += 1) {
-		  result.set(idx,  (v & 1)? verinum::V1 : verinum::V0);
-		  v >>= 1;
+	    if (use_len <= 8*sizeof(long)) {
+		    /* Use native signed modulus to do the work. */
+		  long l = left.as_long();
+		  long r = right.as_long();
+		  long v = l % r;
+		  for (unsigned idx = 0 ;  idx < use_len ;  idx += 1) {
+			result.set(idx,  (v & 1)? verinum::V1 : verinum::V0);
+			v >>= 1;
+		  }
+	    } else {
+		  verinum use_left, use_right;
+		  verinum zero(verinum::V0, 1, false);
+		  zero.has_sign(true);
+		  bool negative = false;
+		  if (left < zero) {
+			use_left = zero - left;
+			negative = true;
+		  } else {
+			use_left = left;
+		  }
+		  if (right < zero) {
+			use_right = zero - right;
+		  } else {
+			use_right = right;
+		  }
+		  result = unsigned_modulus(use_left, use_right);
+		  result.has_sign(true);
+		  if (negative) result = zero - result;
 	    }
-
       } else {
-
-	      /* XXXX FIXME XXXX Use native unsigned division to do
-		 the work. This does not work if the result is too
-		 large for the native integer. */
-	    assert(use_len <= 8*sizeof(unsigned long));
-	    unsigned long l = left.as_ulong();
-	    unsigned long r = right.as_ulong();
-	    unsigned long v = l % r;
-	    for (unsigned idx = 0 ;  idx < use_len ;  idx += 1) {
-		  result.set(idx,  (v & 1)? verinum::V1 : verinum::V0);
-		  v >>= 1;
+	    if (use_len <= 8*sizeof(unsigned long)) {
+		    /* Use native unsigned modulus to do the work. */
+		  unsigned long l = left.as_ulong();
+		  unsigned long r = right.as_ulong();
+		  unsigned long v = l % r;
+		  for (unsigned idx = 0 ;  idx < use_len ;  idx += 1) {
+			result.set(idx,  (v & 1)? verinum::V1 : verinum::V0);
+			v >>= 1;
+		  }
+	    } else {
+		  result = unsigned_modulus(left, right);
 	    }
       }
 
-      return result;
+      return trim_vnum(result);
 }
 
 verinum concat(const verinum&left, const verinum&right)
@@ -958,6 +1331,18 @@ verinum concat(const verinum&left, const verinum&right)
       return res;
 }
 
+verinum::V operator ~ (verinum::V l)
+{
+      switch (l) {
+	  case verinum::V0:
+	    return verinum::V1;
+	  case verinum::V1:
+	    return verinum::V0;
+	  default:
+	    return verinum::Vx;
+      }
+}
+
 verinum::V operator | (verinum::V l, verinum::V r)
 {
       if (l == verinum::V1)
@@ -987,90 +1372,11 @@ verinum::V operator & (verinum::V l, verinum::V r)
 verinum::V operator ^ (verinum::V l, verinum::V r)
 {
       if (l == verinum::V0)
-	    return r;
+	    return bit4_z2x(r);
       if (r == verinum::V0)
-	    return l;
+	    return bit4_z2x(l);
       if ((l == verinum::V1) && (r == verinum::V1))
 	    return verinum::V0;
 
       return verinum::Vx;
 }
-
-/*
- * $Log: verinum.cc,v $
- * Revision 1.43.2.4  2007/03/23 23:02:31  steve
- *  Fix compile time eval of <= comparison.
- *
- * Revision 1.43.2.3  2007/03/23 20:59:26  steve
- *  Fix compile time evaluation of < operator.
- *
- * Revision 1.43.2.2  2005/12/07 03:28:44  steve
- *  Support constant concatenation of constants.
- *
- * Revision 1.43.2.1  2005/08/13 00:45:55  steve
- *  Fix compilation warnings/errors with newer compilers.
- *
- * Revision 1.43  2004/05/18 18:43:15  steve
- *  Handle null string as a single nul character.
- *
- * Revision 1.42  2004/02/17 06:52:55  steve
- *  Support unsigned divide of huge numbers.
- *
- * Revision 1.41  2003/10/26 04:54:56  steve
- *  Support constant evaluation of binary ^ operator.
- *
- * Revision 1.40  2003/05/25 03:01:19  steve
- *  Get length of trimed unsigned value right.
- *
- * Revision 1.39  2003/04/14 03:40:21  steve
- *  Make some effort to preserve bits while
- *  operating on constant values.
- *
- * Revision 1.38  2003/04/03 04:30:00  steve
- *  Prevent overrun comparing verinums to zero.
- *
- * Revision 1.37  2003/02/02 00:43:16  steve
- *  Fix conversion of signed numbes to long
- *
- * Revision 1.36  2003/01/30 16:23:08  steve
- *  Spelling fixes.
- *
- * Revision 1.35  2002/08/19 02:39:17  steve
- *  Support parameters with defined ranges.
- *
- * Revision 1.34  2002/08/12 01:35:01  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.33  2002/04/27 23:26:24  steve
- *  Trim leading nulls from string forms.
- *
- * Revision 1.32  2002/04/27 04:48:43  steve
- *  Display string verinums as strings.
- *
- * Revision 1.31  2002/02/01 05:09:14  steve
- *  Propagate sign in unary minus.
- *
- * Revision 1.30  2001/12/31 00:02:33  steve
- *  Include s indicator in dump of signed numbers.
- *
- * Revision 1.29  2001/11/19 02:54:12  steve
- *  Handle division and modulus by zero while
- *  evaluating run-time constants.
- *
- * Revision 1.28  2001/11/06 06:11:55  steve
- *  Support more real arithmetic in delay constants.
- *
- * Revision 1.27  2001/07/25 03:10:50  steve
- *  Create a config.h.in file to hold all the config
- *  junk, and support gcc 3.0. (Stephan Boettcher)
- *
- * Revision 1.26  2001/02/09 05:44:23  steve
- *  support evaluation of constant < in expressions.
- *
- * Revision 1.25  2001/02/08 05:38:18  steve
- *  trim the length of unsized numbers.
- *
- * Revision 1.24  2001/02/07 21:47:13  steve
- *  Fix expression widths for rvalues and parameters (PR#131,132)
- */
-
