@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-3 Tony Bybell.
+ * Copyright (c) 2001-7 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,16 +20,57 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
 #include "lxt_write.h"
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
-#ifndef fseeko
-#define fseeko fseek
-#endif
-#ifndef ftello
-#define ftello ftell
-#endif
-#endif
+/*
+ * in-place sort to keep chained facs from migrating...
+ */
+static void wave_mergesort(struct lt_symbol **a, struct lt_symbol **b, int lo, int hi)
+{
+int i, j, k;
+
+if (lo<hi)
+	{
+        int mid=(lo+hi)/2;
+
+        wave_mergesort(a, b, lo, mid);
+        wave_mergesort(a, b, mid+1, hi);
+
+    	i=0; j=lo;
+    	while (j<=mid)
+		{
+        	b[i++]=a[j++];
+		}
+
+    	i=0; k=lo;
+    	while ((k<j)&&(j<=hi))
+		{
+        	if (strcmp(b[i]->name, a[j]->name) <= 0)
+			{
+            		a[k++]=b[i++];
+			}
+			else
+			{
+	            	a[k++]=a[j++];
+			}
+		}
+
+    	while (k<j)
+		{
+        	a[k++]=b[i++];
+		}
+	}
+}
+
+static void wave_msort(struct lt_symbol **a, int num)
+{
+struct lt_symbol **b = malloc(((num/2)+1) * sizeof(struct lt_symbol *));
+
+wave_mergesort(a, b, 0, num-1);
+
+free(b);
+}
 
 /************************ splay ************************/
 
@@ -91,7 +132,7 @@ static dslxt_Tree * dslxt_insert(char *i, dslxt_Tree * t, unsigned int val) {
 /* Return a pointer to the resulting tree.                 */
     dslxt_Tree * n;
     int dir;
-
+    
     n = (dslxt_Tree *) calloc (1, sizeof (dslxt_Tree));
     if (n == NULL) {
 	fprintf(stderr, "dslxt_insert: ran out of memory, exiting.\n");
@@ -149,7 +190,7 @@ static dslxt_Tree * dslxt_delete(char *i, dslxt_Tree * t) {
 /*
  * functions which emit various big endian
  * data to a file
- */
+ */ 
 static int lt_emit_u8(struct lt_trace *lt, int value)
 {
 unsigned char buf[1];
@@ -230,7 +271,7 @@ return(nmemb);
 static int lt_emit_string(struct lt_trace *lt, char *value)
 {
 int rc=1;
-do
+do  
 	{
         rc&=lt_emit_u8(lt, *value);
         } while(*(value++));
@@ -243,7 +284,7 @@ return(rc);
  * data to a file.  (lt->position needs to be
  * fixed up on gzclose so the tables don't
  * get out of sync!)
- */
+ */ 
 static int lt_emit_u8z(struct lt_trace *lt, int value)
 {
 unsigned char buf[1];
@@ -330,7 +371,7 @@ return(nmemb);
 static int lt_emit_stringz(struct lt_trace *lt, char *value)
 {
 int rc=1;
-do
+do  
 	{
         rc&=lt_emit_u8z(lt, *value);
         } while(*(value++));
@@ -342,7 +383,7 @@ return(rc);
  * data to a file.  (lt->position needs to be
  * fixed up on BZ2_bzclose so the tables don't
  * get out of sync!)
- */
+ */ 
 static int lt_emit_u8bz(struct lt_trace *lt, int value)
 {
 unsigned char buf[1];
@@ -429,7 +470,7 @@ return(nmemb);
 static int lt_emit_stringbz(struct lt_trace *lt, char *value)
 {
 int rc=1;
-do
+do  
 	{
         rc&=lt_emit_u8bz(lt, *value);
         } while(*(value++));
@@ -496,7 +537,7 @@ for(p=s;*p;p++)
                 {
                 h=h^(g>>24);
                 h=h^g;
-                }
+                }   
         }
 
 h^=h2;						/* combine the two hashes */
@@ -523,17 +564,17 @@ struct lt_symbol *temp;
 
 hv=lt_hash(s);
 if(!(temp=lt->sym[hv])) return(NULL); /* no hash entry, add here wanted to add */
-
+	
 while(temp)
         {
         if(!strcmp(temp->name,s))
                 {
-                return(temp); /* in table already */
+                return(temp); /* in table already */    
                 }
         if(!temp->next) break;
         temp=temp->next;
         }
-
+	
 return(NULL); /* not found, add here if you want to add*/
 }
 
@@ -558,38 +599,18 @@ if(lt->compress_fac_str)
 	lt_emit_u16z(lt, i);
 	lt_emit_stringz(lt, str+i);
         free(lt->compress_fac_str);
-        }
+        } 
         else
         {
 	lt_emit_u16z(lt, 0);
 	lt_emit_stringz(lt, str);
         }
-
+         
 lt->compress_fac_str = (char *) malloc((lt->compress_fac_len=len)+1);
 strcpy(lt->compress_fac_str, str);
 }
 
 
-/*
- * emit facs in sorted order along with geometry
- * and sync table info
- */
-static int lt_compare(const void *v1, const void *v2)
-{
-struct lt_symbol *s1 = *(struct lt_symbol **)v1;
-struct lt_symbol *s2 = *(struct lt_symbol **)v2;
-int rc = strcmp(s1->name, s2->name);
-if(rc)
-	{
-	return(rc);
-	}
-	else
-	{
-	return(s1->msb - s2->msb);
-	}
-}
-
-
 static void strip_brack(struct lt_symbol *s)
 {
 char *lastch = s->name+s->namlen - 1;
@@ -598,12 +619,17 @@ if(s->namlen<3) return;
 lastch--;
 while(lastch!=s->name)
 	{
+	if(*lastch=='.')
+		{
+		return;	/* MTI SV [0.3] notation for implicit vars */
+		}
+
 	if(*lastch=='[')
 		{
-		*lastch=0x00;
+		*lastch=0x00;	
 		return;
 		}
-	lastch--;
+	lastch--;	
 	}
 return;
 }
@@ -625,17 +651,17 @@ if((lt)&&(lt->numfacs))
 			if(lt->do_strip_brackets)
 			for(i=0;i<lt->numfacs;i++)
 				{
-				lt->sorted_facs[i] = s;
+				lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing */
 				strip_brack(s);
 				s=s->symchain;
 				}
-			else
+			else	
 			for(i=0;i<lt->numfacs;i++)
 				{
-				lt->sorted_facs[i] = s;
+				lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing*/
 				s=s->symchain;
-				}
-			qsort((void *)lt->sorted_facs, lt->numfacs, sizeof(struct lt_symbol *), lt_compare);
+				}	
+			wave_msort(lt->sorted_facs, lt->numfacs);
 
 			for(i=0;i<lt->numfacs;i++)
 				{
@@ -662,7 +688,7 @@ if((lt)&&(lt->numfacs))
 		free(lt->compress_fac_str); lt->compress_fac_str=NULL;
 		lt->compress_fac_len=0;
 		lt->zfacname_predec_size = lt->zpackcount;
-
+	
 		gzclose(lt->zhandle);
 		fseeko(lt->handle, 0L, SEEK_END);
 		lt->position=ftello(lt->handle);
@@ -697,13 +723,13 @@ if((lt)&&(lt->numfacs))
 		if(is_interlaced_trace)
 			{
 			lt->zhandle = gzdopen(dup(fileno(lt->handle)), "wb9");
-
+	
 			lt->sync_table_offset = lt->position;
 			for(i=0;i<lt->numfacs;i++)
 				{
 				lt_emit_u32z(lt, lt->sorted_facs[i]->last_change);
 				}
-
+	
 			gzclose(lt->zhandle); lt->zhandle = NULL;
 			fseeko(lt->handle, 0L, SEEK_END);
 			lt->position=ftello(lt->handle);
@@ -714,7 +740,7 @@ if((lt)&&(lt->numfacs))
 }
 
 
-/*
+/* 
  * initialize the trace and get back an lt context
  */
 struct lt_trace *lt_init(const char *name)
@@ -840,7 +866,7 @@ switch(numbytes_trans&3)
 	case 3: lt->lt_emit_u32(lt, numtrans); break;
 	}
 
-/* printf("Clock finish for '%s' at %Ld ending with '%c' for %d repeats over a switch delta of %d\n",
+/* printf("Clock finish for '%s' at %lld ending with '%c' for %d repeats over a switch delta of %d\n", 
 	s->name, lt->timeval, s->clk_prevval, s->clk_numtrans - LT_CLKPACK, s->clk_delta); */
 s->clk_prevtrans = ULLDescriptor(~0);
 s->clk_numtrans = 0;
@@ -941,7 +967,7 @@ switch(numbytes_trans&3)
 	case 3: lt->lt_emit_u32(lt, numtrans); break;
 	}
 
-/* printf("Clock finish for '%s' at %Ld ending with '%08x' for %d repeats over a switch delta of %Ld\n",
+/* printf("Clock finish for '%s' at %lld ending with '%08x' for %d repeats over a switch delta of %lld\n", 
 	s->name, lt->timeval, s->clk_prevval, s->clk_numtrans - LT_CLKPACK_M, s->clk_delta); */
 s->clk_prevtrans = ULLDescriptor(~0);
 s->clk_numtrans = 0;
@@ -1009,7 +1035,7 @@ for(i=0;i<lt->num_dict_entries;i++)
 	/* fprintf(stderr, "%8d) '%s'\n", ds->val, ds->item); */
 	lt_emit_stringz(lt, ds->item+1);
 	}
-
+	
 gzclose(lt->zhandle);
 fseeko(lt->handle, 0L, SEEK_END);
 lt->position=ftello(lt->handle);
@@ -1046,13 +1072,13 @@ if(lt)
                                 if(s->clk_numtrans > LT_CLKPACK_M) lt_flushclock_m(lt, s);
                                 }
                                 else
-                                {
+                                {   
                                 if(s->clk_numtrans > LT_CLKPACK) lt_flushclock(lt, s);
                                 }
                         }
-
+                         
                 s=s->symchain;
-		}
+		}	
 
 	lt_set_dumpon(lt);	/* in case it was turned off */
 
@@ -1110,7 +1136,7 @@ if(lt)
 		while(t)
 			{
 			lt_emit_u32z(lt, t->position - lastposition); lastposition = t->position;
-			t=t->next;
+			t=t->next;			
 			}
 
 		t=lt->timehead;
@@ -1120,9 +1146,9 @@ if(lt)
 				{
 				lxttime_t delta = t->timeval - lasttime;
 				lt_emit_u64z(lt, (int)(delta>>32), (int)delta); lasttime = t->timeval;
-
+				
 				t2=t->next;
-				free(t);
+				free(t);			
 				t=t2;
 				}
 			}
@@ -1131,12 +1157,12 @@ if(lt)
 			while(t)
 				{
 				lt_emit_u32z(lt, (int)(t->timeval - lasttime)); lasttime = t->timeval;
-
+				
 				t2=t->next;
-				free(t);
+				free(t);			
 				t=t2;
 				}
-
+	
 			lt->timehead = lt->timecurr = NULL;
 			}
 
@@ -1146,7 +1172,7 @@ if(lt)
 		lt->ztime_table_size = lt->position - lt->ztime_table_size;
 		}
 
-	if(lt->initial_value>=0)
+	if(lt->initial_value>=0) 
 		{
 		lt->initial_value_offset = lt->position;
 		lt_emit_u8(lt, lt->initial_value);
@@ -1161,7 +1187,7 @@ if(lt)
 	if(lt->double_used)
 		{
 		lt->double_test_offset = lt->position;
-		lt_emit_double(lt, 3.14159);
+		lt_emit_double(lt, 3.14159);		
 		}
 
 	if(lt->dumpoffcount)
@@ -1177,7 +1203,7 @@ if(lt)
 			lt_emit_u64(lt, (int)((ltt->timeval)>>32), (int)ltt->timeval);
 			ltt2 = ltt;
 			ltt=ltt->next;
-			free(ltt2);
+			free(ltt2);			
 			}
 
 		lt->dumpoffhead = lt->dumpoffcurr = NULL;
@@ -1220,7 +1246,7 @@ if(lt)
 		{
 		struct lt_symbol *s = lt->symchain;
 		struct lt_symbol *s2;
-
+		
 		while(s)
 			{
 			free(s->name);
@@ -1229,7 +1255,7 @@ if(lt)
 			s=s2;
 			}
 		}
-
+	
 	free(lt->sorted_facs);
 	fclose(lt->handle);
 	free(lt);
@@ -1335,7 +1361,7 @@ return(sa);
 }
 
 
-/*
+/* 
  * set current time
  */
 int lt_inc_time_by_delta(struct lt_trace *lt, unsigned int timeval)
@@ -1374,7 +1400,7 @@ if(lt)
 				else
 				{
 				free(trl);
-				goto bail;
+				goto bail;				
 				}
 			}
 			else
@@ -1455,7 +1481,7 @@ if((lt)&&(!lt->emitted))
 }
 
 /*
- * sets change interlace
+ * sets change interlace 
  */
 void lt_set_no_interlace(struct lt_trace *lt)
 {
@@ -1476,17 +1502,17 @@ if((lt)&&(!lt->emitted)&&(!lt->sorted_facs))
 		if(lt->do_strip_brackets)
 		for(i=0;i<lt->numfacs;i++)
 			{
-			lt->sorted_facs[i] = s;
+			lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing */
 			strip_brack(s);
 			s=s->symchain;
 			}
-		else
+		else	
 		for(i=0;i<lt->numfacs;i++)
 			{
-			lt->sorted_facs[i] = s;
+			lt->sorted_facs[lt->numfacs - i - 1] = s;	/* facs were chained backwards so reverse to restore bitslicing */
 			s=s->symchain;
-			}
-		qsort((void *)lt->sorted_facs, lt->numfacs, sizeof(struct lt_symbol *), lt_compare);
+			}	
+		wave_msort(lt->sorted_facs, lt->numfacs);
 
 		for(i=0;i<lt->numfacs;i++)
 			{
@@ -1525,12 +1551,12 @@ if(lt)
 	{
 	int tag;
 	switch(value)
-		{
+		{		
 		case '0':	tag = 0; break;
 		case '1':	tag = 1; break;
-		case 'Z':
+		case 'Z':	
 		case 'z':	tag = 2; break;
-		case 'X':
+		case 'X':	
 		case 'x':	tag = 3; break;
 		case 'H':
 		case 'h':	tag = 4; break;
@@ -1688,7 +1714,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						s->clk_prevval1 = s->clk_prevval;
 						s->clk_prevval  = ivalue;
 
-						/* printf("Clock value '%08x' for '%s' at %Ld (#%d)\n", ivalue, s->name, lt->timeval, s->clk_numtrans); */
+						/* printf("Clock value '%08x' for '%s' at %lld (#%d)\n", ivalue, s->name, lt->timeval, s->clk_numtrans); */
 						return(1);
 						}
 					}
@@ -1790,6 +1816,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 		s->clk_prevval = ivalue + '0';
 		}
 	}
+
 	/* normal trace handling */
 
 	last_change_delta = lt->position - s->last_change - 2;
@@ -1831,7 +1858,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 				{
 				tag = (numbytes<<4);
 				}
-
+	
 			lt->lt_emit_u8(lt, tag);
 			switch(numbytes&3)
 				{
@@ -1852,7 +1879,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 				}
 			lt->lt_emit_u8(lt, optimized ? (3+optimized1) : 0);
 			}
-
+	
 		s->last_change = start_position;
 
 		if(s->rows>0)
@@ -1918,7 +1945,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						{
 						if(lt->num_dict_entries==(256*65536)) lt->dict32_offset = lt->position;
 						}
-
+			
 					lt->num_dict_entries++;
 					}
 
@@ -1963,14 +1990,14 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 				value <<= (24-len);
 				rc=lt->lt_emit_u24(lt, value);
 				}
-			else
+			else			
 				{
 				value <<= (32-len);
 				rc=lt->lt_emit_u32(lt, value);
 				}
 			}
-		}
-
+		}	
+	
 	if(lt->timebuff)
 		{
 		lt->timechangecount++;
@@ -2032,7 +2059,7 @@ if((s->flags)&LT_SYM_F_DOUBLE)
 			{
 			numbytes = 0;
 			}
-
+	
 		start_position = lt->position;
 		s->last_change = start_position;
 
@@ -2088,7 +2115,7 @@ if((s->flags)&LT_SYM_F_DOUBLE)
 		}
 
 	rc=lt->lt_emit_double(lt, value);
-
+	
 	if(lt->timebuff)
 		{
 		lt->timechangecount++;
@@ -2130,7 +2157,7 @@ if((s->flags)&LT_SYM_F_STRING)
 	int numbytes;				/* number of bytes to store value minus one */
 	unsigned int last_change_delta = lt->position - s->last_change - 2;
 
-	if(lt->numfacs_bytes)
+	if(!lt->numfacs_bytes)
 		{
 		if(last_change_delta >= 256*65536)
 			{
@@ -2150,7 +2177,7 @@ if((s->flags)&LT_SYM_F_STRING)
 			{
 			numbytes = 0;
 			}
-
+	
 		start_position = lt->position;
 		s->last_change = start_position;
 
@@ -2206,7 +2233,7 @@ if((s->flags)&LT_SYM_F_STRING)
 		}
 
 	rc=lt->lt_emit_string(lt, value);
-
+	
 	if(lt->timebuff)
 		{
 		lt->timechangecount++;
@@ -2319,7 +2346,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						s->clk_prevval1 = s->clk_prevval;
 						s->clk_prevval  = ivalue;
 
-						/* printf("Clock value '%08x' for '%s' [len=%d] at %Ld (#%d)\n",
+						/* printf("Clock value '%08x' for '%s' [len=%d] at %lld (#%d)\n", 
 							ivalue, s->name, len, lt->timeval, s->clk_numtrans); */
 						return(1);
 						}
@@ -2385,7 +2412,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						{
 						s->clk_prevval = value[0];
 
-						/* printf("Clock value '%c' for '%s' at %Ld (#%d)\n", value[0], s->name, lt->timeval, s->clk_numtrans); */
+						/* printf("Clock value '%c' for '%s' at %lld (#%d)\n", value[0], s->name, lt->timeval, s->clk_numtrans); */
 						return(1);
 						}
 					}
@@ -2420,6 +2447,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 		s->clk_prevval = value[0];
 		}
 	}
+
 	/* normal trace handling */
 
 	last_change_delta = lt->position - s->last_change - 2;
@@ -2448,12 +2476,12 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 	while((ch=*(pnt++)))
 		{
 		switch(ch)
-			{
+			{		
 			case '0':
 			case '1':	mvl|=LT_MVL_2; break;
-			case 'Z':
-			case 'z':
-			case 'X':
+			case 'Z':	
+			case 'z':	
+			case 'X':	
 			case 'x':	mvl|=LT_MVL_4; break;
 			default:	mvl|=LT_MVL_9; break;
 			}
@@ -2462,13 +2490,13 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 		}
 
 	switch(prevch)
-		{
+		{		
 		case 0x00:	tagadd = 0; break;
 		case '0':	tagadd = 3; break;
 		case '1':	tagadd = 4; break;
-		case 'Z':
+		case 'Z':	
 		case 'z':	tagadd = 5; break;
-		case 'X':
+		case 'X':	
 		case 'x':	tagadd = 6; break;
 		case 'H':
 		case 'h':	tagadd = 7; break;
@@ -2565,7 +2593,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 				int outval = 0;
 				int thisval= 0;
 
-				pnt = value;
+				pnt = value;				
 
 				if((lt->dictmode)&&(len>lt->mindictwidth))
 					{
@@ -2596,7 +2624,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 							{
 							if(lt->num_dict_entries==(256*65536)) lt->dict32_offset = lt->position;
 							}
-
+			
 						lt->num_dict_entries++;
 						}
 
@@ -2634,8 +2662,8 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 					outval |= (thisval<<bitpos);
 					bitpos--;
 					if((bitpos==-1)||(i==len-1))
-						{
-						lt->lt_emit_u8(lt, outval);
+						{					
+						lt->lt_emit_u8(lt, outval); 
 						outval = 0;
 						bitpos = 7;
 						}
@@ -2649,7 +2677,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 				int outval = 0;
 				int thisval= 0;
 
-				pnt = value;
+				pnt = value;				
 
 				for(i=0;i<len;i++)
 					{
@@ -2668,8 +2696,8 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 					outval |= (thisval<<bitpos);
 					bitpos-=2;
 					if((bitpos==-2)||(i==len-1))
-						{
-						lt->lt_emit_u8(lt, outval);
+						{					
+						lt->lt_emit_u8(lt, outval); 
 						outval = 0;
 						bitpos = 6;
 						}
@@ -2683,7 +2711,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 				int outval = 0;
 				int thisval= 0;
 
-				pnt = value;
+				pnt = value;				
 
 				for(i=0;i<len;i++)
 					{
@@ -2712,8 +2740,8 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 					outval |= (thisval<<bitpos);
 					bitpos-=4;
 					if((bitpos==-4)||(i==len-1))
-						{
-						lt->lt_emit_u8(lt, outval);
+						{					
+						lt->lt_emit_u8(lt, outval); 
 						outval = 0;
 						bitpos = 4;
 						}
@@ -2723,7 +2751,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 
 		rc=1;
 		}
-
+	
 	if(lt->timebuff)
 		{
 		lt->timechangecount++;
