@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2002-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,9 +16,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: sys_lxt.c,v 1.26 2004/10/04 01:10:58 steve Exp $"
-#endif
 
 # include "sys_priv.h"
 # include "lxt_write.h"
@@ -26,7 +23,7 @@
 # include "sys_priv.h"
 
 /*
- * This file contains the implementations of the VCD related
+ * This file contains the implementations of the LXT related
  * funcitons.
  */
 
@@ -108,6 +105,7 @@ static char *create_full_name(const char *name)
 {
       char *n, *n2;
       int len = 0;
+      int is_esc_id = is_escaped_id(name);
       struct lxt_scope *t = lxt_scope_head;
 
 	/* Figure out how long the combined string will be. */
@@ -117,6 +115,7 @@ static char *create_full_name(const char *name)
       }
 
       len += strlen(name) + 1;
+      if (is_esc_id) len += 1;
 
 	/* Allocate a string buffer. */
       n = n2 = malloc(len);
@@ -130,6 +129,10 @@ static char *create_full_name(const char *name)
 	    t=t->next;
       }
 
+      if (is_esc_id) {
+	    *n2 = '\\';
+	    n2++;
+      }
       strcpy(n2, name);
       n2 += strlen(n2);
       assert( (n2 - n + 1) == len );
@@ -138,6 +141,7 @@ static char *create_full_name(const char *name)
 }
 
 
+static char *dump_path = 0;
 static struct lt_trace *dump_file = 0;
 
 struct vcd_info {
@@ -155,13 +159,16 @@ static struct vcd_info*vcd_list = 0;
 static struct vcd_info*vcd_dmp_list = 0;
 static PLI_UINT64 vcd_cur_time = 0;
 static int dump_is_off = 0;
+static long dump_limit = 0;
+static int dump_is_full = 0;
+static int finish_status = 0;
 
 
 static void show_this_item(struct vcd_info*info)
 {
       s_vpi_value value;
 
-      if (vpi_get(vpiType,info->item) == vpiRealVar) {
+      if (vpi_get(vpiType, info->item) == vpiRealVar) {
 	    value.format = vpiRealVal;
 	    vpi_get_value(info->item, &value);
 	    lt_emit_value_double(dump_file, info->sym, 0, value.value.real);
@@ -170,8 +177,8 @@ static void show_this_item(struct vcd_info*info)
 	    value.format = vpiBinStrVal;
 	    vpi_get_value(info->item, &value);
 	    lt_emit_value_bit_string(dump_file, info->sym,
-				     0 /* array row */,
-				     value.value.str);
+	                             0 /* array row */,
+	                             value.value.str);
       }
 }
 
@@ -195,7 +202,7 @@ struct vcd_names_list_s lxt_tab;
 
 static int dumpvars_status = 0; /* 0:fresh 1:cb installed, 2:callback done */
 static PLI_UINT64 dumpvars_time;
-inline static int dump_header_pending(void)
+__inline__ static int dump_header_pending(void)
 {
       return dumpvars_status != 2;
 }
@@ -220,7 +227,7 @@ static void vcd_checkpoint_x()
 	    show_this_item_x(cur);
 }
 
-static int variable_cb_2(p_cb_data cause)
+static PLI_INT32 variable_cb_2(p_cb_data cause)
 {
       struct vcd_info* info = vcd_dmp_list;
       PLI_UINT64 now = timerec_to_time64(cause->time);
@@ -240,14 +247,22 @@ static int variable_cb_2(p_cb_data cause)
       return 0;
 }
 
-static int variable_cb_1(p_cb_data cause)
+static PLI_INT32 variable_cb_1(p_cb_data cause)
 {
       struct t_cb_data cb;
       struct vcd_info*info = (struct vcd_info*)cause->user_data;
 
-      if (dump_is_off) 		 return 0;
+      if (dump_is_full) return 0;
+      if (dump_is_off) return 0;
       if (dump_header_pending()) return 0;
-      if (info->scheduled)       return 0;
+      if (info->scheduled) return 0;
+
+      if ((dump_limit > 0) && (ftell(dump_file->handle) > dump_limit)) {
+            dump_is_full = 1;
+            vpi_printf("WARNING: Dump file limit (%ld bytes) "
+                       "exceeded.\n", dump_limit);
+            return 0;
+      }
 
       if (!vcd_dmp_list) {
           cb = *cause;
@@ -263,10 +278,9 @@ static int variable_cb_1(p_cb_data cause)
       return 0;
 }
 
-static int dumpvars_cb(p_cb_data cause)
+static PLI_INT32 dumpvars_cb(p_cb_data cause)
 {
-      if (dumpvars_status != 1)
-	    return 0;
+      if (dumpvars_status != 1) return 0;
 
       dumpvars_status = 2;
 
@@ -281,24 +295,49 @@ static int dumpvars_cb(p_cb_data cause)
       return 0;
 }
 
-inline static int install_dumpvars_callback(void)
+static PLI_INT32 finish_cb(p_cb_data cause)
+{
+      struct vcd_info *cur, *next;
+
+      if (finish_status != 0) return 0;
+
+      finish_status = 1;
+
+      dumpvars_time = timerec_to_time64(cause->time);
+      if (!dump_is_off && !dump_is_full && dumpvars_time != vcd_cur_time) {
+            lt_set_time64(dump_file, dumpvars_time);
+      }
+
+      for (cur = vcd_list ;  cur ;  cur = next) {
+	    next = cur->next;
+	    free(cur);
+      }
+      vcd_list = 0;
+
+      vcd_names_delete(&lxt_tab);
+      nexus_ident_delete();
+      free(dump_path);
+      dump_path = 0;
+
+      return 0;
+}
+
+__inline__ static int install_dumpvars_callback(void)
 {
       struct t_cb_data cb;
-      static struct t_vpi_time time;
+      static struct t_vpi_time now;
 
-      if (dumpvars_status == 1)
-	    return 0;
+      if (dumpvars_status == 1) return 0;
 
       if (dumpvars_status == 2) {
-	    vpi_mcd_printf(1, "VCD Error:"
-			   " $dumpvars ignored,"
-			   " previously called at simtime %lu\n",
-			   dumpvars_time);
+	    vpi_printf("LXT warning: $dumpvars ignored, previously"
+	               " called at simtime %" PLI_UINT64_FMT "\n",
+	               dumpvars_time);
 	    return 1;
       }
 
-      time.type = vpiSimTime;
-      cb.time = &time;
+      now.type = vpiSimTime;
+      cb.time = &now;
       cb.reason = cbReadOnlySynch;
       cb.cb_rtn = dumpvars_cb;
       cb.user_data = 0x0;
@@ -306,33 +345,35 @@ inline static int install_dumpvars_callback(void)
 
       vpi_register_cb(&cb);
 
+      cb.reason = cbEndOfSimulation;
+      cb.cb_rtn = finish_cb;
+
+      vpi_register_cb(&cb);
+
       dumpvars_status = 1;
       return 0;
 }
 
-static int sys_dumpoff_calltf(char*name)
+static PLI_INT32 sys_dumpoff_calltf(PLI_BYTE8*name)
 {
       s_vpi_time now;
       PLI_UINT64 now64;
 
-      if (dump_is_off)
-	    return 0;
+      if (dump_is_off) return 0;
 
       dump_is_off = 1;
 
-      if (dump_file == 0)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      if (dump_file == 0) return 0;
+      if (dump_header_pending()) return 0;
 
       now.type = vpiSimTime;
       vpi_get_time(0, &now);
       now64 = timerec_to_time64(&now);
 
-      if (now64 > vcd_cur_time)
-            lt_set_time64(dump_file, now64);
-      vcd_cur_time = now64;
+      if (now64 > vcd_cur_time) {
+	    lt_set_time64(dump_file, now64);
+	    vcd_cur_time = now64;
+      }
 
       lt_set_dumpoff(dump_file);
       vcd_checkpoint_x();
@@ -340,29 +381,26 @@ static int sys_dumpoff_calltf(char*name)
       return 0;
 }
 
-static int sys_dumpon_calltf(char*name)
+static PLI_INT32 sys_dumpon_calltf(PLI_BYTE8*name)
 {
       s_vpi_time now;
       PLI_UINT64 now64;
 
-      if (!dump_is_off)
-	    return 0;
+      if (!dump_is_off) return 0;
 
       dump_is_off = 0;
 
-      if (dump_file == 0)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      if (dump_file == 0) return 0;
+      if (dump_header_pending()) return 0;
 
       now.type = vpiSimTime;
       vpi_get_time(0, &now);
       now64 = timerec_to_time64(&now);
 
-      if (now64 > vcd_cur_time)
-            lt_set_time64(dump_file, now64);
-      vcd_cur_time = now64;
+      if (now64 > vcd_cur_time) {
+	    lt_set_time64(dump_file, now64);
+	    vcd_cur_time = now64;
+      }
 
       lt_set_dumpon(dump_file);
       vcd_checkpoint();
@@ -370,24 +408,23 @@ static int sys_dumpon_calltf(char*name)
       return 0;
 }
 
-static int sys_dumpall_calltf(char*name)
+static PLI_INT32 sys_dumpall_calltf(PLI_BYTE8*name)
 {
       s_vpi_time now;
       PLI_UINT64 now64;
 
-      if (dump_file == 0)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      if (dump_is_off) return 0;
+      if (dump_file == 0) return 0;
+      if (dump_header_pending()) return 0;
 
       now.type = vpiSimTime;
       vpi_get_time(0, &now);
       now64 = timerec_to_time64(&now);
 
-      if (now64> vcd_cur_time)
-            lt_set_time64(dump_file, now64);
-      vcd_cur_time = now64;
+      if (now64 > vcd_cur_time) {
+	    lt_set_time64(dump_file, now64);
+	    vcd_cur_time = now64;
+      }
 
       vcd_checkpoint();
 
@@ -396,73 +433,68 @@ static int sys_dumpall_calltf(char*name)
 
 static void *close_dumpfile(void)
 {
-lt_close(dump_file);
-return(dump_file = NULL);
+      lt_close(dump_file);
+      return (dump_file = NULL);
 }
 
-static void open_dumpfile(const char*path)
+static void open_dumpfile(vpiHandle callh)
 {
-      dump_file = lt_init(path);
+      if (dump_path == 0) dump_path = strdup("dump.lxt");
+
+      dump_file = lt_init(dump_path);
 
       if (dump_file == 0) {
-	    vpi_mcd_printf(1,
-			   "LXT Error: Unable to open %s for output.\n",
-			   path);
+	    vpi_printf("LXT Error: %s:%d: ", vpi_get_str(vpiFile, callh),
+	               (int)vpi_get(vpiLineNo, callh));
+	    vpi_printf("Unable to open %s for output.\n", dump_path);
+	    vpi_control(vpiFinish, 1);
 	    return;
       } else {
 	    int prec = vpi_get(vpiTimePrecision, 0);
 
-	    vpi_mcd_printf(1,
-			   "LXT info: dumpfile %s opened for output.\n",
-			   path);
+	    vpi_printf("LXT info: dumpfile %s opened for output.\n",
+	               dump_path);
 
 	    assert(prec >= -15);
 	    lt_set_timescale(dump_file, prec);
 
 	    lt_set_initial_value(dump_file, 'x');
-            lt_set_clock_compress(dump_file);
+	    lt_set_clock_compress(dump_file);
 
             atexit((void(*)(void))close_dumpfile);
       }
 }
 
-static int sys_dumpfile_calltf(char*name)
+static PLI_INT32 sys_dumpfile_calltf(PLI_BYTE8*name)
 {
-      char*path;
-
-      vpiHandle sys = vpi_handle(vpiSysTfCall, 0);
-      vpiHandle argv = vpi_iterate(vpiArgument, sys);
-      vpiHandle item;
-
-      if (argv && (item = vpi_scan(argv))) {
-	    s_vpi_value value;
-
-	    if (vpi_get(vpiType, item) != vpiConstant
-		|| vpi_get(vpiConstType, item) != vpiStringConst) {
-		  vpi_mcd_printf(1,
-				 "LXT Error:"
-				 " %s parameter must be a string constant\n",
-				 name);
-		  return 0;
-	    }
-
-	    value.format = vpiStringVal;
-	    vpi_get_value(item, &value);
-	    path = strdup(value.value.str);
+      vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv = vpi_iterate(vpiArgument, callh);
+      char *path;
 
+        /* $dumpfile must be called before $dumpvars starts! */
+      if (dumpvars_status != 0) {
+	    char msg [64];
+	    snprintf(msg, 64, "LXT warning: %s:%d:",
+	             vpi_get_str(vpiFile, callh),
+	             (int)vpi_get(vpiLineNo, callh));
+	    vpi_printf("%s %s called after $dumpvars started,\n", msg, name);
+	    vpi_printf("%*s using existing file (%s).\n",
+	               (int) strlen(msg), " ", dump_path);
 	    vpi_free_object(argv);
-
-      } else {
-	    path = strdup("dumpfile.lxt");
+	    return 0;
       }
 
-      if (dump_file)
-	    close_dumpfile();
+      path = get_filename(callh, name, vpi_scan(argv));
+      vpi_free_object(argv);
+      if (! path) return 0;
 
-      assert(dump_file == 0);
-      open_dumpfile(path);
-
-      free(path);
+      if (dump_path) {
+	    vpi_printf("LXT warning: %s:%d: ", vpi_get_str(vpiFile, callh),
+	               (int)vpi_get(vpiLineNo, callh));
+	    vpi_printf("Overriding dump file %s with %s.\n", dump_path, path);
+	    free(dump_path);
+      }
+      dump_path = path;
 
       return 0;
 }
@@ -470,11 +502,26 @@ static int sys_dumpfile_calltf(char*name)
 /*
  * The LXT1 format has no concept of file flushing.
  */
-static int sys_dumpflush_calltf(char*name)
+static PLI_INT32 sys_dumpflush_calltf(PLI_BYTE8*name)
 {
       return 0;
 }
 
+static PLI_INT32 sys_dumplimit_calltf(PLI_BYTE8 *name)
+{
+      vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv = vpi_iterate(vpiArgument, callh);
+      s_vpi_value val;
+
+      /* Get the value and set the dump limit. */
+      val.format = vpiIntVal;
+      vpi_get_value(vpi_scan(argv), &val);
+      dump_limit = val.value.integer;
+
+      vpi_free_object(argv);
+      return 0;
+}
+
 static void scan_item(unsigned depth, vpiHandle item, int skip)
 {
       struct t_cb_data cb;
@@ -503,21 +550,13 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
       switch (vpi_get(vpiType, item)) {
 
-	  case vpiMemory:
-	      /* don't know how to watch memories. */
-	    break;
-
-	  case vpiNamedEvent:
-	      /* There is nothing in named events to dump. */
-	    break;
-
 	  case vpiNet:  type = "wire";    if(0){
 	  case vpiIntegerVar:
+	  case vpiMemoryWord:
 	  case vpiTimeVar:
 	  case vpiReg:  type = "reg";    }
 
-	    if (skip)
-		  break;
+            if (skip || vpi_get(vpiAutomatic, item)) break;
 
 	    name = vpi_get_str(vpiName, item);
 	    nexus_id = vpi_get(_vpiNexusId, item);
@@ -532,14 +571,17 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 		  ident = strdup_sh(&name_heap, tmp);
 		  free(tmp);
 
-		  if (nexus_id)
-			set_nexus_ident(nexus_id, ident);
+		  if (nexus_id) set_nexus_ident(nexus_id, ident);
 
 		  info = malloc(sizeof(*info));
 
 		  info->time.type = vpiSimTime;
-		  info->item  = item;
-		  info->sym   = lt_symbol_add(dump_file, ident, 0 /* array rows */, vpi_get(vpiLeftRange, item), vpi_get(vpiRightRange, item), LT_SYM_F_BITS);
+		  info->item = item;
+		  info->sym  = lt_symbol_add(dump_file, ident,
+		                             0 /* array rows */,
+		                             vpi_get(vpiLeftRange, item),
+		                             vpi_get(vpiRightRange, item),
+		                             LT_SYM_F_BITS);
 		  info->scheduled = 0;
 
 		  cb.time      = &info->time;
@@ -565,8 +607,7 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
 	  case vpiRealVar:
 
-	    if (skip)
-		  break;
+            if (skip || vpi_get(vpiAutomatic, item)) break;
 
 	    name = vpi_get_str(vpiName, item);
 	    { char*tmp = create_full_name(name);
@@ -576,8 +617,11 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	    info = malloc(sizeof(*info));
 
 	    info->time.type = vpiSimTime;
-	    info->item  = item;
-	    info->sym   = lt_symbol_add(dump_file, ident, 0 /* array rows */, vpi_get(vpiSize, item)-1, 0, LT_SYM_F_DOUBLE);
+	    info->item = item;
+	    info->sym  = lt_symbol_add(dump_file, ident,
+	                               0 /* array rows */,
+	                               vpi_get(vpiSize, item)-1,
+	                               0, LT_SYM_F_DOUBLE);
 	    info->scheduled = 0;
 
 	    cb.time      = &info->time;
@@ -608,21 +652,16 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 			vpi_get_str(vpiFullName, item);
 
 #if 0
-		  vpi_mcd_printf(1,
-				 "LXT info:"
-				 " scanning scope %s, %u levels\n",
-				 fullname, depth);
+		  vpi_printf("LXT info: scanning scope %s, %u levels\n",
+		             fullname, depth);
 #endif
 		  nskip = 0 != vcd_names_search(&lxt_tab, fullname);
 
 		  if (!nskip)
 			vcd_names_add(&lxt_tab, fullname);
 		  else
-		    vpi_mcd_printf(1,
-				   "LXT warning:"
-				   " ignoring signals"
-				   " in previously scanned scope %s\n",
-				   fullname);
+		    vpi_printf("LXT warning: ignoring signals in "
+		               "previously scanned scope %s\n", fullname);
 
 		  name = vpi_get_str(vpiName, item);
 
@@ -641,9 +680,8 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	    break;
 
 	  default:
-	    vpi_mcd_printf(1,
-			   "LXT Error: $lxtdumpvars: Unsupported parameter "
-			   "type (%d)\n", vpi_get(vpiType, item));
+	    vpi_printf("LXT warning: $dumpvars: Unsupported parameter "
+	               "type (%s)\n", vpi_get_str(vpiType, item));
       }
 
 }
@@ -652,97 +690,68 @@ static int draw_scope(vpiHandle item)
 {
       int depth;
       const char *name;
-      char *type;
 
       vpiHandle scope = vpi_handle(vpiScope, item);
-      if (!scope)
-	    return 0;
+      if (!scope) return 0;
 
       depth = 1 + draw_scope(scope);
       name = vpi_get_str(vpiName, scope);
 
-      switch (vpi_get(vpiType, item)) {
-	  case vpiNamedBegin:  type = "begin";      break;
-	  case vpiTask:        type = "task";       break;
-	  case vpiFunction:    type = "function";   break;
-	  case vpiNamedFork:   type = "fork";       break;
-      	  default:             type = "module";     break;
-      }
-
-      push_scope(name);	/* keep in type info determination for possible future usage */
+      push_scope(name);
 
       return depth;
 }
 
-static int sys_dumpvars_calltf(char*name)
+static PLI_INT32 sys_dumpvars_calltf(PLI_BYTE8*name)
 {
-      unsigned depth;
+      vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv = vpi_iterate(vpiArgument, callh);
+      vpiHandle item;
       s_vpi_value value;
-      vpiHandle item = 0;
-      vpiHandle sys = vpi_handle(vpiSysTfCall, 0);
-      vpiHandle argv;
+      unsigned depth = 0;
 
       if (dump_file == 0) {
-	    open_dumpfile("dumpfile.lxt");
-	    if (dump_file == 0)
+	    open_dumpfile(callh);
+	    if (dump_file == 0) {
+		  vpi_free_object(argv);
 		  return 0;
+	    }
       }
 
       if (install_dumpvars_callback()) {
+	    vpi_free_object(argv);
 	    return 0;
       }
 
-      argv = vpi_iterate(vpiArgument, sys);
+        /* Get the depth if it exists. */
+      if (argv) {
+	    value.format = vpiIntVal;
+	    vpi_get_value(vpi_scan(argv), &value);
+	    depth = value.value.integer;
+      }
+      if (!depth) depth = 10000;
 
-      depth = 0;
-      if (argv && (item = vpi_scan(argv)))
-	    switch (vpi_get(vpiType, item)) {
-		case vpiConstant:
-		case vpiNet:
-		case vpiIntegerVar:
-		case vpiReg:
-		case vpiMemoryWord:
-		  value.format = vpiIntVal;
-		  vpi_get_value(item, &value);
-		  depth = value.value.integer;
-		  break;
-	    }
-
-      if (!depth)
-	    depth = 10000;
-
-      if (!argv) {
-	    // $dumpvars;
-	    // search for the toplevel module
-	    vpiHandle parent = vpi_handle(vpiScope, sys);
-	    while (parent) {
-		  item = parent;
-		  parent = vpi_handle(vpiScope, item);
-	    }
-
-      } else if (!item  ||  !(item = vpi_scan(argv))) {
-	    // $dumpvars(level);
-	    // $dumpvars();
-	    // dump the current scope
-	    item = vpi_handle(vpiScope, sys);
-	    argv = 0x0;
+        /* This dumps all the modules in the design if none are given. */
+      if (!argv || !(item = vpi_scan(argv))) {
+	    argv = vpi_iterate(vpiModule, 0x0);
+	    assert(argv);  /* There must be at least one top level module. */
+	    item = vpi_scan(argv);
       }
 
-      for ( ; item; item = argv ? vpi_scan(argv) : 0x0) {
+      for ( ; item; item = vpi_scan(argv)) {
 
 	    int dep = draw_scope(item);
 
 	    vcd_names_sort(&lxt_tab);
 	    scan_item(depth, item, 0);
 
-	    while (dep--) {
-		  pop_scope();
-	    }
+	    while (dep--) pop_scope();
       }
 
 	/* Most effective compression. */
-      if (lxm_optimum_mode == LXM_SPACE)
+      if (lxm_optimum_mode == LXM_SPACE) {
 	    lt_set_no_interlace(dump_file);
+      }
 
       return 0;
 }
@@ -768,34 +777,20 @@ void sys_lxt_register()
 	    }
       }
 
+      /* All the compiletf routines are located in vcd_priv.c. */
+
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpall";
       tf_data.calltf    = sys_dumpall_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_no_arg_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpall";
       vpi_register_systf(&tf_data);
 
-      tf_data.type      = vpiSysTask;
-      tf_data.tfname    = "$dumpoff";
-      tf_data.calltf    = sys_dumpoff_calltf;
-      tf_data.compiletf = 0;
-      tf_data.sizetf    = 0;
-      tf_data.user_data = "$dumpoff";
-      vpi_register_systf(&tf_data);
-
-      tf_data.type      = vpiSysTask;
-      tf_data.tfname    = "$dumpon";
-      tf_data.calltf    = sys_dumpon_calltf;
-      tf_data.compiletf = 0;
-      tf_data.sizetf    = 0;
-      tf_data.user_data = "$dumpon";
-      vpi_register_systf(&tf_data);
-
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpfile";
       tf_data.calltf    = sys_dumpfile_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_one_string_arg_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpfile";
       vpi_register_systf(&tf_data);
@@ -803,112 +798,40 @@ void sys_lxt_register()
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpflush";
       tf_data.calltf    = sys_dumpflush_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_no_arg_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpflush";
       vpi_register_systf(&tf_data);
 
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumplimit";
+      tf_data.calltf    = sys_dumplimit_calltf;
+      tf_data.compiletf = sys_one_numeric_arg_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumplimit";
+      vpi_register_systf(&tf_data);
+
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumpoff";
+      tf_data.calltf    = sys_dumpoff_calltf;
+      tf_data.compiletf = sys_no_arg_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumpoff";
+      vpi_register_systf(&tf_data);
+
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumpon";
+      tf_data.calltf    = sys_dumpon_calltf;
+      tf_data.compiletf = sys_no_arg_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumpon";
+      vpi_register_systf(&tf_data);
+
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpvars";
       tf_data.calltf    = sys_dumpvars_calltf;
-      tf_data.compiletf = sys_vcd_dumpvars_compiletf;
+      tf_data.compiletf = sys_dumpvars_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpvars";
       vpi_register_systf(&tf_data);
 }
-
-/*
- * $Log: sys_lxt.c,v $
- * Revision 1.26  2004/10/04 01:10:58  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.25  2004/02/15 20:46:01  steve
- *  Add the $dumpflush function
- *
- * Revision 1.24  2004/01/21 01:22:53  steve
- *  Give the vip directory its own configure and vpi_config.h
- *
- * Revision 1.23  2003/09/30 01:33:39  steve
- *  dumpers must be aware of 64bit time.
- *
- * Revision 1.22  2003/08/22 23:14:27  steve
- *  Preserve variable ranges all the way to the vpi.
- *
- * Revision 1.21  2003/05/15 16:51:09  steve
- *  Arrange for mcd id=00_00_00_01 to go to stdout
- *  as well as a user specified log file, set log
- *  file to buffer lines.
- *
- *  Add vpi_flush function, and clear up some cunfused
- *  return codes from other vpi functions.
- *
- *  Adjust $display and vcd/lxt messages to use the
- *  standard output/log file.
- *
- * Revision 1.20  2003/04/28 01:03:11  steve
- *  Fix stringheap list management failure.
- *
- * Revision 1.19  2003/04/27 02:22:27  steve
- *  Capture VCD dump value in the rosync time period.
- *
- * Revision 1.18  2003/02/21 01:36:25  steve
- *  Move dumpon/dumpoff around to the right times.
- *
- * Revision 1.17  2003/02/20 00:50:06  steve
- *  Update lxt_write implementation, and add compression control flags.
- *
- * Revision 1.16  2003/02/13 18:13:28  steve
- *  Make lxt use stringheap to perm-allocate strings.
- *
- * Revision 1.15  2003/02/12 05:28:01  steve
- *  Set dumpoff of real variables to NaN.
- *
- * Revision 1.14  2003/02/11 05:21:33  steve
- *  Support dump of vpiRealVar objects.
- *
- * Revision 1.13  2002/12/21 00:55:58  steve
- *  The $time system task returns the integer time
- *  scaled to the local units. Change the internal
- *  implementation of vpiSystemTime the $time functions
- *  to properly account for this. Also add $simtime
- *  to get the simulation time.
- *
- * Revision 1.12  2002/11/17 22:28:42  steve
- *  Close old file if $dumpfile is called again.
- *
- * Revision 1.11  2002/08/15 02:12:20  steve
- *  add dumpvars_compiletf to check first argument.
- *
- * Revision 1.10  2002/08/12 01:35:04  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.9  2002/07/17 05:13:43  steve
- *  Implementation of vpi_handle_by_name, and
- *  add the vpiVariables iterator.
- *
- * Revision 1.8  2002/07/15 03:57:30  steve
- *  Fix dangling pointer in pop_scope.
- *
- * Revision 1.7  2002/07/12 17:09:21  steve
- *  Remember to scan IntegerVars.
- *
- * Revision 1.6  2002/07/12 17:08:13  steve
- *  Eliminate use of vpiInternalScope.
- *
- * Revision 1.5  2002/06/21 04:59:36  steve
- *  Carry integerness throughout the compilation.
- *
- * Revision 1.4  2002/06/03 03:56:06  steve
- *  Ignore memories and named events.
- *
- * Revision 1.3  2002/04/06 21:33:29  steve
- *  allow runtime selection of VCD vs LXT.
- *
- * Revision 1.2  2002/04/06 20:25:45  steve
- *  cbValueChange automatically replays.
- *
- * Revision 1.1  2002/03/09 21:54:49  steve
- *  Add LXT dumper support. (Anthony Bybell)
- *
- */
-
