@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2003 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,9 +16,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: sys_vcd.c,v 1.56 2004/10/04 01:10:58 steve Exp $"
-#endif
 
 # include "sys_priv.h"
 
@@ -83,24 +80,22 @@ static struct vcd_info *vcd_list = 0;
 static struct vcd_info *vcd_dmp_list = 0;
 PLI_UINT64 vcd_cur_time = 0;
 static int dump_is_off = 0;
+static long dump_limit = 0;
+static int dump_is_full = 0;
+static int finish_status = 0;
 
 static char *truncate_bitvec(char *s)
 {
       char l, r;
 
       r=*s;
-      if(r=='1')
-	    return s;
-      else
-	    s += 1;
+      if(r=='1') return s;
+      else s += 1;
 
       for(;;s++) {
 	    l=r; r=*s;
 	    if(!r) return (s-1);
-
-	    if(l!=r)
-		  return(((l=='0')&&(r=='1'))?s:s-1);
-
+	    if(l!=r) return(((l=='0')&&(r=='1'))?s:s-1);
       }
 }
 
@@ -120,8 +115,7 @@ static void show_this_item(struct vcd_info*info)
       } else {
 	    value.format = vpiBinStrVal;
 	    vpi_get_value(info->item, &value);
-	    fprintf(dump_file, "b%s %s\n",
-		    truncate_bitvec(value.value.str),
+	    fprintf(dump_file, "b%s %s\n", truncate_bitvec(value.value.str),
 		    info->ident);
       }
 }
@@ -141,15 +135,16 @@ static void show_this_item_x(struct vcd_info*info)
 
 
 /*
- * managed qsorted list of scope names for duplicates bsearching
+ * managed qsorted list of scope names/variables for duplicates bsearching
  */
 
 struct vcd_names_list_s vcd_tab = { 0 };
+struct vcd_names_list_s vcd_var = { 0 };
 
 
 static int dumpvars_status = 0; /* 0:fresh 1:cb installed, 2:callback done */
 static PLI_UINT64 dumpvars_time;
-inline static int dump_header_pending(void)
+__inline__ static int dump_header_pending(void)
 {
       return dumpvars_status != 2;
 }
@@ -174,7 +169,7 @@ static void vcd_checkpoint_x()
 	    show_this_item_x(cur);
 }
 
-static int variable_cb_2(p_cb_data cause)
+static PLI_INT32 variable_cb_2(p_cb_data cause)
 {
       struct vcd_info* info = vcd_dmp_list;
       PLI_UINT64 now = timerec_to_time64(cause->time);
@@ -194,14 +189,24 @@ static int variable_cb_2(p_cb_data cause)
       return 0;
 }
 
-static int variable_cb_1(p_cb_data cause)
+static PLI_INT32 variable_cb_1(p_cb_data cause)
 {
       struct t_cb_data cb;
       struct vcd_info*info = (struct vcd_info*)cause->user_data;
 
-      if (dump_is_off) 		 return 0;
+      if (dump_is_full) return 0;
+      if (dump_is_off) return 0;
       if (dump_header_pending()) return 0;
-      if (info->scheduled)       return 0;
+      if (info->scheduled) return 0;
+
+      if ((dump_limit > 0) && (ftell(dump_file) > dump_limit)) {
+            dump_is_full = 1;
+            vpi_printf("WARNING: Dump file limit (%ld bytes) "
+                               "exceeded.\n", dump_limit);
+            fprintf(dump_file, "$comment Dump file limit (%ld bytes) "
+                               "exceeded. $end\n", dump_limit);
+            return 0;
+      }
 
       if (!vcd_dmp_list) {
           cb = *cause;
@@ -217,10 +222,9 @@ static int variable_cb_1(p_cb_data cause)
       return 0;
 }
 
-static int dumpvars_cb(p_cb_data cause)
+static PLI_INT32 dumpvars_cb(p_cb_data cause)
 {
-      if (dumpvars_status != 1)
-	    return 0;
+      if (dumpvars_status != 1) return 0;
 
       dumpvars_status = 2;
 
@@ -239,24 +243,53 @@ static int dumpvars_cb(p_cb_data cause)
       return 0;
 }
 
-inline static int install_dumpvars_callback(void)
+static PLI_INT32 finish_cb(p_cb_data cause)
+{
+      struct vcd_info *cur, *next;
+
+      if (finish_status != 0) return 0;
+
+      finish_status = 1;
+
+      dumpvars_time = timerec_to_time64(cause->time);
+
+      if (!dump_is_off && !dump_is_full && dumpvars_time != vcd_cur_time) {
+	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", dumpvars_time);
+      }
+
+      fclose(dump_file);
+
+      for (cur = vcd_list ;  cur ;  cur = next) {
+	    next = cur->next;
+	    free((char *)cur->ident);
+	    free(cur);
+      }
+      vcd_list = 0;
+      vcd_names_delete(&vcd_tab);
+      vcd_names_delete(&vcd_var);
+      nexus_ident_delete();
+      free(dump_path);
+      dump_path = 0;
+
+      return 0;
+}
+
+__inline__ static int install_dumpvars_callback(void)
 {
       struct t_cb_data cb;
-      static struct t_vpi_time time;
+      static struct t_vpi_time now;
 
-      if (dumpvars_status == 1)
-	    return 0;
+      if (dumpvars_status == 1) return 0;
 
       if (dumpvars_status == 2) {
-	    vpi_mcd_printf(1, "VCD Error:"
-			   " $dumpvars ignored,"
-			   " previously called at simtime %" PLI_UINT64_FMT "\n",
-			   dumpvars_time);
+	    vpi_printf("VCD warning: $dumpvars ignored, previously"
+	               " called at simtime %" PLI_UINT64_FMT "\n",
+	               dumpvars_time);
 	    return 1;
       }
 
-      time.type = vpiSimTime;
-      cb.time = &time;
+      now.type = vpiSimTime;
+      cb.time = &now;
       cb.reason = cbReadOnlySynch;
       cb.cb_rtn = dumpvars_cb;
       cb.user_data = 0x0;
@@ -264,33 +297,35 @@ inline static int install_dumpvars_callback(void)
 
       vpi_register_cb(&cb);
 
+      cb.reason = cbEndOfSimulation;
+      cb.cb_rtn = finish_cb;
+
+      vpi_register_cb(&cb);
+
       dumpvars_status = 1;
       return 0;
 }
 
-static int sys_dumpoff_calltf(char*name)
+static PLI_INT32 sys_dumpoff_calltf(PLI_BYTE8*name)
 {
       s_vpi_time now;
       PLI_UINT64 now64;
 
-      if (dump_is_off)
-	    return 0;
+      if (dump_is_off) return 0;
 
       dump_is_off = 1;
 
-      if (dump_file == 0)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      if (dump_file == 0) return 0;
+      if (dump_header_pending()) return 0;
 
       now.type = vpiSimTime;
       vpi_get_time(0, &now);
       now64 = timerec_to_time64(&now);
 
-      if (now64 > vcd_cur_time)
+      if (now64 > vcd_cur_time) {
 	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now64);
-      vcd_cur_time = now64;
+	    vcd_cur_time = now64;
+      }
 
       fprintf(dump_file, "$dumpoff\n");
       vcd_checkpoint_x();
@@ -299,29 +334,26 @@ static int sys_dumpoff_calltf(char*name)
       return 0;
 }
 
-static int sys_dumpon_calltf(char*name)
+static PLI_INT32 sys_dumpon_calltf(PLI_BYTE8*name)
 {
       s_vpi_time now;
       PLI_UINT64 now64;
 
-      if (!dump_is_off)
-	    return 0;
+      if (!dump_is_off) return 0;
 
       dump_is_off = 0;
 
-      if (dump_file == 0)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      if (dump_file == 0) return 0;
+      if (dump_header_pending()) return 0;
 
       now.type = vpiSimTime;
       vpi_get_time(0, &now);
       now64 = timerec_to_time64(&now);
 
-      if (now64 > vcd_cur_time)
+      if (now64 > vcd_cur_time) {
 	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now64);
-      vcd_cur_time = now64;
+	    vcd_cur_time = now64;
+      }
 
       fprintf(dump_file, "$dumpon\n");
       vcd_checkpoint();
@@ -330,24 +362,23 @@ static int sys_dumpon_calltf(char*name)
       return 0;
 }
 
-static int sys_dumpall_calltf(char*name)
+static PLI_INT32 sys_dumpall_calltf(PLI_BYTE8*name)
 {
       s_vpi_time now;
       PLI_UINT64 now64;
 
-      if (dump_file == 0)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      if (dump_is_off) return 0;
+      if (dump_file == 0) return 0;
+      if (dump_header_pending()) return 0;
 
       now.type = vpiSimTime;
       vpi_get_time(0, &now);
       now64 = timerec_to_time64(&now);
 
-      if (now64 > vcd_cur_time)
+      if (now64 > vcd_cur_time) {
 	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now64);
-      vcd_cur_time = now.low;
+	    vcd_cur_time = now64;
+      }
 
       fprintf(dump_file, "$dumpall\n");
       vcd_checkpoint();
@@ -356,18 +387,17 @@ static int sys_dumpall_calltf(char*name)
       return 0;
 }
 
-static void open_dumpfile(void)
+static void open_dumpfile(vpiHandle callh)
 {
-      if (dump_path == 0) {
-	    dump_path = strdup("dump.vcd");
-      }
+      if (dump_path == 0) dump_path = strdup("dump.vcd");
 
       dump_file = fopen(dump_path, "w");
 
       if (dump_file == 0) {
-	    vpi_mcd_printf(1,
-			   "VCD Error: Unable to open %s for output.\n",
-			   dump_path);
+	    vpi_printf("VCD Error: %s:%d: ", vpi_get_str(vpiFile, callh),
+	               (int)vpi_get(vpiLineNo, callh));
+	    vpi_printf("Unable to open %s for output.\n", dump_path);
+	    vpi_control(vpiFinish, 1);
 	    return;
       } else {
 	    int prec = vpi_get(vpiTimePrecision, 0);
@@ -375,9 +405,8 @@ static void open_dumpfile(void)
 	    unsigned udx = 0;
 	    time_t walltime;
 
-	    vpi_mcd_printf(1,
-			   "VCD info: dumpfile %s opened for output.\n",
-			   dump_path);
+	    vpi_printf("VCD info: dumpfile %s opened for output.\n",
+	               dump_path);
 
 	    time(&walltime);
 
@@ -403,58 +432,60 @@ static void open_dumpfile(void)
       }
 }
 
-static int sys_dumpfile_compiletf(char*name)
+static PLI_INT32 sys_dumpfile_calltf(PLI_BYTE8*name)
 {
-      vpiHandle sys = vpi_handle(vpiSysTfCall, 0);
-      vpiHandle argv = vpi_iterate(vpiArgument, sys);
-      vpiHandle item;
-
-      char*path;
-
-      if (argv && (item = vpi_scan(argv))) {
-	    s_vpi_value value;
-
-	    if (vpi_get(vpiType, item) != vpiConstant
-		|| vpi_get(vpiConstType, item) != vpiStringConst) {
-		  vpi_mcd_printf(1,
-				 "VCD Error:"
-				 " %s parameter must be a string constant\n",
-				 name);
-		  return 0;
-	    }
-
-	    value.format = vpiStringVal;
-	    vpi_get_value(item, &value);
-	    path = strdup(value.value.str);
+      vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv = vpi_iterate(vpiArgument, callh);
+      char *path;
 
+        /* $dumpfile must be called before $dumpvars starts! */
+      if (dumpvars_status != 0) {
+	    char msg [64];
+	    snprintf(msg, 64, "VCD warning: %s:%d:",
+	             vpi_get_str(vpiFile, callh),
+	             (int)vpi_get(vpiLineNo, callh));
+	    vpi_printf("%s %s called after $dumpvars started,\n", msg, name);
+	    vpi_printf("%*s using existing file (%s).\n",
+	               (int) strlen(msg), " ", dump_path);
 	    vpi_free_object(argv);
-
-      } else {
-	    path = strdup("dump.vcd");
+	    return 0;
       }
 
+      path = get_filename(callh, name, vpi_scan(argv));
+      vpi_free_object(argv);
+      if (! path) return 0;
+
       if (dump_path) {
-	    vpi_mcd_printf(1, "VCD Warning:"
-			   " Overriding dumpfile path %s with %s\n",
-			   dump_path, path);
+	    vpi_printf("VCD warning: %s:%d: ", vpi_get_str(vpiFile, callh),
+	               (int)vpi_get(vpiLineNo, callh));
+	    vpi_printf("Overriding dump file %s with %s.\n", dump_path, path);
 	    free(dump_path);
       }
-
       dump_path = path;
 
       return 0;
 }
 
-static int sys_dumpfile_calltf(char*name)
+static PLI_INT32 sys_dumpflush_calltf(PLI_BYTE8*name)
 {
+      if (dump_file) fflush(dump_file);
+
       return 0;
 }
 
-static int sys_dumpflush_calltf(char*name)
+static PLI_INT32 sys_dumplimit_calltf(PLI_BYTE8 *name)
 {
-      if (dump_file)
-	    fflush(dump_file);
+      vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv = vpi_iterate(vpiArgument, callh);
+      s_vpi_value val;
 
+      /* Get the value and set the dump limit. */
+      assert(argv);
+      val.format = vpiIntVal;
+      vpi_get_value(vpi_scan(argv), &val);
+      dump_limit = val.value.integer;
+
+      vpi_free_object(argv);
       return 0;
 }
 
@@ -465,6 +496,7 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
       const char* type;
       const char* name;
+      const char* prefix;
       const char* ident;
       int nexus_id;
 
@@ -486,23 +518,24 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
       switch (vpi_get(vpiType, item)) {
 
-	  case vpiMemory:
-	      /* don't know how to watch memories. */
-	    break;
-
-	  case vpiNamedEvent:
-	      /* There is nothing in named events to dump. */
-	    break;
-
 	  case vpiNet:  type = "wire";    if(0){
 	  case vpiIntegerVar:
+	  case vpiMemoryWord:
 	  case vpiTimeVar:
 	  case vpiReg:  type = "reg";    }
 
-	    if (skip)
+	      /* Skip this signal if it has already been included. */
+	    if (vcd_names_search(&vcd_var, vpi_get_str(vpiFullName, item))) {
+		  vpi_printf("VCD warning: skipping signal %s, it was "
+		             "previously included.\n",
+		             vpi_get_str(vpiFullName, item));
 		  break;
+	    }
+
+            if (skip || vpi_get(vpiAutomatic, item)) break;
 
 	    name = vpi_get_str(vpiName, item);
+	    prefix = is_escaped_id(name) ? "\\" : "";
 
 	    nexus_id = vpi_get(_vpiNexusId, item);
 
@@ -516,8 +549,7 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 		  ident = strdup(vcdid);
 		  gen_new_vcd_id();
 
-		  if (nexus_id)
-			set_nexus_ident(nexus_id, ident);
+		  if (nexus_id) set_nexus_ident(nexus_id, ident);
 
 		  info = malloc(sizeof(*info));
 
@@ -541,15 +573,15 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 		  info->cb    = vpi_register_cb(&cb);
 	    }
 
-	    fprintf(dump_file, "$var %s %u %s %s",
+	    fprintf(dump_file, "$var %s %u %s %s%s",
 		    type, vpi_get(vpiSize, item), ident,
-		    name);
+		    prefix, name);
       /* FIXME
 	    if (vpi_get(vpiVector, item)
       */
 	    if (vpi_get(vpiSize, item) > 1
 		|| vpi_get(vpiLeftRange, item) != 0) {
-		  fprintf(dump_file, "[%i:%i]", vpi_get(vpiLeftRange, item),
+		  fprintf(dump_file, " [%i:%i]", vpi_get(vpiLeftRange, item),
 			  vpi_get(vpiRightRange, item));
 	    }
 	    fprintf(dump_file, " $end\n");
@@ -557,15 +589,23 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
 	  case vpiRealVar:
 
-	    if (skip)
+	      /* Skip this signal if it has already been included. */
+	    if (vcd_names_search(&vcd_var, vpi_get_str(vpiFullName, item))) {
+		  vpi_printf("VCD warning: skipping signal %s, it was "
+		             "previously included.\n",
+		             vpi_get_str(vpiFullName, item));
 		  break;
+	    }
+
+            if (skip || vpi_get(vpiAutomatic, item)) break;
 
 	      /* Declare the variable in the VCD file. */
 	    name = vpi_get_str(vpiName, item);
+	    prefix = is_escaped_id(name) ? "\\" : "";
 	    ident = strdup(vcdid);
 	    gen_new_vcd_id();
-	    fprintf(dump_file, "$var real 1 %s %s $end\n",
-		    ident, name);
+	    fprintf(dump_file, "$var real 1 %s %s%s $end\n",
+		    ident, prefix, name);
 
 	      /* Add a callback for the variable. */
 	    info = malloc(sizeof(*info));
@@ -604,21 +644,16 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 			vpi_get_str(vpiFullName, item);
 
 #if 0
-		  vpi_mcd_printf(1,
-				 "VCD info:"
-				 " scanning scope %s, %u levels\n",
-				 fullname, depth);
+		  vpi_printf("VCD info: scanning scope %s, %u levels\n",
+		             fullname, depth);
 #endif
 		  nskip = 0 != vcd_names_search(&vcd_tab, fullname);
 
 		  if (!nskip)
 			vcd_names_add(&vcd_tab, fullname);
 		  else
-		    vpi_mcd_printf(1,
-				   "VCD warning:"
-				   " ignoring signals"
-				   " in previously scanned scope %s\n",
-				   fullname);
+		    vpi_printf("VCD warning: ignoring signals in "
+		               "previously scanned scope %s\n", fullname);
 
 		  name = vpi_get_str(vpiName, item);
 
@@ -637,31 +672,34 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	    break;
 
 	  default:
-	    vpi_mcd_printf(1,
-			   "VCD Error: $dumpvars: Unsupported parameter "
-			   "type (%d)\n", vpi_get(vpiType, item));
+	    vpi_printf("VCD warning: $dumpvars: Unsupported parameter "
+	               "type (%s)\n", vpi_get_str(vpiType, item));
       }
 }
 
-static int draw_scope(vpiHandle item)
+static int draw_scope(vpiHandle item, vpiHandle callh)
 {
       int depth;
       const char *name;
       char *type;
 
       vpiHandle scope = vpi_handle(vpiScope, item);
-      if (!scope)
-	    return 0;
+      if (!scope) return 0;
 
-      depth = 1 + draw_scope(scope);
+      depth = 1 + draw_scope(scope, callh);
       name = vpi_get_str(vpiName, scope);
 
-      switch (vpi_get(vpiType, item)) {
+      switch (vpi_get(vpiType, scope)) {
 	  case vpiNamedBegin:  type = "begin";      break;
 	  case vpiTask:        type = "task";       break;
 	  case vpiFunction:    type = "function";   break;
 	  case vpiNamedFork:   type = "fork";       break;
-      	  default:             type = "module";     break;
+	  case vpiModule:      type = "module";     break;
+	  default:
+	    vpi_printf("VCD Error: %s:%d: $dumpvars: Unsupported scope "
+	               "type (%d)\n", vpi_get_str(vpiFile, callh),
+	               (int)vpi_get(vpiLineNo, callh), vpi_get(vpiType, item));
+            assert(0);
       }
 
       fprintf(dump_file, "$scope %s %s $end\n", type, name);
@@ -669,111 +707,81 @@ static int draw_scope(vpiHandle item)
       return depth;
 }
 
-/*
- * This function is also used in sys_lxt to check the arguments of the
- * lxt variant of $dumpvars.
- */
-int sys_vcd_dumpvars_compiletf(char*name)
+static PLI_INT32 sys_dumpvars_calltf(PLI_BYTE8*name)
 {
-      vpiHandle sys   = vpi_handle(vpiSysTfCall, 0);
-      vpiHandle argv  = vpi_iterate(vpiArgument, sys);
-      vpiHandle tmp;
-
-      if (argv == 0)
-	    return 0;
-
-      tmp = vpi_scan(argv);
-      assert(tmp);
-
-      switch (vpi_get(vpiType, tmp)) {
-	  case vpiConstant:
-	    if (vpi_get(vpiConstType, tmp) == vpiStringConst) {
-		  vpi_printf("ERROR: %s argument must be "
-			     "a number constant.\n", name);
-		  vpi_control(vpiFinish, 1);
-	    }
-	    break;
-
-	  case vpiNet:
-	  case vpiReg:
-	  case vpiIntegerVar:
-	  case vpiMemoryWord:
-	    break;
-
-	  default:
-	    vpi_printf("ERROR: %s argument must be "
-		       "a number constant.\n", name);
-	    vpi_control(vpiFinish, 1);
-	    break;
-      }
-
-      vpi_free_object(argv);
-      return 0;
-}
-
-static int sys_dumpvars_calltf(char*name)
-{
-      unsigned depth;
+      vpiHandle callh = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv = vpi_iterate(vpiArgument, callh);
+      vpiHandle item;
       s_vpi_value value;
-      vpiHandle item = 0;
-      vpiHandle sys = vpi_handle(vpiSysTfCall, 0);
-      vpiHandle argv;
+      unsigned depth = 0;
 
       if (dump_file == 0) {
-	    open_dumpfile();
-	    if (dump_file == 0)
+	    open_dumpfile(callh);
+	    if (dump_file == 0) {
+		  vpi_free_object(argv);
 		  return 0;
+	    }
       }
 
       if (install_dumpvars_callback()) {
+	    vpi_free_object(argv);
 	    return 0;
       }
 
-      argv = vpi_iterate(vpiArgument, sys);
+        /* Get the depth if it exists. */
+      if (argv) {
+	    value.format = vpiIntVal;
+	    vpi_get_value(vpi_scan(argv), &value);
+	    depth = value.value.integer;
+      }
+      if (!depth) depth = 10000;
 
-      depth = 0;
-      if (argv && (item = vpi_scan(argv)))
-	    switch (vpi_get(vpiType, item)) {
-		case vpiConstant:
-		case vpiNet:
-		case vpiReg:
-		case vpiIntegerVar:
-		case vpiMemoryWord:
-		  value.format = vpiIntVal;
-		  vpi_get_value(item, &value);
-		  depth = value.value.integer;
-		  break;
-	    }
-
-      if (!depth)
-	    depth = 10000;
-
-      if (!argv) {
-	    // $dumpvars;
-	    // search for the toplevel module
-	    vpiHandle parent = vpi_handle(vpiScope, sys);
-	    while (parent) {
-		  item = parent;
-		  parent = vpi_handle(vpiScope, item);
-	    }
-
-      } else if (!item  ||  !(item = vpi_scan(argv))) {
-	    // $dumpvars(level);
-	    // $dumpvars();
-	    // dump the current scope
-	    item = vpi_handle(vpiScope, sys);
-	    argv = 0x0;
+        /* This dumps all the modules in the design if none are given. */
+      if (!argv || !(item = vpi_scan(argv))) {
+	    argv = vpi_iterate(vpiModule, 0x0);
+	    assert(argv);  /* There must be at least one top level module. */
+	    item = vpi_scan(argv);
       }
 
-      for ( ; item; item = argv ? vpi_scan(argv) : 0x0) {
-
-	    int dep = draw_scope(item);
+      for ( ; item; item = vpi_scan(argv)) {
+	    const char *scname;
+	    int add_var = 0;
+	    int dep;
 
 	    vcd_names_sort(&vcd_tab);
+
+	      /* If this is a signal make sure it has not already
+	       * been included. */
+	    switch (vpi_get(vpiType, item)) {
+	        case vpiIntegerVar:
+	        case vpiMemoryWord:
+	        case vpiNet:
+	        case vpiRealVar:
+	        case vpiReg:
+	        case vpiTimeVar:
+		  scname = vpi_get_str(vpiFullName, vpi_handle(vpiScope, item));
+		  if (vcd_names_search(&vcd_tab, scname)) {
+		        vpi_printf("VCD warning: skipping signal %s, "
+		                   "it was previously included.\n",
+		                   vpi_get_str(vpiFullName, item));
+		        continue;
+		  } else {
+		        add_var = 1;
+		  }
+	    }
+
+	    dep = draw_scope(item, callh);
+
 	    scan_item(depth, item, 0);
 
-	    while (dep--) {
-		  fprintf(dump_file, "$upscope $end\n");
+	    while (dep--) fprintf(dump_file, "$upscope $end\n");
+
+	      /* Add this signal to the variable list so we can verify it
+	       * is not included twice. This must be done after it has
+	       * been added */
+	    if (add_var) {
+		  vcd_names_add(&vcd_var, vpi_get_str(vpiFullName, item));
+		  vcd_names_sort(&vcd_var);
 	    }
       }
 
@@ -784,34 +792,20 @@ void sys_vcd_register()
 {
       s_vpi_systf_data tf_data;
 
+      /* All the compiletf routines are located in vcd_priv.c. */
+
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpall";
       tf_data.calltf    = sys_dumpall_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_no_arg_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpall";
       vpi_register_systf(&tf_data);
 
-      tf_data.type      = vpiSysTask;
-      tf_data.tfname    = "$dumpoff";
-      tf_data.calltf    = sys_dumpoff_calltf;
-      tf_data.compiletf = 0;
-      tf_data.sizetf    = 0;
-      tf_data.user_data = "$dumpoff";
-      vpi_register_systf(&tf_data);
-
-      tf_data.type      = vpiSysTask;
-      tf_data.tfname    = "$dumpon";
-      tf_data.calltf    = sys_dumpon_calltf;
-      tf_data.compiletf = 0;
-      tf_data.sizetf    = 0;
-      tf_data.user_data = "$dumpon";
-      vpi_register_systf(&tf_data);
-
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpfile";
       tf_data.calltf    = sys_dumpfile_calltf;
-      tf_data.compiletf = sys_dumpfile_compiletf;
+      tf_data.compiletf = sys_one_string_arg_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpfile";
       vpi_register_systf(&tf_data);
@@ -819,56 +813,40 @@ void sys_vcd_register()
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpflush";
       tf_data.calltf    = sys_dumpflush_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_no_arg_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpflush";
       vpi_register_systf(&tf_data);
 
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumplimit";
+      tf_data.calltf    = sys_dumplimit_calltf;
+      tf_data.compiletf = sys_one_numeric_arg_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumplimit";
+      vpi_register_systf(&tf_data);
+
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumpoff";
+      tf_data.calltf    = sys_dumpoff_calltf;
+      tf_data.compiletf = sys_no_arg_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumpoff";
+      vpi_register_systf(&tf_data);
+
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumpon";
+      tf_data.calltf    = sys_dumpon_calltf;
+      tf_data.compiletf = sys_no_arg_compiletf;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumpon";
+      vpi_register_systf(&tf_data);
+
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpvars";
       tf_data.calltf    = sys_dumpvars_calltf;
-      tf_data.compiletf = sys_vcd_dumpvars_compiletf;
+      tf_data.compiletf = sys_dumpvars_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpvars";
       vpi_register_systf(&tf_data);
 }
-
-/*
- * $Log: sys_vcd.c,v $
- * Revision 1.56  2004/10/04 01:10:58  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.55  2004/02/15 20:46:01  steve
- *  Add the $dumpflush function
- *
- * Revision 1.54  2004/02/15 03:17:15  steve
- *  dumpfile selects file at compiletf time.
- *
- * Revision 1.53  2004/01/21 01:22:53  steve
- *  Give the vip directory its own configure and vpi_config.h
- *
- * Revision 1.52  2003/12/19 01:27:10  steve
- *  Fix various unsigned compare warnings.
- *
- * Revision 1.51  2003/10/29 03:28:27  steve
- *  Add the PLU_UINT64_FMT string for formatting output.
- *
- * Revision 1.50  2003/10/29 03:23:12  steve
- *  Portably handle time format of VCD prints.
- *
- * Revision 1.49  2003/10/08 23:14:29  steve
- *  TIME_FMT does not include the % character.
- *
- * Revision 1.48  2003/10/02 21:30:06  steve
- *  Use configured TIME_FMT in vcd dump printf.
- *
- * Revision 1.47  2003/09/30 01:33:39  steve
- *  dumpers must be aware of 64bit time.
- *
- * Revision 1.46  2003/08/22 23:14:27  steve
- *  Preserve variable ranges all the way to the vpi.
- *
- * Revision 1.45  2003/08/06 18:24:55  steve
- *  Fix error truncating bitvec in output.
- */
-
