@@ -1,7 +1,7 @@
 #ifndef __vpi_user_H
 #define __vpi_user_H
 /*
- * Copyright (c) 1999 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,9 +18,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_user.h,v 1.36 2004/10/04 01:10:56 steve Exp $"
-#endif
 
 
 #if defined(__MINGW32__) || defined (__CYGWIN32__)
@@ -58,10 +55,10 @@ typedef struct t_vpi_systf_data {
       PLI_INT32 type;
       PLI_INT32 sysfunctype;
       const char *tfname;
-      PLI_INT32 (*calltf)(char*);
-      PLI_INT32 (*compiletf)(char*);
-      PLI_INT32 (*sizetf)(char*);
-      char      *user_data;
+      PLI_INT32 (*calltf)   (PLI_BYTE8*);
+      PLI_INT32 (*compiletf)(PLI_BYTE8*);
+      PLI_INT32 (*sizetf)   (PLI_BYTE8*);
+      PLI_BYTE8 *user_data;
 } s_vpi_systf_data, *p_vpi_systf_data;
 
 /* The type in the above structure can have one of the following
@@ -79,6 +76,15 @@ typedef struct t_vpi_vlog_info
 
 
 typedef struct t_vpi_time {
+
+      /*
+	Type can be :
+
+	vpiScaledRealTime == 1
+	vpiSimTime        == 2
+	vpiSuppressTime   == 3
+      */
+
       PLI_INT32 type;
       PLI_UINT32 high;
       PLI_UINT32 low;
@@ -116,6 +122,114 @@ typedef struct t_vpi_value {
       } value;
 } s_vpi_value, *p_vpi_value;
 
+
+/*
+
+  Conform the IEEE 1364, We add the
+  Standard vpi_delay structure to
+  enable the modpath delay values
+
+
+  Conform IEEE 1364, Pg 670 :
+
+  The "da" field of the s_vpi_delay
+  structure shall be a user allocated
+  array of "s_vpi_time" structure
+
+  The array shall store delay values returned
+  by vpi_get_delay(). The number of elements in
+  the array shall be determined by
+
+  (1) The number of delays to be retrieved
+      ( normally this is used in vpi_get_delays (..) )
+  {
+    (1.1) Set by "no_of_delays" field
+
+    (1.2) For the primitive_object, the no_of_delays
+        shall be 2 or 3
+
+    (1.3) For path_delay object the no_of_delays shall
+        be 1,2,3,6, 12
+
+    (1.4) For timing_check_object, the no_of_delays shall
+        be match the number of limits existing in the
+	Time Check
+
+    (1.5) For intermodule_path object, the no_of_delays shall
+        be 2 or 3
+  }
+
+
+
+
+  (2) The "mtm_flag" && "pulsere_flag"
+
+
+  Normally, if you set mtm = X, pulsere = Y
+  then, you will need allocate (num * no_of_delay)
+  s_vpi_time elements for 'da' array before calling
+  the vpi_get/put_delays (..)
+
+  ---------------------------------------------------------------------------
+  |                |                         |                              |
+  | mtm_flag       | No of s_vpi_time array  |   order in which delay       |
+  | pulsere_flag   | element required by the |   elements shall be filed    |
+  |                | s_vpi_delay->da         |                              |
+  |                |                         |                              |
+  |----------------|-------------------------|------------------------------|
+  |                |                         | 1o delay  da[0]--> 1o delay  |
+  | mtm = false    | no_of_delay             | 2o delay  da[1]--> 2o delay  |
+  | pulere = false |                         |                              |
+  |                |                         |                              |
+  |----------------|-------------------------|------------------------------|
+  |                |                         | 1o delay  da[0]--> min delay |
+  | mtm = true     |                         |           da[1]--> typ delay |
+  | pulere = false | 3*no_of_delay           |           da[2]--> max delay |
+  |                |                         | 2o delay  da[3]--> min delay |
+  |                |                         |           da[4]--> typ delay |
+  |                |                         |           ....               |
+  |----------------|-------------------------|------------------------------|
+  |                |                         | 1o delay  da[0]--> delay     |
+  | mtm = false    |                         |           da[1]--> rej limit |
+  | pulere = true  | 3*no_of_delay           |           da[2]--> err limit |
+  |                |                         | 2o delay  da[3]--> delay     |
+  |                |                         |           da[4]--> rej limit |
+  |                |                         |           ....               |
+  |----------------|-------------------------|------------------------------|
+  |                |                         | 1o delay da[0]--> min delay  |
+  | mtm = true     |                         |          da[1]--> typ delay  |
+  | pulere = true  | 9*no_of_delay           |          da[2]--> max delay  |
+  |                |                         |          da[3]--> min delay  |
+  |                |                         |          da[4]--> typ delay  |
+  |                |                         |          da[5]--> max delay  |
+  |                |                         |          da[6]--> min delay  |
+  |                |                         |          da[7]--> typ delay  |
+  |                |                         |          da[8]--> max delay  |
+  |                |                         | 2o delay da[9]--> min delay  |
+  |                |                         |          ....                |
+   -------------------------------------------------------------------------
+
+   IMPORTANT :
+
+   The delay Structure has to be allocated before passing a pointer to
+   "vpi_get_delays ( )".
+
+*/
+
+
+typedef struct t_vpi_delay  {
+      struct t_vpi_time  *da; /* Array of delay data */
+      PLI_INT32  no_of_delays ;
+      PLI_INT32  time_type; /* vpiScaledRealTime, vpiSimTime */
+      PLI_INT32  mtm_flag;
+      PLI_INT32  append_flag;
+      PLI_INT32  plusere_flag;
+} s_vpi_delay, *p_vpi_delay;
+
+
+
+
+
 /* These are valid codes for the format of the t_vpi_value structure. */
 #define vpiBinStrVal    1
 #define vpiOctStrVal    2
@@ -158,26 +272,34 @@ typedef struct t_vpi_value {
 #define vpiIterator    27
 #define vpiMemory      29
 #define vpiMemoryWord  30
+#define vpiModPath     31
 #define vpiModule      32
 #define vpiNamedBegin  33
 #define vpiNamedEvent  34
 #define vpiNamedFork   35
 #define vpiNet         36
 #define vpiParameter   41
+#define vpiPartSelect  42
+#define vpiPathTerm    43
 #define vpiRealVar     47
 #define vpiReg         48
 #define vpiSysFuncCall 56
 #define vpiSysTaskCall 57
 #define vpiTask        59
 #define vpiTimeVar     63
+#define vpiNetArray   114
 #define vpiIndex       78
 #define vpiLeftRange   79
+#define vpiParent      81
 #define vpiRightRange  83
 #define vpiScope       84
 #define vpiSysTfCall   85
 #define vpiArgument    89
 #define vpiInternalScope 92
+#define vpiModPathIn     95
+#define vpiModPathOut    96
 #define vpiVariables   100
+#define vpiExpr        102
 
 #define vpiCallback  1000
 
@@ -187,11 +309,28 @@ typedef struct t_vpi_value {
 #define vpiName       2
 #define vpiFullName   3
 #define vpiSize       4
+#define vpiFile       5
+#define vpiLineNo     6
+#define vpiTopModule  7
 #define vpiDefName    9
 #define vpiTimeUnit      11
 #define vpiTimePrecision 12
-#define vpiNetType    22
+#define vpiDefFile   15
+#define vpiDefLineNo 16
+#define vpiNetType   22
 #   define vpiWire        1
+#define vpiArray     28
+#define vpiEdge      36
+#   define vpiNoEdge     0x00 /* No edge */
+#   define vpiEdge01     0x01 /* 0 --> 1 */
+#   define vpiEdge10     0x02 /* 1 --> 0 */
+#   define vpiEdge0x     0x04 /* 0 --> x */
+#   define vpiEdgex1     0x08 /* x --> 1 */
+#   define vpiEdge1x     0x10 /* 1 --> x */
+#   define vpiEdgex0     0x20 /* x --> 0 */
+#   define vpiPosedge    (vpiEdgex1|vpiEdge01|vpiEdge0x)
+#   define vpiNegedge    (vpiEdgex0|vpiEdge10|vpiEdge1x)
+#   define vpiAnyEdge    (vpiPosedge|vpiNegedge)
 #define vpiConstType 40
 #   define vpiDecConst    1
 #   define vpiRealConst   2
@@ -210,8 +349,10 @@ typedef struct t_vpi_value {
 #   define vpiSysFuncReal  vpiRealFunc
 #   define vpiSysFuncTime  vpiTimeFunc
 #   define vpiSysFuncSized vpiSizedFunc
-#define vpiSigned    65
-/* IVL private properties */
+#define vpiAutomatic      50
+#define vpiConstantSelect 53
+#define vpiSigned         65
+/* IVL private properties, also see vvp/vpi_priv.h for other properties */
 #define _vpiNexusId 0x1000000
 
 /* DELAY MODES */
@@ -222,7 +363,7 @@ typedef struct t_vpi_value {
 
 #define vpiForceFlag   5
 #define vpiReleaseFlag 6
-
+#define vpiReturnEvent 0x1000
 
 /* VPI FUNCTIONS */
 extern void vpi_register_systf(const struct t_vpi_systf_data*ss);
@@ -305,8 +446,8 @@ extern PLI_INT32 vpi_remove_cb(vpiHandle ref);
  * This function allows a vpi application to control the simulation
  * engine. The operation parameter specifies the function to
  * perform. The remaining parameters (if any) are interpreted by the
- * operation. The vpi_sim_control definition was added to P1364-2000
- * 14 July 1999. See PLI Task Force ID: PTF-161
+ * operation. The vpi_sim_control definition (now named vpi_control)
+ * was added to P1364-2000 14 July 1999. See PLI Task Force ID: PTF-161
  *
  * vpiFinish - perform the $finish operation, as soon as the user
  *             function returns. This operation takes a single
@@ -361,6 +502,18 @@ extern vpiHandle vpi_put_value(vpiHandle obj, p_vpi_value value,
 extern PLI_INT32 vpi_free_object(vpiHandle ref);
 extern PLI_INT32 vpi_get_vlog_info(p_vpi_vlog_info vlog_info_p);
 
+/*
+  These Routines will enable the modpath vpiHandle
+  to read/write delay values
+*/
+extern void vpi_get_delays(vpiHandle expr, p_vpi_delay delays);
+
+extern void vpi_put_delays(vpiHandle expr, p_vpi_delay delays);
+
+
+
+
+
 /*
  * These functions support attaching user data to an instance of a
  * system task or function. These functions only apply to
@@ -413,94 +566,10 @@ extern DLLEXPORT void (*vlog_startup_routines[])();
 
   /* Format a scalar a la %v. The str points to a 4byte character
      buffer. The value must be a vpiStrengthVal. */
-extern void vpip_format_strength(char*str, s_vpi_value*value);
+extern void vpip_format_strength(char*str, s_vpi_value*value, unsigned bit);
+extern void vpip_set_return_value(int value);
+extern s_vpi_vecval vpip_calc_clog2(vpiHandle arg);
 
 EXTERN_C_END
 
-/*
- * $Log: vpi_user.h,v $
- * Revision 1.36  2004/10/04 01:10:56  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.35  2004/09/05 21:30:16  steve
- *  Better type safety.
- *
- * Revision 1.34  2004/03/09 04:29:26  steve
- *  Define function types.
- *
- * Revision 1.33  2004/02/19 21:32:46  steve
- *  Add sysfunctype to calltf structure.
- *
- * Revision 1.32  2004/01/13 02:55:50  steve
- *  Get value for vpoiConstType correct.
- *
- * Revision 1.31  2003/12/07 20:06:24  steve
- *  tfname can be constant.
- *
- * Revision 1.30  2003/10/30 03:42:51  steve
- *  Details on the vpi_get_file function.
- *
- * Revision 1.29  2003/07/15 03:49:22  steve
- *  Spelling fixes.
- *
- * Revision 1.28  2003/06/04 01:56:20  steve
- * 1) Adds configure logic to clean up compiler warnings
- * 2) adds acc_compare_handle, acc_fetch_range, acc_next_scope and
- *    tf_isetrealdelay, acc_handle_scope
- * 3) makes acc_next reentrant
- * 4) adds basic vpiWire type support
- * 5) fills in some acc_object_of_type() and acc_fetch_{full}type()
- * 6) add vpiLeftRange/RigthRange to signals
- *
- * Revision 1.27  2003/05/29 02:21:45  steve
- *  Implement acc_fetch_defname and its infrastructure in vvp.
- *
- * Revision 1.26  2003/05/23 04:04:02  steve
- *  Add vpi_fopen and vpi_get_file.
- *
- * Revision 1.25  2003/05/15 16:51:08  steve
- *  Arrange for mcd id=00_00_00_01 to go to stdout
- *  as well as a user specified log file, set log
- *  file to buffer lines.
- *
- *  Add vpi_flush function, and clear up some cunfused
- *  return codes from other vpi functions.
- *
- *  Adjust $display and vcd/lxt messages to use the
- *  standard output/log file.
- *
- * Revision 1.24  2003/04/20 02:49:07  steve
- *  acc_fetch_value support for %v format.
- *
- * Revision 1.23  2003/03/13 18:26:12  steve
- *  Verilog 2001 standart types.
- *
- * Revision 1.22  2003/03/13 04:34:35  steve
- *  Add VPI_TRACE tracing of VPI calls.
- *  vpi_handle_by_name takes a const char*.
- *
- * Revision 1.21  2003/03/12 02:49:38  steve
- *  Move _vpiNexisId safely out of the way.
- *
- * Revision 1.20  2003/03/10 23:40:54  steve
- *  Keep parameter constants for the ivl_target API.
- *
- * Revision 1.19  2003/02/17 06:39:47  steve
- *  Add at least minimal implementations for several
- *  acc_ functions. Add support for standard ACC
- *  string handling.
- *
- *  Add the _pli_types.h header file to carry the
- *  IEEE1364-2001 standard PLI type declarations.
- *
- * Revision 1.18  2003/02/01 05:50:27  steve
- *  Make $time and $realtime available to $display uniquely.
- *
- * Revision 1.17  2003/01/26 21:15:59  steve
- *  Rework expression parsing and elaboration to
- *  accommodate real/realtime values and expressions.
- *
- * Revision 1.16  2003/01/09 04:10:17  steve
- *  Add vpi_put_userdata
- */
 #endif
