@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,12 +16,13 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: codes.cc,v 1.14 2003/08/01 00:58:03 steve Exp $"
-#endif
 
 # include  "codes.h"
 # include  "statistics.h"
+# include  "config.h"
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+#endif
 # include  <string.h>
 # include  <assert.h>
 
@@ -95,21 +96,24 @@ vvp_code_t codespace_null(void)
       return first_chunk + 0;
 }
 
-/*
- * $Log: codes.cc,v $
- * Revision 1.14  2003/08/01 00:58:03  steve
- *  Initialize allocated memory.
- *
- * Revision 1.13  2003/07/03 20:03:36  steve
- *  Remove the vvp_cpoint_t indirect code pointer.
- *
- * Revision 1.12  2002/08/12 01:35:07  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.11  2002/07/05 03:46:43  steve
- *  Track opcode memory space.
- *
- * Revision 1.10  2002/07/05 02:50:58  steve
- *  Remove the vpi object symbol table after compile.
- */
+#ifdef CHECK_WITH_VALGRIND
+void codespace_delete(void)
+{
+      vvp_code_t cur = first_chunk;
 
+      do {
+	    vvp_code_t next = cur[code_chunk_size-1].cptr;
+	    for (unsigned idx = 0 ; idx < code_chunk_size; idx += 1) {
+		  count_opcodes -= 1;
+		  if ((cur+idx)->opcode == &of_VPI_CALL) {
+			vpi_call_delete((cur+idx)->handle);
+		  }
+		  if (count_opcodes == 0) break;
+	    }
+	      /* Don't count the &of_CHUNK_LINK opcode. */
+	    if (count_opcodes != 0) count_opcodes += 1;
+	    delete [] cur;
+	    cur = next;
+      } while (cur != 0);
+}
+#endif
