@@ -1,7 +1,7 @@
 #ifndef __schedule_H
 #define __schedule_H
 /*
- * Copyright (c) 2001-2003 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2008 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,12 +18,11 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: schedule.h,v 1.16 2003/09/09 00:56:45 steve Exp $"
-#endif
 
 # include  "vthread.h"
 # include  "pointers.h"
+# include  "vvp_net.h"
+# include  "array.h"
 
 /*
  * This causes a thread to be scheduled for execution. The schedule
@@ -39,11 +38,47 @@ extern void schedule_vthread(vthread_t thr, vvp_time64_t delay,
 
 /*
  * Create an assignment event. The val passed here will be assigned to
- * the specified input when the delay times out.
+ * the specified input when the delay times out. This is scheduled
+ * like a non-blocking assignment. This is in fact mostly used to
+ * implement the non-blocking assignment.
  */
-extern void schedule_assign(vvp_ipoint_t fun, unsigned char val,
-			    vvp_time64_t delay);
+extern void schedule_assign_vector(vvp_net_ptr_t ptr,
+				   unsigned base, unsigned vwid,
+				   const vvp_vector4_t&val,
+				   vvp_time64_t  delay);
 
+extern void schedule_assign_plucked_vector(vvp_net_ptr_t ptr,
+					   vvp_time64_t  delay,
+					   const vvp_vector4_t&val,
+					   unsigned adr, unsigned wid);
+
+extern void schedule_assign_array_word(vvp_array_t mem,
+				       unsigned word_address,
+				       unsigned off,
+				       vvp_vector4_t val,
+				       vvp_time64_t delay);
+/*
+ * This is very similar to schedule_assign_vector, but generates an
+ * event in the active queue. It is used at link time to assign a
+ * constant value (i.e. C4<...>) to the input of a functor. This
+ * creates an event in the active queue.
+ */
+extern void schedule_set_vector(vvp_net_ptr_t ptr, const vvp_vector4_t&val);
+extern void schedule_set_vector(vvp_net_ptr_t ptr, vvp_vector8_t val);
+extern void schedule_set_vector(vvp_net_ptr_t ptr, double val);
+
+/*
+ * The schedule_init_vector function assigns an initial value to a
+ * functor. The assignment is put into a pre-simulation queue that is
+ * run before the rest of the simulation runs. This is used to assign
+ * initial values to variables and have them propagate through the
+ * net. Doing the init before the rest of the scheduler starts
+ * prevents threads being triggered by the initialization of the
+ * variable, but still allows the initial value to be driven
+ * (propagated as events) through the rest of the net.
+ */
+extern void schedule_init_vector(vvp_net_ptr_t ptr, vvp_vector4_t val);
+extern void schedule_init_vector(vvp_net_ptr_t ptr, double val);
 
 /*
  * Create a generic event. This supports scheduled events that are not
@@ -52,16 +87,29 @@ extern void schedule_assign(vvp_ipoint_t fun, unsigned char val,
  * up to the user to allocate/free the vvp_get_event_s object. The
  * object is never referenced by the scheduler after the run method is
  * called.
+ *
+ * The sync_flag is true if this is intended to be a sync event. These
+ * are placed in the stratified event queue after nb assigns. If the
+ * ro_flag is true as well, then it is a Read-only sync event, with
+ * all that means.
+ *
+ * If the sync_flag is false, then the event is ACTIVE, and the
+ * ro_flag is ignored.
  */
 
 typedef struct vvp_gen_event_s *vvp_gen_event_t;
 
-extern void schedule_generic(vvp_gen_event_t obj, unsigned char val,
-			     vvp_time64_t delay, bool sync_flag);
+extern void schedule_generic(vvp_gen_event_t obj, vvp_time64_t delay,
+			     bool sync_flag, bool ro_flag =true,
+			     bool delete_obj_when_done =false);
+
+/* Use this is schedule thread deletion (after rosync). */
+extern void schedule_del_thr(vthread_t thr);
 
 struct vvp_gen_event_s
 {
-      void (*run)(vvp_gen_event_t obj, unsigned char val);
+      virtual ~vvp_gen_event_s() =0;
+      virtual void run_run() =0;
 };
 
 /*
@@ -106,30 +154,4 @@ extern unsigned long count_prop_events;
 extern unsigned long count_thread_events;
 extern unsigned long count_event_pool;
 
-/*
- * $Log: schedule.h,v $
- * Revision 1.16  2003/09/09 00:56:45  steve
- *  Reimpelement scheduler to divide nonblocking assign queue out.
- *
- * Revision 1.15  2003/02/22 02:52:06  steve
- *  Check for stopped flag in certain strategic points.
- *
- * Revision 1.14  2003/02/21 03:40:35  steve
- *  Add vpiStop and interactive mode.
- *
- * Revision 1.13  2003/02/09 23:33:26  steve
- *  Spelling fixes.
- *
- * Revision 1.12  2003/01/06 23:57:26  steve
- *  Schedule wait lists of threads as a single event,
- *  to save on events. Also, improve efficiency of
- *  event_s allocation. Add some event statistics to
- *  get an idea where performance is really going.
- *
- * Revision 1.11  2002/08/12 01:35:08  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.10  2002/05/12 23:44:41  steve
- *  task calls and forks push the thread event in the queue.
- */
 #endif
