@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,11 +16,13 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_const.cc,v 1.32 2004/10/04 01:10:59 steve Exp $"
-#endif
 
 # include  "vpi_priv.h"
+# include  "compile.h"
+# include  "config.h"
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+#endif
 # include  <stdio.h>
 #ifdef HAVE_MALLOC_H
 # include  <malloc.h>
@@ -49,6 +51,14 @@ static int string_get(int code, vpiHandle ref)
 	  case vpiConstType:
 	      return vpiStringConst;
 
+          case vpiAutomatic:
+	      return 0;
+
+#ifdef CHECK_WITH_VALGRIND
+          case _vpiFromThr:
+	      return _vpiNoThr;
+#endif
+
 	  default:
 	      fprintf(stderr, "vvp error: get %d not supported "
 		      "by vpiStringConst\n", code);
@@ -60,6 +70,7 @@ static int string_get(int code, vpiHandle ref)
 static void string_value(vpiHandle ref, p_vpi_value vp)
 {
       unsigned uint_value;
+      p_vpi_vecval vecp;
       struct __vpiStringConst*rfp = (struct __vpiStringConst*)ref;
       int size = strlen(rfp->value);
       char*rbuf = 0;
@@ -125,6 +136,36 @@ static void string_value(vpiHandle ref, p_vpi_value vp)
 	      assert(0);
 	      break;
 
+          case vpiIntVal:
+	      vp->value.integer = 0;
+	      for(int i=0; i<size;i ++){
+		  for(int bit=7;bit>=0; bit--){
+		      vp->value.integer <<= 1;
+		      vp->value.integer += (rfp->value[i]>>bit)&1;
+		  }
+	      }
+	      break;
+
+          case vpiVectorVal:
+              vp->value.vector = (p_vpi_vecval)
+                                 need_result_buf((size+3)/4*
+                                                  sizeof(s_vpi_vecval),
+                                                 RBUF_VAL);
+              uint_value = 0;
+              vecp = vp->value.vector;
+              vecp->aval = vecp->bval = 0;
+	      for(int i=0; i<size;i ++){
+		  vecp->aval |= rfp->value[i] << uint_value*8;
+		  uint_value += 1;
+		  if (uint_value > 3) {
+		      uint_value = 0;
+		      vecp += 1;
+		      vecp->aval = vecp->bval = 0;
+		  }
+	      }
+	      break;
+
+
 	  default:
 	    fprintf(stderr, "ERROR (vpi_const.cc): vp->format: %d\n", vp->format);
 	    assert(0);
@@ -149,7 +190,7 @@ static int free_temp_string(vpiHandle obj)
       struct __vpiStringConst*rfp = (struct __vpiStringConst*)obj;
       assert(obj->vpi_type->type_code == vpiConstant);
 
-      free(rfp->value);
+      delete [] rfp->value;
       free(rfp);
       return 1;
 }
@@ -168,6 +209,36 @@ static const struct __vpirt vpip_string_temp_rt = {
       free_temp_string
 };
 
+/*
+ * Strings are described at the level of the vvp source as a string
+ * with literal characters or octal escapes. No other escapes are
+ * included, they are processed already by the compiler that generated
+ * the vvp source.
+ */
+static void vpip_process_string(struct __vpiStringConst*obj)
+{
+      char*chr = obj->value;
+      char*dp = obj->value;
+
+      while (*chr) {
+	    char next_char = *chr;
+
+	      /* Process octal escapes that I might find. */
+	    if (*chr == '\\') {
+		  for (int idx = 1 ;  idx <= 3 ;  idx += 1) {
+			assert(chr[idx] != 0);
+			assert(chr[idx] < '8');
+			assert(chr[idx] >= '0');
+			next_char = next_char*8 + chr[idx] - '0';
+		  }
+		  chr += 3;
+	    }
+	    *dp++ = next_char;
+	    chr += 1;
+      }
+      *dp = 0;
+      obj->value_len = dp - obj->value;
+}
 
 vpiHandle vpip_make_string_const(char*text, bool persistent_flag)
 {
@@ -179,6 +250,8 @@ vpiHandle vpip_make_string_const(char*text, bool persistent_flag)
 	    ? &vpip_string_rt
 	    : &vpip_string_temp_rt;
       obj->value = text;
+      obj->value_len = 0;
+      vpip_process_string(obj);
 
       return &obj->base;
 }
@@ -187,23 +260,34 @@ vpiHandle vpip_make_string_const(char*text, bool persistent_flag)
 struct __vpiStringParam  : public __vpiStringConst {
       const char*basename;
       struct __vpiScope* scope;
+      unsigned file_idx;
+      unsigned lineno;
 };
 
+static int string_param_get(int code, vpiHandle ref)
+{
+      struct __vpiStringParam*rfp = (struct __vpiStringParam*)ref;
+
+      assert(ref->vpi_type->type_code == vpiParameter);
+
+      if (code == vpiLineNo) {
+	    return rfp->lineno;
+      }
+
+      return string_get(code, ref);
+}
+
 static char* string_param_get_str(int code, vpiHandle obj)
 {
       struct __vpiStringParam*rfp = (struct __vpiStringParam*)obj;
-      char *rbuf = need_result_buf(strlen(rfp->basename) + 1, RBUF_STR);
 
       assert(obj->vpi_type->type_code == vpiParameter);
 
-      switch (code) {
-	  case vpiName:
-	    strcpy(rbuf, rfp->basename);
-	    return rbuf;
-
-	  default:
-	    return 0;
+      if (code == vpiFile) {
+	    return simple_set_rbuf_str(file_names[rfp->file_idx]);
       }
+
+      return generic_get_str(code, &rfp->scope->base, rfp->basename, NULL);
 }
 
 static vpiHandle string_param_handle(int code, vpiHandle obj)
@@ -216,6 +300,9 @@ static vpiHandle string_param_handle(int code, vpiHandle obj)
 	  case vpiScope:
 	    return &rfp->scope->base;
 
+	  case vpiModule:
+	    return vpip_module(rfp->scope);
+
 	  default:
 	    return 0;
       }
@@ -223,7 +310,7 @@ static vpiHandle string_param_handle(int code, vpiHandle obj)
 
 static const struct __vpirt vpip_string_param_rt = {
       vpiParameter,
-      string_get,
+      string_param_get,
       string_param_get_str,
       string_value,
       0,
@@ -236,7 +323,8 @@ static const struct __vpirt vpip_string_param_rt = {
 };
 
 
-vpiHandle vpip_make_string_param(char*name, char*text)
+vpiHandle vpip_make_string_param(char*name, char*text,
+                                 long file_idx, long lineno)
 {
       struct __vpiStringParam*obj;
 
@@ -244,8 +332,13 @@ vpiHandle vpip_make_string_param(char*name, char*text)
 	    malloc(sizeof (struct __vpiStringParam));
       obj->base.vpi_type = &vpip_string_param_rt;
       obj->value = text;
+      obj->value_len = 0;
       obj->basename = name;
       obj->scope = vpip_peek_current_scope();
+      obj->file_idx = (unsigned) file_idx;
+      obj->lineno = (unsigned) lineno;
+
+      vpip_process_string(obj);
 
       return &obj->base;
 }
@@ -253,17 +346,29 @@ vpiHandle vpip_make_string_param(char*name, char*text)
 static int binary_get(int code, vpiHandle ref)
 {
       struct __vpiBinaryConst*rfp = (struct __vpiBinaryConst*)ref;
-      assert(ref->vpi_type->type_code == vpiConstant);
+      assert(ref->vpi_type->type_code == vpiConstant
+	     || ref->vpi_type->type_code == vpiParameter);
 
       switch (code) {
 	  case vpiConstType:
 	    return vpiBinaryConst;
 
+	  case vpiLineNo:
+	    return 0;  // Not implemented for now!
+
 	  case vpiSigned:
 	    return rfp->signed_flag? 1 : 0;
 
 	  case vpiSize:
-	    return rfp->nbits;
+	    return rfp->bits.size();
+
+          case vpiAutomatic:
+	    return 0;
+
+#ifdef CHECK_WITH_VALGRIND
+          case _vpiFromThr:
+	      return _vpiNoThr;
+#endif
 
 	  default:
 	    fprintf(stderr, "vvp error: get %d not supported "
@@ -273,212 +378,29 @@ static int binary_get(int code, vpiHandle ref)
       }
 }
 
-static void binary_vpiStringVal(struct __vpiBinaryConst*rfp, p_vpi_value vp)
-{
-      unsigned nchar = rfp->nbits / 8;
-      unsigned tail = rfp->nbits%8;
-
-      char*rbuf = need_result_buf(nchar + 1, RBUF_VAL);
-      char*cp = rbuf;
-
-      if (tail > 0) {
-	    char char_val = 0;
-	    for (unsigned idx = rfp->nbits-tail; idx < rfp->nbits; idx += 1) {
-		  unsigned nibble = idx/4;
-		  unsigned shift  = 2 * (idx%4);
-		  unsigned val = (rfp->bits[nibble] >> shift) & 3;
-		  if (val & 1)
-			char_val |= 1 << idx;
-	    }
-
-	    if (char_val != 0)
-		  *cp++ = char_val;
-      }
-
-      for (unsigned idx = 0 ;  idx < nchar ;  idx += 1) {
-	    unsigned bit = (nchar - idx - 1) * 8;
-	    unsigned nibble = bit/4;
-	    unsigned vall = rfp->bits[nibble+0];
-	    unsigned valh = rfp->bits[nibble+1];
-
-	    char char_val = 0;
-	    if (vall&0x01) char_val |= 0x01;
-	    if (vall&0x04) char_val |= 0x02;
-	    if (vall&0x10) char_val |= 0x04;
-	    if (vall&0x40) char_val |= 0x08;
-	    if (valh&0x01) char_val |= 0x10;
-	    if (valh&0x04) char_val |= 0x20;
-	    if (valh&0x10) char_val |= 0x40;
-	    if (valh&0x40) char_val |= 0x80;
-
-	    if (char_val != 0)
-		  *cp++ = char_val;
-      }
-
-      *cp = 0;
-      vp->value.str = rbuf;
-}
-
-static int bits2int(struct __vpiBinaryConst*rfp)
-{
-      unsigned val = 0;
-      unsigned bit_val = 0;
-      unsigned bit_limit = rfp->nbits;
-      if (bit_limit > 8*sizeof(val))
-	bit_limit = 8*sizeof(val);
-
-      for (unsigned idx = 0 ;  idx < bit_limit ;  idx += 1) {
-	unsigned nibble = idx/4;
-	unsigned shift  = 2 * (idx%4);
-	bit_val = (rfp->bits[nibble] >> shift) & 3;
-	if (bit_val > 1) {
-	      return 0;
-	} else {
-	      val |= bit_val << idx;
-	}
-      }
-
-      /* sign extend */
-      if (rfp->signed_flag && bit_val) {
-	  for (unsigned idx = rfp->nbits; idx <sizeof(val)*8; idx++)
-	  {
-	  val |= bit_val << idx;
-	  }
-      }
-
-      return val;
-}
 
 static void binary_value(vpiHandle ref, p_vpi_value vp)
 {
-      assert(ref->vpi_type->type_code == vpiConstant);
+      assert(ref->vpi_type->type_code == vpiConstant
+	     || ref->vpi_type->type_code == vpiParameter);
 
       struct __vpiBinaryConst*rfp = (struct __vpiBinaryConst*)ref;
-      char*rbuf = 0;
 
 
       switch (vp->format) {
 
 	  case vpiObjTypeVal:
-	  case vpiBinStrVal: {
-	    rbuf = need_result_buf(rfp->nbits + 1, RBUF_VAL);
-	    for (unsigned idx = 0 ;  idx < rfp->nbits ;  idx += 1) {
-		  unsigned nibble = idx/4;
-		  unsigned shift  = 2 * (idx%4);
-		  unsigned val = (rfp->bits[nibble] >> shift) & 3;
-
-		  rbuf[rfp->nbits-idx-1] = "01xz"[val];
-	    }
-	    rbuf[rfp->nbits] = 0;
-	    vp->value.str = rbuf;
-	    break;
-	  }
-
-	  case vpiDecStrVal: {
-		unsigned wid = rfp->nbits;
-		rbuf = need_result_buf(rfp->nbits + 1, RBUF_VAL);
-		unsigned char*tmp = new unsigned char[wid];
-		for (unsigned idx = 0 ;  idx < wid ;  idx += 1)
-		      tmp[idx] = (rfp->bits[idx/4] >> 2*(idx%4)) & 3;
-
-		vpip_bits_to_dec_str(tmp, wid, rbuf, wid + 1,
-				     rfp->signed_flag);
-
-		delete[]tmp;
-		vp->value.str = rbuf;
-		break;
-	  }
-
-	  case vpiHexStrVal: {
-		unsigned nchar = (rfp->nbits+3)/4;
-	        rbuf = need_result_buf(nchar + 1, RBUF_VAL);
-		for (unsigned idx = 0 ;  idx < rfp->nbits ;  idx += 4) {
-		      unsigned nibble = idx/4;
-		      unsigned vals = rfp->bits[nibble];
-
-		      if (vals == 0xff) {
-			    rbuf[nchar-idx/4-1] = 'z';
-		      } else if (vals == 0xaa) {
-			    rbuf[nchar-idx/4-1] = 'x';
-		      } else if (vals & 0xaa) {
-			    rbuf[nchar-idx/4-1] = 'X';
-		      } else {
-			    unsigned val = vals&1;
-			    if (vals&0x04) val |= 2;
-			    if (vals&0x10) val |= 4;
-			    if (vals&0x40) val |= 8;
-			    rbuf[nchar-idx/4-1] = "0123456789abcdef"[val];
-		      }
-		}
-
-		rbuf[nchar] = 0;
-		vp->value.str = rbuf;
-		break;
-	  }
-
-	  case vpiOctStrVal: {
-		unsigned nchar = (rfp->nbits+2)/3;
-	        rbuf = need_result_buf(nchar + 1, RBUF_VAL);
-		vpip_bits_to_oct_str(rfp->bits, rfp->nbits,
-				     rbuf, nchar+1, rfp->signed_flag);
-		vp->value.str = rbuf;
-		break;
-	  }
-
-	  case vpiIntVal: {
-		vp->value.integer = bits2int(rfp);
-		break;
-	  }
-
-	  case vpiVectorVal: {
-	      unsigned int obit = 0;
-	      unsigned hwid = (rfp->nbits - 1)/32 + 1;
-	      rbuf = need_result_buf(hwid*sizeof(s_vpi_vecval), RBUF_VAL);
-
-	      s_vpi_vecval *op = (p_vpi_vecval)rbuf;
-	      vp->value.vector = op;
-
-	      op->aval = op->bval = 0;
-	      for (unsigned idx = 0 ;  idx < rfp->nbits ;  idx += 1) {
-		unsigned nibble = idx/4;
-		unsigned shift  = 2 * (idx%4);
-		unsigned bit_val = (rfp->bits[nibble] >> shift) & 3;
-
-		switch (bit_val) {
-		case 0:
-		  op->aval &= ~(1 << obit);
-		  op->bval &= ~(1 << obit);
-		  break;
-		case 1:
-		  op->aval |= (1 << obit);
-		  op->bval &= ~(1 << obit);
-		  break;
-		case 2:
-		  op->aval |= (1 << obit);
-		  op->bval |= (1 << obit);
-		  break;
-		case 3:
-		  op->aval &= ~(1 << obit);
-		  op->bval |= (1 << obit);
-		  break;
-		}
-		obit++;
-		if (!(obit % 32)) {
-		      op += 1;
-		      if ((op - vp->value.vector) < (long)hwid)
-			    op->aval = op->bval = 0;
-		      obit = 0;
-		}
-	      }
-	      break;
-	  }
-
-	  case vpiRealVal:
-	      vp->value.real = (double)bits2int(rfp);
-	    break;
-
+	  case vpiBinStrVal:
+	  case vpiDecStrVal:
+	  case vpiOctStrVal:
+	  case vpiHexStrVal:
+          case vpiScalarVal:
+	  case vpiIntVal:
+	  case vpiVectorVal:
 	  case vpiStringVal:
-	    binary_vpiStringVal(rfp, vp);
+	  case vpiRealVal:
+	    vpip_vec4_get_value(rfp->bits, rfp->bits.size(),
+				rfp->signed_flag, vp);
 	    break;
 
 	  default:
@@ -504,18 +426,16 @@ static const struct __vpirt vpip_binary_rt = {
  * ASCII string, with each letter a 4-value bit. The first character
  * may be an 's' if the vector is signed.
  */
-vpiHandle vpip_make_binary_const(unsigned wid, char*bits)
+vpiHandle vpip_make_binary_const(unsigned wid, const char*bits)
 {
       struct __vpiBinaryConst*obj;
 
-      obj = (struct __vpiBinaryConst*)
-	    malloc(sizeof (struct __vpiBinaryConst));
+      obj = new __vpiBinaryConst;
       obj->base.vpi_type = &vpip_binary_rt;
 
       obj->signed_flag = 0;
-      obj->nbits = wid;
-      obj->bits = (unsigned char*)malloc((obj->nbits + 3) / 4);
-      memset(obj->bits, 0, (obj->nbits + 3) / 4);
+      obj->sized_flag = 0;
+      obj->bits = vvp_vector4_t(wid);
 
       const char*bp = bits;
       if (*bp == 's') {
@@ -523,31 +443,111 @@ vpiHandle vpip_make_binary_const(unsigned wid, char*bits)
 	    obj->signed_flag = 1;
       }
 
-      for (unsigned idx = 0 ;  idx < obj->nbits ;  idx += 1) {
-	    unsigned nibble = idx / 4;
-	    unsigned val = 0;
+      for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
+	    vvp_bit4_t val = BIT4_0;
 	    switch (bp[wid-idx-1]) {
 		case '0':
-		  val = 0;
+		  val = BIT4_0;
 		  break;
 		case '1':
-		  val = 1;
+		  val = BIT4_1;
 		  break;
 		case 'x':
-		  val = 2;
+		  val = BIT4_X;
 		  break;
 		case 'z':
-		  val = 3;
+		  val = BIT4_Z;
 		  break;
 	    }
 
-	    obj->bits[nibble] |= val << (2 * (idx%4));
+	    obj->bits.set_bit(idx, val);
       }
 
-      free(bits);
       return &(obj->base);
 }
 
+struct __vpiBinaryParam  : public __vpiBinaryConst {
+      const char*basename;
+      struct __vpiScope*scope;
+      unsigned file_idx;
+      unsigned lineno;
+};
+
+static int binary_param_get(int code, vpiHandle ref)
+{
+      struct __vpiBinaryParam*rfp = (struct __vpiBinaryParam*)ref;
+
+      assert(ref->vpi_type->type_code == vpiParameter);
+
+      if (code == vpiLineNo) {
+	    return rfp->lineno;
+      }
+
+      return binary_get(code, ref);
+}
+
+static char* binary_param_get_str(int code, vpiHandle obj)
+{
+      struct __vpiBinaryParam*rfp = (struct __vpiBinaryParam*)obj;
+
+      assert(obj->vpi_type->type_code == vpiParameter);
+
+      if (code == vpiFile) {
+	    return simple_set_rbuf_str(file_names[rfp->file_idx]);
+      }
+
+      return generic_get_str(code, &rfp->scope->base, rfp->basename, NULL);
+}
+
+static vpiHandle binary_param_handle(int code, vpiHandle obj)
+{
+      struct __vpiBinaryParam*rfp = (struct __vpiBinaryParam*)obj;
+
+      assert(obj->vpi_type->type_code == vpiParameter);
+
+      switch (code) {
+	  case vpiScope:
+	    return &rfp->scope->base;
+
+	  case vpiModule:
+	    return vpip_module(rfp->scope);
+
+	  default:
+	    return 0;
+      }
+}
+
+static const struct __vpirt vpip_binary_param_rt = {
+      vpiParameter,
+      binary_param_get,
+      binary_param_get_str,
+      binary_value,
+      0,
+
+      binary_param_handle,
+      0,
+      0,
+
+      0
+};
+
+vpiHandle vpip_make_binary_param(char*name, const vvp_vector4_t&bits,
+				 bool signed_flag,
+				 long file_idx, long lineno)
+{
+      struct __vpiBinaryParam*obj = new __vpiBinaryParam;
+
+      obj->base.vpi_type = &vpip_binary_param_rt;
+      obj->bits = bits;
+      obj->signed_flag = signed_flag? 1 : 0;
+      obj->sized_flag = 0;
+      obj->basename = name;
+      obj->scope = vpip_peek_current_scope();
+      obj->file_idx = (unsigned) file_idx;
+      obj->lineno = (unsigned) lineno;
+
+      return &obj->base;
+}
 
 
 static int dec_get(int code, vpiHandle ref)
@@ -563,6 +563,14 @@ static int dec_get(int code, vpiHandle ref)
 	  case vpiSize:
 	    return 32;
 
+          case vpiAutomatic:
+	    return 0;
+
+#ifdef CHECK_WITH_VALGRIND
+          case _vpiFromThr:
+	      return _vpiNoThr;
+#endif
+
 	  default:
 	    fprintf(stderr, "vvp error: get %d not supported "
 		    "by vpiDecConst\n", code);
@@ -650,54 +658,210 @@ vpiHandle vpip_make_dec_const(int value)
 }
 
 
-/*
- * $Log: vpi_const.cc,v $
- * Revision 1.32  2004/10/04 01:10:59  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.31  2004/05/18 18:43:38  steve
- *  Allow vpiParamter as a string type.
- *
- * Revision 1.30  2003/05/30 04:22:13  steve
- *  Add tf_strgetp functions.
- *
- * Revision 1.29  2003/05/29 03:46:21  steve
- *  Add tf_getp/putp support for integers
- *  and real valued arguments.
- *
- *  Add tf_mipname function.
- *
- * Revision 1.28  2003/03/17 23:47:25  steve
- *  Make a safe copy of const string values.
- *
- * Revision 1.27  2003/03/15 05:44:50  steve
- *  Remove excess assignment.
- *
- * Revision 1.26  2003/03/14 05:02:13  steve
- *  Streamline parameter string value, get paramete scope.
- *
- * Revision 1.25  2003/03/13 04:59:21  steve
- *  Use rbufs instead of static buffers.
- *
- * Revision 1.24  2003/03/10 23:37:07  steve
- *  Direct support for string parameters.
- *
- * Revision 1.23  2003/03/10 19:14:27  steve
- *  More carful about shifting beyond word size.
- *
- * Revision 1.22  2003/02/24 06:35:45  steve
- *  Interactive task calls take string arguments.
- *
- * Revision 1.21  2002/11/03 20:33:43  steve
- *  Compiler error wrt ptrdiff_t.
- *
- * Revision 1.20  2002/11/03 02:07:24  steve
- *  Get VectorVals from constant values.
- *
- * Revision 1.19  2002/08/12 01:35:08  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.18  2002/06/23 18:23:09  steve
- *  trivial performance boost.
- */
+static int real_get(int code, vpiHandle ref)
+{
 
+      switch (code) {
+	  case vpiLineNo:
+	    return 0;  // Not implemented for now!
+
+	  case vpiSize:
+	    return 1;
+
+	  case vpiConstType:
+	    return vpiRealConst;
+
+	  case vpiSigned:
+	    return 1;
+
+          case vpiAutomatic:
+	    return 0;
+
+#ifdef CHECK_WITH_VALGRIND
+          case _vpiFromThr:
+	      return _vpiNoThr;
+#endif
+
+	  default:
+	    fprintf(stderr, "vvp error: get %d not supported "
+		    "by vpiDecConst\n", code);
+	    assert(0);
+	    return 0;
+      }
+}
+
+static void real_value(vpiHandle ref, p_vpi_value vp)
+{
+      struct __vpiRealConst*rfp = (struct __vpiRealConst*)ref;
+      assert((ref->vpi_type->type_code == vpiConstant) ||
+             (ref->vpi_type->type_code == vpiParameter));
+
+      switch (vp->format) {
+	  case vpiObjTypeVal:
+	    vp->format = vpiRealVal;
+	  case vpiRealVal:
+	    vp->value.real = rfp->value;
+	    break;
+	  default:
+	    fprintf(stderr, "vvp error: unsupported format %d.\n", vp->format);
+	    assert(0);
+      }
+}
+
+static const struct __vpirt vpip_real_rt = {
+      vpiConstant,
+      real_get,
+      0,
+      real_value,
+      0,
+      0,
+      0
+};
+
+vpiHandle vpip_make_real_const(struct __vpiRealConst*obj, double value)
+{
+      obj->base.vpi_type = &vpip_real_rt;
+      obj->value = value;
+      return &(obj->base);
+}
+
+vpiHandle vpip_make_real_const(double value)
+{
+      struct __vpiRealConst*obj;
+      obj =(struct __vpiRealConst*) malloc(sizeof (struct __vpiRealConst));
+      return vpip_make_real_const(obj, value);
+}
+
+struct __vpiRealParam  : public __vpiRealConst {
+      const char*basename;
+      struct __vpiScope* scope;
+      unsigned file_idx;
+      unsigned lineno;
+};
+
+static int real_param_get(int code, vpiHandle ref)
+{
+      struct __vpiRealParam*rfp = (struct __vpiRealParam*)ref;
+
+      assert(ref->vpi_type->type_code == vpiParameter);
+
+      if (code == vpiLineNo) {
+	    return rfp->lineno;
+      }
+
+      return real_get(code, ref);
+}
+
+static char* real_param_get_str(int code, vpiHandle obj)
+{
+      struct __vpiRealParam*rfp = (struct __vpiRealParam*)obj;
+
+      assert(obj->vpi_type->type_code == vpiParameter);
+
+      if (code == vpiFile) {
+            return simple_set_rbuf_str(file_names[rfp->file_idx]);
+      }
+
+      return generic_get_str(code, &rfp->scope->base, rfp->basename, NULL);
+}
+
+static vpiHandle real_param_handle(int code, vpiHandle obj)
+{
+      struct __vpiRealParam*rfp = (struct __vpiRealParam*)obj;
+
+      assert(obj->vpi_type->type_code == vpiParameter);
+
+      switch (code) {
+          case vpiScope:
+            return &rfp->scope->base;
+
+	  case vpiModule:
+	    return vpip_module(rfp->scope);
+
+          default:
+            return 0;
+      }
+}
+
+static const struct __vpirt vpip_real_param_rt = {
+      vpiParameter,
+      real_param_get,
+      real_param_get_str,
+      real_value,
+      0,
+
+      real_param_handle,
+      0,
+      0,
+
+      0
+};
+
+vpiHandle vpip_make_real_param(char*name, double value,
+                               long file_idx, long lineno)
+{
+      struct __vpiRealParam*obj;
+
+      obj = (struct __vpiRealParam*)
+            malloc(sizeof (struct __vpiRealParam));
+      obj->base.vpi_type = &vpip_real_param_rt;
+      obj->value = value;
+      obj->basename = name;
+      obj->scope = vpip_peek_current_scope();
+      obj->file_idx = (unsigned) file_idx;
+      obj->lineno = (unsigned) lineno;
+
+      return &obj->base;
+}
+
+#ifdef CHECK_WITH_VALGRIND
+void constant_delete(vpiHandle item)
+{
+      assert(item->vpi_type->type_code == vpiConstant);
+      switch(vpi_get(vpiConstType, item)) {
+	  case vpiStringConst: {
+	    struct __vpiStringConst*rfp = (struct __vpiStringConst*)item;
+	    delete [] rfp->value;
+	    free(rfp);
+	    break; }
+	  case vpiDecConst: {
+	    struct __vpiDecConst*rfp = (struct __vpiDecConst*)item;
+	    free(rfp);
+	    break; }
+	  case vpiBinaryConst: {
+	    struct __vpiBinaryConst*rfp = (struct __vpiBinaryConst*)item;
+	    delete rfp;
+	    break; }
+	  case vpiRealConst: {
+	    struct __vpiRealConst*rfp = (struct __vpiRealConst*)item;
+	    free(rfp);
+	    break; }
+	  default:
+	    assert(0);
+      }
+}
+
+void parameter_delete(vpiHandle item)
+{
+      switch(vpi_get(vpiConstType, item)) {
+	  case vpiStringConst: {
+	    struct __vpiStringParam*rfp = (struct __vpiStringParam*)item;
+	    delete [] rfp->basename;
+	    delete [] rfp->value;
+	    free(rfp);
+	    break; }
+	  case vpiBinaryConst: {
+	    struct __vpiBinaryParam*rfp = (struct __vpiBinaryParam*)item;
+	    delete [] rfp->basename;
+	    delete rfp;
+	    break; }
+	  case vpiRealConst: {
+	    struct __vpiRealParam*rfp = (struct __vpiRealParam*)item;
+	    delete [] rfp->basename;
+	    free(rfp);
+	    break; }
+	  default:
+	    assert(0);
+      }
+}
+#endif
