@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Stephen G. Tell <steve@telltronics.org>
+ * Copyright (c) 2000-2009 Stephen G. Tell <steve@telltronics.org>
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,11 +16,12 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_mcd.cc,v 1.12 2004/10/04 01:10:59 steve Exp $"
-#endif
 
 # include  "vpi_priv.h"
+# include  "config.h"
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+#endif
 # include  <assert.h>
 # include  <stdarg.h>
 # include  <stdio.h>
@@ -58,18 +59,39 @@ static FILE* logfile;
 void vpi_mcd_init(FILE *log)
 {
 	mcd_table[0].fp = stdout;
-	mcd_table[0].filename = "stdout";
+	mcd_table[0].filename = strdup("stdout");
 
 	fd_table[0].fp = stdin;
-	fd_table[0].filename = "stdin";
+	fd_table[0].filename = strdup("stdin");
 	fd_table[1].fp = stdout;
-	fd_table[1].filename = "stdout";
+	fd_table[1].filename = strdup("stdout");
 	fd_table[2].fp = stderr;
-	fd_table[2].filename = "stderr";
+	fd_table[2].filename = strdup("stderr");
 
 	logfile = log;
 }
 
+#ifdef CHECK_WITH_VALGRIND
+void vpi_mcd_delete(void)
+{
+      free(mcd_table[0].filename);
+      mcd_table[0].filename = 0;
+      mcd_table[0].fp = 0;
+
+      free(fd_table[0].filename);
+      fd_table[0].filename = 0;
+      fd_table[0].fp = 0;
+
+      free(fd_table[1].filename);
+      fd_table[1].filename = 0;
+      fd_table[1].fp = 0;
+
+      free(fd_table[2].filename);
+      fd_table[2].filename = 0;
+      fd_table[2].fp = 0;
+}
+#endif
+
 /*
  * close one or more channels.  we silently refuse to close the preopened ones.
  */
@@ -152,7 +174,15 @@ vpi_mcd_vprintf(PLI_UINT32 mcd, const char*fmt, va_list ap)
 		    mcd, fmt);
       }
 
+#ifdef __MINGW32__
+	/*
+	 * The MinGW runtime (version 3.14) fixes some things, but breaks
+	 * %f for us, so we have to us the underlying version.
+	 */
+      rc = _vsnprintf(buffer, sizeof buffer, fmt, ap);
+#else
       rc = vsnprintf(buffer, sizeof buffer, fmt, ap);
+#endif
 
       for(int i = 0; i < 31; i++) {
 	    if((mcd>>i) & 1) {
@@ -200,6 +230,14 @@ extern "C" PLI_INT32 vpi_mcd_flush(PLI_UINT32 mcd)
 /*
  * MCD/FD Extensions
  */
+
+/*
+ * The vpi_fopen function opens a file with the given path, and
+ * returns a file descriptor that includes bit 31 set. This is to
+ * differentiate the fd from a mcd descriptor. Note that these
+ * descriptors are distinct from the mcd descriptors, so uses a
+ * different fd table.
+ */
 extern "C" PLI_INT32 vpi_fopen(const char*name, const char*mode)
 {
 	unsigned i;
@@ -222,38 +260,9 @@ extern "C" FILE *vpi_get_file(PLI_INT32 fd)
 	// Only deal with FD's
 	if (IS_MCD(fd)) return NULL;
 
-	// Only know about FD_MAX indicies
+	// Only know about FD_MAX indices
 	if (FD_IDX(fd) >= FD_MAX) return NULL;
 
 	return fd_table[FD_IDX(fd)].fp;
 }
 
-/*
- * $Log: vpi_mcd.cc,v $
- * Revision 1.12  2004/10/04 01:10:59  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.11  2003/07/21 01:20:59  steve
- *  vpi_mcd_vprintf can only scan the va_list once.
- *
- * Revision 1.10  2003/05/23 04:04:02  steve
- *  Add vpi_fopen and vpi_get_file.
- *
- * Revision 1.9  2003/05/15 16:51:09  steve
- *  Arrange for mcd id=00_00_00_01 to go to stdout
- *  as well as a user specified log file, set log
- *  file to buffer lines.
- *
- *  Add vpi_flush function, and clear up some cunfused
- *  return codes from other vpi functions.
- *
- *  Adjust $display and vcd/lxt messages to use the
- *  standard output/log file.
- *
- * Revision 1.8  2002/08/12 01:35:09  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.7  2002/08/11 23:47:05  steve
- *  Add missing Log and Ident strings.
- *
- */
