@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2008-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,9 +16,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_priv.cc,v 1.46 2004/05/19 03:26:24 steve Exp $"
-#endif
 
 # include  "vpi_priv.h"
 # include  "schedule.h"
@@ -40,7 +37,7 @@ static s_vpi_error_info vpip_last_error = { 0, 0, 0, 0, 0, 0, 0 };
 
 /*
  * The vpip_string function creates a constant string from the pass
- * input. This constant string is permanently allocate from an
+ * input. This constant string is permanently allocated from an
  * efficient string buffer store.
  */
 struct vpip_string_chunk {
@@ -48,6 +45,36 @@ struct vpip_string_chunk {
       char data[64*1024 - sizeof (struct vpip_string_chunk*)];
 };
 
+unsigned vpip_size(__vpiSignal *sig)
+{
+      return abs(sig->msb - sig->lsb) + 1;
+}
+
+struct __vpiScope* vpip_scope(__vpiSignal*sig)
+{
+      if (sig->is_netarray)
+	    return  (struct __vpiScope*) vpi_handle(vpiScope, sig->within.parent);
+      else
+	    return sig->within.scope;
+}
+
+struct __vpiScope* vpip_scope(__vpiRealVar*sig)
+{
+      if (sig->is_netarray)
+	    return  (struct __vpiScope*) vpi_handle(vpiScope, sig->within.parent);
+      else
+	    return sig->within.scope;
+}
+
+vpiHandle vpip_module(struct __vpiScope*scope)
+{
+      while(scope && scope->base.vpi_type->type_code != vpiModule) {
+	    scope = scope->scope;
+      }
+      assert(scope);
+      return &scope->base;
+}
+
 const char *vpip_string(const char*str)
 {
       static vpip_string_chunk first_chunk = {0, {0}};
@@ -113,7 +140,7 @@ PLI_INT32 vpi_chk_error(p_vpi_error_info info)
       info->level = vpip_last_error.level;
       info->message = vpip_last_error.message;
       info->product = vpi_vlog_info.product;
-      info->code = "";
+      info->code = (char *) "";
       info->file = 0;
       info->line = 0;
 
@@ -151,6 +178,7 @@ static int vpip_get_global(int property)
 {
       switch (property) {
 
+	  case vpiTimeUnit:
 	  case vpiTimePrecision:
 	    return vpip_get_time_precision();
 
@@ -173,6 +201,8 @@ static const char* vpi_property_str(PLI_INT32 code)
 	    return "vpiFullName";
 	  case vpiTimeUnit:
 	    return "vpiTimeUnit";
+	  case vpiTimePrecision:
+	    return "vpiTimePrecision";
 	  default:
 	    sprintf(buf, "%d", code);
       }
@@ -185,12 +215,44 @@ static const char* vpi_type_values(PLI_INT32 code)
       switch (code) {
 	  case vpiConstant:
 	    return "vpiConstant";
+	  case vpiFunction:
+	    return "vpiFunction";
+	  case vpiIntegerVar:
+	    return "vpiIntegerVar";
+	  case vpiIterator:
+	    return "vpiIterator";
+	  case vpiMemory:
+	    return "vpiMemory";
+	  case vpiMemoryWord:
+	    return "vpiMemoryWord";
 	  case vpiModule:
 	    return "vpiModule";
+	  case vpiNamedBegin:
+	    return "vpiNamedBegin";
+	  case vpiNamedEvent:
+	    return "vpiNamedEvent";
+	  case vpiNamedFork:
+	    return "vpiNamedFork";
 	  case vpiNet:
 	    return "vpiNet";
+	  case vpiNetArray:
+	    return "vpiNetArray";
+	  case vpiParameter:
+	    return "vpiParameter";
+	  case vpiPartSelect:
+	    return "vpiPartSelect";
+	  case vpiRealVar:
+	    return "vpiRealVar";
 	  case vpiReg:
 	    return "vpiReg";
+	  case vpiSysFuncCall:
+	    return "vpiSysFuncCall";
+	  case vpiSysTaskCall:
+	    return "vpiSysTaskCall";
+	  case vpiTask:
+	    return "vpiTask";
+	  case vpiTimeVar:
+	    return "vpiTimeVar";
 	  default:
 	    sprintf(buf, "%d", code);
       }
@@ -211,6 +273,8 @@ PLI_INT32 vpi_get(int property, vpiHandle ref)
 	    struct __vpiSignal*rfp = (struct __vpiSignal*)ref;
 	    if (ref->vpi_type->type_code == vpiReg && rfp->isint_)
 		  return vpiIntegerVar;
+	    else if (ref->vpi_type->type_code == vpiMemory && is_net_array(ref))
+		  return vpiNetArray;
 	    else
 		  return ref->vpi_type->type_code;
       }
@@ -242,7 +306,23 @@ char* vpi_get_str(PLI_INT32 property, vpiHandle ref)
 	    return 0;
       }
 
-      assert(ref);
+      if (property == vpiType) {
+	    if (vpi_trace) {
+		  fprintf(vpi_trace, "vpi_get(vpiType, %p) --> %s\n",
+			  ref, vpi_type_values(ref->vpi_type->type_code));
+	    }
+
+	    struct __vpiSignal*rfp = (struct __vpiSignal*)ref;
+            PLI_INT32 type;
+	    if (ref->vpi_type->type_code == vpiReg && rfp->isint_)
+		  type = vpiIntegerVar;
+	    else if (ref->vpi_type->type_code == vpiMemory && is_net_array(ref))
+		  type = vpiNetArray;
+	    else
+		  type = ref->vpi_type->type_code;
+	    return (char *)vpi_type_values(type);
+      }
+
       if (ref->vpi_type->vpi_get_str_ == 0) {
 	    if (vpi_trace) {
 		  fprintf(vpi_trace, "vpi_get_str(%s, %p) --X\n",
@@ -281,12 +361,45 @@ int vpip_time_units_from_handle(vpiHandle obj)
 
 	  case vpiNet:
 	  case vpiReg:
-	    signal = (struct __vpiSignal*)obj;
- 	    return signal->scope->time_units;
+	    signal = vpip_signal_from_handle(obj);
+	    scope = vpip_scope(signal);
+	    return scope->time_units;
 
 	  default:
-	    fprintf(stderr, "ERROR: vpi_get_time called with object "
-		    "handle type=%u\n", obj->vpi_type->type_code);
+	    fprintf(stderr, "ERROR: vpip_time_units_from_handle called with "
+		    "object handle type=%u\n", obj->vpi_type->type_code);
+	    assert(0);
+	    return 0;
+      }
+}
+
+int vpip_time_precision_from_handle(vpiHandle obj)
+{
+      struct __vpiSysTaskCall*task;
+      struct __vpiScope*scope;
+      struct __vpiSignal*signal;
+
+      if (obj == 0)
+	    return vpip_get_time_precision();
+
+      switch (obj->vpi_type->type_code) {
+	  case vpiSysTaskCall:
+	    task = (struct __vpiSysTaskCall*)obj;
+	    return task->scope->time_precision;
+
+	  case vpiModule:
+	    scope = (struct __vpiScope*)obj;
+	    return scope->time_precision;
+
+	  case vpiNet:
+	  case vpiReg:
+	    signal = vpip_signal_from_handle(obj);
+	    scope = vpip_scope(signal);
+	    return scope->time_precision;
+
+	  default:
+	    fprintf(stderr, "ERROR: vpip_time_precision_from_handle called "
+		    "with object handle type=%u\n", obj->vpi_type->type_code);
 	    assert(0);
 	    return 0;
       }
@@ -309,7 +422,7 @@ void vpi_get_time(vpiHandle obj, s_vpi_time*vp)
 
           case vpiScaledRealTime:
 	    units = vpip_time_units_from_handle(obj);
-            vp->real = pow(10, vpip_get_time_precision() - units);
+            vp->real = pow(10.0L, vpip_get_time_precision() - units);
             vp->real *= time;
 	    break;
 
@@ -332,8 +445,10 @@ PLI_INT32 vpi_get_vlog_info(p_vpi_vlog_info vlog_info_p)
 
 void vpi_set_vlog_info(int argc, char** argv)
 {
-    vpi_vlog_info.product = "Icarus Verilog";
-    vpi_vlog_info.version = "$Name:  $";
+    static char icarus_product[] = "Icarus Verilog";
+    static char icarus_version[] = "$Name:  $";
+    vpi_vlog_info.product = icarus_product;
+    vpi_vlog_info.version = icarus_version;
     vpi_vlog_info.argc    = argc;
     vpi_vlog_info.argv    = argv;
 
@@ -353,6 +468,191 @@ void vpi_set_vlog_info(int argc, char** argv)
     }
 }
 
+/*
+ * This is a generic function to convert a vvp_vector4_t value into a
+ * vpi_value structure. The format is selected by the format of the
+ * value pointer. The width is the real width of the word, in case the
+ * word_val width is not accurate.
+ */
+
+static void vec4_get_value_string(const vvp_vector4_t&word_val, unsigned width,
+				  s_vpi_value*vp);
+
+void vpip_vec4_get_value(const vvp_vector4_t&word_val, unsigned width,
+			 bool signed_flag, s_vpi_value*vp)
+{
+      char *rbuf = 0;
+
+      switch (vp->format) {
+	  default:
+	    fprintf(stderr, "internal error: Format %d not implemented\n",
+		    vp->format);
+	    assert(0 && "format not implemented");
+
+	  case vpiObjTypeVal:
+	    vp->format = vpiVectorVal;
+	    break;
+
+	  case vpiSuppressVal:
+	    break;
+
+	  case vpiBinStrVal:
+	    rbuf = need_result_buf(width+1, RBUF_VAL);
+	    for (unsigned idx = 0 ;  idx < width ;  idx += 1) {
+		  vvp_bit4_t bit = word_val.value(idx);
+		  rbuf[width-idx-1] = vvp_bit4_to_ascii(bit);
+	    }
+	    rbuf[width] = 0;
+	    vp->value.str = rbuf;
+	    break;
+
+	  case vpiOctStrVal: {
+		unsigned hwid = (width+2) / 3;
+		rbuf = need_result_buf(hwid+1, RBUF_VAL);
+		vpip_vec4_to_oct_str(word_val, rbuf, hwid+1, signed_flag);
+		vp->value.str = rbuf;
+		break;
+	  }
+
+	  case vpiDecStrVal: {
+		rbuf = need_result_buf(width+1, RBUF_VAL);
+		vpip_vec4_to_dec_str(word_val, rbuf, width+1, signed_flag);
+		vp->value.str = rbuf;
+		break;
+	  }
+
+	  case vpiHexStrVal: {
+		unsigned  hwid = (width + 3) / 4;
+
+		rbuf = need_result_buf(hwid+1, RBUF_VAL);
+		rbuf[hwid] = 0;
+
+		vpip_vec4_to_hex_str(word_val, rbuf, hwid+1, signed_flag);
+		vp->value.str = rbuf;
+		break;
+	  }
+
+          case vpiScalarVal: {
+	        // scalars should be of size 1
+	        assert(width == 1);
+	        switch(word_val.value(0)) {
+		    case BIT4_0:
+		      vp->value.scalar = vpi0;
+		      break;
+	            case BIT4_1:
+                      vp->value.scalar = vpi1;
+                      break;
+	            case BIT4_X:
+                      vp->value.scalar = vpiX;
+                      break;
+	            case BIT4_Z:
+                      vp->value.scalar = vpiZ;
+                      break;
+		}
+                break;
+          }
+
+	  case vpiIntVal: {
+		long val = 0;
+		vvp_bit4_t pad = BIT4_0;
+		if (signed_flag && word_val.size() > 0)
+		      pad = word_val.value(word_val.size()-1);
+
+		for (unsigned idx = 0 ; idx < 8*sizeof(val) ;  idx += 1) {
+		      vvp_bit4_t val4 = pad;
+		      if (idx < word_val.size())
+			    val4 = word_val.value(idx);
+		      if (val4 == BIT4_1)
+			    val |= 1L << idx;
+		}
+
+		vp->value.integer = val;
+		break;
+	  }
+
+	  case vpiVectorVal: {
+		unsigned hwid = (width - 1)/32 + 1;
+
+		rbuf = need_result_buf(hwid * sizeof(s_vpi_vecval), RBUF_VAL);
+		s_vpi_vecval *op = (p_vpi_vecval)rbuf;
+		vp->value.vector = op;
+
+		op->aval = op->bval = 0;
+		for (unsigned idx = 0 ;  idx < width ;  idx += 1) {
+		      switch (word_val.value(idx)) {
+			  case BIT4_0:
+			    op->aval &= ~(1 << idx % 32);
+			    op->bval &= ~(1 << idx % 32);
+			    break;
+			  case BIT4_1:
+			    op->aval |=  (1 << idx % 32);
+			    op->bval &= ~(1 << idx % 32);
+			    break;
+			  case BIT4_X:
+			    op->aval |= (1 << idx % 32);
+			    op->bval |= (1 << idx % 32);
+			    break;
+			  case BIT4_Z:
+			    op->aval &= ~(1 << idx % 32);
+			    op->bval |=  (1 << idx % 32);
+			    break;
+		      }
+		      if (!((idx+1) % 32) && (idx+1 < width)) {
+			    op++;
+			    op->aval = op->bval = 0;
+		      }
+		}
+		break;
+	  }
+
+	  case vpiStringVal:
+	    vec4_get_value_string(word_val, width, vp);
+	    break;
+
+	  case vpiRealVal: {
+		vector4_to_value(word_val, vp->value.real, signed_flag);
+		break;
+	  }
+      }
+}
+
+static void vec4_get_value_string(const vvp_vector4_t&word_val, unsigned width,
+				  s_vpi_value*vp)
+{
+      unsigned nchar = width / 8;
+      unsigned tail = width % 8;
+
+      char*rbuf = need_result_buf(nchar + 1, RBUF_VAL);
+      char*cp = rbuf;
+
+      if (tail > 0) {
+	    char char_val = 0;
+	    for (unsigned idx = width-tail; idx < width ;  idx += 1) {
+		  vvp_bit4_t val = word_val.value(idx);
+		  if (val == BIT4_1)
+			char_val |= 1 << idx;
+	    }
+
+	    if (char_val != 0)
+		  *cp++ = char_val;
+      }
+
+      for (unsigned idx = 0 ;  idx < nchar ;  idx += 1) {
+	    unsigned bit = (nchar - idx - 1) * 8;
+	    char char_val = 0;
+	    for (unsigned bdx = 0 ;  bdx < 8 ;  bdx += 1) {
+		  vvp_bit4_t val = word_val.value(bit+bdx);
+		  if (val == BIT4_1)
+			char_val |= 1 << bdx;
+	    }
+	    if (char_val != 0)
+		  *cp++ = char_val;
+      }
+
+      *cp = 0;
+      vp->value.str = rbuf;
+}
+
 void vpi_get_value(vpiHandle expr, s_vpi_value*vp)
 {
       assert(expr);
@@ -394,13 +694,30 @@ void vpi_get_value(vpiHandle expr, s_vpi_value*vp)
 struct vpip_put_value_event : vvp_gen_event_s {
       vpiHandle handle;
       s_vpi_value value;
+      int flags;
+      virtual void run_run();
+      ~vpip_put_value_event() { }
 };
 
-static void vpip_put_value_callback(vvp_gen_event_t eobj, unsigned char)
+void vpip_put_value_event::run_run()
 {
-      vpip_put_value_event*put = (vpip_put_value_event*)eobj;
-
-      put->handle->vpi_type->vpi_put_value_ (put->handle, &put->value);
+      handle->vpi_type->vpi_put_value_ (handle, &value, flags);
+      switch (value.format) {
+	    /* Free the copied string. */
+	  case vpiBinStrVal:
+	  case vpiOctStrVal:
+	  case vpiDecStrVal:
+	  case vpiHexStrVal:
+	  case vpiStringVal:
+	    free(value.value.str);
+	    break;
+	    /* If these are every copied then free them too. */
+	  case vpiTimeVal:
+	  case vpiVectorVal:
+	  case vpiStrengthVal:
+	  default:
+	    break;
+      }
 }
 
 vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
@@ -411,33 +728,61 @@ vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
       if (obj->vpi_type->vpi_put_value_ == 0)
 	    return 0;
 
-      if (flags != vpiNoDelay) {
+      flags &= ~vpiReturnEvent;
+
+      if (flags!=vpiNoDelay && flags!=vpiForceFlag && flags!=vpiReleaseFlag) {
 	    vvp_time64_t dly;
 
- 	    switch (when->type) {
- 		case vpiScaledRealTime:
- 		  dly = (vvp_time64_t)(when->real *
-				       (pow(10,
+            if (vpi_get(vpiAutomatic, obj)) {
+                  fprintf(stderr, "vpi error: cannot put a value with "
+                                  "a delay on automatically allocated "
+                                  "variable '%s'\n",
+                                  vpi_get_str(vpiName, obj));
+                  return 0;
+            }
+
+	    assert(when != 0);
+
+	    switch (when->type) {
+		case vpiScaledRealTime:
+		  dly = (vvp_time64_t)(when->real *
+				       (pow(10.0L,
 					    vpip_time_units_from_handle(obj) -
 					    vpip_get_time_precision())));
- 		  break;
- 		case vpiSimTime:
+		  break;
+		case vpiSimTime:
 		  dly = vpip_timestruct_to_time(when);
- 		  break;
- 		default:
+		  break;
+		default:
 		  dly = 0;
 		  break;
- 	    }
+	    }
 
 	    vpip_put_value_event*put = new vpip_put_value_event;
 	    put->handle = obj;
 	    put->value = *vp;
-	    put->run = &vpip_put_value_callback;
-	    schedule_generic(put, 0, dly, false);
+	    switch (put->value.format) {
+		  /* If this is scheduled make a copy of the string. */
+		case vpiBinStrVal:
+		case vpiOctStrVal:
+		case vpiDecStrVal:
+		case vpiHexStrVal:
+		case vpiStringVal:
+		  put->value.value.str = strdup(put->value.value.str);
+		  break;
+		  /* Do these also need to be copied? */
+		case vpiTimeVal:
+		case vpiVectorVal:
+		case vpiStrengthVal:
+		default:
+		  break;
+	    }
+	    put->flags = flags;
+	    schedule_generic(put, dly, false, true, true);
 	    return 0;
       }
 
-      (obj->vpi_type->vpi_put_value_)(obj, vp);
+      (obj->vpi_type->vpi_put_value_)(obj, vp, flags);
 
       return 0;
 }
@@ -588,7 +933,10 @@ static vpiHandle find_scope(const char *name, vpiHandle handle, int depth)
 		  rtn = find_scope(cp+1, hand, depth + 1);
 
 	    /* found it yet ? */
-	    if (rtn) break;
+	    if (rtn) {
+		  vpi_free_object(iter);
+		  break;
+	    }
       }
 
       /* matched up to here */
@@ -613,10 +961,22 @@ vpiHandle vpi_handle_by_name(const char *name, vpiHandle scope)
        * traverse the hierarchy specified in name to find the leaf module
        * and try finding it there.
        */
-      if (scope)
-	    hand = vpi_handle(vpiModule, scope);
-      else
+      if (scope) {
+	    /* Some implementations support either a module or a scope. */
+	    switch (vpi_get(vpiType, scope)) {
+		case vpiScope:
+	          hand = vpi_handle(vpiModule, scope);
+	          break;
+		case vpiModule:
+	          hand = scope;
+	          break;
+		default:
+	          // Use vpi_chk_error() here when it is implemented.
+	          return 0;
+	    }
+      } else {
 	    hand = find_scope(name, NULL, 0);
+      }
 
       if (hand) {
 	    /* remove hierarchical portion of name */
@@ -633,6 +993,70 @@ vpiHandle vpi_handle_by_name(const char *name, vpiHandle scope)
       return 0;
 }
 
+
+/*
+  We increment the two vpi methods to enable the
+  read/write of SDF delay values from/into
+  the modpath vpiHandle
+
+
+  basically, they will redirect the generic vpi_interface
+
+  vpi_get_delay ( .. )
+  vpi_put_delay ( .. )
+
+
+  to the
+
+  modpath_get_delay ( .. ) ;
+  modpath_put_delay ( .. ) ;
+
+*/
+
+
+
+void vpi_get_delays(vpiHandle expr, p_vpi_delay delays)
+{
+      assert(expr);
+      assert(delays);
+
+      if (expr->vpi_type->vpi_get_delays_)
+	{
+	  (expr->vpi_type->vpi_get_delays_)(expr, delays);
+
+	  if (vpi_trace)
+	    {
+	      fprintf(vpi_trace,
+		      "vpi_get_delays(%p, %p) -->\n", expr, delays);
+	    }
+	}
+}
+
+
+void vpi_put_delays(vpiHandle expr, p_vpi_delay delays)
+{
+      assert(expr  );
+      assert(delays );
+
+      if (expr->vpi_type->vpi_put_delays_)
+	{
+	  (expr->vpi_type->vpi_put_delays_)(expr, delays);
+
+	  if (vpi_trace)
+	    {
+	      fprintf(vpi_trace,
+		      "vpi_put_delays(%p, %p) -->\n", expr, delays);
+	    }
+	}
+}
+
+
+
+
+
+
+
+
 extern "C" PLI_INT32 vpi_vprintf(const char*fmt, va_list ap)
 {
       return vpi_mcd_vprintf(1, fmt, ap);
@@ -655,16 +1079,21 @@ extern "C" PLI_INT32 vpi_flush(void)
 
 extern "C" void vpi_sim_vcontrol(int operation, va_list ap)
 {
+      long diag_msg;
+
       switch (operation) {
 	  case vpiFinish:
-	    schedule_finish(0);
+            diag_msg = va_arg(ap, long);
+	    schedule_finish(diag_msg);
 	    break;
 
 	  case vpiStop:
-	    schedule_stop(0);
+            diag_msg = va_arg(ap, long);
+	    schedule_stop(diag_msg);
 	    break;
 
 	  default:
+	    fprintf(stderr, "Unsupported operation %d.\n", operation);
 	    assert(0);
       }
 }
@@ -686,99 +1115,61 @@ extern "C" void vpi_control(PLI_INT32 operation, ...)
 }
 
 /*
- * $Log: vpi_priv.cc,v $
- * Revision 1.46  2004/05/19 03:26:24  steve
- *  Support delayed/non-blocking assignment to reals and others.
- *
- * Revision 1.45  2004/02/18 17:52:00  steve
- *  PRototypes match the standard.
- *
- * Revision 1.44  2004/02/18 02:51:59  steve
- *  Fix type mismatches of various VPI functions.
- *
- * Revision 1.43  2003/06/25 04:04:19  steve
- *  Fix mingw portability problems.
- *
- * Revision 1.42  2003/06/22 04:19:26  steve
- *  vpi_handle diagnostic message.
- *
- * Revision 1.41  2003/06/17 16:55:08  steve
- *  1) setlinebuf() for vpi_trace
- *  2) Addes error checks for trace file opens
- *  3) removes now extraneous flushes
- *  4) fixes acc_next() bug
- *
- * Revision 1.40  2003/05/30 04:08:28  steve
- *  vpi_trace of vpi_free_object.
- *
- * Revision 1.39  2003/05/15 16:51:09  steve
- *  Arrange for mcd id=00_00_00_01 to go to stdout
- *  as well as a user specified log file, set log
- *  file to buffer lines.
- *
- *  Add vpi_flush function, and clear up some cunfused
- *  return codes from other vpi functions.
- *
- *  Adjust $display and vcd/lxt messages to use the
- *  standard output/log file.
- *
- * Revision 1.38  2003/05/15 01:24:46  steve
- *  Return all 64bits of time in vpi_get_time.
- *
- * Revision 1.37  2003/05/02 04:29:57  steve
- *  Add put_value with transport delay.
- *
- * Revision 1.36  2003/04/27 04:19:24  steve
- *  Support vpiScaledRealTime.
- *
- * Revision 1.35  2003/03/14 05:02:34  steve
- *  More detail in vpi tracing.
- *
- * Revision 1.34  2003/03/13 04:34:18  steve
- *  Add VPI_TRACE tracing of VPI calls.
- *  vpi_handle_by_name takes a const char*.
- *
- * Revision 1.33  2003/03/12 02:50:32  steve
- *  Add VPI tracing.
- *
- * Revision 1.32  2003/03/06 04:32:00  steve
- *  Use hashed name strings for identifiers.
- *
- * Revision 1.31  2003/02/21 03:40:35  steve
- *  Add vpiStop and interactive mode.
- *
- * Revision 1.30  2003/02/09 23:33:26  steve
- *  Spelling fixes.
- *
- * Revision 1.29  2003/02/02 01:40:24  steve
- *  Five vpi_free_object a default behavior.
- *
- * Revision 1.28  2003/01/10 19:02:21  steve
- *  Add missing vpi entry points.
- *
- * Revision 1.27  2003/01/10 03:06:32  steve
- *  Remove vpithunk, and move libvpi to vvp directory.
- *
- * Revision 1.26  2002/12/21 00:55:58  steve
- *  The $time system task returns the integer time
- *  scaled to the local units. Change the internal
- *  implementation of vpiSystemTime the $time functions
- *  to properly account for this. Also add $simtime
- *  to get the simulation time.
- *
- * Revision 1.25  2002/12/11 23:55:22  steve
- *  Add vpi_handle_by_name to the VPI interface,
- *  and bump the vpithunk magic number.
- *
- * Revision 1.24  2002/08/24 05:02:58  steve
- *  Fix = vs == error.
- *
- * Revision 1.23  2002/08/12 01:35:09  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.22  2002/07/19 01:57:26  steve
- *  Add vpi_chk_error and vpi_control functions.
- *
- * Revision 1.21  2002/07/19 01:12:50  steve
- *  vpi_iterate returns 0 on error.
+ * This routine calculated the return value for $clog2.
+ * It is easier to do it here vs trying to to use the VPI interface.
  */
+extern "C" s_vpi_vecval vpip_calc_clog2(vpiHandle arg)
+{
+      s_vpi_vecval rtn;
+      s_vpi_value val;
+      vvp_vector4_t vec4;
+      bool is_neg = false;  // At this point only a real can be negative.
+
+	/* Get the value as a vvp_vector4_t. */
+      val.format = vpiObjTypeVal;
+      vpi_get_value(arg, &val);
+      if (val.format == vpiRealVal) {
+	    vpi_get_value(arg, &val);
+	      /* All double values can be represented in 1024 bits. */
+	    vec4 = vvp_vector4_t(1024, val.value.real);
+	    if (val.value.real < 0) is_neg = true;
+      } else {
+	    val.format = vpiVectorVal;
+	    vpi_get_value(arg, &val);
+	    unsigned wid = vpi_get(vpiSize, arg);
+	    vec4 = vvp_vector4_t(wid, BIT4_0);
+	    for (unsigned idx=0; idx < wid; idx += 1) {
+		PLI_INT32 aval = val.value.vector[idx/32].aval;
+		PLI_INT32 bval = val.value.vector[idx/32].bval;
+		aval >>= idx % 32;
+		bval >>= idx % 32;
+		int bitmask = (aval&1) | ((bval<<1)&2);
+		vvp_bit4_t bit = scalar_to_bit4(bitmask);
+		vec4.set_bit(idx, bit);
+	    }
+      }
+
+      if (vec4.has_xz()) {
+	    rtn.aval = rtn.bval = 0xFFFFFFFFU;  /* Set to 'bx. */
+	    return rtn;
+      }
+
+      vvp_vector2_t vec2(vec4);
+
+      if (is_neg) vec2.trim_neg(); /* This is a special trim! */
+      else vec2.trim(); /* This makes less work shifting. */
+
+	/* Calculate the clog2 result. */
+      PLI_INT32 res = 0;
+      if (!vec2.is_zero()) {
+	    vec2 -= vvp_vector2_t(1, vec2.size());
+	    while(!vec2.is_zero()) {
+		  res += 1;
+		  vec2 >>= 1;
+	    }
+      }
+
+      rtn.aval = res;
+      rtn.bval = 0;
+      return rtn;
+}
