@@ -1,7 +1,7 @@
 #ifndef __vpi_priv_H
 #define __vpi_priv_H
 /*
- * Copyright (c) 2001-2003 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,13 +18,18 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_priv.h,v 1.59.2.1 2006/02/19 16:57:31 steve Exp $"
-#endif
 
 # include  "vpi_user.h"
 # include  "pointers.h"
-# include  "memory.h"
+# include  "vvp_net.h"
+# include  "config.h"
+
+/*
+ * Added to use some "vvp_fun_modpath_src"
+ * and "vvp_fun_modpath" classes definitions
+ */
+#include  "delay.h"
+
 
 /*
  * This header file contains the internal definitions that the vvp
@@ -32,6 +37,18 @@
  * header file elsewhere.
  */
 
+/*
+ * Private VPI properties that are only used in the cleanup code.
+ */
+#ifdef CHECK_WITH_VALGRIND
+#define _vpiFromThr 0x1000001
+#   define _vpiNoThr  0
+#   define _vpiVThr   1
+#   define _vpiWord   2
+#   define _vpi_at_PV 3
+#   define _vpi_at_A  4
+#endif
+
 
 /*
  * The vpi_mode_flag contains the major mode for VPI use. This is used
@@ -74,7 +91,7 @@ struct __vpirt {
       int   (*vpi_get_)(int, vpiHandle);
       char* (*vpi_get_str_)(int, vpiHandle);
       void  (*vpi_get_value_)(vpiHandle, p_vpi_value);
-      vpiHandle (*vpi_put_value_)(vpiHandle, p_vpi_value);
+      vpiHandle (*vpi_put_value_)(vpiHandle, p_vpi_value, int flags);
 
 	/* These methods follow references. */
       vpiHandle (*handle_)(int, vpiHandle);
@@ -83,8 +100,23 @@ struct __vpirt {
 
 	/* This implements the vpi_free_object method. */
       int (*vpi_free_object_)(vpiHandle);
+
+       /*
+	 These two methods are used to read/write delay
+	 values from/into modpath records
+       */
+      void  (*vpi_get_delays_)(vpiHandle, p_vpi_delay);
+      void  (*vpi_put_delays_)(vpiHandle, p_vpi_delay);
 };
 
+/*
+ * In general a vpi object is a structure that contains the member
+ * "base" that is a __vpiHandle object. This template can convert any
+ * of those structures into a vpiHandle object.
+ */
+template <class T> vpiHandle vpi_handle(T obj)
+{ return &obj->base; }
+
 /*
  * The vpiHandle for an iterator has this structure. The definition of
  * the methods lives in vpi_iter.c
@@ -117,15 +149,20 @@ struct __vpiCallback {
 	// user supplied callback data
       struct t_cb_data cb_data;
       struct t_vpi_time cb_time;
+      struct t_vpi_value cb_value;
 
 	// scheduled event
       struct sync_cb* cb_sync;
 
+	// The callback holder may use this for various purposes.
+      long extra_data;
+
 	// Used for listing callbacks.
       struct __vpiCallback*next;
 };
 
 extern struct __vpiCallback* new_vpi_callback();
+extern void delete_vpi_callback(struct __vpiCallback* ref);
 extern void callback_execute(struct __vpiCallback*cur);
 
 struct __vpiSystemTime {
@@ -144,19 +181,36 @@ struct __vpiScope {
 	/* The scope has a name. */
       const char*name;
       const char*tname;
+      unsigned file_idx;
+      unsigned lineno;
+      unsigned def_file_idx;
+      unsigned def_lineno;
+      bool is_automatic;
 	/* The scope has a system time of its own. */
       struct __vpiSystemTime scoped_time;
+      struct __vpiSystemTime scoped_stime;
       struct __vpiSystemTime scoped_realtime;
 	/* Keep an array of internal scope items. */
       struct __vpiHandle**intern;
       unsigned nintern;
+        /* Keep an array of items to be automatically allocated */
+      struct automatic_hooks_s**item;
+      unsigned nitem;
+        /* Keep a list of live contexts. */
+      vvp_context_t live_contexts;
+        /* Keep a list of freed contexts. */
+      vvp_context_t free_contexts;
 	/* Keep a list of threads in the scope. */
       vthread_t threads;
       signed int time_units :8;
+      signed int time_precision :8;
 };
 
 extern struct __vpiScope* vpip_peek_current_scope(void);
 extern void vpip_attach_to_current_scope(vpiHandle obj);
+extern struct __vpiScope* vpip_peek_context_scope(void);
+extern unsigned vpip_add_item_to_context(automatic_hooks_s*item,
+                                         struct __vpiScope*scope);
 extern vpiHandle vpip_make_root_iterator(void);
 extern void vpip_make_root_iterator(struct __vpiHandle**&table,
 				    unsigned&ntable);
@@ -168,25 +222,118 @@ extern void vpip_make_root_iterator(struct __vpiHandle**&table,
  */
 struct __vpiSignal {
       struct __vpiHandle base;
-      struct __vpiScope* scope;
-	/* The name of this reg/net object */
-      const char*name;
+#ifdef CHECK_WITH_VALGRIND
+      struct __vpiSignal *pool;
+#endif
+      union { // The scope or parent array that contains me.
+	    vpiHandle parent;
+	    struct __vpiScope* scope;
+      } within;
+      union { // The name of this reg/net, or the index for array words.
+            const char*name;
+            vpiHandle index;
+      } id;
 	/* The indices that define the width and access offset. */
       int msb, lsb;
 	/* Flags */
       unsigned signed_flag  : 1;
-      unsigned isint_ : 1;	// original type was integer
+      unsigned isint_       : 1; // original type was integer
+      unsigned is_netarray  : 1; // This is word of a net array
 	/* The represented value is here. */
-      vvp_fvector_t bits;
-        /* This is the callback event functor */
-      struct callback_functor_s *callback;
+      vvp_net_t*node;
 };
+extern unsigned vpip_size(__vpiSignal *sig);
+extern struct __vpiScope* vpip_scope(__vpiSignal*sig);
+
 extern vpiHandle vpip_make_int(const char*name, int msb, int lsb,
-			       vvp_fvector_t vec);
+			       vvp_net_t*vec);
 extern vpiHandle vpip_make_reg(const char*name, int msb, int lsb,
-			       bool signed_flag, vvp_fvector_t vec);
+			       bool signed_flag, vvp_net_t*net);
 extern vpiHandle vpip_make_net(const char*name, int msb, int lsb,
-			       bool signed_flag, vvp_fvector_t vec);
+			       bool signed_flag, vvp_net_t*node);
+
+/*
+ * This is used by system calls to represent a bit/part select of
+ * a simple variable or constant array word.
+ */
+struct __vpiPV {
+      struct __vpiHandle base;
+      vpiHandle parent;
+      vvp_net_t*net;
+      vpiHandle sbase;
+      int tbase;
+      unsigned twid, width;
+};
+extern vpiHandle vpip_make_PV(char*name, int base, int width);
+extern vpiHandle vpip_make_PV(char*name, char*symbol, int width);
+extern vpiHandle vpip_make_PV(char*name, vpiHandle handle, int width);
+extern vpiHandle vpip_make_PV(char*name, int tbase, int twid, int width);
+
+extern struct __vpiPV* vpip_PV_from_handle(vpiHandle obj);
+extern void vpip_part_select_value_change(struct __vpiCallback*cbh, vpiHandle obj);
+
+
+/*
+ * This function safely converts a vpiHandle back to a
+ * __vpiSignal. Return a nil if the type is not appropriate.
+ */
+extern __vpiSignal* vpip_signal_from_handle(vpiHandle obj);
+
+
+struct __vpiModPathTerm {
+      struct __vpiHandle base;
+      vpiHandle expr;
+	/* The value returned by vpi_get(vpiEdge, ...); */
+      int edge;
+};
+
+struct __vpiModPathSrc {
+      struct __vpiHandle   base;
+      struct __vpiModPath *dest;
+      int   type;
+
+	/* This is the input expression for this modpath. */
+      struct __vpiModPathTerm path_term_in;
+
+	/* This is the input net for the modpath. signals on this net
+	   are used to determine the modpath. They are *not* propagated
+	   anywhere. */
+      vvp_net_t *net;
+} ;
+
+
+/*
+ *
+ * The vpiMoaPath vpiHandle will define
+ * a vpiModPath of record .modpath as defined
+ * in the IEEE 1364
+ *
+ */
+
+struct __vpiModPath {
+      struct __vpiScope   *scope ;
+
+      class vvp_fun_modpath*modpath;
+
+      struct __vpiModPathTerm path_term_out;
+      vvp_net_t *input_net  ;
+};
+
+extern struct __vpiModPathTerm* vpip_modpath_term_from_handle(vpiHandle ref);
+extern struct __vpiModPathSrc* vpip_modpath_src_from_handle(vpiHandle ref);
+
+
+/*
+ * The Function is used to create the vpiHandle
+ * for vpiModPath && vpiModPathIn objects
+ */
+
+extern struct __vpiModPathSrc* vpip_make_modpath_src  (struct __vpiModPath*path_dest,
+					 vvp_time64_t use_delay[12] ,
+					 vvp_net_t *net ) ;
+
+extern struct __vpiModPath* vpip_make_modpath(vvp_net_t *net) ;
+
 
 /*
  * These methods support the vpi creation of events. The name string
@@ -200,21 +347,15 @@ struct __vpiNamedEvent {
 	/* Parent scope of this object. */
       struct __vpiScope*scope;
 	/* The functor, used for %set operations. */
-      vvp_ipoint_t funct;
+      vvp_net_t*funct;
 	/* List of callbacks interested in this event. */
       struct __vpiCallback*callbacks;
 };
 
-extern vpiHandle vpip_make_named_event(const char*name, vvp_ipoint_t f);
+extern vpiHandle vpip_make_named_event(const char*name, vvp_net_t*f);
 extern void vpip_run_named_event_callbacks(vpiHandle ref);
 extern void vpip_real_value_change(struct __vpiCallback*cbh,
 				   vpiHandle ref);
-/*
- * Callback for vpiMemory type
- */
-void vpip_memory_value_change(struct __vpiCallback*cbh,
-			      vpiHandle ref);
-
 
 /*
  * Memory is an array of bits that is accessible in N-bit chunks, with
@@ -223,12 +364,29 @@ void vpip_memory_value_change(struct __vpiCallback*cbh,
  * the memory.
  */
 
-extern vpiHandle vpip_make_memory(vvp_memory_t mem);
+extern bool is_net_array(vpiHandle obj);
 
 /*
  * These are the various variable types.
  */
-extern vpiHandle vpip_make_real_var(const char*name);
+struct __vpiRealVar {
+      struct __vpiHandle base;
+      union { // The scope or parent array that contains me.
+	    vpiHandle parent;
+	    struct __vpiScope* scope;
+      } within;
+	/* The name of this variable, or the index for array words. */
+      union {
+            const char*name;
+            vpiHandle index;
+      } id;
+      unsigned is_netarray  : 1; // This is word of a net array
+      vvp_net_t*net;
+};
+
+extern struct __vpiScope* vpip_scope(__vpiRealVar*sig);
+extern vpiHandle vpip_make_real_var(const char*name, vvp_net_t*net);
+extern struct __vpiRealVar* vpip_realvar_from_handle(vpiHandle obj);
 
 /*
  * When a loaded VPI module announces a system task/function, one
@@ -253,6 +411,9 @@ struct __vpiUserSystf {
       s_vpi_systf_data info;
 };
 
+extern struct __vpiUserSystf* vpip_find_systf(const char*name);
+
+
 struct __vpiSysTaskCall {
       struct __vpiHandle base;
       struct __vpiScope* scope;
@@ -264,6 +425,10 @@ struct __vpiSysTaskCall {
 	/* These represent where in the vthread to put the return value. */
       unsigned vbit;
       signed   vwid;
+      class vvp_net_t*fnet;
+      unsigned file_idx;
+      unsigned lineno;
+      bool put_value;
 };
 
 extern struct __vpiSysTaskCall*vpip_cur_task;
@@ -279,19 +444,26 @@ extern struct __vpiSysTaskCall*vpip_cur_task;
 struct __vpiStringConst {
       struct __vpiHandle base;
       char*value;
+      size_t value_len;
 };
 
 vpiHandle vpip_make_string_const(char*text, bool persistent =true);
-vpiHandle vpip_make_string_param(char*name, char*value);
+vpiHandle vpip_make_string_param(char*name, char*value,
+                                 long file_idx, long lineno);
 
 struct __vpiBinaryConst {
       struct __vpiHandle base;
-      unsigned nbits;
-      unsigned char*bits;
-      unsigned signed_flag :1;
+      vvp_vector4_t bits;
+	/* TRUE if this constant is signed. */
+      int signed_flag :1;
+	/* TRUE if this constant has an explicit size (i.e. 19'h0 vs. 'h0) */
+      int sized_flag   :1;
 };
 
-vpiHandle vpip_make_binary_const(unsigned wid, char*bits);
+vpiHandle vpip_make_binary_const(unsigned wid, const char*bits);
+vpiHandle vpip_make_binary_param(char*name, const vvp_vector4_t&bits,
+				 bool signed_flag,
+				 long file_idx, long lineno);
 
 struct __vpiDecConst {
       struct __vpiHandle base;
@@ -301,6 +473,15 @@ struct __vpiDecConst {
 vpiHandle vpip_make_dec_const(int value);
 vpiHandle vpip_make_dec_const(struct __vpiDecConst*obj, int value);
 
+struct __vpiRealConst {
+      struct __vpiHandle base;
+      double value;
+};
+
+vpiHandle vpip_make_real_const(double value);
+vpiHandle vpip_make_real_param(char*name, double value,
+                               long file_idx, long lineno);
+
 /*
  *  This one looks like a constant, but really is a vector in the current
  *  thread.
@@ -310,6 +491,11 @@ vpiHandle vpip_make_vthr_vector(unsigned base, unsigned wid, bool signed_flag);
 
 vpiHandle vpip_make_vthr_word(unsigned base, const char*type);
 
+vpiHandle vpip_make_vthr_A(char*label, unsigned index);
+vpiHandle vpip_make_vthr_A(char*label, char*symbol);
+vpiHandle vpip_make_vthr_A(char*label, unsigned tbase, unsigned twid);
+vpiHandle vpip_make_vthr_A(char*label, vpiHandle handle);
+
 /*
  * This function is called before any compilation to load VPI
  * modules. This gives the modules a chance to announce their
@@ -342,8 +528,11 @@ extern unsigned vpip_module_path_cnt;
  */
 extern vpiHandle vpip_build_vpi_call(const char*name,
 				     unsigned vbit, int vwid,
+				     class vvp_net_t*fnet,
 				     unsigned argc,
-				     vpiHandle*argv);
+				     vpiHandle*argv,
+				     long file_idx,
+				     long lineno);
 
 extern vthread_t vpip_current_vthread;
 
@@ -355,17 +544,21 @@ extern void vpip_execute_vpi_call(vthread_t thr, vpiHandle obj);
  * and to finish compilation in preparation for execution.
  */
 
-vpiHandle vpip_sim_time(struct __vpiScope*scope);
+vpiHandle vpip_sim_time(struct __vpiScope*scope, bool is_stime);
 vpiHandle vpip_sim_realtime(struct __vpiScope*scope);
 
 extern int vpip_get_time_precision(void);
 extern void vpip_set_time_precision(int pres);
 
 extern int vpip_time_units_from_handle(vpiHandle obj);
+extern int vpip_time_precision_from_handle(vpiHandle obj);
 
 extern void vpip_time_to_timestruct(struct t_vpi_time*ts, vvp_time64_t ti);
 extern vvp_time64_t vpip_timestruct_to_time(const struct t_vpi_time*ts);
 
+extern double vpip_time_to_scaled_real(vvp_time64_t ti, struct __vpiScope*sc);
+extern vvp_time64_t vpip_scaled_real_to_time64(double val, struct __vpiScope*sc);
+
 /*
  * These functions are used mostly as compile time to strings into
  * permallocated memory. The vpip_string function is the most general,
@@ -377,38 +570,32 @@ extern vvp_time64_t vpip_timestruct_to_time(const struct t_vpi_time*ts);
 extern const char* vpip_string(const char*str);
 extern const char* vpip_name_string(const char*str);
 
-/*
-**  Functions defined in vpi_scope.cc, to keep track of functor scope.
-*/
-
-extern vpiHandle ipoint_get_scope(vvp_ipoint_t ipt);
-extern void functor_set_scope(vpiHandle scope);
 
 /*
  * This function is used to make decimal string versions of various
- * vectors. The input format is an array of bit values (0, 1, 2, 3)
- * lsb first, and the result is written into buf, without overflowing
- * nbuf.
+ * vectors. The input format is a vvp_vector4_t, and the result is
+ * written into buf, without overflowing nbuf.
  */
-extern unsigned vpip_bits_to_dec_str(const unsigned char *bits,
-				     unsigned int nbits,
+extern unsigned vpip_vec4_to_dec_str(const vvp_vector4_t&vec4,
 				     char *buf, unsigned int nbuf,
 				     int signed_flag);
 
-extern void vpip_dec_str_to_bits(unsigned char*bits, unsigned nbits,
-				 const char*buf, bool signed_flag);
 
-extern void vpip_bin_str_to_bits(unsigned char*bits, unsigned nbits,
-				 const char*buf, bool signed_flag);
+extern void vpip_vec4_to_hex_str(const vvp_vector4_t&bits, char*buf,
+				 unsigned nbuf, bool signed_flag);
 
-extern void vpip_hex_str_to_bits(unsigned char*bits, unsigned nbits,
-				 const char*buf, bool signed_flag);
+extern void vpip_vec4_to_oct_str(const vvp_vector4_t&bits, char*buf,
+				 unsigned nbuf, bool signed_flag);
 
-extern void vpip_bits_to_oct_str(const unsigned char*bits, unsigned nbits,
-				 char*buf, unsigned nbuf, bool signed_flag);
+extern void vpip_bin_str_to_vec4(vvp_vector4_t&val, const char*buf);
+extern void vpip_oct_str_to_vec4(vvp_vector4_t&val, const char*str);
+extern void vpip_dec_str_to_vec4(vvp_vector4_t&val, const char*str);
+extern void vpip_hex_str_to_vec4(vvp_vector4_t&val, const char*str);
 
-extern void vpip_oct_str_to_bits(unsigned char*bits, unsigned nbits,
-				 const char*buf, bool signed_flag);
+extern vvp_vector4_t vec4_from_vpi_value(s_vpi_value*vp, unsigned wid);
+
+extern void vpip_vec4_get_value(const vvp_vector4_t&word_val, unsigned width,
+				bool signed_flag, s_vpi_value*vp);
 
 /*
  * Function defined in vpi_signal.cc to manage vpi_get_* persistent
@@ -417,55 +604,17 @@ extern void vpip_oct_str_to_bits(unsigned char*bits, unsigned nbits,
 enum vpi_rbuf_t {
       RBUF_VAL =0,
 	/* Storage for *_get_value() */
-      RBUF_STR
+      RBUF_STR,
 	/* Storage for *_get_str() */
+      RBUF_DEL
+	/* Delete the storage for both buffers. */
 };
 extern char *need_result_buf(unsigned cnt, vpi_rbuf_t type);
+/* following two routines use need_result_buf(, RBUF_STR) */
+extern char *simple_set_rbuf_str(const char *s1);
+extern char *generic_get_str(int code, vpiHandle ref, const char *name, const char *index);
+
+/* A routine to find the enclosing module. */
+extern vpiHandle vpip_module(struct __vpiScope*scope);
 
-/*
- * $Log: vpi_priv.h,v $
- * Revision 1.59.2.1  2006/02/19 16:57:31  steve
- *  Add change callback to vpiMemory objects.
- *
- * Revision 1.59  2004/10/04 01:10:59  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.58  2004/06/30 03:00:36  steve
- *  Some explination of vpi_func arguments.
- *
- * Revision 1.57  2004/05/19 03:26:24  steve
- *  Support delayed/non-blocking assignment to reals and others.
- *
- * Revision 1.56  2003/06/17 19:17:42  steve
- *  Remove short int restrictions from vvp opcodes.
- *
- * Revision 1.55  2003/05/29 02:21:45  steve
- *  Implement acc_fetch_defname and its infrastructure in vvp.
- *
- * Revision 1.54  2003/05/02 04:29:57  steve
- *  Add put_value with transport delay.
- *
- * Revision 1.53  2003/04/23 03:09:25  steve
- *  VPI Access to named events.
- *
- * Revision 1.52  2003/03/10 23:37:07  steve
- *  Direct support for string parameters.
- *
- * Revision 1.51  2003/03/06 04:32:00  steve
- *  Use hashed name strings for identifiers.
- *
- * Revision 1.50  2003/02/24 06:35:45  steve
- *  Interactive task calls take string arguments.
- *
- * Revision 1.49  2003/02/23 06:41:54  steve
- *  Add to interactive stop mode support for
- *  current scope, the ability to scan/traverse
- *  scopes, and the ability to call system tasks.
- *
- * Revision 1.48  2003/02/10 05:20:10  steve
- *  Add value change callbacks to real variables.
- *
- * Revision 1.47  2003/02/09 23:33:26  steve
- *  Spelling fixes.
- */
 #endif
