@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,15 +16,15 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_scope.cc,v 1.32 2004/10/04 01:10:59 steve Exp $"
-#endif
 
 # include  "compile.h"
 # include  "vpi_priv.h"
 # include  "symbols.h"
-# include  "functor.h"
 # include  "statistics.h"
+# include  "config.h"
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+#endif
 #ifdef HAVE_MALLOC_H
 # include  <malloc.h>
 #endif
@@ -49,6 +49,69 @@ void vpip_make_root_iterator(struct __vpiHandle**&table, unsigned&ntable)
       ntable = vpip_root_table_cnt;
 }
 
+#ifdef CHECK_WITH_VALGRIND
+static void delete_sub_scopes(struct __vpiScope *scope)
+{
+      for (unsigned idx = 0; idx < scope->nintern; idx += 1) {
+	    struct __vpiScope*lscope = (__vpiScope*)(scope->intern)[idx];
+	    switch(scope->intern[idx]->vpi_type->type_code) {
+		case vpiFunction:
+		case vpiTask:
+		  contexts_delete(lscope);
+		case vpiModule:
+		case vpiNamedBegin:
+		case vpiNamedFork:
+		  delete_sub_scopes(lscope);
+		  vthreads_delete(lscope->threads);
+		  delete (scope->intern)[idx];
+		  break;
+		case vpiIntegerVar:
+		  break;
+		case vpiMemory:
+		case vpiNetArray:
+		  memory_delete((scope->intern)[idx]);
+		  break;
+		case vpiModPath:
+		    /* The destination ModPath is cleaned up later. */
+		  free((scope->intern)[idx]);
+		  break;
+		case vpiNamedEvent:
+		  named_event_delete((scope->intern)[idx]);
+		  break;
+		case vpiNet:
+		case vpiReg:
+		  signal_delete((scope->intern)[idx]);
+		  break;
+		case vpiParameter:
+		  parameter_delete((scope->intern)[idx]);
+		  break;
+		case vpiRealVar:
+		  real_delete((scope->intern)[idx]);
+		  break;
+		default:
+		  fprintf(stderr, "Need support for type: %d\n",
+		          scope->intern[idx]->vpi_type->type_code);
+		  assert(0);
+		  break;
+	    }
+      }
+      free(scope->intern);
+}
+
+void root_table_delete(void)
+{
+      for (unsigned idx = 0; idx < vpip_root_table_cnt; idx += 1) {
+	    struct __vpiScope *scope = (__vpiScope *)vpip_root_table_ptr[idx];
+	    delete_sub_scopes(scope);
+	    vthreads_delete(scope->threads);
+	    delete scope;
+      }
+      free(vpip_root_table_ptr);
+      vpip_root_table_ptr = 0;
+      vpip_root_table_cnt = 0;
+}
+#endif
+
 static bool handle_is_scope(vpiHandle obj)
 {
       return (obj->vpi_type->type_code == vpiModule)
@@ -65,10 +128,23 @@ static int scope_get(int code, vpiHandle obj)
       assert(handle_is_scope(obj));
 
       switch (code) {
+	  case vpiDefLineNo:
+	    return ref->def_lineno;
+
+	  case vpiLineNo:
+	    return ref->lineno;
+
 	  case vpiTimeUnit:
 	    return ref->time_units;
+
 	  case vpiTimePrecision:
-	    return vpip_get_time_precision();
+	    return ref->time_precision;
+
+	  case vpiTopModule:
+	    return 0x0 == ref->scope;
+
+          case vpiAutomatic:
+	    return (int) ref->is_automatic;
       }
 
       return 0;
@@ -84,37 +160,66 @@ static void construct_scope_fullname(struct __vpiScope*ref, char*buf)
       strcat(buf, ref->name);
 }
 
+static const char* scope_get_type(int code)
+{
+      switch (code) {
+          case vpiModule:
+            return "vpiModule";
+          case vpiFunction:
+            return "vpiFunction";
+          case vpiTask:
+            return "vpiTask";
+          case vpiNamedBegin:
+            return "vpiNamedBegin";
+          case vpiNamedFork:
+            return "vpiNamedFork";
+          default:
+            fprintf(stderr, "ERROR: invalid code %d.", code);
+            assert(0);
+      }
+}
+
 static char* scope_get_str(int code, vpiHandle obj)
 {
       struct __vpiScope*ref = (struct __vpiScope*)obj;
-      char *rbuf;
 
       assert(handle_is_scope(obj));
 
+      char buf[4096];  // XXX is a fixed buffer size really reliable?
+      const char *p=0;
       switch (code) {
-	  case vpiFullName: {
-		char buf[4096];
-		buf[0] = 0;
-		construct_scope_fullname(ref, buf);
-		rbuf = need_result_buf(strlen(buf) + 1, RBUF_STR);
-		strcpy(rbuf, buf);
-		return rbuf;
-	  }
+	  case vpiDefFile:
+	    p = file_names[ref->def_file_idx];
+	    break;
+
+	  case vpiFile:
+	    p = file_names[ref->file_idx];
+	    break;
+
+	  case vpiFullName:
+	    buf[0] = 0;
+	    construct_scope_fullname(ref, buf);
+	    p = buf;
+	    break;
 
 	  case vpiName:
-	    rbuf = need_result_buf(strlen(ref->name) + 1, RBUF_STR);
-	    strcpy(rbuf, ref->name);
-	    return rbuf;
+	    p = ref->name;
+	    break;
 
 	  case vpiDefName:
-	    rbuf = need_result_buf(strlen(ref->tname) + 1, RBUF_STR);
-	    strcpy(rbuf, ref->tname);
-	    return rbuf;
+	    p = ref->tname;
+	    break;
+
+	  case vpiType:
+	    p = scope_get_type(code);
+	    break;
 
 	  default:
+	    fprintf(stderr, "ERROR: invalid code %d.", code);
 	    assert(0);
 	    return 0;
       }
+      return simple_set_rbuf_str(p);
 }
 
 static vpiHandle scope_get_handle(int code, vpiHandle obj)
@@ -163,7 +268,7 @@ static int compare_types(int code, int type)
 	     (type == vpiIntegerVar ||
 	      type == vpiTimeVar    ||
 	      type == vpiRealVar))
-      	    return 1;
+	    return 1;
 
       return 0;
 }
@@ -208,92 +313,6 @@ static vpiHandle module_iter(int code, vpiHandle obj)
       return module_iter_subset(code, ref);
 }
 
-/*
-**  Keeping track of functor scope.  When the scope changes during
-**  compilation, we record the current number of functors in a list.
-**
-**  Why are we doing this?  The SDF annotator needs this for
-**  INTERCONNECT delays.  The INTERCONNECT delay is specified between
-**  a source modules output port and a target module input port, which
-**  are connected with a wire.  The vpiSignal for both ports point to
-**  the same functor output, together with all other ports that may be
-**  connected to the same wire.  The SDF annotator need to find those
-**  functors which are inside the scope of the target module, which
-**  are driven by the source functor.  And even this is only an
-**  approximation, in case the wire is connected to multiple inputs of
-**  the same module.  But those should have the same delays anyway.
-**
-*/
-
-struct functor_scope_s {
-      vpiHandle scope;
-      unsigned start;
-};
-
-static struct functor_scope_s * functor_scopes = 0;
-static unsigned n_functor_scopes = 0;
-static unsigned a_functor_scopes = 0;
-
-void functor_set_scope(vpiHandle scope)
-{
-      unsigned nfun = functor_limit();
-
-      if (n_functor_scopes) {
-	    functor_scope_s *last = &functor_scopes[n_functor_scopes - 1];
-
-	    if (last->scope == scope) {
-		  return;
-	    }
-
-	    if (last->start == nfun) {
-		  last->scope = scope;
-		  return;
-	    }
-      }
-
-      n_functor_scopes += 1;
-      if (n_functor_scopes >= a_functor_scopes) {
-	    a_functor_scopes += 512;
-	    functor_scopes = (struct functor_scope_s *)
-		  realloc(functor_scopes,
-			  a_functor_scopes*sizeof(struct functor_scope_s));
-	    assert(functor_scopes);
-      }
-
-      functor_scope_s *last = &functor_scopes[n_functor_scopes - 1];
-      last->start = nfun;
-      last->scope = scope;
-}
-
-/*
-**  Look up the scope of a functor.
-**
-**  Cannot use bsearch, since we are not looking for an exact match
-*/
-vpiHandle ipoint_get_scope(vvp_ipoint_t ipt)
-{
-      if (n_functor_scopes == 0)
-	    return NULL;
-
-      unsigned fidx = ipt/4;
-
-      unsigned first = 0;
-      unsigned last = n_functor_scopes;
-      while (first < last) {
-	    unsigned next = (first+last)/2;
-	    functor_scope_s *cur = &functor_scopes[next];
-
-	    if (cur->start > fidx)
-		  last = next;
-	    else if (next == first)
-		  break;
-	    else
-		  first = next;
-      }
-
-      functor_scope_s *cur = &functor_scopes[first];
-      return cur->scope;
-}
 
 static const struct __vpirt vpip_scope_module_rt = {
       vpiModule,
@@ -374,28 +393,35 @@ static void attach_to_scope_(struct __vpiScope*scope, vpiHandle obj)
  * symbol table and the name is used to construct the actual object.
  */
 void
-compile_scope_decl(char*label, char*type, char*name, char*tname, char*parent)
+compile_scope_decl(char*label, char*type, char*name, char*tname,
+                   char*parent, long file_idx, long lineno,
+                   long def_file_idx, long def_lineno)
 {
       struct __vpiScope*scope = new struct __vpiScope;
       count_vpi_scopes += 1;
 
-      switch(type[2]) {
-	  case 'd': /* type == moDule */
+      char*base_type = 0;
+      if (strncmp(type,"auto",4) == 0) {
+	    scope->is_automatic = true;
+            base_type = &type[4];
+      } else {
+	    scope->is_automatic = false;
+            base_type = &type[0];
+      }
+
+      if (strcmp(base_type,"module") == 0) {
 	    scope->base.vpi_type = &vpip_scope_module_rt;
-	    break;
-	  case 'n': /* type == fuNction */
+      } else if (strcmp(base_type,"function") == 0) {
 	    scope->base.vpi_type = &vpip_scope_function_rt;
-	    break;
-	  case 's': /* type == taSk */
+      } else if (strcmp(base_type,"task") == 0) {
 	    scope->base.vpi_type = &vpip_scope_task_rt;
-	    break;
-	  case 'r': /* type == foRk */
+      } else if (strcmp(base_type,"fork") == 0) {
 	    scope->base.vpi_type = &vpip_scope_fork_rt;
-	    break;
-	  case 'g': /* type == beGin */
+      } else if (strcmp(base_type,"begin") == 0) {
 	    scope->base.vpi_type = &vpip_scope_begin_rt;
-	    break;
-	  default:
+      } else if (strcmp(base_type,"generate") == 0) {
+	    scope->base.vpi_type = &vpip_scope_begin_rt;
+      } else {
 	    scope->base.vpi_type = &vpip_scope_module_rt;
 	    assert(0);
       }
@@ -403,9 +429,18 @@ compile_scope_decl(char*label, char*type, char*name, char*tname, char*parent)
       assert(scope->base.vpi_type);
 
       scope->name = vpip_name_string(name);
-      scope->tname = vpip_name_string(tname);
+      if (tname) scope->tname = vpip_name_string(tname);
+      else scope->tname = vpip_name_string("");
+      scope->file_idx = (unsigned) file_idx;
+      scope->lineno  = (unsigned) lineno;
+      scope->def_file_idx = (unsigned) def_file_idx;
+      scope->def_lineno  = (unsigned) def_lineno;
       scope->intern = 0;
       scope->nintern = 0;
+      scope->item = 0;
+      scope->nitem = 0;
+      scope->live_contexts = 0;
+      scope->free_contexts = 0;
       scope->threads = 0;
 
       current_scope = scope;
@@ -414,7 +449,8 @@ compile_scope_decl(char*label, char*type, char*name, char*tname, char*parent)
 
       free(label);
       free(type);
-      free(name);
+      delete[] name;
+      delete[] tname;
 
       if (parent) {
 	    static vpiHandle obj;
@@ -424,8 +460,9 @@ compile_scope_decl(char*label, char*type, char*name, char*tname, char*parent)
 	    attach_to_scope_(sp, &scope->base);
 	    scope->scope = (struct __vpiScope*)obj;
 
-	      /* Inherit time units from the parent scope. */
+	      /* Inherit time units and precision from the parent scope. */
 	    scope->time_units = sp->time_units;
+	    scope->time_precision = sp->time_precision;
 
       } else {
 	    scope->scope = 0x0;
@@ -436,28 +473,28 @@ compile_scope_decl(char*label, char*type, char*name, char*tname, char*parent)
 	    vpip_root_table_ptr[vpip_root_table_cnt] = &scope->base;
 	    vpip_root_table_cnt = cnt;
 
-	      /* Root scopes inherit time_units from system precision. */
+	      /* Root scopes inherit time_units and precision from the
+	         system precision. */
 	    scope->time_units = vpip_get_time_precision();
+	    scope->time_precision = vpip_get_time_precision();
       }
-
-      functor_set_scope(&current_scope->base);
 }
 
 void compile_scope_recall(char*symbol)
 {
       compile_vpi_lookup((vpiHandle*)&current_scope, symbol);
       assert(current_scope);
-      functor_set_scope(&current_scope->base);
 }
 
 /*
  * This function handles the ".timescale" directive in the vvp
  * source. It sets in the current scope the specified units value.
  */
-void compile_timescale(long units)
+void compile_timescale(long units, long precision)
 {
       assert(current_scope);
       current_scope->time_units = units;
+      current_scope->time_precision = precision;
 }
 
 struct __vpiScope* vpip_peek_current_scope(void)
@@ -470,71 +507,36 @@ void vpip_attach_to_current_scope(vpiHandle obj)
       attach_to_scope_(current_scope, obj);
 }
 
+struct __vpiScope* vpip_peek_context_scope(void)
+{
+      struct __vpiScope*scope = current_scope;
 
-/*
- * $Log: vpi_scope.cc,v $
- * Revision 1.32  2004/10/04 01:10:59  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.31  2003/05/29 02:21:45  steve
- *  Implement acc_fetch_defname and its infrastructure in vvp.
- *
- * Revision 1.30  2003/05/27 16:22:10  steve
- *  PLI get time units/precision.
- *
- * Revision 1.29  2003/03/14 05:01:22  steve
- *  vpiModule handle of scope is parent scope.
- *
- * Revision 1.28  2003/03/06 04:32:00  steve
- *  Use hashed name strings for identifiers.
- *
- * Revision 1.27  2003/03/03 01:47:50  steve
- *  .scope directives store only the base names.
- *
- * Revision 1.26  2003/02/27 21:54:44  steve
- *  Add scope type have a vpi_get function.
- *
- * Revision 1.25  2003/02/23 06:41:54  steve
- *  Add to interactive stop mode support for
- *  current scope, the ability to scan/traverse
- *  scopes, and the ability to call system tasks.
- *
- * Revision 1.24  2003/02/11 05:20:45  steve
- *  Include vpiRealVar objects in vpiVariables scan.
- *
- * Revision 1.23  2003/02/09 23:33:26  steve
- *  Spelling fixes.
- *
- * Revision 1.22  2002/12/21 00:55:58  steve
- *  The $time system task returns the integer time
- *  scaled to the local units. Change the internal
- *  implementation of vpiSystemTime the $time functions
- *  to properly account for this. Also add $simtime
- *  to get the simulation time.
- *
- * Revision 1.21  2002/11/15 22:14:12  steve
- *  Add vpiScope iterate on vpiScope objects.
- *
- * Revision 1.20  2002/08/12 01:35:09  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.19  2002/07/17 05:13:43  steve
- *  Implementation of vpi_handle_by_name, and
- *  add the vpiVariables iterator.
- *
- * Revision 1.18  2002/07/14 02:52:05  steve
- *  Fix vpiScope iterator.
- *
- * Revision 1.17  2002/07/12 18:23:30  steve
- *  Use result buf for event and scope names.
- *
- * Revision 1.16  2002/07/05 17:14:15  steve
- *  Names of vpi objects allocated as vpip_strings.
- *
- * Revision 1.15  2002/05/18 02:34:11  steve
- *  Add vpi support for named events.
- *
- *  Add vpi_mode_flag to track the mode of the
- *  vpi engine. This is for error checking.
- */
+        /* A context is allocated for each automatic task or function.
+           Storage for nested scopes (named blocks) is allocated in
+           the parent context. */
+      while (scope->scope && scope->scope->is_automatic)
+            scope = scope->scope;
 
+      return scope;
+}
+
+unsigned vpip_add_item_to_context(automatic_hooks_s*item,
+                                  struct __vpiScope*scope)
+{
+      assert(scope);
+      assert(scope->is_automatic);
+
+      unsigned idx = scope->nitem++;
+
+      if (scope->item == 0)
+	    scope->item = (automatic_hooks_s**)
+		  malloc(sizeof(automatic_hooks_s*));
+      else
+	    scope->item = (automatic_hooks_s**)
+		  realloc(scope->item, sizeof(automatic_hooks_s*)*scope->nitem);
+
+      scope->item[idx] = item;
+
+        /* Offset the context index by 2 to leave space for the list links. */
+      return 2 + idx;
+}
