@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,9 +16,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_tasks.cc,v 1.29 2004/10/04 01:10:59 steve Exp $"
-#endif
 
 /*
  * This file keeps the table of system/task definitions. This table is
@@ -27,6 +24,11 @@
  */
 # include  "vpi_priv.h"
 # include  "vthread.h"
+# include  "compile.h"
+# include  "config.h"
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+#endif
 # include  <stdio.h>
 #ifdef HAVE_MALLOC_H
 # include  <malloc.h>
@@ -64,6 +66,7 @@ static vpiHandle systask_handle(int type, vpiHandle ref)
       switch (type) {
 	  case vpiScope:
 	    return &rfp->scope->base;
+
 	  default:
 	    return 0;
       };
@@ -77,8 +80,36 @@ static int systask_get(int type, vpiHandle ref)
 	     || (ref->vpi_type->type_code == vpiSysFuncCall));
 
       switch (type) {
+	    /* This is not the correct way to get this information, but
+	     * some of the code that implements the acc and tf routines
+	     * use this method so we will keep it in for now. */
 	  case vpiTimeUnit:
 	    return rfp->scope->time_units;
+	  case vpiTimePrecision:
+	    return rfp->scope->time_precision;
+
+	  case vpiLineNo:
+	    return rfp->lineno;
+
+	  default:
+	    return vpiUndefined;
+      }
+}
+
+// support getting vpiSize for a system function call
+static int sysfunc_get(int type, vpiHandle ref)
+{
+      struct __vpiSysTaskCall*rfp = (struct __vpiSysTaskCall*)ref;
+
+      assert(ref->vpi_type->type_code == vpiSysFuncCall);
+
+      switch (type) {
+	  case vpiSize:
+	    return rfp->vwid;
+
+	  case vpiLineNo:
+	    return rfp->lineno;
+
 	  default:
 	    return vpiUndefined;
       }
@@ -95,14 +126,13 @@ static char *systask_get_str(int type, vpiHandle ref)
       assert((ref->vpi_type->type_code == vpiSysTaskCall)
 	     || (ref->vpi_type->type_code == vpiSysFuncCall));
 
-      const char *bn = rfp->defn->info.tfname;
-      char *rbuf = need_result_buf(strlen(bn) + 1, RBUF_STR);
-
       switch (type) {
+          case vpiFile:
+            assert(rfp->file_idx < file_names.size());
+            return simple_set_rbuf_str(file_names[rfp->file_idx]);
 
           case vpiName:
-            strcpy(rbuf,bn);
-	    return rbuf;
+            return simple_set_rbuf_str(rfp->defn->info.tfname);
       }
 
       return 0;
@@ -141,12 +171,14 @@ static const struct __vpirt vpip_systask_rt = {
  * bits and set into the thread space bits that were selected at
  * compile time.
  */
-static vpiHandle sysfunc_put_value(vpiHandle ref, p_vpi_value vp)
+static vpiHandle sysfunc_put_value(vpiHandle ref, p_vpi_value vp, int)
 {
       assert(ref->vpi_type->type_code == vpiSysFuncCall);
 
       struct __vpiSysTaskCall*rfp = (struct __vpiSysTaskCall*)ref;
 
+      rfp->put_value = true;
+
       assert(rfp->vbit >= 4);
 
       switch (vp->format) {
@@ -155,7 +187,7 @@ static vpiHandle sysfunc_put_value(vpiHandle ref, p_vpi_value vp)
 		long val = vp->value.integer;
 		for (int idx = 0 ;  idx < rfp->vwid ;  idx += 1) {
 		      vthread_put_bit(vpip_current_vthread,
-				      rfp->vbit+idx, val&1);
+				      rfp->vbit+idx, (val&1)? BIT4_1 :BIT4_0);
 		      val >>= 1;
 		}
 		break;
@@ -172,30 +204,53 @@ static vpiHandle sysfunc_put_value(vpiHandle ref, p_vpi_value vp)
 		      word >>= idx % 32;
 
 		      vthread_put_bit(vpip_current_vthread,
-				      rfp->vbit+idx, word&1);
+				      rfp->vbit+idx, (word&1)? BIT4_1 :BIT4_0);
 		}
 		break;
 
-
 	  case vpiScalarVal:
 	    switch (vp->value.scalar) {
 		case vpi0:
-		  vthread_put_bit(vpip_current_vthread, rfp->vbit, 0);
+		  vthread_put_bit(vpip_current_vthread, rfp->vbit, BIT4_0);
 		  break;
 		case vpi1:
-		  vthread_put_bit(vpip_current_vthread, rfp->vbit, 1);
+		  vthread_put_bit(vpip_current_vthread, rfp->vbit, BIT4_1);
 		  break;
 		case vpiX:
-		  vthread_put_bit(vpip_current_vthread, rfp->vbit, 2);
+		  vthread_put_bit(vpip_current_vthread, rfp->vbit, BIT4_X);
 		  break;
 		case vpiZ:
-		  vthread_put_bit(vpip_current_vthread, rfp->vbit, 3);
+		  vthread_put_bit(vpip_current_vthread, rfp->vbit, BIT4_Z);
 		  break;
 		default:
+		  fprintf(stderr, "Unsupported value %d.\n", vp->value.scalar);
 		  assert(0);
 	    }
 	    break;
 
+	  case vpiStringVal: {
+	    unsigned len = strlen(vp->value.str) - 1;
+	    assert(len*8 <= (unsigned)rfp->vwid);
+	    for (unsigned wdx = 0 ;  wdx < (unsigned)rfp->vwid ;  wdx += 8) {
+		  unsigned word = wdx / 8;
+		  char bits;
+		  if (word <= len) {
+			bits = vp->value.str[len-word];
+		  } else {
+			bits = 0;
+		  }
+		  for (unsigned idx = 0 ;  (wdx+idx) < (unsigned)rfp->vwid &&
+		       idx < 8; idx += 1) {
+			vvp_bit4_t bit4 = BIT4_0;
+			if (bits & 1) bit4 = BIT4_1;
+			vthread_put_bit(vpip_current_vthread,
+					rfp->vbit+wdx+idx, bit4);
+			bits >>= 1;
+		  }
+	    }
+	    break;
+	  }
+
 	  case vpiVectorVal:
 
 	    for (unsigned wdx = 0 ;  wdx < (unsigned)rfp->vwid ;  wdx += 32) {
@@ -203,27 +258,32 @@ static vpiHandle sysfunc_put_value(vpiHandle ref, p_vpi_value vp)
 		  unsigned long aval = vp->value.vector[word].aval;
 		  unsigned long bval = vp->value.vector[word].bval;
 
-		  for (unsigned idx = 0 ;  (wdx+idx) < (unsigned)rfp->vwid ;
-		      idx += 1)
+		  for (unsigned idx = 0 ;  (wdx+idx) < (unsigned)rfp->vwid &&
+		       idx < 32; idx += 1)
 		  {
 			int bit = (aval&1) | ((bval<<1)&2);
+			vvp_bit4_t bit4;
 
 			switch (bit) {
 			    case 0:
-			      bit = 0;
+			      bit4 = BIT4_0;
 			      break;
 			    case 1:
-			      bit = 1;
+			      bit4 = BIT4_1;
 			      break;
 			    case 2:
-			      bit = 3;
+			      bit4 = BIT4_Z;
 			      break;
 			    case 3:
-			      bit = 2;
+			      bit4 = BIT4_X;
 			      break;
+			    default:
+			      fprintf(stderr, "Unsupported bit value %d.\n",
+			              bit);
+			      assert(0);
 			}
 			vthread_put_bit(vpip_current_vthread,
-					rfp->vbit+wdx+idx, bit);
+					rfp->vbit+wdx+idx, bit4);
 
 			aval >>= 1;
 			bval >>= 1;
@@ -232,18 +292,21 @@ static vpiHandle sysfunc_put_value(vpiHandle ref, p_vpi_value vp)
 	    break;
 
 	  default:
+	    fprintf(stderr, "Unsupported format %d.\n", vp->format);
 	    assert(0);
       }
 
       return 0;
 }
 
-static vpiHandle sysfunc_put_real_value(vpiHandle ref, p_vpi_value vp)
+static vpiHandle sysfunc_put_real_value(vpiHandle ref, p_vpi_value vp, int)
 {
       assert(ref->vpi_type->type_code == vpiSysFuncCall);
 
       struct __vpiSysTaskCall*rfp = (struct __vpiSysTaskCall*)ref;
 
+      rfp->put_value = true;
+
 	/* Make sure this is a real valued function. */
       assert(rfp->vwid == -vpiRealConst);
 
@@ -256,6 +319,7 @@ static vpiHandle sysfunc_put_real_value(vpiHandle ref, p_vpi_value vp)
 	    break;
 
 	  default:
+	    fprintf(stderr, "Unsupported format %d.\n", vp->format);
 	    assert(0);
       }
 
@@ -263,10 +327,138 @@ static vpiHandle sysfunc_put_real_value(vpiHandle ref, p_vpi_value vp)
       return 0;
 }
 
+static vpiHandle sysfunc_put_4net_value(vpiHandle ref, p_vpi_value vp, int)
+{
+      assert(ref->vpi_type->type_code == vpiSysFuncCall);
+
+      struct __vpiSysTaskCall*rfp = (struct __vpiSysTaskCall*)ref;
+
+      rfp->put_value = true;
+
+      unsigned vwid = (unsigned) rfp->vwid;
+      vvp_vector4_t val (vwid);
+
+      switch (vp->format) {
+
+          case vpiScalarVal: {
+	        switch(vp->value.scalar) {
+		      case vpi0:
+			val.set_bit(0, BIT4_0);
+                        break;
+		      case vpi1:
+                        val.set_bit(0, BIT4_1);
+                        break;
+		      case vpiX:
+                        val.set_bit(0, BIT4_X);
+                        break;
+		      case vpiZ:
+                        val.set_bit(0, BIT4_Z);
+                        break;
+		      default:
+                        fprintf(stderr, "Unsupported bit value %d.\n",
+                                vp->value.scalar);
+                        assert(0);
+                }
+          }
+
+	  case vpiIntVal: {
+		long tmp = vp->value.integer;
+		for (unsigned idx = 0 ;  idx < vwid ;  idx += 1) {
+		      val.set_bit(idx, (tmp&1)? BIT4_1 : BIT4_0);
+		      tmp >>= 1;
+		}
+		break;
+	  }
+
+          case vpiTimeVal: {
+                unsigned long tmp = vp->value.time->low;
+                for (unsigned idx = 0 ;  idx < vwid ;  idx += 1) {
+                      val.set_bit(idx, (tmp&1)? BIT4_1 : BIT4_0);
+
+                      if (idx == 31)
+                            tmp = vp->value.time->high;
+                      else
+                            tmp >>= 1;
+                }
+                break;
+          }
+
+	  case vpiVectorVal:
+
+	    for (unsigned wdx = 0 ;  wdx < vwid ;  wdx += 32) {
+		  unsigned word = wdx / 32;
+		  unsigned long aval = vp->value.vector[word].aval;
+		  unsigned long bval = vp->value.vector[word].bval;
+
+		  for (unsigned idx = 0 ;  (wdx+idx) < vwid && idx < 32;
+		       idx += 1) {
+			int bit = (aval&1) | ((bval<<1)&2);
+			vvp_bit4_t bit4;
+
+			switch (bit) {
+			    case 0:
+			      bit4 = BIT4_0;
+			      break;
+			    case 1:
+			      bit4 = BIT4_1;
+			      break;
+			    case 2:
+			      bit4 = BIT4_Z;
+			      break;
+			    case 3:
+			      bit4 = BIT4_X;
+			      break;
+			    default:
+			      fprintf(stderr, "Unsupported bit value %d.\n",
+			              bit);
+			      assert(0);
+			}
+			val.set_bit(wdx+idx, bit4);
+
+			aval >>= 1;
+			bval >>= 1;
+		  }
+	    }
+	    break;
+
+	  default:
+	    fprintf(stderr, "XXXX format=%d, vwid=%u\n", vp->format, rfp->vwid);
+	    assert(0);
+      }
+
+      vvp_send_vec4(rfp->fnet->out, val, vthread_get_wt_context());
+      return 0;
+}
+
+static vpiHandle sysfunc_put_rnet_value(vpiHandle ref, p_vpi_value vp, int)
+{
+      assert(ref->vpi_type->type_code == vpiSysFuncCall);
+
+      struct __vpiSysTaskCall*rfp = (struct __vpiSysTaskCall*)ref;
+
+      rfp->put_value = true;
+
+      double val;
+      switch (vp->format) {
+
+	  case vpiRealVal:
+	    val = vp->value.real;
+	    break;
+
+	  default:
+	    fprintf(stderr, "Unsupported format %d.\n", vp->format);
+	    assert(0);
+      }
+
+      vvp_send_real(rfp->fnet->out, val, vthread_get_wt_context());
+
+      return 0;
+}
+
 
 static const struct __vpirt vpip_sysfunc_rt = {
       vpiSysFuncCall,
-      0,
+      sysfunc_get,
       systask_get_str,
       0,
       sysfunc_put_value,
@@ -276,7 +468,7 @@ static const struct __vpirt vpip_sysfunc_rt = {
 
 static const struct __vpirt vpip_sysfunc_real_rt = {
       vpiSysFuncCall,
-      0,
+      sysfunc_get,
       systask_get_str,
       0,
       sysfunc_put_real_value,
@@ -284,7 +476,27 @@ static const struct __vpirt vpip_sysfunc_real_rt = {
       systask_iter
 };
 
-  /* **** Manipulate the internal datastructures. **** */
+static const struct __vpirt vpip_sysfunc_4net_rt = {
+      vpiSysFuncCall,
+      sysfunc_get,
+      systask_get_str,
+      0,
+      sysfunc_put_4net_value,
+      systask_handle,
+      systask_iter
+};
+
+static const struct __vpirt vpip_sysfunc_rnet_rt = {
+      vpiSysFuncCall,
+      sysfunc_get,
+      systask_get_str,
+      0,
+      sysfunc_put_rnet_value,
+      systask_handle,
+      systask_iter
+};
+
+  /* **** Manipulate the internal data structures. **** */
 
 static struct __vpiUserSystf**def_table = 0;
 static unsigned def_count = 0;
@@ -311,8 +523,20 @@ static struct __vpiUserSystf* allocate_def(void)
       return def_table[def_count++];
 }
 
+#ifdef CHECK_WITH_VALGRIND
+void def_table_delete(void)
+{
+      for (unsigned idx = 0; idx < def_count; idx += 1) {
+	    free(const_cast<char *>(def_table[idx]->info.tfname));
+	    free(def_table[idx]);
+      }
+      free(def_table);
+      def_table = 0;
+      def_count = 0;
+}
+#endif
 
-static struct __vpiUserSystf* vpip_find_systf(const char*name)
+struct __vpiUserSystf* vpip_find_systf(const char*name)
 {
       for (unsigned idx = 0 ;  idx < def_count ;  idx += 1)
 	    if (strcmp(def_table[idx]->info.tfname, name) == 0)
@@ -334,7 +558,9 @@ static struct __vpiUserSystf* vpip_find_systf(const char*name)
  * vbit is also a non-zero value, the address in thread space of the result.
  */
 vpiHandle vpip_build_vpi_call(const char*name, unsigned vbit, int vwid,
-			      unsigned argc, vpiHandle*argv)
+			      class vvp_net_t*fnet,
+			      unsigned argc, vpiHandle*argv,
+			      long file_idx, long lineno)
 {
       struct __vpiUserSystf*defn = vpip_find_systf(name);
       if (defn == 0) {
@@ -345,7 +571,7 @@ vpiHandle vpip_build_vpi_call(const char*name, unsigned vbit, int vwid,
 
       switch (defn->info.type) {
 	  case vpiSysTask:
-	    if (vwid != 0) {
+	    if (vwid != 0 || fnet != 0) {
 		  fprintf(stderr, "%s: This is a system Task, "
 			  "you cannot call it as a Function\n", name);
 		  return 0;
@@ -354,7 +580,7 @@ vpiHandle vpip_build_vpi_call(const char*name, unsigned vbit, int vwid,
 	    break;
 
 	  case vpiSysFunc:
-	    if (vwid == 0) {
+	    if (vwid == 0 && fnet == 0) {
 		  fprintf(stderr, "%s: This is a system Function, "
 			  "you cannot call it as a Task\n", name);
 		  return 0;
@@ -362,6 +588,7 @@ vpiHandle vpip_build_vpi_call(const char*name, unsigned vbit, int vwid,
 	    break;
 
 	  default:
+	    fprintf(stderr, "Unsupported type %d.\n", defn->info.type);
 	    assert(0);
       }
 
@@ -373,18 +600,20 @@ vpiHandle vpip_build_vpi_call(const char*name, unsigned vbit, int vwid,
 	    break;
 
 	  case vpiSysFunc:
-	    if (vwid > 0) {
-		  obj->base.vpi_type = &vpip_sysfunc_rt;
+	    if (fnet && vwid == -vpiRealConst) {
+		  obj->base.vpi_type = &vpip_sysfunc_rnet_rt;
 
-	    } else switch (vwid) {
+	    } else if (fnet && vwid > 0) {
+		  obj->base.vpi_type = &vpip_sysfunc_4net_rt;
 
-		case -vpiRealConst:
+	    } else if (vwid == -vpiRealConst) {
 		  obj->base.vpi_type = &vpip_sysfunc_real_rt;
-		  break;
 
-		default:
-		  assert(0);
+	    } else if (vwid > 0) {
 		  obj->base.vpi_type = &vpip_sysfunc_rt;
+
+	    } else {
+		  assert(0);
 	    }
 	    break;
       }
@@ -395,22 +624,53 @@ vpiHandle vpip_build_vpi_call(const char*name, unsigned vbit, int vwid,
       obj->args  = argv;
       obj->vbit  = vbit;
       obj->vwid  = vwid;
-      obj->userdata = 0;
+      obj->fnet  = fnet;
+      obj->file_idx  = (unsigned) file_idx;
+      obj->lineno   = (unsigned) lineno;
+      obj->userdata  = 0;
+      obj->put_value = false;
 
-	/* If there is a compiletf function, call it here. */
-      if (obj->defn->info.compiletf) {
-
-	    assert(vpi_mode_flag == VPI_MODE_NONE);
-	    vpi_mode_flag = VPI_MODE_COMPILETF;
-	    vpip_cur_task = obj;
-	    obj->defn->info.compiletf (obj->defn->info.user_data);
-	    vpip_cur_task = 0;
-	    vpi_mode_flag = VPI_MODE_NONE;
-      }
+      compile_compiletf(obj);
 
       return &obj->base;
 }
 
+#ifdef CHECK_WITH_VALGRIND
+void vpi_call_delete(vpiHandle item)
+{
+      struct __vpiSysTaskCall*obj = (struct __vpiSysTaskCall *) item;
+      for (unsigned arg = 0; arg < obj->nargs; arg += 1) {
+	    switch (obj->args[arg]->vpi_type->type_code) {
+		case vpiConstant:
+		  switch (vpi_get(_vpiFromThr, obj->args[arg])) {
+		      case _vpiNoThr:
+			constant_delete(obj->args[arg]);
+			break;
+		      case _vpiVThr:
+			thread_vthr_delete(obj->args[arg]);
+			break;
+		      case _vpiWord:
+			thread_word_delete(obj->args[arg]);
+			break;
+		      default:
+			assert(0);
+		  }
+		  break;
+		case vpiMemoryWord:
+		  if (vpi_get(_vpiFromThr, obj->args[arg]) == _vpi_at_A) {
+			A_delete(obj->args[arg]);
+		  }
+		  break;
+		case vpiPartSelect:
+		  assert(vpi_get(_vpiFromThr, obj->args[arg]) == _vpi_at_PV);
+		  PV_delete(obj->args[arg]);
+		  break;
+	    }
+      }
+      free(obj->args);
+      delete obj;
+}
+#endif
 
 /*
  * This function is used by the %vpi_call instruction to actually
@@ -432,8 +692,23 @@ void vpip_execute_vpi_call(vthread_t thr, vpiHandle ref)
       if (vpip_cur_task->defn->info.calltf) {
 	    assert(vpi_mode_flag == VPI_MODE_NONE);
 	    vpi_mode_flag = VPI_MODE_CALLTF;
+	    vpip_cur_task->put_value = false;
 	    vpip_cur_task->defn->info.calltf(vpip_cur_task->defn->info.user_data);
 	    vpi_mode_flag = VPI_MODE_NONE;
+	      /* If the function call did not set a value then put a
+	       * default value (0). */
+	    if (ref->vpi_type->type_code == vpiSysFuncCall &&
+	        !vpip_cur_task->put_value) {
+		  s_vpi_value val;
+		  if (vpip_cur_task->vwid == -vpiRealConst) {
+			val.format = vpiRealVal;
+			val.value.real = 0.0;
+		  } else {
+			val.format = vpiIntVal;
+			val.value.integer = 0;
+		  }
+		  vpi_put_value(ref, &val, 0, vpiNoDelay);
+	    }
       }
 }
 
@@ -455,6 +730,7 @@ void vpi_register_systf(const struct t_vpi_systf_data*ss)
 	    cur->base.vpi_type = &vpip_sysfunc_def_rt;
 	    break;
 	  default:
+	    fprintf(stderr, "Unsupported type %d.\n", ss->type);
 	    assert(0);
       }
 
@@ -464,12 +740,14 @@ void vpi_register_systf(const struct t_vpi_systf_data*ss)
 
 PLI_INT32 vpi_put_userdata(vpiHandle ref, void*data)
 {
+      if (ref->vpi_type->type_code != vpiSysTaskCall
+	  && ref->vpi_type->type_code != vpiSysFuncCall)
+	    return 0;
+
       struct __vpiSysTaskCall*rfp = (struct __vpiSysTaskCall*)ref;
-      assert((ref->vpi_type->type_code == vpiSysTaskCall)
-	     || (ref->vpi_type->type_code == vpiSysFuncCall));
 
       rfp->userdata = data;
-      return 0;
+      return 1;
 }
 
 void* vpi_get_userdata(vpiHandle ref)
@@ -480,108 +758,3 @@ void* vpi_get_userdata(vpiHandle ref)
 
       return rfp->userdata;
 }
-
-
-/*
- * $Log: vpi_tasks.cc,v $
- * Revision 1.29  2004/10/04 01:10:59  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.28  2004/06/30 03:00:36  steve
- *  Some explination of vpi_func arguments.
- *
- * Revision 1.27  2004/05/19 03:30:46  steve
- *  Support delayed/non-blocking assignment to reals and others.
- *
- * Revision 1.26  2004/02/18 02:51:59  steve
- *  Fix type mismatches of various VPI functions.
- *
- * Revision 1.25  2003/12/07 20:05:56  steve
- *  Ducument lxt2 access.
- *
- * Revision 1.24  2003/06/18 00:54:28  steve
- *  Account for all 64 bits in results of $time.
- *
- * Revision 1.23  2003/03/14 05:00:44  steve
- *  Support vpi_get of vpiTimeUnit.
- *
- * Revision 1.22  2003/03/13 05:07:10  steve
- *  signed/unsigned warnings.
- *
- * Revision 1.21  2003/03/07 02:44:14  steve
- *  Support vector put of function return values.
- *
- * Revision 1.20  2003/03/03 03:27:35  steve
- *  Support vpiName for system task/function calls.
- *
- * Revision 1.19  2003/01/27 00:14:37  steve
- *  Support in various contexts the $realtime
- *  system task.
- *
- * Revision 1.18  2003/01/09 04:09:44  steve
- *  Add vpi_put_userdata
- *
- * Revision 1.17  2002/12/21 00:55:58  steve
- *  The $time system task returns the integer time
- *  scaled to the local units. Change the internal
- *  implementation of vpiSystemTime the $time functions
- *  to properly account for this. Also add $simtime
- *  to get the simulation time.
- *
- * Revision 1.16  2002/08/12 01:35:09  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.15  2002/06/02 19:05:50  steve
- *  Check for null pointers from users.
- *
- * Revision 1.14  2002/05/18 02:34:11  steve
- *  Add vpi support for named events.
- *
- *  Add vpi_mode_flag to track the mode of the
- *  vpi engine. This is for error checking.
- *
- * Revision 1.13  2002/05/09 03:34:31  steve
- *  Handle null time and calltf pointers.
- *
- * Revision 1.12  2002/05/03 15:44:11  steve
- *  Add vpiModule iterator to vpiScope objects.
- *
- * Revision 1.11  2002/04/07 02:34:10  steve
- *  Set vpip_cur_task while calling compileft
- *
- * Revision 1.10  2001/09/15 18:27:05  steve
- *  Make configure detect malloc.h
- *
- * Revision 1.9  2001/08/03 06:50:44  steve
- *  Detect system function used as a task.
- *
- * Revision 1.8  2001/06/25 03:12:06  steve
- *  Give task/function definitions a vpi type object.
- *
- * Revision 1.7  2001/05/20 00:46:12  steve
- *  Add support for system function calls.
- *
- * Revision 1.6  2001/05/10 00:26:53  steve
- *  VVP support for memories in expressions,
- *  including general support for thread bit
- *  vectors as system task parameters.
- *  (Stephan Boettcher)
- *
- * Revision 1.5  2001/04/18 04:21:23  steve
- *  Put threads into scopes.
- *
- * Revision 1.4  2001/03/22 22:38:14  steve
- *  Detect undefined system tasks at compile time.
- *
- * Revision 1.3  2001/03/18 04:35:18  steve
- *  Add support for string constants to VPI.
- *
- * Revision 1.2  2001/03/18 00:37:55  steve
- *  Add support for vpi scopes.
- *
- * Revision 1.1  2001/03/16 01:44:34  steve
- *  Add structures for VPI support, and all the %vpi_call
- *  instruction. Get linking of VPI modules to work.
- *
- */
-
