@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
  * Copyright (c) 2001 Stephan Boettcher <stephan@nevis.columbia.edu>
  *
  *    This source code is free software; you can redistribute it
@@ -17,9 +17,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: vpi_vthr_vector.cc,v 1.21 2004/10/04 01:11:00 steve Exp $"
-#endif
 
 /*
  * vpiReg handles are handled here. These objects represent vectors of
@@ -28,6 +25,11 @@
 
 # include  "vpi_priv.h"
 # include  "vthread.h"
+# include  "config.h"
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+# include <map>
+#endif
 # include  <stdio.h>
 #ifdef HAVE_MALLOC_H
 # include  <malloc.h>
@@ -40,17 +42,17 @@ struct __vpiVThrVec {
       unsigned bas;
       unsigned wid;
       unsigned signed_flag : 1;
-      char *name;
+      const char *name;
 };
 
 inline static
-unsigned get_bit(struct __vpiVThrVec *rfp, unsigned idx)
+vvp_bit4_t get_bit(struct __vpiVThrVec *rfp, unsigned idx)
 {
       return vthread_get_bit(vpip_current_vthread, rfp->bas+idx);
 }
 
 inline static
-void set_bit(struct __vpiVThrVec *rfp, unsigned idx, unsigned bit)
+void set_bit(struct __vpiVThrVec *rfp, unsigned idx, vvp_bit4_t bit)
 {
       return vthread_put_bit(vpip_current_vthread, rfp->bas+idx, bit);
 }
@@ -70,7 +72,7 @@ void set_bit(struct __vpiVThrVec *rfp, unsigned idx, unsigned bit)
 
 extern const char hex_digits[256];
 
-extern const char oct_digits[256];
+extern const char oct_digits[64];
 
 /*
  *  vpi_get
@@ -88,9 +90,17 @@ static int vthr_vec_get(int code, vpiHandle ref)
 	  case vpiSigned:
 	    return rfp->signed_flag;
 
+	  case vpiConstType:
+	    return vpiBinaryConst; // If this is a constant it is Binary.
+
 	  case vpiSize:
 	    return rfp->wid;
 
+#ifdef CHECK_WITH_VALGRIND
+	  case _vpiFromThr:
+	    return _vpiVThr;
+#endif
+
 	  default:
 	    return 0;
       }
@@ -105,8 +115,8 @@ static char* vthr_vec_get_str(int code, vpiHandle ref)
 
       switch (code) {
 
-	  case vpiFullName:
-	    return (char*)rfp->name;
+	  case vpiFullName:   /* should this be vpiName? */
+	    return simple_set_rbuf_str(rfp->name);
       }
 
       return 0;
@@ -114,13 +124,14 @@ static char* vthr_vec_get_str(int code, vpiHandle ref)
 
 static void vthr_vec_DecStrVal(struct __vpiVThrVec*rfp, s_vpi_value*vp)
 {
-      unsigned char*bits = new unsigned char[rfp->wid];
-      char *rbuf = need_result_buf((rfp->wid+2)/3 + 1, RBUF_VAL);
+      int nbuf = (rfp->wid+2)/3 + 1;
+      char *rbuf = need_result_buf(nbuf, RBUF_VAL);
 
+      vvp_vector4_t tmp (rfp->wid);
       for (unsigned idx = 0 ;  idx < rfp->wid ;  idx += 1)
-	    bits[idx] = get_bit(rfp, idx);
+	    tmp.set_bit(idx, get_bit(rfp, idx));
 
-      vpip_bits_to_dec_str(bits, rfp->wid, rbuf, rfp->wid+1, rfp->signed_flag);
+      vpip_vec4_to_dec_str(tmp, rbuf, nbuf, rfp->signed_flag);
       vp->value.str = rbuf;
 
       return;
@@ -136,9 +147,9 @@ static void vthr_vec_StringVal(struct __vpiVThrVec*rfp, s_vpi_value*vp)
 	tmp <<= 1;
 
 	switch(get_bit(rfp, bitnr)){
-	  case 0:
+	  case BIT4_0:
 	      break;
-	  case 1:
+	  case BIT4_1:
 	      tmp |= 1;
 	      break;
 	  default:
@@ -181,7 +192,7 @@ static void vthr_vec_get_value(vpiHandle ref, s_vpi_value*vp)
 	  case vpiBinStrVal:
 	    rbuf = need_result_buf(wid+1, RBUF_VAL);
 	    for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
-		  rbuf[wid-idx-1] = "01xz"[get_bit(rfp, idx)];
+		  rbuf[wid-idx-1] = vvp_bit4_to_ascii(get_bit(rfp, idx));
 	    }
 	    rbuf[wid] = 0;
 	    vp->value.str = rbuf;
@@ -194,7 +205,22 @@ static void vthr_vec_get_value(vpiHandle ref, s_vpi_value*vp)
 		rbuf[hwid] = 0;
 		hval = 0;
 		for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
-		      hval = hval | (get_bit(rfp, idx) << 2*(idx % 4));
+		      unsigned tmp = 0;
+		      switch (get_bit(rfp, idx)) {
+			  case BIT4_0:
+			    tmp = 0;
+			    break;
+			  case BIT4_1:
+			    tmp = 1;
+			    break;
+			  case BIT4_X:
+			    tmp = 2;
+			    break;
+			  case BIT4_Z:
+			    tmp = 3;
+			    break;
+		      }
+		      hval = hval | (tmp << 2*(idx % 4));
 
 		      if (idx%4 == 3) {
 			    hwid -= 1;
@@ -219,7 +245,22 @@ static void vthr_vec_get_value(vpiHandle ref, s_vpi_value*vp)
 		rbuf[hwid] = 0;
 		hval = 0;
 		for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
-		      hval = hval | (get_bit(rfp,idx) << 2*(idx % 3));
+		      unsigned tmp = 0;
+		      switch (get_bit(rfp, idx)) {
+			  case BIT4_0:
+			    tmp = 0;
+			    break;
+			  case BIT4_1:
+			    tmp = 1;
+			    break;
+			  case BIT4_X:
+			    tmp = 2;
+			    break;
+			  case BIT4_Z:
+			    tmp = 3;
+			    break;
+		      }
+		      hval = hval | (tmp << 2*(idx % 3));
 
 		      if (idx%3 == 2) {
 			    hwid -= 1;
@@ -245,17 +286,34 @@ static void vthr_vec_get_value(vpiHandle ref, s_vpi_value*vp)
 	    vthr_vec_StringVal(rfp, vp);
 	    break;
 
-	  case vpiIntVal:
-	    vp->value.integer = 0;
+	  case vpiIntVal: {
+	    long ival = 0;
 	    for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
 		  switch (get_bit(rfp, idx)) {
-		      case 0:
+		      case BIT4_0:
 			break;
-		      case 1:
-			vp->value.integer |= 1 << idx;
+		      case BIT4_1:
+			ival |= 1 << idx;
 			break;
-		      case 2:
-		      case 3:
+		      default:
+			break;
+		  }
+	    }
+	    vp->value.integer = ival;
+	    }
+	    break;
+
+	  case vpiRealVal:
+	    vp->value.real = 0;
+	    for (unsigned idx = wid ;  idx > 0 ;  idx -= 1) {
+		  vp->value.real *= 2.0;
+		  switch (get_bit(rfp, idx-1)) {
+		      case BIT4_0:
+			break;
+		      case BIT4_1:
+			vp->value.real += 1.0;
+			break;
+		      default:
 			break;
 		  }
 	    }
@@ -271,19 +329,19 @@ static void vthr_vec_get_value(vpiHandle ref, s_vpi_value*vp)
 		  PLI_INT32 mask = 1 << (idx%32);
 
 		  switch (get_bit(rfp,idx)) {
-		      case 0:
+		      case BIT4_0:
 			vp->value.vector[word].aval &= ~mask;
 			vp->value.vector[word].bval &= ~mask;
 			break;
-		      case 1:
+		      case BIT4_1:
 			vp->value.vector[word].aval |=  mask;
 			vp->value.vector[word].bval &= ~mask;
 			break;
-		      case 2:
+		      case BIT4_X:
 			vp->value.vector[word].aval |=  mask;
 			vp->value.vector[word].bval |=  mask;
 			break;
-		      case 3:
+		      case BIT4_Z:
 			vp->value.vector[word].aval &= ~mask;
 			vp->value.vector[word].bval |=  mask;
 			break;
@@ -302,7 +360,7 @@ static void vthr_vec_get_value(vpiHandle ref, s_vpi_value*vp)
 /*
  * The put_value method writes the value into the vector.
  */
-static vpiHandle vthr_vec_put_value(vpiHandle ref, s_vpi_value*vp)
+static vpiHandle vthr_vec_put_value(vpiHandle ref, s_vpi_value*vp, int)
 {
       assert((ref->vpi_type->type_code==vpiNet)
 	     || (ref->vpi_type->type_code==vpiReg));
@@ -318,7 +376,7 @@ static vpiHandle vthr_vec_put_value(vpiHandle ref, s_vpi_value*vp)
 
 		long val = vp->value.integer;
 		for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
-		      set_bit(rfp, idx, val&1);
+		      set_bit(rfp, idx, (val&1)? BIT4_1 : BIT4_0);
 		      val >>= 1;
 		}
 		break;
@@ -327,18 +385,20 @@ static vpiHandle vthr_vec_put_value(vpiHandle ref, s_vpi_value*vp)
 	  case vpiScalarVal:
 	    switch (vp->value.scalar) {
 		case vpi0:
-		  set_bit(rfp, 0, 0);
+		  set_bit(rfp, 0, BIT4_0);
 		  break;
 		case vpi1:
-		  set_bit(rfp, 0, 1);
+		  set_bit(rfp, 0, BIT4_1);
 		  break;
 		case vpiX:
-		  set_bit(rfp, 0, 2);
+		  set_bit(rfp, 0, BIT4_X);
 		  break;
 		case vpiZ:
-		  set_bit(rfp, 0, 3);
+		  set_bit(rfp, 0, BIT4_Z);
 		  break;
 		default:
+		  fprintf(stderr, "Unsupported scalar value %d.\n",
+		          vp->value.scalar);
 		  assert(0);
 	    }
 	    break;
@@ -350,7 +410,20 @@ static vpiHandle vthr_vec_put_value(vpiHandle ref, s_vpi_value*vp)
 		unsigned long bval = vp->value.vector->bval;
 		for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
 		      int bit = (aval&1) | (((bval^aval)<<1)&2);
-		      set_bit(rfp, idx, bit);
+		      switch (bit) {
+			  case 0:
+			    set_bit(rfp, idx, BIT4_0);
+			    break;
+			  case 1:
+			    set_bit(rfp, idx, BIT4_1);
+			    break;
+			  case 2:
+			    set_bit(rfp, idx, BIT4_X);
+			    break;
+			  case 3:
+			    set_bit(rfp, idx, BIT4_Z);
+			    break;
+		      }
 		      aval >>= 1;
 		      bval >>= 1;
 		}
@@ -358,6 +431,7 @@ static vpiHandle vthr_vec_put_value(vpiHandle ref, s_vpi_value*vp)
 	  }
 
 	  default:
+	    fprintf(stderr, "Unsupported format %d.\n", vp->format);
 	    assert(0);
 
       }
@@ -392,14 +466,29 @@ vpiHandle vpip_make_vthr_vector(unsigned base, unsigned wid, bool signed_flag)
       assert(wid < 65536);
       obj->wid = wid;
       obj->signed_flag = signed_flag? 1 : 0;
-      obj->name = "T<>";
+      obj->name = vpip_name_string("T<>");
 
       return &obj->base;
 }
 
+#ifdef CHECK_WITH_VALGRIND
+static map<vpiHandle, bool> handle_map;
+
+void thread_vthr_delete(vpiHandle item)
+{
+      handle_map[item] = true;
+}
+
+static void thread_vthr_delete_real(vpiHandle item)
+{
+      struct __vpiVThrVec*obj = (struct __vpiVThrVec*)item;
+      free (obj);
+}
+#endif
+
 struct __vpiVThrWord {
       struct __vpiHandle base;
-      char* name;
+      const char* name;
       int subtype;
       unsigned index;
 };
@@ -415,6 +504,11 @@ static int vthr_word_get(int code, vpiHandle ref)
 	  case vpiConstType:
 	    return rfp->subtype;
 
+#ifdef CHECK_WITH_VALGRIND
+	  case _vpiFromThr:
+	    return _vpiWord;
+#endif
+
 	  default:
 	    return 0;
       }
@@ -510,7 +604,7 @@ vpiHandle vpip_make_vthr_word(unsigned base, const char*type)
       assert(type[0] == 'r');
 
       obj->base.vpi_type = &vpip_vthr_const_real_rt;
-      obj->name = "W<>";
+      obj->name = vpip_name_string("W<>");
       obj->subtype = vpiRealConst;
       assert(base < 65536);
       obj->index = base;
@@ -518,77 +612,24 @@ vpiHandle vpip_make_vthr_word(unsigned base, const char*type)
       return &obj->base;
 }
 
-/*
- * $Log: vpi_vthr_vector.cc,v $
- * Revision 1.21  2004/10/04 01:11:00  steve
- *  Clean up spurious trailing white space.
- *
- * Revision 1.20  2004/05/19 03:30:46  steve
- *  Support delayed/non-blocking assignment to reals and others.
- *
- * Revision 1.19  2004/02/20 01:52:25  steve
- *  vpiStringVal does not include leading nulls.
- *
- * Revision 1.18  2003/06/17 19:17:42  steve
- *  Remove short int restrictions from vvp opcodes.
- *
- * Revision 1.17  2003/06/11 05:07:31  steve
- *  support vpiVectorVal for value of thread vector.
- *
- * Revision 1.16  2003/05/29 03:46:21  steve
- *  Add tf_getp/putp support for integers
- *  and real valued arguments.
- *
- *  Add tf_mipname function.
- *
- * Revision 1.15  2003/05/28 03:10:52  steve
- *  Some asserts that check for thread vector overflow.
- *
- * Revision 1.14  2003/03/13 04:59:21  steve
- *  Use rbufs instead of static buffers.
- *
- * Revision 1.13  2003/03/01 00:46:13  steve
- *  Careful about compiletf calls.
- *
- * Revision 1.12  2003/02/09 23:33:26  steve
- *  Spelling fixes.
- *
- * Revision 1.11  2003/02/06 17:41:33  steve
- *  Support constant types for thread words.
- *
- * Revision 1.10  2003/02/04 04:03:40  steve
- *  Add hex and binary formatting of real values.
- *
- * Revision 1.9  2003/01/26 18:16:22  steve
- *  Add %cvt/ir and %cvt/ri instructions, and support
- *  real values passed as arguments to VPI tasks.
- *
- * Revision 1.8  2002/08/12 01:35:09  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.7  2002/05/12 00:32:21  steve
- *  Get thread vectors by vpiIntVal.
- *
- * Revision 1.6  2002/04/14 02:56:19  steve
- *  Support signed expressions through to VPI.
- *
- * Revision 1.5  2002/01/10 01:54:04  steve
- *  odd width thread vectors as strings.
- *
- * Revision 1.4  2001/12/30 21:31:38  steve
- *  Support vpiStringVal in vhtread vectors.
- *
- * Revision 1.3  2001/09/15 18:27:05  steve
- *  Make configure detect malloc.h
- *
- * Revision 1.2  2001/05/20 00:40:12  steve
- *  Get bit ordering right when making decimal strings.
- *
- * Revision 1.1  2001/05/10 00:26:53  steve
- *  VVP support for memories in expressions,
- *  including general support for thread bit
- *  vectors as system task parameters.
- *  (Stephan Boettcher)
- *
- */
+#ifdef CHECK_WITH_VALGRIND
+void thread_word_delete(vpiHandle item)
+{
+      handle_map[item] = false;
+}
 
+static void thread_word_delete_real(vpiHandle item)
+{
+      struct __vpiVThrWord*obj = (struct __vpiVThrWord*)item;
+      free(obj);
+}
+
+void vpi_handle_delete()
+{
+      map<vpiHandle, bool>::iterator iter;
+      for (iter = handle_map.begin(); iter != handle_map.end(); iter++) {
+	    if (iter->second) thread_vthr_delete_real(iter->first);
+	    else thread_word_delete_real(iter->first);
+      }
+}
+#endif
